#=============================================================================
# BBS Note Comet 1.x
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2007/11/26
# Version : 0.40.000 build 1
#=============================================================================
package BN::CMS;
use strict;
use vars qw( $bn $VERSION );

$VERSION = '0.40.000';

sub new {
  my $class = shift;
  my $cms = bless { }, $class;
  $bn = $_[0];
  $cms;
}

sub menu {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $bbsnoteurl = $cms->{bbsnoteurl} = $bn->get($cf, 'BBSNoteUrl');
  $cms->{error} = [];

  if ($bn->{flag}{AUTHORITY} ne 'ADMIN') {
    my($pwd, $usr) = $bn->get_admin_pass;
    $bn->check_authority($pwd) or $bn->error('PASSWORD_ERROR');
  }

  my $item = $cms->{item} = $bn->text(undef, 'setting');
  my $_item = $bn->{sys}{CMS}{ITEM};
  map { $item->{$_} = $_item->{$_} } keys %$_item;

  # default
  $in->{mode} = 'view_bbs' unless $in->{mode};

  $cms->design;
  $bn->call( 'CMS_Menu' );
  $cms->write if $in->{cmd} eq 'write';
  $cms->preview if $in->{design_preview};
  $cms->config;
}

sub write { $bn->mo( 'Write' ) }

sub design {
  return unless defined $bn->get( $bn->{skin}, 'Design' );
  my $mo = $bn->{mo};
  my $pm = $bn->get( $bn->{skin}, 'Design');
  my $dir = $bn->get( $bn->{cfg}, 'SkinDir' ) ? $bn->get( $bn->{cfg}, 'SkinDir' ) : $bn->{sys}{DirSys};
  my $mod = "$dir/$pm";

  $bn->set_call( 'CMS_WriteMode', sub{
    eval { require $mod; $mo->{SkinDesign} = BN::CMS::Design->new( $bn ) };
    delete $INC{$mod};
    if ( $@ ) { print $@; return }
    if ( $bn->{in}{mode} eq 'skin_design' ) {
      eval { $mo->{SkinDesign}->design }; my $e = $@;
      $bn->error( 'ERR_BOOT_MOD', $pm, '<tt>'. $e. '</tt>' ) if $e;
    }
  } );

  $bn->set_call( 'CMS_ConfigMode', sub{
    return unless defined $bn->get( $bn->{skin}, 'Design' );
    unless ( exists $mo->{SkinDesign} ) {
      eval { require $mod; $mo->{SkinDesign} = BN::CMS::Design->new( $bn ) };
      if ( $@ ) { print $@; return }
    }
    delete $INC{$mod};
    if ( $bn->{in}{mode} eq 'skin_design' ) {
      eval { $mo->{SkinDesign}->config }; my $e = $@;
      $bn->error( 'ERR_BOOT_MOD', $pm, '<tt>'. $e. '</tt>' ) if $e;
    }
  } );
}

sub preview {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $mo = $bn->{module};
  my $sys = $bn->{sys};
  my $mode = $in->{design_mode};

  undef %$in;
  $bn->cfg_init->boot_init(
    $sys->{Config},
    "$sys->{DirCfgSys}/$sys->{Skin}.$sys->{Exe}",
    $sys->{SkinCfg}
  );

  if ($mode =~ /^_(\w+)/) {
    my $opt = $1;
    if ($opt =~ /(normal|thumbnail|catalog|noimage)/) {
      $in->{mode} = $1;
      $bn->open_page;
    }
    elsif ($opt eq 'edit') { $bn->open_edit }
    elsif ($opt eq 'help') { $bn->open_help }
    elsif ($opt eq 'post') { $bn->open_post }
    elsif ($opt eq 'admin') { $bn->open_admin }
    elsif ($opt eq 'delete') { $bn->open_delete }
    elsif ($opt eq 'reply' || $opt eq 'thread') {
      $bn->open_index;
      $in->{log} = $bn->read_index(0, 0);
      $bn->close_index;
      $opt eq 'reply' ? $bn->open_post : $bn->open_thread;
    }
    elsif ($opt eq 'tree') {
      $bn->set($cf, 'UseTree', 1);
      $bn->set($cf, 'UseTreeFrame', 0);
      $bn->open_page;
    }
  } elsif ($mode =~ /^paint/ && exists $mo->{Paint}) {
    $in->{fc} = $mode;
    my $pm = $bn->pm;
    for my $c(@{$mo->{Paint}{applets}}){
      if ($in->{fc} eq 'paint_' . $c->{key}) {
        $pm->paint($c);
      }
    }
  }
  $bn->EXIT;
}

sub config {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $mo = $bn->{module};
  my $mode = $in->{mode};
  my $item = $cms->{item};

  $bn->check_access('cms check');

  $cms->setting_menu;

  my $menu = "cfg_$mode";
     if ( defined &$menu )           { $cms->$menu                      }
  elsif ($mode eq 'text_head')       { $cms->cfg_text('TxtHead')        }
  elsif ($mode eq 'text_top')        { $cms->cfg_text('TxtTop')         }
  elsif ($mode eq 'text_bottom')     { $cms->cfg_text('TxtBottom')      }
  elsif ($mode eq 'text_bbs_head')   { $cms->cfg_text('TxtBbsHead')     }
  elsif ($mode eq 'text_bbs_top')    { $cms->cfg_text('TxtBbsTop')      }
  elsif ($mode eq 'text_bbs_bottom') { $cms->cfg_text('TxtBbsBottom')   }
  elsif ($mode eq 'text_msg_top')    { $cms->cfg_text('TxtMsgTop')      }
  elsif ($mode eq 'text_msg_bottom') { $cms->cfg_text('TxtMsgBottom')   }
  elsif ($mode eq 'text_tree_top')   { $cms->cfg_text('TxtTreeTop')     }
  elsif ($mode eq 'text_paint_head') { $cms->cfg_text('TxtPaintHead')   }
  elsif ($mode eq 'text_paint_top')  { $cms->cfg_text('TxtPaintTop')    }
  elsif ($mode eq 'text_paint_down') { $cms->cfg_text('TxtPaintBottom') }
  elsif ($mode eq 'text_spa_head')   { $cms->cfg_text('TxtSpaHead')     }
  elsif ($mode eq 'text_spa_top')    { $cms->cfg_text('TxtSpaTop')      }
  elsif ($mode eq 'text_spa_down')   { $cms->cfg_text('TxtSpaBottom')   }
  elsif ($mode eq 'text_sii_head')   { $cms->cfg_text('TxtSiiHead')     }
  elsif ($mode eq 'text_sii_top')    { $cms->cfg_text('TxtSiiTop')      }
  elsif ($mode eq 'text_sii_down')   { $cms->cfg_text('TxtSiiBottom')   }
  elsif ($mode eq 'text_spv_head')   { $cms->cfg_text('TxtSpvHead')     }
  elsif ($mode eq 'text_spv_top')    { $cms->cfg_text('TxtSpvTop')      }
  elsif ($mode eq 'text_spv_down')   { $cms->cfg_text('TxtSpvBottom')   }
  elsif ($mode eq 'text_pic_head')   { $cms->cfg_text('TxtPicHead')     }
  elsif ($mode eq 'text_pic_top')    { $cms->cfg_text('TxtPicTop')      }
  elsif ($mode eq 'text_pic_down')   { $cms->cfg_text('TxtPicBottom')   }
  elsif ($mode eq 'text_pai_head')   { $cms->cfg_text('TxtPaiHead')     }
  elsif ($mode eq 'text_pai_top')    { $cms->cfg_text('TxtPaiTop')      }
  elsif ($mode eq 'text_pai_down')   { $cms->cfg_text('TxtPaiBottom')   }

  elsif ($mode eq 'skin_stylesheet') { $cms->cfg_css('StyleSheet')      }
  elsif ($mode eq 'skin_number')     { $cms->cfg_skin_tags('Number')    }
  elsif ($mode eq 'skin_title')      { $cms->cfg_skin_tags('Title')     }
  elsif ($mode eq 'skin_user')       { $cms->cfg_skin_tags('User')      }
  elsif ($mode eq 'skin_message')    { $cms->cfg_skin_tags('Message')   }
  elsif ($mode eq 'skin_edit')       { $cms->cfg_skin_tags('Edit')      }

  elsif ($mode eq 'design_head')     { $cms->cfg_design('HEAD')         }
  elsif ($mode eq 'design_menu')     { $cms->cfg_design('MENU')         }
  elsif ($mode eq 'design_main')     { $cms->cfg_design('MAIN')         }
  elsif ($mode eq 'design_object')   { $cms->cfg_design('OBJECT')       }
  elsif ($mode eq 'design_catalog')  { $cms->cfg_design('CATALOG')      }
  elsif ($mode eq 'design_tree')     { $cms->cfg_design('TREE')         }
  elsif ($mode eq 'design_frame')    { $cms->cfg_design('FRAME')        }
  elsif ($mode eq 'design_foot')     { $cms->cfg_design('FOOT')         }
  elsif ($mode eq 'design_post')     { $cms->cfg_design('POST')         }
  elsif ($mode eq 'design_search')   { $cms->cfg_design('SEARCH')       }

  $bn->call( 'CMS_ConfigMode' );

  unless ($in->{edit} || $mode =~ /system_recover|archive/) {

    $bn->print('<tr><td colspan="2"><hr class="ConfigHr"></td></tr><tr><td colspan="2" align="right" nowrap>');

    if ($cms->{design_preview}) {
      $bn->print('<script type="text/javascript">'."\n",
      "function CMS_design_preview() {\n",
      "d = document.config; d.target = d.target == '_self' ? 'CMSPreview' : '_self'}\n</script>\n",
      '<input type="checkbox" name="design_preview" value="1" onclick="CMS_design_preview()"> ',
      $item->{design_preview}, '<select name="design_mode">');

      for (qw/normal thumbnail catalog noimage delete edit admin help post reply thread tree/) {
        $bn->print('<option value="_'.$_.'">'. $item->{'design_'.$_});
      }
      if (exists $mo->{Paint}) {
        my $pm = $mo->{Paint};
        my $app = $pm->{applets};
        for (@{$pm->{applets}}) { $bn->print('<option value="paint_' . $_->{key} . '">' . $_->{name}) }
      }
      $bn->print('</select>');
    }

    $bn->print('<input type="submit" class="ConfigBtn" value="',$item->{submit},'"',$item->{submit_ext},'> ',
      '<input type="reset" class="ConfigBtn" value="',$item->{clear},'"></td></tr></form>');

  }

  $bn->print('</table></td></tr></table></td></tr></table></div>');
  $bn->foot;

}

sub load_picker {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $mode = $bn->{in}{mode};

  return unless ( $bn->{global}{UserOS} =~ /Win/ && $bn->{global}{UserBrowserVer} =~ /IE/ ) ||
  ( $bn->{global}{UserBrowser} =~ /^(?:Firefox|Safari|Opera)$/ );

  return unless $cms->{load_picker} ||
   $mode =~ /^(?:paint_palette|user_names|skin_page|skin_design|skin_number|skin_title|skin_user|skin_message|skin_edit|skin_form|skin_catalog|skin_frame|skin_shipainter|skin_shipaintbbs|skin_pchviewer|skin_picturebbs|skin_dynpalette)$/;

  my $sk = $bn->{skin};
  my $c1 = ref $bn->get($sk, 'TagTableOut') eq 'HASH' ? $bn->get($sk, 'TagTableOut')->{bgcolor} :
    ref $bn->get($sk, 'TagBody') eq 'HASH' ? $bn->get($sk, 'TagBody')->{text} : '#000000';
  my $c2 = ref $bn->get($sk, 'TagTableIn') eq 'HASH' ? $bn->get($sk, 'TagTableIn')->{bgcolor} :
    ref $bn->get($sk, 'TagBody') eq 'HASH' ? $bn->get($sk, 'TagBody')->{bgcolor} : '#FFFFFF';

  $bn->print(<<'_HTML_');
<script type="text/javascript">
var S = new Array('<br>','&nbsp;','textarea',' border="0"',' width=',' align=',' type=',
'</td></form></tr></table>',' size="2"',' name=',' color=',' value=',' bgcolor=',' tabindex=',
' onclick=',' cellpadding=',' cellpadding="0"',' cellspacing="0"',' class="button"','<input type="hidden" name=',
'<input','<select','</select>','<option','</option>','><tr class="layerd1"><td>','</td><td>','</td><tr>',
'</td></tr></table>','<table','<font',' colspan="2"','><tr><td')

var TC = new Array(
_HTML_
  $bn->print("'$c1','$c2'");
  $bn->print(<<'_HTML_');
)
S[50] = S[29]+S[3]+" style='border:1px solid "+TC[1]+"'"+S[17]+S[15]+"0"+S[12];
var CTable =
"000000,CCFFFF,CCFFCC,CCFF99,CCFF66,CCFF33,CCFF00,66FF00,66FF33,66FF66,66FF99,66FFCC,66FFFF,11FFFF,00FFCC,00FF99,00FF66,00FF33,11FF11,"+
"333333,CCCCFF,CCCCCC,CCCC99,CCCC66,CCCC33,CCCC00,66CC00,66CC33,66CC66,66CC99,66CCCC,66CCFF,00CCFF,00CCCC,00CC99,00CC66,00CC33,00CC00,"+
"666666,CC99FF,CC99CC,CC9999,CC9966,CC9933,CC9900,669900,669933,669966,669999,6699CC,6699FF,0099FF,0099CC,009999,009966,009933,009900,"+
"999999,CC66FF,CC66CC,CC6699,CC6666,CC6633,CC6600,666600,666633,666666,666699,6666CC,6666FF,0066FF,0066CC,006699,006666,006633,006600,"+
"CCCCCC,CC33FF,CC33CC,CC3399,CC3366,CC3333,CC3300,663300,663333,663366,663399,6633CC,6633FF,0033FF,0033CC,003399,003366,003333,003300,"+
"FFFFFF,CC00FF,CC00CC,CC0099,CC0066,CC0033,CC0000,660000,660033,660066,660099,6600CC,6600FF,1111FF,0000CC,000099,000066,000033,001100,"+
"FF0000,FF11FF,FF00CC,FF0099,FF0066,FF0033,FF1111,990000,990033,990066,990099,9900CC,9900FF,3300FF,3300CC,330099,330066,330033,330000,"+
"00FF00,FF33FF,FF33CC,FF3399,FF3366,FF3333,FF3300,993300,993333,993366,993399,9933CC,9933FF,3333FF,3333CC,333399,333366,333333,333300,"+
"0000FF,FF66FF,FF66CC,FF6699,FF6666,FF6633,FF6600,996600,996633,996666,996699,9966CC,9966FF,3399FF,3399CC,339999,339966,339933,339900,"+
"FFFF00,FF99FF,FF99CC,FF9999,FF9966,FF9933,FF9900,999900,999933,999966,999999,9999CC,9999FF,3399FF,3399CC,339999,339966,339933,339900,"+
"00FFFF,FFCCFF,FFCCCC,FFCC99,FFCC66,FFCC33,FFCC00,99CC00,99CC33,99CC66,99CC99,99CCCC,99CCFF,33CCFF,33CCCC,33CC99,33CC66,33CC33,33CC00,"+
"FF00FF,FFEEFF,FFFFCC,FFFF99,FFFF66,FFFF33,FFFF11,99FF00,99FF33,99FF66,99FF99,99FFCC,99FFFF,33FFFF,33FFCC,33FF99,33FF66,33FF33,33FF00";
C=CTable.split(",");CTable="";
CTable="<table border=0 cellspacing=1 cellpadding=0><tr><td><table style='border:1px solid "+TC[1]+"' cellspacing=0 cellpadding=1><tr>"
for(n = 0 ,cnt=0; n < C.length; n++,cnt++){
CTable += "<td style='border:1px solid "+TC[1]+"' bgcolor=#"+C[n]+" onclick=\"PC_set('"+C[n]+"')\" onmouseover=\"PC_focus(this,'"+C[n]+"')\" onmouseout=\"fc_flag=0\"><div style=\"font-size:10px;width:10px;height:10px\" >"+S[1]+"</div></td>"
if( cnt>17  && n < 220){ CTable += "</tr><tr>" ; cnt = -1}}
CTable+="</tr></table></td></tr></table>"
var obj;var tobj;var cobj;var tid;var flag;var fc_flag;var picker;
document.write('<div id="picker" style="position:absolute;visibility:hidden;width:120px;height:30px;left:50px;top:0px"></div>');
function PC_focus(t,c) {cobj=c="#"+c;fc_flag=1;status = c;
if(tobj)tobj.runtimeStyle.borderColor = '';t.runtimeStyle.borderColor='red';tobj=t;}
function PC_color(c){c=c.toUpperCase();c=c.match("([0-9A-F]+)");
if(c==null){return}else{if(c[0].length < 6){for(p=c[0].length;p<6;p++){c[0]+='0'}}
else if(c[0].length > 6){c[0]=c[0].substr(0,6)}return '#'+c[0]}}
function PC_set(c){fc_flag=0;if(!obj){obj=event.srcElement}
if(c.length<1){obj.value='';obj.style.backgroundColor='';return}
c=PC_color(c);obj.value=c;obj.style.backgroundColor=c
r=parseInt("0x"+c.substr(1,2));g=parseInt("0x"+c.substr(3,2));b=parseInt("0x"+c.substr(5,2));
obj.style.color = 99>parseInt((r+g+b)/3)?"#FFFFFF":"#000000";PC_hide();obj.focus();status = ""}
function PC_hide(){if(fc_flag|!picker){return}picker.style.visibility="hidden"}
function PC_open(o,e){obj=o;d=document;if(document.all){cW=d.body.clientWidth;
L=parseInt(window.event.clientX);T=parseInt(window.event.clientY+document.body.scrollTop);
picker=d.all.picker;s=picker.style;}else{cW=window.innerWidth;L=e.pageX;
T=e.pageY;picker=document.getElementById('picker');s=picker.style;}
if(cW<(L+290)){L=cW-290}s.left=L;s.top=T+15;s.width=242;s.height=10;s.visibility="visible";
if(!flag){picker.innerHTML=S[50]+TC[0]+" style='cursor:crosshair'"+S[25]+S[29]+S[12]+TC[0]+S[25]+CTable+S[28]+S[28]}flag=1
}
</script>
_HTML_

  $cms->{picker} = 1;
}

sub cfg_archive {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $dir = $bn->{sys}{DirImages};
  my $dir_url = $bn->get($cf, 'DirImagesUrl');
  my $item = $cms->{item};
  my $args = $bn->{arg}{SettingMenu};
  $args->{fc} = 'setting';
  $args->{mode} = $in->{mode};
  $args->{name} = $in->{name};
  $args->{password} = $in->{password};
  my $arg = $bn->set_form_arg($bn->{arg}{SettingMenu});
  my $body = $bn->get($sk, 'TagBody');
  my $table_in = $bn->get($sk, 'TagTableIn');
  my $table_out = $bn->get($sk, 'TagTableOut');
  my $limit_num = $bn->{sys}{CMS}{LIMIT_NUM};

  $bn->error('ERR_DIR_NOTFOUND',$dir) unless -d $dir;

  my $limit_list = 100;
  my $limit_file = 1024000;
  my $limit_mime = '.*';
  if (exists $limit_num->{archive}){
    $limit_list = $limit_num->{archive}{list};
    $limit_file = $limit_num->{archive}{file};
    $limit_mime = $limit_num->{archive}{mime};
    $dir_url = $limit_num->{archive}{url} if exists $limit_num->{archive}{url};
  }

  my(@list, %files);
  my $cnt = 0;
  my $dh = $bn->opendir($dir);
  $dh or $bn->error('ERR_DATA_NOTOPEN', $dir);
  while ($_ = readdir $dh) {
    next if /^\.\.?/;
    $cnt++;
    push @list, $_;
    $files{$_} = -s "$dir/$_";
  }
  close $dh;

  @list = sort @list;

  $bn->print('<tr><td><select name="list" size="20" style="width:300" onclick="CMS_Preview()" onchange="CMS_Preview()">');

  for (@list) { $bn->print("<option value='$_'>$_  ($files{$_} byte)\n") }

$bn->print(<<_HTML_);
</select></td></form>
<td valign="top" width="100%" style="height:100%"><table cellpadding="5" cellspacing="0"
 style="width:100%;background-color:$table_in->{bgcolor};height:100%">
  <tr style="font-size:10pt;color:$table_in->{bgcolor};background-color:$table_out->{bgcolor}"><td>$item->{archive_preview}</td></tr>
  <tr><td id="CMSpreview" align="center" style="width:200px;height:100%;
  background-repeat: no-repeat; background-position: center center">&nbsp;</td></tr>
</table></td>
</tr>
<tr><td colspan="2" class="ConfigTable">

<table style="font-size:10pt">
<tr><form action="$cms->{bbsnoteurl}" method="post" enctype="multipart/form-data">$arg
<input type="hidden" name="opt" value="upload">
<td><input type="submit" class="ConfigBtn" value="$item->{archive_upload}"> : <input type="file" name="file"></td></form></tr>
<tr><form name="del" action="$cms->{bbsnoteurl}" method="post">$arg
<input type="hidden" name="opt" value="delete">
<td><input type="submit" class="ConfigBtn" value="$item->{archive_delete}"> : <input type="text" name="filename"></td></form></tr>
<tr><form name="rename" action="$cms->{bbsnoteurl}" method="post">$arg
<input type="hidden" name="opt" value="rename">
<td><input type="submit" class="ConfigBtn" value="$item->{archive_rename}"> : <input type="text" name="from" style="text-align:right;border:none;background-color:$body->{bgcolor};color:$body->{text}"> &gt; <input type="text" name="to"></td>
</form></tr>
<tr><td><b>$item->{archive_limit_list}</b> : $cnt / $limit_list</td></tr>
<tr><td><b>$item->{archive_limit_file}</b> : $limit_file $item->{byte}</td></tr>
</table>$item->{manual_archive}</td></tr><script type="text/javascript">
<!--
dir = "$dir_url/";
function CMS_Preview(){
  var s='';d=document;c=d.config;
  p=d.all?d.all.CMSpreview:d.getElementById?d.getElementById('CMSpreview'):null;
  if(p){i=c.list.selectedIndex;if(i>=0){s=c.list.options[i].value}
  else if(c.list.options.length){s=c.list.options[0].value}
  d.del.filename.value=d.rename.from.value=d.rename.to.value=s;
  p.style.backgroundImage=s.match("jpg|gif|png\$")?'url('+dir+s+')':''}
}
CMS_Preview()
//-->
</script></td></tr>
_HTML_



}

sub cfg_design_copyright {
  my $cms = $_[0];
  my $sk = $bn->{skin};

  $cms->txt('skin_name', $bn->get($sk, 'SkinName'));
  $cms->txt('skin_ver', $bn->get($sk, 'SkinVersion'));
  $cms->txt('skin_lastmodify', $bn->get($sk, 'SkinLastmodify'));
  $cms->txt('skin_auth', $bn->get($sk, 'SkinAuthor'));
  $cms->txt('skin_copy', $bn->get($sk, 'SkinCopyright'));
  $cms->txt('skin_authurl', $bn->get($sk, 'SkinAuthorSiteURL'), 50);

}

sub cfg_design {
  my( $cms, $tmpl ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $item = $cms->{item};
  my %foot = %{$sk->{template}{FOOT}};
  $cms->{design_preview} = 1;
  my $tmplate;
  if ( exists $in->{ $in->{mode} } && @{ $cms->{error} } ) {
    $tmplate = $in->{ $in->{mode} };
  } else {
    $bn->read_skin( "$sys->{DirCfgSys}/$sys->{Skin}.$sys->{Exe}" );
    $tmplate = $sk->{template}{ $tmpl }{data};
  }

  $cms->txa( $in->{mode}, $tmplate, 60, 20 );
  $bn->print( $item->{manual_design} ) if exists $item->{manual_design};
  $sk->{template}{FOOT} = \%foot;
}

sub cfg_skin_dynpalette {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'AppletColorDynPalette');
  $cms->{design_preview} = 1;

  $cms->col('palette_border', $cfg->{palette_border}, 10);
  $cms->col('palette_back', $cfg->{palette_back}, 10);
  $cms->col('palette_font', $cfg->{palette_font}, 10);
  $cms->col('gradation_border', $cfg->{gradation_border}, 10);
  $cms->col('gradation_back', $cfg->{gradation_back}, 10);
  $cms->col('gradation_font', $cfg->{gradation_font}, 10);
}

sub cfg_skin_picturebbs {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'AppletColorPictureBBS');
  $cms->{design_preview} = 1;

  $cms->col('bgcolor', $cfg->{_BGCOLOR}, 10);
  $cms->col('window_color', $cfg->{_WINDOWCOLOR}, 10);
  $cms->col('caption_color', $cfg->{_CAPTIONCOLOR}, 10);
}

sub cfg_skin_pchviewer {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'AppletColorShiPCHViewer');

  $cms->col('color_text', $cfg->{color_text}, 10);
  $cms->col('color_back', $cfg->{color_back}, 10);
  $cms->col('color_icon', $cfg->{color_icon}, 10);
  $cms->col('color_iconselect', $cfg->{color_iconselect}, 10);
  $cms->col('color_bar', $cfg->{color_bar}, 10);
  $cms->col('color_frame', $cfg->{color_frame}, 10);
  $cms->col('color_bar_select', $cfg->{color_bar_select}, 10);
}

sub cfg_skin_shipaintbbs {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'AppletColorShiPaintBBS');
  $cms->{design_preview} = 1;

  for ( qw/color_text color_bk color_bk2 color_icon color_frame
  color_iconselect color_bar color_bar_frame_hl color_bar_hl
  tool_color_text tool_color_frame tool_color_button tool_color_button2 tool_color_bar/ )
   { $cms->col( $_, $cfg->{$_}, 10) }
}


sub cfg_skin_shipainter {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'AppletColorShiPainter');
  $cms->{design_preview} = 1;

  for ( qw/
  color_text color_bk color_bk2
  tool_color_text tool_color_bk tool_color_frame
  tool_color_button tool_color_button2 tool_color_button_hl tool_color_button_dk color_iconselect tool_color_bar
  color_bar color_bar_hl color_bar_shadow color_frame color_icon
  bar_color_text bar_color_bk bar_color_frame bar_color_off bar_color_off_hl bar_color_off_dk
  bar_color_on bar_color_on_hl bar_color_on_dk
  window_color_text window_color_bk window_color_frame window_color_bar_text window_color_bar window_color_bar_hl
  pro_menu_color_text pro_menu_color_on pro_menu_color_on_hl pro_menu_color_on_dk
  pro_menu_color_off pro_menu_color_off_dk pro_menu_color_off_hl
  l_m_color l_m_color_text dlg_color_bk dlg_color_text
  / ) {
     $cms->col( exists $item->{'spa_'.$_} ? 'spa_'.$_ : $_, $cfg->{$_}, 10 );
  }

}

sub cfg_skin_frame {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'TagTableOut');
  $cms->{design_preview} = 1;
  for ('bgcolor','width','cellpadding','cellspacing') {
    $item->{'out_' . $_} = $item->{$_};
    $item->{'in_' . $_} = $item->{$_};
  }

  $cms->fld_set('skin_frame_out');
  $cms->col('out_bgcolor', $cfg->{bgcolor}, 10);
  $cms->txt('out_width', $cfg->{width}, 5);
  $cms->txt('out_cellpadding', $cfg->{cellpadding}, 5);
  $cms->txt('out_cellspacing', $cfg->{cellspacing}, 5);
  $cms->fld_end('skin_frame_out');

  $cfg = $bn->get($sk, 'TagTableIn');
  $cms->fld_set('skin_frame_in');
  $cms->col('in_bgcolor', $cfg->{bgcolor}, 10);
  $cms->txt('in_width', $cfg->{width}, 5);
  $cms->txt('in_cellpadding', $cfg->{cellpadding}, 5);
  $cms->txt('in_cellspacing', $cfg->{cellspacing}, 5);
  $cms->fld_end('skin_frame_in');

  $cfg = $bn->get($sk, 'TagTableFont');
  $cms->fld_set('skin_frame_font');
  $cms->col('font_color', $cfg->{color}, 10);
  $cms->txt('font_size', $cfg->{size}, 5);
  $cms->fld_end('skin_frame_font');

}
sub cfg_skin_catalog {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg = $bn->get($sk, 'TagCatalogTableOut');
  $cms->{design_preview} = 1;
  for ('bgcolor','width','cellpadding','cellspacing') {
    $item->{'out_' . $_} = $item->{$_};
    $item->{'in_' . $_} = $item->{$_};
  }

  $cms->fld_set('skin_catalog_out');
  $cms->col('out_bgcolor', $cfg->{bgcolor}, 10);
  $cms->txt('out_width', $cfg->{width}, 5);
  $cms->txt('out_cellpadding', $cfg->{cellpadding}, 5);
  $cms->txt('out_cellspacing', $cfg->{cellspacing}, 5);
  $cms->fld_end('skin_catalog_out');

  $cfg = $bn->get($sk, 'TagCatalogTableIn');
  $cms->fld_set('skin_catalog_in');
  $cms->col('in_bgcolor', $cfg->{bgcolor}, 10);
  $cms->txt('in_width', $cfg->{width}, 5);
  $cms->txt('in_cellpadding', $cfg->{cellpadding}, 5);
  $cms->txt('in_cellspacing', $cfg->{cellspacing}, 5);
  $cms->fld_end('skin_catalog_in');

}

sub cfg_skin_form {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_form = $bn->get($sk, 'TagFormItems');
  my $cfg_ast = $bn->get($sk, 'TagFormAstMark');
  $cms->{design_preview} = 1;

  for ('string','font_color','font_size') {
    $item->{'skin_form_ast_' . $_} = $item->{$_};
  }

  $cms->cfg_tags($bn->get($sk, 'TagFormItems'), 'item', 0, 'skin_form');
  $cms->fld_set('skin_form_ast');
  $cms->txt('skin_form_ast_string', $cfg_ast->{string}, 5);
  $cms->col('skin_form_ast_font_color', $cfg_ast->{color}, 10);
  $cms->txt('skin_form_ast_font_size', $cfg_ast->{size}, 3);
  $cms->fld_end('skin_form_ast');

}

sub cfg_skin_tags {
  my( $cms, $tag ) = @_;
  my $sk = $bn->{skin};
  $cms->{design_preview} = 1;
  $cms->cfg_tags($bn->get($sk, 'TagMain' . $tag), 'main', 0, 'skin_main');
  $cms->cfg_tags($bn->get($sk, 'TagRes' . $tag), 'res', 0, 'skin_res');
}
sub cfg_tags {
  my( $cms, $cfg, $tag, $strflag, $field ) = @_;
  my $item = $cms->{item};

  my $c = ' checked';
  my $b = $c if $cfg->{bold};
  my $i = $c if $cfg->{italic};
  my $u = $c if $cfg->{under};
  my $t = $c if $cfg->{teletype};
  $cms->{design_preview} = 1;

  my $cp;
  if ($cms->{picker}) {
    $cp = ' onclick="PC_open(this,event)" onblur="PC_hide()" onchange="PC_set(this.value)"';
    my $tc;
    if ($cfg->{color} =~ /\#(\w\w)(\w\w)(\w\w)/) {
       $tc = ';color:';
       $tc .= 99 > int (((hex $1) + (hex $2) + (hex $3)) / 3) ? '#FFFFFF' : '#000000';
    }
    $cp .= ' style="background-color:'.$cfg->{color}.$tc.'"';
  }

  if ($field) {
    $bn->print('<td colspan="2"><fieldset class="ConfigFld"><legend>',
     $item->{$field}, '</legend><table style="font-size:10pt">');
  }

  if ($strflag) {
    $bn->print('<tr><td align="right">',$item->{string},
    '</td><td><input type="text" name="',$tag,'_string" value="',$cfg->{string},'" size="40"></td></tr>');
  }


$bn->print(<<_HTML_);
<tr><td align="right">$item->{font_color}</td>
<td><input type="text" name="$tag\_color" value="$cfg->{color}" size="9"$cp>
$item->{font_size} <input type="text" name="$tag\_size" value="$cfg->{size}" size="2">
</td></tr>
<tr><td align="right">$item->{font_face}</td>
<td><input type="text" name="$tag\_face" value="$cfg->{face}" size="40"></td></tr>
<tr><td align="right">$item->{style}</td>
<td><input type="text" name="$tag\_style" value="$cfg->{style}" size="40"></td></tr>
<tr><td>&nbsp;</td><td>
<input type="checkbox" name="$tag\_bold" value="1"$b>$item->{bold}&nbsp;
<input type="checkbox" name="$tag\_italic" value="1"$i>$item->{italic}&nbsp;
<input type="checkbox" name="$tag\_under" value="1"$u>$item->{under}&nbsp;
<input type="checkbox" name="$tag\_teletype" value="1"$t>$item->{teletype}</td></tr>
_HTML_

  if ($field) {
    $bn->print('</table></fieldset><br></td></tr>');
  }

}

sub cfg_skin_page {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  $cms->{design_preview} = 1;

  my $cfg = $bn->get($sk, 'TagBody');
  for ('bgcolor','text','link','alink','vlink','background',
  'leftmargin','topmargin','marginwidth','marginheight') {
    $item->{'skin_page_' . $_} = $item->{$_};
  }

  $cms->col('skin_page_bgcolor', $cfg->{bgcolor}, 10);
  $cms->col('skin_page_text', $cfg->{text}, 10);
  $cms->col('skin_page_link', $cfg->{link}, 10);
  $cms->col('skin_page_alink', $cfg->{alink}, 10);
  $cms->col('skin_page_vlink', $cfg->{vlink}, 10);

  $cms->txt('skin_page_background', $cfg->{background}, 40);
  $cms->inf('skin_page_txt');
  $cms->hr;

  $cms->txt('skin_page_leftmargin', $cfg->{leftmargin}, 10);
  $cms->txt('skin_page_topmargin', $cfg->{topmargin}, 10);
  $cms->txt('skin_page_marginwidth', $cfg->{marginwidth}, 10);
  $cms->txt('skin_page_marginheight', $cfg->{marginheight}, 10);

}

sub cfg_css {
  my $cms = $_[0];
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $txt = $bn->get($sk, $_[1]);
  $cms->{design_preview} = 1;
  $cms->txa($bn->{in}{mode}, (join "\n", @$txt), 60, 20);
}

sub cfg_text {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $cms->{item};
  $item->{'manual_'.$bn->{in}{mode}} = $item->{'manual_text'};
  $cms->txa($bn->{in}{mode},
  (@{ $cms->{error} } ? $in->{$in->{mode}} :
     join "\n", @{$bn->get($cf, $_[1])}), 60, 20);
}
sub cfg_system_recover {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $bbs = $bn->get($cf, 'BBSNoteUrl');
  my $args = $bn->{arg}{AdminMenuForm};
  $args->{fc} = 'admin';
  $args->{name} = $in->{name};
  $args->{password} = $in->{password};
  my $arg = $bn->set_form_arg($args);
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  $bn->error('ERR_FUNCTION') if exists $disable_menu->{system_recover};

  $bn->print('</form><tr><td colspan="2">');


$bn->print(<<_HTML_) unless exists $disable_menu->{system_recover_index};
<form method="post" action="$bbs">$arg
<input type="hidden" name="cmd" value="recover_sysindex">
<input class="ConfigBtn" type="submit" value="$item->{system_recover_index}"><br>
$item->{system_recover_index_txt}
</form>
_HTML_


$bn->print(<<_HTML_) unless exists $disable_menu->{system_recover_backup};
<form method="post" action="$bbs">$arg
<input type="hidden" name="cmd" value="recover_backupindex">
<input class="ConfigBtn" type="submit" value="$item->{system_recover_backup}"><br>
$item->{system_recover_backup_txt}
</form>
_HTML_

$bn->print(<<_HTML_) unless exists $disable_menu->{system_recover_file};
<form method="post" action="$bbs" enctype="multipart/form-data">$arg
<input type="hidden" name="cmd" value="recover_fileindex">
<input class="ConfigBtn" type="submit" value="$item->{system_recover_file}">
<input type="file" name="file"><br>
$item->{system_recover_file_txt}
</form>
_HTML_

  $bn->print('</td></tr>');


}

sub cfg_system_counter_tmpl_image {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $item = $cms->{item};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $bn->error('ERR_FUNCTION') if exists $disable_menu->{system_log};

  my $cfg = 'TxtCounterImage';
  my $txt = $bn->get($cf, $cfg);
  if ( ref $txt ne 'ARRAY' || !@$txt ) {
    $bn->define($cf, $cfg, undef, '@');
    my @tmpl = (
      '<SKIN ACCESSIMAGECOUNTER>',
    );
    for (@tmpl) { $bn->set($cf, $cfg, $_) }
  }

  $cms->cfg_text($cfg);

}

sub cfg_system_counter_tmpl_text {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $item = $cms->{item};
  my $cfg = 'TxtCounterText';
  my $txt = $bn->get($cf, $cfg);
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $bn->error('ERR_FUNCTION') if exists $disable_menu->{system_log};

  if ( ref $txt ne 'ARRAY' || !@$txt ) {
    $bn->define($cf, $cfg, undef, '@');
    my @tmpl = (
      '<div class="AccessCount">$AccessTotal</div>',
    );
    for (@tmpl) { $bn->set($cf, $cfg, $_) }
  }
  $cms->cfg_text($cfg);
}

sub cfg_system_counter_image_config {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $item = $cms->{item};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $bn->error('ERR_FUNCTION') if exists $disable_menu->{system_log};

  my $dir = $bn->{sys}{DirSys}.'/access/images/';
  my $is = join '', map{ "<option value=\"$_\">$_$item->{image}</option>" }(0..9);

  $bn->print(
    '<tr><td nowrap>', $item->{counter_image_upload}, '</td>',
    '<td width="90%"><select name="num">', $is, '</select>',
    '<input type="file" name="file" accept="image/gif"></td></tr>'
  );

  my $cnt_dither = $bn->get($cf, 'ViewCounterImageDither');
  $cnt_dither = 0 if $cnt_dither eq '';
  my $cnt_color = $bn->get($cf, 'ViewCounterImageColors');
  $cnt_color = 64 if $cnt_color eq '';
  my $cnt_trans = $bn->get($cf, 'ViewCounterImageTransparent');
  $cnt_trans = -1 if $cnt_trans eq '';
  $cms->txt('counter_image_trans', $cnt_trans, 4);
  $cms->inf('counter_image_trans_txt');
  $cms->txt('counter_image_colors', $cnt_color, 4);
  $cms->chk('counter_image_dither', $cnt_dither);

  for (0..9) {
    $bn->print( '<tr><td align="right">'.$_.$item->{image}.'</td><td>' );
#    for my $exe (qw/gif png jpg/) { $flag = 1 and last if -e $dir.$_.'.'.$exe }
    my $size = -s $dir.$_.'.gif';
    $size = $size ? $size . $item->{byte} :'-';
    $bn->print( $size, '</td></tr>' );
  }

  1;
}

sub cfg_system_log {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $item = $cms->{item};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  my $limit_str = $bn->{sys}{CMS}{LIMIT_STR};

  my $cntup = $bn->get($cf, 'LimitCountUpMin');
  $cntup = 60 if $cntup eq '';
  my $acc_log = $bn->get($cf, 'LimitAccessLog');
  $acc_log = 365 if $acc_log eq '';
  my $cnt_pos = $bn->get($cf, 'ViewCounterPos');
  $cnt_pos = 'msgbottom' if $cnt_pos eq '';
  my $cnt_mode = $bn->get($cf, 'ViewCounterMode');
  $cnt_mode = 'add' if $cnt_mode eq '';
  my $skin_pos =$bn->get($sk, 'ViewCounter');
  $cnt_pos = 'skin' if $skin_pos;

  $bn->print(<<_HTML_);
<script type="text/javascript">
d=document;c=d.config;
c.cmd.value='write';c.mode.value='system_log';
function CF_reset(){c.cmd.value='write';c.mode.value='system_log';c.fc.value='setting'}
function CF_jump(m,v,f){c.cmd.value=v;c.mode.value=m;c.fc.value=f?f:'setting';c.submit()}
function CF_view(m){c.imageconfig.style.visibility= (m=='image'?'visible':'hidden')}
</script>
_HTML_

  $item->{submit_ext} = ' onmouseover="CF_reset()"';

 unless ( exists $disable_menu->{use_access_log} ) {
    my $cfg = 'use_access_log';
    my $val = $bn->get($cf, 'UseAccessLog');
    my $s = $val ? ' checked' : '';
    $bn->print( '<tr><td align="right"><input type="checkbox" name="', $cfg,
      '" value="1"', $s, '></td><td>', $cms->{item}{$cfg} );
    $bn->print( '<script type="text/javascript">',
      'd.write(\'<input class="ConfigBtn" type="button" value="', $item->{system_access_log},
      '" onclick="CF_jump\\(\\\'\\\',\\\'\\\',\\\'access_log\\\'\\)">\');', "\n",
      '</script>'
    ) if $val;
    $bn->print( '</td></tr>' );
 }

  $cms->txt('limit_access_log', $acc_log, 5, $item->{day});
  $cms->opt('limit_access_log_lv', $bn->get($cf, 'LimitAccessLogLevel'), ['0','1','2','debug']);
  $cms->inf('limit_access_log_lv_txt') unless exists $disable_menu->{limit_access_log_lv};
  $cms->hr;

  $item->{use_counter_0} = $item->{off};

 unless ( exists $disable_menu->{use_counter} ) {
    my $cfg = 'use_counter';
    my $val = $bn->get($cf, 'UseCounter');
    my $list = [qw/0 text image/];
    $list = $limit_str->{$cfg} if exists $limit_str->{$cfg};

    $bn->print(<<_HTML_);
<tr><td width="10%" align="right" nowrap>$cms->{item}{$cfg}</td>
<td><select name="$cfg" onchange="CF_view(this.value)">
_HTML_

    for (@$list) {
      $bn->print("<option value='$_'" . ($val eq $_ ? ' selected' : '') . '>'
      . $cms->{item}{$cfg . '_' . $_} . '</option>')}
    $bn->print(<<_HTML_);
</select>
<script type="text/javascript">
d.write('<input name="imageconfig" class="ConfigBtn" type="button" value="$item->{system_counter_image_config}" onclick="CF_jump\\(\\'system_counter_image_config\\'\\)">');
CF_view(c.$cfg.options[c.$cfg.selectedIndex].value)
</script>
</td></tr>
_HTML_

  $cms->opt('view_counter_pos', $cnt_pos, $skin_pos ? ['skin'] : ['bbstop','bbsbottom','msgtop','msgbottom']);
  $cms->opt('view_counter_mode', $cnt_mode, ['add','insert']);
  $cms->txt('limit_countup_min', $cntup, 5, $item->{min});
  $cms->inf('limit_countup_min_txt') unless exists $disable_menu->{limit_countup_min};

  unless ( exists $disable_menu->{counter_value} ) {
    $bn->print(<<_HTML_);
<tr><td align="right">$item->{counter_value}</td><td><noscript>$item->{noscript}</noscript>
<script type="text/javascript">
d.write(
'$item->{counter_value_total} : <input name="total" type="text" value="$gl->{accesslog}[1]" size="9" maxlength="9">&nbsp;'+
'$item->{counter_value_today} : <input name="today" type="text" value="$gl->{accesslog}[2]" size="9" maxlength="9">&nbsp;'+
'$item->{counter_value_yesterday} : <input name="yesterday" type="text" value="$gl->{accesslog}[3]" size="9" maxlength="9">&nbsp;'+
'<input class="ConfigBtn" type="button" value="$item->{counter_value_submit}" onclick="CF_jump\\(\\'system_counter_value\\',\\'write\\'\\)">'
);
</script>
</td></tr>
_HTML_
  }

  unless ( exists $disable_menu->{system_counter_tmpl} ) {
  my $templ_text = 'd.write(\'<input class="ConfigBtn" type="button" value="'.$item->{system_counter_tmpl_text}.'" onclick="CF_jump\\(\\\'system_counter_tmpl_text\\\'\\)">\')'
    unless exists $disable_menu->{system_counter_tmpl_text};
  my $templ_image = 'd.write(\'&nbsp;<input class="ConfigBtn" type="button" value="'.$item->{system_counter_tmpl_image}.'" onclick="CF_jump\\(\\\'system_counter_tmpl_image\\\'\\)">\')'
    unless exists $disable_menu->{system_counter_tmpl_image};
    $bn->print(<<_HTML_);
<tr><td align="right" nowrap>$item->{system_counter_tmpl}</td><td><noscript>$item->{noscript}</noscript>
<script type="text/javascript">
d=document;c=d.config;
$templ_text
$templ_image
</script>
</td></tr>
_HTML_
  }
 }

}

sub cfg_system_imagetools {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};

  $cms->chk('use_gd', $bn->get($cf, 'UseGD'));
  $cms->chk('use_perlmagick', $bn->get($cf, 'UsePerlMagick'));
  $cms->chk('use_imagemagick', $bn->get($cf, 'UseImageMagick'));
  $cms->inf('imagetools_txt');
  $cms->txt('imagemagick_path', $bn->get($cf, 'ImageMagickPath'), 50);
  $cms->inf('dir_path_txt');

}

sub cfg_system_dir {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  $cms->txt('dir_bbsnote_url', $bn->get($cf, 'BBSNoteUrl'), 50);
  $cms->txt('dir_images_url', $bn->get($cf, 'DirImagesUrl'), 50);
  $cms->txt('dir_data_url', $bn->get($cf, 'DirFileUrl'), 50);
  $cms->inf('dir_url_txt');
  $cms->hr;
  $cms->txt('dir_images_path', $bn->get($cf, 'DirImagesPath'), 50);
  $cms->txt('dir_data_path', $bn->get($cf, 'DirFilePath'), 50);
  $cms->txt('dir_index_path', $bn->get($cf, 'IndexFilePath'), 50);
  $cms->txt('dir_sendmail_path', $bn->get($cf, 'SendMailPath'), 50);
  $cms->inf('dir_path_txt');
  $cms->hr;
  $cms->cap('dir_head');
  $cms->txt('dir_head_logs', $bn->get($cf, 'FileHeadLogs'), 5);
  $cms->txt('dir_head_image', $bn->get($cf, 'FileHeadImage'), 5);
  $cms->txt('dir_head_anime', $bn->get($cf, 'FileHeadAnime'), 5);
  $cms->txt('dir_head_thumb', $bn->get($cf, 'FileHeadThumb'), 5);
  $cms->txt('dir_head_object', $bn->get($cf, 'FileHeadObject'), 5);
  $cms->inf('dir_head_txt');
}

sub cfg_system_base {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};

  $cms->chk('system_crypt_password', $bn->get($cf, 'UseCrypt'));
  $cms->chk('system_crypt_mail', $bn->get($cf, 'UseMailCrypt'));
  $cms->inf('system_crypt_txt') unless exists $d->{system_crypt_mail};
  $cms->txt('system_crypt_id', $bn->get($cf, 'UseCryptKey'), 3);
  $cms->inf('system_crypt_id_txt') unless exists $d->{system_crypt_id};
  $cms->hr unless exists $d->{system_crypt_password} && exists $d->{system_crypt_mail} && exists $d->{system_crypt_id};

  $cms->chk('system_meta_cookie', $bn->get($cf, 'UseMetaCookie'));
  $cms->inf('system_meta_cookie_txt');
  $cms->chk('system_pragma', $bn->get($cf, 'HttpPragma'));
  $cms->inf('system_pragma_txt');
  $cms->txt('system_time_diff', $bn->get($cf, 'TimeDiffSec'), 5, $item->{sec});
  $cms->inf('system_time_diff_txt');
  $cms->txt('system_permission', $bn->get($cf, 'Permission'), 5);
  $cms->inf('system_permission_txt');
}

sub cfg_user_names_edit {
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $bn->error('ERR_FUNCTION') if exists $disable_menu->{user_names_edit};
  $bn->usr->editor;
}

sub cfg_user_names {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $menu_arg = $bn->{arg}{SettingMenu};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  if ($in->{edit} && ! exists $disable_menu->{user_names_edit}) {
    $cms->cfg_user_names_edit;
    return;
  }

  $menu_arg->{fc} = 'setting';
  $menu_arg->{name} = $in->{name};
  $menu_arg->{password} = $in->{password};
  my $arg = $bn->set_form_arg($menu_arg);

  unless (exists $disable_menu->{user_names_edit}) {

$bn->print(<<_HTML_);
  <input type="hidden" name="mode" value="user_names_edit">
  <input type="hidden" name="edit" value="1">
  <input type="hidden" name="cmd" value="">
  </form><form name="cfg" action="$cms->{bbsnoteurl}" method="post">$arg
_HTML_

  my $s = $bn->get( $cf, 'UseNameCap' ) ? ' checked' : '';
  my $c = 'user_names_cap';
  $bn->print( '<tr><td colspan="2"><fieldset class="ConfigFld" style="padding:10px"><legend>',
    '<input type="checkbox" name="', $c, '" id="cms_',
    $c, '" value="1"', $s, '><label for="cms_', $c, '">',
    $item->{$c}, '</label></legend><table style="font-size:10pt">' );

  $cms->inf('user_names_cap_txt');
  $bn->print('<tr><td>&nbsp;</td><td><input class="ConfigBtn" type="button" value="'.$item->{user_names_edit}.'" onclick="document.config.submit()"></td></tr>');
  $cms->cfg_tags($bn->get($sk, 'TagNameCap'), 'cap', 1);
  $bn->print('</table></fieldset></td></tr>');

  $s = $bn->get( $cf, 'UseNameTrip' ) ? ' checked' : '';
  $c = 'user_names_trip';
  $bn->print( '<tr><td colspan="2"><fieldset class="ConfigFld" style="padding:10px"><legend>',
    '<input type="checkbox" name="', $c, '" id="cms_',
    $c, '" value="1"', $s, '><label for="cms_', $c, '">',
    $item->{$c}, '</label></legend><table style="font-size:10pt">' );
  $cms->inf('user_names_trip_txt');
  $cms->cfg_tags($bn->get($sk, 'TagNameTrip'), 'trip', 1);
  $bn->print('</table></fieldset></td></tr>');

  }
}

sub cfg_user_profile {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  my $cfg_name = $bn->get($cf, 'AdminName');
  my $cfg_mail = $bn->get($cf, 'AdminMail');
  my $cfg_sitename = $bn->get($cf, 'AdminSiteName');
  my $cfg_siteurl = $bn->get($cf, 'AdminSiteUrl');
  my $cfg_comment = $bn->get($cf, 'AdminComment');

  $cms->txt('user_admin_name', $bn->get($cf, 'AdminName'));
  $cms->txt('user_admin_mail', $bn->get($cf, 'AdminMail'));
  $cms->txt('user_admin_site', $bn->get($cf, 'AdminSiteName'));
  $cms->txt('user_admin_url', $bn->get($cf, 'AdminSiteUrl'), 50);
  $cms->txt('user_admin_comment', $bn->get($cf, 'AdminComment'), 50);
  $cms->chk('user_admin_anonymous', $bn->get($cf, 'AdminAnonymous'));
}

sub cfg_user_password {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_admin = $bn->get($cf, 'AdminPassword');
  my $cfg_sub = $bn->get($cf, 'SubPassword');

  $cms->fld_set('password_admin');

$bn->print(<<_HTML_);
<tr><td align="right" nowrap>
<input class="ConfigBtn" type="button" name="ad_pass_btn" value="$item->{password_pass}" onclick="set_pass('ad','pass')"><br>
<input class="ConfigBtn" type="button" name="ad_named_btn" value="$item->{password_named}" onclick="set_pass('ad','name')">
</td><td>
$item->{password} <input type="text" name="ad_pass" size="9" maxlength="8"><br>
$item->{password} <input type="text" name="ad_npass" size="9" maxlength="8"> $item->{name} <input type="text" name="ad_name">
</td></tr>
<tr><td align="right" valign="top" nowrap>$item->{password_list}</td>
<td>
_HTML_

  my $i = 1;
  for (@$cfg_admin) { $i++ unless $_ eq '' }
  $bn->print('<select name="ad_list" size="'.$i.'" onclick="view_pass(this,\'ad\')" onchange="view_pass(this,\'ad\')">');
  $bn->print('<option value="">'.$item->{password_new}.'</option>');
  for (@$cfg_admin) {
    my($pass, $name) = split /:/,$_,2;
    my $val =  $name eq '' ? $pass : "$pass:$name";
    $bn->print('<option value="'.$val.'">'.$val.'</option>') unless $val eq '';
  }
  $bn->print('</select><input class="ConfigBtn" type="button" onclick="del_pass(\'ad_list\')" value="'.$item->{password_del}.'"></td></tr>');

  $cms->inf('password_txt');
  $cms->fld_end('password_admin');

  $cms->fld_set('password_sub');

$bn->print(<<_HTML_);
<tr><td align="right" nowrap>
<input class="ConfigBtn" type="button" name="sb_pass_btn" value="$item->{password_pass}" onclick="set_pass('sb','pass')"><br>
<input class="ConfigBtn" type="button" name="sb_named_btn" value="$item->{password_named}" onclick="set_pass('sb','name')">
</td>
<td>
$item->{password} <input type="text" name="sb_pass" size="9" maxlength="8"><br>
$item->{password} <input type="text" name="sb_npass" size="9" maxlength="8"> $item->{name} <input type="text" name="sb_name">
</td></tr>
<tr><td align="right" valign="top" nowrap>$item->{password_list}</td>
<td>
_HTML_

  $i = 1;
  for (@$cfg_sub) { $i++ unless $_ eq '' }
  $bn->print('<select name="sb_list" size="'.$i.'" onclick="view_pass(this,\'sb\')" onchange="view_pass(this,\'sb\')">');
  $bn->print('<option value="">'.$item->{password_new}.'</option>');
  for (@$cfg_sub) {
    my($pass, $name) = split /:/,$_,2;
    my $val =  $name eq '' ? $pass : "$pass:$name";
    $bn->print('<option value="'.$val.'">'.$val.'</option>') unless $val eq '';
  }
  $bn->print('</select><input class="ConfigBtn" type="button" onclick="del_pass(\'sb_list\')" value="'.$item->{password_del}.'"></td></tr>');
  $cms->inf('password_sub_txt');
  $cms->fld_end('password_sub');

  $bn->print('<tr><td colspan="2">'.$item->{manual_user_password}.'</td></tr>') if exists $item->{manual_user_password};

  $bn->{arg}{SettingMenu}{ad_pass} = '';
  $bn->{arg}{SettingMenu}{sb_pass} = '';
  my $arg = $bn->set_form_arg($bn->{arg}{SettingMenu});
  $bn->print('</form><form name="cfg" action="'.$cms->{bbsnoteurl}.'" method="post">'.$arg);

$bn->print(<<_HTML_);
<script type="text/javascript">
function set_pass (a,t) {
  d = document.config;
  var s;
  if (t == 'pass') {
    s = d[a+'_'+t].value;
    if(!s.length)return
  } else {
    if(!d[a+'_npass'].value.length)return
    if(!d[a+'_'+t].value.length)return
    s = d[a+'_npass'].value + ':' + d[a+'_'+t].value
  }
  o = d[a+'_list'];
  l = o.options.length;
  o.options[l] = new Option(s)
  o.options[l].value = s
  o.size = l+1;
  ref()
}

function view_pass (t,a) {
  d = document.config
  i = t.selectedIndex
  if(!t.options.length>0||i<0)return
  s = t.options[i].value;
  i = s.indexOf(':',0);
  if(i>0){
    d[a+'_npass'].value = s.substr(0,i);
    d[a+'_name'].value = s.substr(i+1,s.length-i-1);
    d[a+'_pass'].value = ''
  } else {
    d[a+'_npass'].value = '';
    d[a+'_name'].value = '';
    d[a+'_pass'].value = s
  }

}

function del_pass (t) {
  d = document.config
  i = d[t].selectedIndex;
  if(i>0){
    d[t].options[i]=null
    d[t].size = d[t].options.length
    d[t].selectedIndex=0
  }
  ref()
}

function ref(){
  d = document.config
  var ad,sb
  i = d['ad_list'].options.length
  if(i>0){
    ad = ''
    for(n=1;n<i;n++){
      ad = ad + d['ad_list'].options[n].value + "\\n"
    }
  }
  i = d['sb_list'].options.length
  if(i>0){
    sb = ''
    for(n=1;n<i;n++){
      sb = sb + d['sb_list'].options[n].value + "\\n"
    }
  }
  d = document.cfg
  d['ad_pass'].value = ad
  d['sb_pass'].value = sb
}
ref()
</script>
_HTML_
}

sub cfg_paint_palette {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  my @p_name;
  my $palettes = $bn->get_palette('all', \@p_name);
  my $cp = ' onchange="set_colors()" onfocus="set_colors()"';
  $cp .= ' onclick="PC_open(this,event)" onBlur="PC_hide()"' if $cms->{picker};
  $bn->print('<script type="text/javascript">'."\n","var palettes = new Array(\n");

  my $max = $#$palettes;
  for (my $n = 0; $n <= $max; $n++) {
    my($key, $name) = split /\t/, $p_name[$n];
    $bn->print("'$key\\t$name\\t" . (join '\t', split /,/,$palettes->[$n]) . "'");
    $bn->print(",\n") if $n < $max;
  }
  my $set_color = 'd[\'p\' + (i - 1)].style.backgroundColor = p;
    d[\'p\' + (i - 1)].style.color = get_bright(p);'
     if $bn->{global}{UserBrowserVer} !~ /NN 4/;

$bn->print(<<_HTML_);

  );

function reflesh() {
  d = document.config
  s = d.list.selectedIndex;
  for (n = d.list.options.length - 1; n >= 0; n--) { d.list.options[n] = null }
  for (n = 0; n < palettes.length; n++) {
    var l = new Array();
    l = palettes[n].split('\\t');
    n = d.list.options.length;
    if(!n) {n = 0};
    d.list.options[n] = new Option(l[1])
    d.list.options[n].value = l[0];
    d.list.options[n].text = l[1];
    if(n == s) d.list.options[n].selected = 1
  }

}

function add_palette() {
  d = document.config
  r = prompt('$item->{palette_add_key}', 'Palette' + d.list.options.length)
  if(r==null) return
  s = r.match("([0-9a-zA-Z_]+)");
  if(s==null) return
  r = prompt('$item->{palette_add_pname}', '')
  if(r==null) return
  p = new Array(14)
  palettes[palettes.length] = s[0]+"\\t"+r+"\\t" + p.join("\\t")
  reflesh()
  d.list.selectedIndex = d.list.options.length -1
  prev_colors()
  set_colors()

}
function del_palette() {
  d = document.config
  i = d.list.selectedIndex
  if(0>i)return
  r = confirm(d.list.options[i].text + ':$item->{palette_del_txt}')
  if(!r)return
  p = new Array();
  for(n=0,a=0;n<palettes.length;n++){
    if(n!=i){p[a] = palettes[n];a++}
  }
  palettes = p
  reflesh()
  prev_colors()
  set_colors()
}

function mov_list(m) {
  d = document.config
  i = d.list.selectedIndex
  if(0>i)return
  if((m == 1 && i == 0)||(m == -1 && i >= palettes.length -1))return
  t = i - m;
  p = new Array();
  for(n=0;n<palettes.length;n++){
    if(n==t){p[i] = palettes[t]}
    else if(n==i){p[t] = palettes[i]}
    else{p[n] = palettes[n]}
  }
  d.list.selectedIndex = t
  palettes = p
  reflesh()
  set_colors()
}

function set_colors() {
  d = document.config
  n = d.list.selectedIndex
  if (n==null) n = 0;
  var l = new Array();
  l[0] = d.key.value;
  l[1] = d.pname.value;
  for (i = 1; i < 15; i++) {
    c = d['p' + i].value.match("([0-9a-fA-F]+)");
    if(c==null){l[i+1] = ''}else{
    if(c[0].length < 6){for(p=c[0].length;p<6;p++){c[0]+='0'}}
    else if(c[0].length > 6){c[0]=c[0].substr(0,6)}
    l[i+1] = c[0] == null ? '' : c[0]}
  }
  palettes[n] = l.join('\\t');
  document.cfg.palettes.value = palettes.join('\\n');
  prev_colors()
}

function prev_colors() {
  d = document.config
  n = d.list.selectedIndex
  if(!palettes.length) return
  if(n==null) n = 0;
  var l = new Array();
  l = palettes[n].split('\\t');
  d.key.value = l[0]
  d.pname.value = l[1]
  for (i = 2; i < 16; i++) {
    c = l[i].match("([0-9a-fA-F]+)");
    a = c==null?'FFFFFF':c[0];
    if(a.length < 6){for(p=c[0].length;p<6;p++){a+='0'}}
    else if(a.length > 6){a=a.substr(0,6)}
    p = l[i] == null || a == null ? '' : '#'+a.toUpperCase() ;
    d['p' + (i - 1)].value = p;
    $set_color
  }

}
function get_bright(c){
  c = '#' + c;
  r=parseInt("0x"+c.substr(1,2));g=parseInt("0x"+c.substr(3,2));b=parseInt("0x"+c.substr(5,2));
  return 99>parseInt((r+g+b)/3)?"#FFFFFF":"#000000";
}


</script>
<tr>
<td colspan="2">
$item->{palette_list} <select class="select" name="list" onchange="prev_colors()"></select>
<input class="ConfigBtn" type="button" value="$item->{palette_add}" onclick="add_palette()">
<input class="ConfigBtn" type="button" value="$item->{palette_del}" onclick="del_palette()">
<input class="ConfigBtn" type="button" value="$item->{palette_list_up}" onclick="mov_list(1)">
<input class="ConfigBtn" type="button" value="$item->{palette_list_down}" onclick="mov_list(-1)">
<br>
$item->{palette_key} <input class="input" type="text" name=key onchange="set_colors()">
&nbsp; $item->{palette_name} <input class="input" type="text" name="pname" onchange="set_colors();reflesh()">
<br>
<input class="ConfigColor" type="text" name="p1"  size="9"$cp> : <input class="ConfigColor" type="text" name="p7"  size="9"$cp><br>
<input class="ConfigColor" type="text" name="p2"  size="9"$cp> : <input class="ConfigColor" type="text" name="p8"  size="9"$cp><br>
<input class="ConfigColor" type="text" name="p3"  size="9"$cp> : <input class="ConfigColor" type="text" name="p9"  size="9"$cp><br>
<input class="ConfigColor" type="text" name="p4"  size="9"$cp> : <input class="ConfigColor" type="text" name="p10" size="9"$cp><br>
<input class="ConfigColor" type="text" name="p5"  size="9"$cp> : <input class="ConfigColor" type="text" name="p11" size="9"$cp><br>
<input class="ConfigColor" type="text" name="p6"  size="9"$cp> : <input class="ConfigColor" type="text" name="p12" size="9"$cp><br>
<input class="ConfigColor" type="text" name="p13" size="9"$cp> : <input class="ConfigColor" type="text" name="p14" size="9"$cp><br>

</td></tr>
_HTML_

  $bn->{arg}{SettingMenu}{palettes} = '';
  my $arg = $bn->set_form_arg($bn->{arg}{SettingMenu});
  $bn->print("\n".'</form><form name="cfg" action="'.$cms->{bbsnoteurl}.'" method="post">'.$arg);
  $bn->set_call('SkinFootTranslate', sub{
   $bn->{data}{txtbbsbottom} .= '<script>reflesh();prev_colors();set_colors()</script>';
  });

}


sub cfg_paint_apple_pai {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_app = $bn->get($cf, 'ConfigBBSPainter');
  $cms->txt('applet_archive', $cfg_app->{archive});
  $cms->txt('applet_code', $cfg_app->{code});
  $cms->txt('applet_codebase', $cfg_app->{applet_codebase});
}

sub cfg_paint_apple_pic {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_app = $bn->get($cf, 'ConfigPictureBBS');
  $cms->txt('applet_archive', $cfg_app->{archive});
  $cms->txt('applet_code', $cfg_app->{code});
  $cms->txt('applet_codebase', $cfg_app->{applet_codebase});
  $cms->hr;
  $cms->txt('applet_gradation_begin', $cfg_app->{gradation_begin}, 9);
  $cms->txt('applet_gradation_end', $cfg_app->{gradation_end}, 9);
}
sub cfg_paint_apple_spv {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};
  my $cfg_app = $bn->get($cf, 'ConfigShiPCHViewer');

  $cms->txt('applet_archive', $cfg_app->{archive});
  $cms->txt('applet_code', $cfg_app->{code});
  $cms->txt('applet_codebase', $cfg_app->{applet_codebase});
  $cms->txt('applet_resource', $cfg_app->{resource});
  $cms->txt('applet_texture', $cfg_app->{texture});
  $cms->txt('applet_res', $cfg_app->{res});
  $cms->txt('applet_layer_count', $cfg_app->{layer_count}, 3);
  $cms->txt('applet_tt_size', $cfg_app->{tt_size}, 3);
  $cms->chk('applet_run', $cfg_app->{run});
  $cms->opt('applet_speed', $cfg_app->{speed}, [ '-1', 0, 10, 20 ]);
}
sub cfg_paint_apple_sii {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_app = $bn->get($cf, 'ConfigShiPaintBBS');
  $cms->txt('applet_archive', $cfg_app->{archive});
  $cms->txt('applet_code', $cfg_app->{code});
  $cms->txt('applet_codebase', $cfg_app->{applet_codebase});
  $cms->txt('applet_anime', $cfg_app->{anime_ext});
}

sub cfg_paint_apple_spa {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_app = $bn->get($cf, 'ConfigShiPainter');
  $cms->txt('applet_archive', $cfg_app->{archive});
  $cms->txt('applet_code', $cfg_app->{code});
  $cms->txt('applet_codebase', $cfg_app->{applet_codebase});
  $cms->txt('applet_anime', $cfg_app->{anime_ext});
  $cms->txt('applet_resource', $cfg_app->{resource});
  $cms->txt('applet_texture', $cfg_app->{texture});
  $cms->txt('applet_pro', $cfg_app->{pro});
  $cms->txt('applet_normal', $cfg_app->{normal});
  $cms->txt('applet_layer_count', $cfg_app->{layer_count}, 3);
  $cms->txt('applet_quality', $cfg_app->{quality}, 3);

}

sub cfg_paint_apple_base {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};
  my $cfg_app = $bn->get($cf, 'ConfigShiApplet');
  unless (exists $d->{applet_image_size}) {
    $cms->txt('applet_image_size', $cfg_app->{image_size}, 5);
    $bn->print('<tr><td align="right" colspan="2" nowrap><table style="font-size:10pt"><tr><td>'.$item->{applet_image_size_txt}.'</td></tr></table></td></tr>');
  }

  $cms->txt('applet_compress_level', $cfg_app->{compress_level}, 5);
  $cms->txt('applet_thumbnail_compress_level', $cfg_app->{thumbnail_compress_level}, 5);
  unless (exists $d->{applet_compress_level} && exists $d->{applet_thumbnail_compress_level}) {
    $cms->inf('applet_compress_level_txt');
    $cms->hr;
  }

  $cms->txt('applet_image_bkcolor', $cfg_app->{image_bkcolor}, 10);
  $cms->txt('applet_undo', $cfg_app->{undo}, 5);
  $cms->txt('applet_undo_in_mg', $cfg_app->{undo_in_mg}, 5);
  $cms->chk('applet_image_jpeg', $cfg_app->{image_jpeg});
  $cms->chk('applet_send_header_timer', $cfg_app->{send_header_timer});
  $cms->chk('applet_send_header_count', $cfg_app->{send_header_count});
  $cms->chk('applet_tool_advance', $cfg_app->{tool_advance});
  $cms->chk('applet_jp', $cfg_app->{jp});
  $cms->chk('applet_dyn_palette', $cfg_app->{dynamic_palette});
  $cms->chk('applet_dyn_color', $cfg_app->{dynamic_pcolor});
  $cms->chk('applet_send_advance', $cfg_app->{send_advance});
  unless (exists $d->{applet_send_advance}) {
    $cms->inf('applet_send_advance_txt');
  }

  $cms->fld_set('applet_security');
  $cms->txt('applet_security_click', $cfg_app->{security_click}, 5);
  $cms->txt('applet_security_timer', $cfg_app->{security_timer}, 5, $item->{sec});
  $cms->txt('applet_security_url', $cfg_app->{security_url}, 40);
  $cms->inf('applet_security_txt');
  $cms->fld_end('applet_security');

}

sub cfg_paint_apple {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_app = $bn->get($cf, 'UseApplet');
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  $bn->{arg}{SettingMenu}{applet_use} = '';
  $bn->{arg}{SettingMenu}{applet_setting} = '';
  my $arg = $bn->set_form_arg($bn->{arg}{SettingMenu});

  unless (exists $disable_menu->{applet_use}) {
  $bn->print('<tr><td width="10%" valign="top" align="right" nowrap>'.$item->{applet_use}.'</td><td><select name="applist" size="8" onclick="cfg_view(this)" onchange="cfg_view(this)">');
  for (@$cfg_app) {
    my $cfg = $bn->get($cf, 'AppletSetting:'.$_);
    $bn->print('<option value="'.$_.'">'.$cfg->{name}.'</option>');
  }

  $bn->print('</select><br>',
    '<input class="ConfigBtn" type="button" value="'.$item->{applet_up}.'" onclick="list_move(-1)"> ',
    '<input class="ConfigBtn" type="button" value="'.$item->{applet_down}.'" onclick="list_move(1)"> ',
    '<input class="ConfigBtn" type="button" value="'.$item->{applet_edit}.'" onclick="list_edit()"> ',
    '<input class="ConfigBtn" type="button" value="'.$item->{applet_add}.'" onclick="list_add()"> ',
    '<input class="ConfigBtn" type="button" value="'.$item->{applet_del}.'" onclick="list_del()"> ',
    '</td></tr>');

  $item->{paint_applet_app_list} ||= 'spanormal:spanormal,spapro:spapro,sii:shipaintbbs,pic:picturebbs,pai:bbspainter';

  my $app_option;
  for ( split /,/, $item->{paint_applet_app_list} ) {
    my( $val, $iname ) = split /:/, $_, 2;
    $app_option .= '<option value="'. $val. '">'. $item->{$iname}. '</option>' if $item->{$iname};
  }

$bn->print(<<_HTML_);
  <tr><td align="right" nowrap>$item->{applet_app}</td><td>
  <select class="ConfigBtn" name="app">
  $app_option
  </select><input class="ConfigBtn" type="button" value="$item->{applet_def}" onclick="list_def()"></td></tr>
_HTML_

  $cms->txt('applet_name','',30);
  $cms->txt('applet_key');
$bn->print(<<_HTML_);
  <tr><td align="right" nowrap>$item->{applet_opt}</td>
  <td><textarea name="opt" cols="30" rows="3" wrap="off"></textarea></td></tr>
</form>
<form name="cfg" action="$cms->{bbsnoteurl}" method="post">$arg
<script type="text/javascript">
  var setting = new Array();
_HTML_

  for (@$cfg_app) {
    my $cfg = $bn->get($cf, 'AppletSetting:'.$_);
    $bn->print("setting['$_'] = new Array();\n",,
      "setting['$_']['name'] = '$cfg->{name}';\n",
      "setting['$_']['applet'] = '$cfg->{applet}';\n",
      "setting['$_']['option'] = '" . (join '\n', grep $_, map{ /^(?:name|applet|key)$/ ? '' : "$_:$cfg->{$_}" }keys %$cfg) . "';\n");
  }
$bn->print(<<_HTML_);

function list_ref(){
  d = document.config;
  l = d.applist.options.length;
  use = new Array();
  for (i=0;i<l;i++) { use.push(d.applist.options[i].value) }
  document.cfg.applet_use.value = use.join();
  var key;var s = '';
  for (key in setting) {
    s += key + '=name:' + setting[key]['name'] + "\\n" + 'applet:' + setting[key]['applet'] + "\\n" + setting[key]['option'] + ';';
  }document.cfg.applet_setting.value = s;
}

function list_def(){
  d = document.config;
  t = d.app;
  ap = t.options[t.selectedIndex].value;
  var name;
  if (ap == 'spanormal') { name = '$item->{spanormal}' }
  else if (ap == 'spapro') { name = '$item->{spapro}' }
  else if (ap == 'sii') { name = '$item->{shipaintbbs}' }
  else if (ap == 'pic') { name = '$item->{picturebbs}' }
  else if (ap == 'pai') { name = '$item->{bbspainter}' }
  d.applet_key.value=ap;
  d.applet_name.value=name;
  list_add()
}

function list_add(){
  d = document.config;
  t = d.applist;
  ap=d.applet_key.value;
  if (setting[ap]) {
    alert('$item->{applet_err1}');
    return;
  }
  setting[ap] = new Array();
  setting[ap]['name'] = d.applet_name.value;
  setting[ap]['applet'] = d.app.options[d.app.selectedIndex].value;
  setting[ap]['option'] = d.opt.value;
  n = t.options.length;
  status = "" + n;
  t.options[n] = new Option(setting[ap]['name']);
  t.options[n].value = ap;
  t.options[n].text = setting[ap]['name'];
  t.options[n].selected = 1;
  list_ref()
}

function list_del(){
  d = document.config.applist;
  i = d.selectedIndex;
  if(i < 0) return;
  delete setting[d.options[i].value]
  d.options[i] = null;
  list_ref()
}

function list_edit(){
  d = document.config;
  t = d.applist;
  ap=d.applet_key.value;
  if (t.selectedIndex < 0) return;
  if (t.options[t.selectedIndex].value != ap) {
    if (setting[ap]) {
      alert('$item->{applet_err1}');
      return;
    }
    delete setting[t.options[t.selectedIndex].value]
  }
  setting[ap] = new Array();
  setting[ap]['name'] = d.applet_name.value;
  setting[ap]['applet'] = d.app.options[d.app.selectedIndex].value;
  setting[ap]['option'] = d.opt.value;
  t.options[t.selectedIndex].value = ap;
  t.options[t.selectedIndex].text = setting[ap]['name'];
  list_ref()
}

function list_move(i){
  d = document.config.applist;
  l = d.options.length;
  if(d.selectedIndex < 0) return;
  i = d.selectedIndex + i;
  if(i < 0 || i+1 > l) return
  val = d.options[d.selectedIndex].value;
  txt = d.options[d.selectedIndex].text;
  d.options[d.selectedIndex].value = d.options[i].value;
  d.options[d.selectedIndex].text = d.options[i].text;
  d.options[i].selected = 1;
  d.options[i].value = val;
  d.options[i].text = txt;
  list_ref()
}

function cfg_view(t){
  d = document.config;
  if (t.selectedIndex < 0) return;
  d.applet_key.value = t.options[t.selectedIndex].value;
  s = setting[d.applet_key.value];
  d.applet_name.value = s['name'];
  d.opt.value = s['option'];
  l = d.app.options.length;
  for (i=0;i<l;i++){
    if (d.app.options[i].value == s['applet']) {
      d.app.options[i].selected = 1;
      break;
    }
  }

}
list_ref()
</script>
_HTML_

}

  for ('width','height') { $item->{'applet_thumbnail_size_' . $_} = $item->{$_} }

  unless ( exists $disable_menu->{applet_cgi_stream} ) {
    $cms->hr;
    $cms->chk('applet_cgi_stream', $bn->get($cf, 'ConfigApplet')->{cgi_stream});
  }
  $bn->print( $item->{manual_paint_applet} ) if exists $item->{manual_paint_applet};
}

sub cfg_paint_base {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $cfg_shi = $bn->get($cf, 'ConfigShiApplet');
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};

  for ('width','height') {
    $item->{'paint_canvace_default_' . $_} = $item->{$_};
    $item->{'paint_canvace_select_min_' . $_} = $item->{$_};
    $item->{'paint_canvace_select_max_' . $_} = $item->{$_};
    $item->{'paint_canvace_select_var_' . $_} = $item->{$_};
  }
  my $i = 0;
  for ('off','on','paint_continue_new','paint_continue_rep') {
    $item->{'paint_continue_' . $i} = $item->{$_};
    $i++;
  }
  $i = 0;
  for ('nolimit','paint_continue_pass_need','paint_continue_pass_rep') {
    $item->{'paint_continue_pass_' . $i} = $item->{$_};
    $i++;
  }
  $i = 0;
  for ('view_all','view_admin','view_auth') {
    $item->{'paint_anime_limit_' . $i} = $item->{$_};
    $i++;
  }

  $cms->fld_set('paint_canvace');
  $cms->opt('paint_canvace_mode', $bn->get($cf, 'CanvasMode'), ['text','fix','select']);
  $cms->siz('paint_canvace_default', [$bn->get($cf, 'DefaultCanvasWidth'),$bn->get($cf, 'DefaultCanvasHeight')],['width','height']);
  $cms->siz('paint_canvace_select_min', [$bn->get($cf, 'CanvasSelectMinWidth'),$bn->get($cf, 'CanvasSelectMinHeight')],['width','height']);
  $cms->siz('paint_canvace_select_max', [$bn->get($cf, 'CanvasSelectMaxWidth'),$bn->get($cf, 'CanvasSelectMaxHeight')],['width','height']);
  $cms->siz('paint_canvace_select_var', [$bn->get($cf, 'CanvasSelectVarWidth'),$bn->get($cf, 'CanvasSelectVarHeight')],['width','height']);
  $cms->inf('paint_canvace_select_txt');
  $cms->fld_end('paint_canvace');

  $cms->fld_set('paint_anime');
  $cms->opt('paint_anime', $cfg_shi->{animation}, ['user','true','false']);
  $cms->chk('paint_anime_default', $cfg_shi->{default_anime});
  $cms->inf('paint_anime_default_txt');
  $cms->opt('paint_anime_limit', $bn->get($cf, 'LimitViewPaintAnime'), ['0','1','2'] );
  $cms->opt('paint_anime_dl_type', $cfg_shi->{anime_dl_type}, ['auto','get','post','cgi']);
  $cms->inf('paint_anime_dl_type_txt') unless exists $d->{paint_anime_dl_type};
  $cms->fld_end('paint_anime');

  $cms->fld_set('paint_continue');
  $cms->opt('paint_continue', $bn->get($cf, 'UsePaintContinue'), ['0','1','2','3']);
  $cms->opt('paint_continue_pass', $bn->get($cf, 'UsePaintContinuePassword'), ['0','1','2']);
  $cms->chk('paint_continue_history', $bn->get($cf, 'ViewPaintContinueHistory'));
  $cms->chk('paint_password_auth', $bn->get($cf, 'PaintPasswordAuth'));
  $cms->inf('paint_password_auth_txt');
  $cms->fld_end('paint_continue');

  $cms->chk('paint_continue_conv', $bn->get($cf, 'UsePaintContinueConvert'));
  $cms->chk('paint_data_over_conv', $bn->get($cf, 'UsePaintDataOverConvert'));
  unless (exists $d->{paint_continue_conv} && exists $d->{paint_data_over_conv}) {
    $cms->inf('paint_conv_txt');
  }

  unless (exists $d->{paint_pool}) {
  $cms->fld_set('paint_pool');
  $cms->inf('paint_pool_txt');
  $cms->txt('paint_pool_max', $bn->get($cf, 'PaintPoolMaxData'), 5, $item->{log_num});
  $cms->txt('paint_pool_time', $bn->get($cf, 'PaintPoolMaxTime'), 5, $item->{min});
  $cms->fld_end('paint_pool');
  }

  $cms->txt('paint_palette_view', $bn->get($cf, 'ViewLimitPaletteList'), 5);
  $cms->chk('paint_applet_fit', $bn->get($cf, 'UseAppletFit'));
  $cms->inf('paint_applet_fit_txt');

}



sub cfg_post_icon {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};
  my $menu_arg = $bn->{arg}{SettingMenu};

  my $i = 0;
  for ( qw/off admin_only admin_sub everyone/ ) {
    $item->{'post_icon_use_' . $i} = $item->{$_};
    $i++;
  }
  $i = 0;
  for ( qw/nolimit admin_only admin_sub/ ) {
    $item->{'post_icon_regist_allow_' . $i} = $item->{$_};
    $i++;
  }

  for ('width','height') {
    $item->{'post_icon_limit_size_' . $_} = $item->{$_};
  }

  if ($bn->get($cf, 'UseIcon')) {

    $menu_arg->{fc} = 'icon_regist';
    my $arg1 = $bn->set_form_arg($menu_arg);
    $menu_arg->{fc} = 'icon_list';
    my $arg2 = $bn->set_form_arg($menu_arg);
    $menu_arg->{fc} = 'setting';
    my $arg3 = $bn->set_form_arg($menu_arg);

    $bn->print(<<_HTML_);
  <tr>
  </form><form name="cfg" action="$cms->{bbsnoteurl}" method="post" target="_icon">$arg1
  <td align="right" nowrap><input class="ConfigBtn" type="submit" value="$item->{post_icon_regist}"></td>
  </form><form name="cfg" action="$cms->{bbsnoteurl}" method="post" target="_icon">$arg2
  <td><input class="ConfigBtn" type="submit" value="$item->{post_icon_list}"></td>
  </form><form name="cfg" action="$cms->{bbsnoteurl}" method="post">$arg3
  <input type="hidden" name="mode" value="post_icon">
  </tr>
_HTML_

  }

  $cms->opt('post_icon_use', $bn->get($cf, 'UseIcon'), ['0','1','2','3']);
  $cms->opt('post_icon_regist_allow', $bn->get($cf, 'IconRegist'), ['0','1','2']);
  $cms->chk('post_icon_ex', $bn->get($cf, 'UseIconEx'));
  $cms->inf('post_icon_ex_txt');
  $cms->chk('post_icon_preview', $bn->get($cf, 'UseIconPreview'));
  $cms->chk('post_icon_view_force', $bn->get($cf, 'IconViewForce'));
  $cms->txt('post_icon_limit_post', $bn->get($cf, 'LimitIcon'), 4);
  $cms->siz('post_icon_limit_size', [$bn->get($cf, 'LimitIconWidth'),$bn->get($cf, 'LimitIconHeight')],['width','height']);
  $cms->txt('post_icon_limit_data', $bn->get($cf, 'LimitIconSize'), 8, $item->{byte});
  $cms->txt('post_icon_limit_msg', $bn->get($cf, 'LimitIconComment'), 8, $item->{byte});

  unless (exists $d->{post_icon_window}) {
  $cms->hr;
  $cms->txt('post_icon_window', $bn->get($cf, 'IconPreviewWindow'), 50);
  $cms->inf('post_icon_window_txt');
  }

}
sub cfg_post_notice {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $item = $cms->{item};

  $cms->chk('post_notice_use', $bn->get($cf, 'UseNoticeMail'));
  $cms->chk('post_notice_all', $bn->get($cf, 'UseAllPostNotice'));
  $cms->txa('post_notice_format', (join "\n", @{$bn->get($cf, 'NoticeMailFormat')}), 50, 16);

}

sub cfg_post_panic_edit {
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $bn->error('ERR_FUNCTION') if exists $disable_menu->{post_panic_edit};
  $bn->pa->editor;
}

sub cfg_post_panic {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $item = $cms->{item};
  my $menu_arg = $bn->{arg}{SettingMenu};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  if ($in->{edit} && ! exists $disable_menu->{post_panic_edit}) {
    $cms->cfg_post_panic_edit;
    return;
  }

  $menu_arg->{fc} = 'setting';
  $menu_arg->{name} = $in->{name};
  $menu_arg->{password} = $in->{password};
  my $arg = $bn->set_form_arg($menu_arg);

  unless (exists $disable_menu->{post_panic_edit}) {

$bn->print(<<_HTML_);
  <input type="hidden" name="mode" value="post_panic_edit">
  <input type="hidden" name="edit" value="1">
  <input type="hidden" name="cmd" value="">
  <tr><td align="right" nowrap></td><td>
  <input class="ConfigBtn" type="submit" value="$item->{post_panic_edit}">
  </td></tr>
  </form><form name="cfg" action="$cms->{bbsnoteurl}" method="post">$arg
_HTML_

  }
  $item->{post_panic_use_0} = $item->{off};
  $item->{post_panic_use_1} = $item->{on};
  $cms->opt('post_panic_use', $bn->get($cf, 'UsePanic'), ['0', '1', '2']);
  $cms->txa('post_panic_color', (join "\n", split /,/, $bn->get($cf, 'PanicColor')), 8, 16);

}
sub cfg_post_font_color {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  $cms->chk('post_font_color_use', $bn->get($cf, 'UseColor'));
  $cms->inf('post_font_color_txt');
  $cms->txa('post_font_color_font', (join "\n", split /,/, $bn->get($cf, 'ColorSetting:font_color')), 8, 16);
  $cms->txa('post_font_color_title', (join "\n", split /,/, $bn->get($cf, 'ColorSetting:title_color')), 8, 16);
  $cms->txa('post_font_color_table', (join "\n", split /,/, $bn->get($cf, 'ColorSetting:table_color')), 8, 16);

}
sub cfg_post_tag {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  my $i = 0;
  for ('off','admin_only','admin_sub','everyone') {
    $item->{'post_tag_all_' . $i} = $item->{$_};
    $i++;
  }
  $cms->chk('post_tag_use', $bn->get($cf, 'UseTag'));
  $cms->inf('post_tag_use_txt');

  unless (exists $disable_menu->{post_tag_ok}) {
    my $a = $bn->get($cf, 'OkTag');
    $bn->print('<tr><td valign="top" align="right" nowrap>'.$item->{post_tag_ok}.'</td><td><textarea name="post_tag_ok" cols="30" rows="14" wrap="off">');
    for (@$a) {
       s/\s//g;
       $bn->print("$_\n") if $_ ne '';
    }
    $bn->print('</textarea></td><tr>');
    $cms->inf('post_tag_txt');
    $cms->hr if !exists $disable_menu->{post_tag_all};
  }

  $cms->opt('post_tag_all', $bn->get($cf, 'UseAllTag'), ['0','1','2','3']);
  $cms->inf('post_tag_all_txt') if !exists $disable_menu->{post_tag_all};

}

sub cfg_post_upload {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};

  my $i = 0;
  for ('off','admin_only','admin_sub','everyone') {
    $item->{'post_upload_new_' . $i} = $item->{$_};
    $item->{'post_upload_reply_' . $i} = $item->{$_};
    $i++;
  }
  $i = 0;
  for ('off','post_upload_ex_new','post_upload_ex_reply','post_upload_ex_all') {
    $item->{'post_upload_ex_' . $i} = $item->{$_};
    $item->{'post_upload_local_' . $i} = $item->{$_};
    $i++;
  }
  for ('width','height') {
    $item->{'post_thumbnail_size_' . $_} = $item->{$_};
    $item->{'post_thumbnail_size_' . $_} = $item->{$_};
  }

  $cms->opt('post_upload_new', $bn->get($cf, 'UseUpload'), ['0','1','2','3']);
  $cms->opt('post_upload_reply', $bn->get($cf, 'UseReplyUpload'), ['0','1','2','3']);
  $cms->inf('post_upload_txt');

  unless (exists $d->{post_thumbnail_use}) {
  $cms->hr;
  $cms->chk('post_thumbnail_use', $bn->get($cf, 'UseThumbnail'));
  $cms->inf('post_thumbnail_txt');
  $cms->siz('post_thumbnail_size', [$bn->get($cf, 'ThumbnailWidth'),$bn->get($cf, 'ThumbnailHeight')],['width','height']);
  }

  unless (exists $d->{post_file_ex}) {
  $cms->hr;
  $cms->cap('post_file_ex');
  $cms->chk('post_file_text', $bn->get($cf, 'UseFileExText'));
  $cms->chk('post_file_bmp', $bn->get($cf, 'UseFileExBmp'));
  $cms->chk('post_file_midi', $bn->get($cf, 'UseFileExMidi'));
  $cms->chk('post_file_swf', $bn->get($cf, 'UseFileExSwf'));
  $cms->chk('post_file_zip', $bn->get($cf, 'UseFileExZip'));
  $cms->chk('post_file_lzh', $bn->get($cf, 'UseFileExLzh'));
  }

}
sub cfg_post_base {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  my $i = 0;
  for ('off','admin_only','admin_sub','everyone') {
    $item->{'post_edit_date_' . $i} = $item->{$_};
    $i++;
  }

  $cms->chk('post_auto_link', $bn->get($cf, 'UseAutoLinkUrl'));
  $cms->chk('post_auto_mail', $bn->get($cf, 'UseAutoLinkMail'));
  $cms->chk('post_quotation', $bn->get($cf, 'UseQuotation'));
  $cms->chk('post_index',     $bn->get($cf, 'UseRefleshIndex'));
  $cms->chk('post_res_title', $bn->get($cf, 'UseReplyTitle'));
  $cms->opt('post_edit_date', $bn->get($cf, 'UseEditDate'), ['0','1','2','3']);

}

sub cfg_limit_filter {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $item = $cms->{item};

  my $f_string =  $bn->get($cf, 'LimitFilterString');
  $f_string = [] unless ref $f_string eq 'ARRAY';

  $cms->txa('limit_filter_string', (join "\n", @$f_string ) , 50, 16);
  $cms->inf('limit_filter_string_txt');
  $cms->chk('limit_filter_spam', $bn->get($cf, 'LimitFilterSpam'));
  $cms->chk('limit_filter_en_chr', $bn->get($cf, 'LimitFilterEnChr'));
  $cms->chk('limit_filter_address', $bn->get($cf, 'LimitFilterAddress'));
  $cms->inf('limit_filter_address_txt');
  $bn->call( 'CMS_CfgLimitFilter' );

}

sub cfg_limit_securty {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  $cms->opt('limit_security_proxy', $bn->get($cf, 'LimitSecurityProxy'), ['1','2','3','4']);
  $cms->inf('limit_security_proxy_txt');
  $cms->hr;
  $cms->txt('limit_security_deny_ip', $bn->get($cf, 'LimitSecurityDenyIp'), 50);
  $cms->txt('limit_security_deny_host', $bn->get($cf, 'LimitSecurityDenyHost'), 50);
  $cms->txt('limit_security_accept_ip', $bn->get($cf, 'LimitSecurityAcceptIp'), 50);
  $cms->txt('limit_security_accept_host', $bn->get($cf, 'LimitSecurityAcceptHost'), 50);
  $cms->inf('limit_deny_txt');
  $cms->txt('limit_security_accept_ref', $bn->get($cf, 'LimitSecurityAcceptRef'), 50);
  $cms->inf('limit_security_accept_ref_txt');
  $cms->hr;
  $cms->chk('limit_security_deny_raw_ip', $bn->get($cf, 'LimitSecurityDenyRawIp'));
  $cms->chk('limit_security_bookmark', $bn->get($cf, 'LimitSecurityBookMark'));
  $cms->chk('limit_no_robots', $bn->get($cf, 'LimitNoRobots'));
  $bn->call( 'CMS_CfgLimitSecurty' );
}

sub cfg_limit_post_multi {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};
  my $m = 'limit_post_multi_';
  my $r = 'limit_post_range_';

  for ( 'new', 'reply' ) { $item->{$r.$_.'_guest'} = $item->{$r.'guest'} }

  $bn->print('<tr><td colspan="2">', $item->{$m.'txt'}, '</td></tr>');
  $cms->hr;

  my $h = $bn->get($cf, 'LimitPostMulti');
  my %mode;
  for (@$h){
    my %ch;
    for (split /\;/) { my @b = split /:/; $ch{$b[0]} = $b[1] if $b[1] }
    if    ($ch{mode} eq 'WRITE_POST'        ) { $mode{new}   = \%ch }
    elsif ($ch{mode} eq 'WRITE_REPLY'       ) { $mode{reply} = \%ch }
    elsif ($ch{mode} eq 'EDIT'              ) { $mode{edit}  = \%ch }
    elsif ($ch{mode} eq 'WRITE_PAINT_CAPTOR') { $mode{paint} = \%ch }
    delete $ch{mode};
  }

  my $chk = sub {
    my( $cfg, $val, $inf ) = @_;
    my $s = $val ? ' checked' : '';
    $bn->print('<td nowrap><input type="checkbox" name="',$cfg,'" id="',$cfg,
      '" value="1" ',$s,'><label for="',$cfg,'"',
       ( $inf ? ' title="'.$inf.'"':'' ),'> ',$item->{$cfg},'</label></td>');
  };

  $bn->print('<tr><td colspan="2"><style>.inpnum{text-align:right;ime-mode:disabled}</style>');
  for ('new','reply','edit','paint') {
    next if exists $d->{$m.$_};
    my $i = 1;
    $bn->print('<fieldset class="ConfigFld"><legend>', $item->{$m.$_}, '</legend>',
      '<table style="font-size:10pt"><tr>');
    for my $val('ip','name','message','title','file_size','num','time') {
      $item->{$m.$_.'_'.$val} = $item->{$m.$val};
      if($val eq 'time') {
        $bn->print('<td colspan="3" nowrap>', $item->{$m.$_.'_'.$val},
        ' <input type="text" name="',$m, $_, '_', $val, '" value="',
        $mode{$_}->{$val}, '" class="inpnum" size="5"> ', $item->{sec}, '</td>');
      } else {
        if ($_ =~ /new|paint/ && $val eq 'num') {
          $bn->print('<td>&nbsp;</td>');
        } else {
          &$chk($m.$_.'_'.$val, $mode{$_}->{$val});
        }
      }
      $bn->print('</tr><tr>') unless ($i % 3);
      $i++;
    }
    $bn->print('</tr></table></fieldset><br>');
  }
  $bn->print('</td></tr>');
  $cms->hr unless exists $d->{$m.'history'} && exists $d->{$m.'log'};
  $cms->txt($m.'history', $bn->get($cf, 'LimitPostMultiHistory'), 5, $item->{log_num});
  $cms->txt($m.'log', $bn->get($cf, 'LimitPostMultiLog'), 5, $item->{log_num});

  $cms->hr;
  $bn->print('<tr><td colspan="2">', $item->{limit_post_range}, '</td></tr>');
  $cms->inf($r.'txt');
  $bn->print('<tr><td colspan="2">');

  my %c;
  for ( split ';', $bn->get( $cf, 'LimitPostRange' ) ) {
    my @a = split ':', $_, 2; for ( split ',', $a[1] )
     { my @v = split '=', $_, 2; $c{$a[0]}{$v[0]} = $v[1] if $v[1] } }
  for ('new','reply') {
    $bn->print(
      '<fieldset class="ConfigFld"><legend>', $item->{$m.$_},
      ( ( $c{$_}{day} || $c{$_}{hour} ) && $c{$_}{log} ? '' : '('.$item->{'nolimit'}.')' ),
      '</legend><table style="font-size:10pt"><tr>',
      '<td>', $item->{$r.'time'},
      ' <input type="text" name="', $r, $_, '_day" value="', $c{$_}{day}, '" size="2" class="inpnum" maxlength="2"> ', $item->{day},
      ' <input type="text" name="', $r, $_, '_hour" value="', $c{$_}{hour}, '" size="2" class="inpnum" maxlength="2"> ', $item->{hour},
      '</td><td>', $item->{$r.'count'},
      ' <input type="text" name="',$r, $_, '_log" value="', $c{$_}{log}, '" size="2" class="inpnum" maxlength="2"> ', $item->{log_num}
    );
    &$chk($r.$_.'_guest', $c{$_}{guest}, $item->{$r.'guest_txt'});
    $bn->print('</tr></table></fieldset><br>');
  }
  $bn->print('</td></tr>');
}

sub cfg_limit_post {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $p = 'limit_post_';
  my $v = 'limit_view_post_';

  my $i = 0;
  my @e = ( 'new', 'reply', 'edit', 'delete', 'paint' );
  for my $a ('nolimit','admin_only','admin_sub')
   { for ( @e ) { $item->{$p.$_.'_'.$i} = $item->{$a} } $i++ }
  $i = 0;
  for my $a ('view_all','view_admin','view_auth')
   { for ( @e, 'upload' ) { $item->{$v.$_.'_'.$i} = $item->{$a} } $i++ }

  $cms->opt($p.'new', $bn->get($cf, 'LimitPostNew'), ['0','1','2']);
  $cms->opt($p.'reply', $bn->get($cf, 'LimitPostReply'), ['0','1','2']);
  $cms->opt($p.'edit', $bn->get($cf, 'LimitPostEdit'), ['0','1','2']);
  $cms->opt($p.'delete', $bn->get($cf, 'LimitPostDelete'), ['0','1','2']);
  $cms->opt($p.'paint', $bn->get($cf, 'LimitPostPaint'), ['0','1','2']);
  $cms->hr;
  $cms->cap('limit_view_post');
  $cms->inf($v.'txt');
  $cms->opt($v.'new', $bn->get($cf, 'LimitViewPostNew'), ['0','1','2']);
  $cms->opt($v.'reply', $bn->get($cf, 'LimitViewPostReply'), ['0','1','2']);
  $cms->opt($v.'edit', $bn->get($cf, 'LimitViewPostEdit'), ['0','1','2']);
  $cms->opt($v.'delete', $bn->get($cf, 'LimitViewPostDelete'), ['0','1','2']);
  $cms->opt($v.'paint', $bn->get($cf, 'LimitViewPostPaint'), ['0','1','2']);
  $cms->opt($v.'upload', $bn->get($cf, 'LimitViewPostUpload'), ['0','2']);
  $cms->inf($v.'upload_txt');

  $cms->cap;
  $cms->cap($v.'all', $v.'all_txt');
  $cms->cap($v.'admin', $v.'admin_txt');
  $cms->cap($v.'auth', $v.'auth_txt');

}


sub cfg_limit_listen {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $d = $bn->{sys}{CMS}{DISABLE_MENU};

  for ('width','height') {
    $item->{'limit_image_size_' . $_} = $item->{$_};
    $item->{'limit_image_view_' . $_} = $item->{$_};
  }

  $cms->txt('limit_log', $bn->get($cf, 'LogMax'), 5, $item->{log_num});
  $cms->txt('limit_log_size', $bn->get($cf, 'LimitResLogSize'), 5, $item->{byte});
  $cms->txt('limit_res_total', $bn->get($cf, 'LimitResTotal'), 5, $item->{log_num});
  $cms->opt('limit_res', $bn->get($cf, 'LimitResType'), ['0','1','2','3']);
  $cms->opt('limit_log_removed', $bn->get($cf, 'LimitLogRemoved'), ['0','1']);
  $cms->inf('limit_log_removed_txt');
  $cms->hr unless $d->{limit_log} && $d->{limit_log_size} && $d->{limit_res} && $d->{limit_log_removed};
  $cms->siz('limit_image_size', [$bn->get($cf, 'LimitImageWidth'),$bn->get($cf, 'LimitImageHeight')],['width','height']);
  $cms->siz('limit_image_view', [$bn->get($cf, 'ViewLimitImageWidth'),$bn->get($cf, 'ViewLimitImageHeight')],['width','height']);
  $cms->txt('limit_data_upload', $bn->get($cf, 'LimitUpLoadData'), 7, $item->{byte});
  $cms->txt('limit_data_paint', $bn->get($cf, 'LimitPaintData'), 7, $item->{byte});
  $cms->txt('limit_data_anime', $bn->get($cf, 'LimitAnimeData'), 7, $item->{byte});
  $cms->txt('limit_data_thumbnail', $bn->get($cf, 'LimitThumbnailData'), 7, $item->{byte});
  $cms->hr;
  $cms->chk('limit_thread_lock', $bn->get($cf, 'UseThreadLock'));
  $cms->inf('limit_thread_lock_txt');

}

sub cfg_limit_len {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $l = 'limit_len_';
  for (qw/title name mail url message/)
   { $cms->txt($l.$_, $bn->get($cf, 'Limit'. ucfirst), 5, $item->{byte}) }
  $cms->txt($l.'message_line', $bn->get($cf, 'LimitLine'), 5, $item->{line});
}

sub cfg_limit_need {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $l = 'limit_need_';
  for (qw/title name mail url message password data/)
   { $cms->chk($l.$_, $bn->get($cf, 'Need'. ucfirst )) }
  $cms->inf($l.'data_txt');
}

sub cfg_view_search {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $v = 'view_search_';
  $cms->chk($v.'use', $bn->get($cf, 'UseSearch'));
  $cms->txt($v.'default_view', $bn->get($cf, 'ViewSearchDefault'), 5, $item->{log_num});
  for (qw/min max var log/)
   { $cms->txt($v.'limit_'.$_, $bn->get($cf, 'ViewSearchLimit'. ucfirst), 5, $item->{log_num}) }
  $cms->chk($v.'form_ex', $bn->get($cf, 'ViewSearchFormEx'));
}

sub cfg_view_tree {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};
  my $v = 'view_tree_';
  $cms->chk($v.'use', $bn->get($cf, 'UseTree'));
  $cms->chk($v.'frame', $bn->get($cf, 'UseTreeFrame'));
  $cms->txt($v.'new_log', $bn->get($cf, 'ViewTreeNewLog'), 5, $item->{hour});
  $cms->txt('view_max_title', $bn->get($cf, 'ViewMaxTitle'), 5, $item->{byte});
  $cms->txt($v.'child_space', $bn->get($cf, 'ViewTreeChildSpace'), 5, $item->{pixcel});
  $cms->hr;
  $cms->chk($v.'spp_check', $bn->get($cf, 'UseTreeSppCheck'));

}


sub cfg_view_position {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  for ('left','right','center') {
    $item->{'view_image_pos_' . $_} = $item->{'view_image_' . $_};
    $item->{'view_reply_image_pos_' . $_} = $item->{'view_image_' . $_};
  }

  my $menu_arg = $bn->{arg}{SettingMenu};
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
  $menu_arg->{fc} = 'setting';
  $menu_arg->{name} = $in->{name};
  $menu_arg->{password} = $in->{password};
  my $arg = $bn->set_form_arg( $menu_arg );

  unless ( exists $disable_menu->{view_rotation} ) {
    my $cfg = $bn->get( $sk, 'TagBody' );
    my @c = ref $cfg eq 'HASH' ? ( $cfg->{bgcolor}, $cfg->{text} ) : ( '#FFFFFF', '#000000' );
    $bn->print(
      '<style type="text/css">', "\n",
      '.CMS_TL{border:1px solid ', $c[1], ';border-right:none}', "\n",
      '.CMS_TM{border-top:1px solid ', $c[1], ';border-bottom:1px solid ', $c[1], '}', "\n",
      '.CMS_TR{border:1px solid ', $c[1], ';border-left:none}', "\n",
      '.CMS_TB{color:',$c[0],';background:'.$c[1],';border:1px solid ', $c[0], ';cursor:pointer}', "\n",
      '</style>', "\n",
      '<tr><td align="right" nowrap>', $item->{view_rotation}, '</td>',
      '<td><table cellspacing="0" cellpadding="1" style="font-size:10pt"><tr>'
    );
    my @ro = split ',', $bn->get( $cf, 'ViewRotation' );
    my $n = 0;
    for ( @ro ) {
      $bn->print(
        '<td class="CMS_T',
        ( $n == 0 ? 'L' : $n == $#ro ? 'R' : 'M' ),
        @ro <= 1 ? ' CMS_TB' : '',
        '">', $item->{ 'design_'. $_ }, '</td>'
      );
      $bn->print(
        '<form action="', $cms->{bbsnoteurl}. '" method="post">',
        $bn->set_form_arg( $menu_arg, { mode => 'view_rotation', pos => $n, move => 'down' } ),
        '<td class="CMS_TR"><input class="CMS_TB" type="submit" value="&gt;"></td></form>',
       ) if $#ro > $n;
      $n++;
      $bn->print(
        '<form action="', $cms->{bbsnoteurl}. '" method="post">',
        $bn->set_form_arg( $menu_arg, { mode => 'view_rotation', pos => $n, move => 'up' } ),
      '<td class="CMS_TL"><input class="CMS_TB" type="submit" value="&lt;"></td></form>',
      ) if @ro > $n;
    }

    $bn->print(
      '</tr></table></td></tr>',
      '<form action="', $cms->{bbsnoteurl}. '" method="post">',
      $bn->set_form_arg( $menu_arg, { mode => 'view_position' } )
    );
  }


  $cms->opt('view_sort', $bn->get($cf, 'ViewSortType'), ['0','1']);
  $cms->opt('view_number', $bn->get($sk, 'ViewThreadNumber'), ['real','virtual','off']);
  $cms->inf('view_number_txt');
  $cms->chk('view_paint_time', $bn->get($cf, 'ViewPaintTime'));
  $cms->chk('view_agent', $bn->get($sk, 'ViewAgent'));
  $cms->chk('view_edit', $bn->get($sk, 'ViewEditCount'));
  $cms->chk('view_id', $bn->get($cf, 'ViewId'));
  $cms->inf('view_id_txt');
  $cms->chk('view_first_date', $bn->get($cf, 'ViewFirstDate'));
  $cms->inf('view_first_date_txt');
  $cms->hr;
  $cms->opt('view_image_pos', lc $bn->get($sk, 'ViewImagePos'), ['left','right','center']);
  $cms->opt('view_reply_image_pos', lc $bn->get($sk, 'ViewReplyImagePos'), ['left','right','center']);
  $cms->opt('view_form_type', lc $bn->get($sk, 'FormType'), ['normal','thread','layer','line']);
  $cms->inf('view_form_txt');
  $cms->chk('view_top_form', $bn->get($sk, 'ViewTopForm'));

}

sub cfg_view_bbs {
  my $cms = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $cms->{item};

  $cms->txt('view_bbs_title', $bn->get($cf, 'BbsTitle'));
  $cms->txt('view_homepage_url', $bn->get($cf, 'HomePageUrl'), 50);
  $cms->txt('view_homepage_url_ta', $bn->get($cf, 'HomePageUrlTarget'), 5);
  $cms->inf('view_homepage_url_ta_txt');
  $cms->hr;
  $cms->txt('view_log_view', $bn->get($cf, 'LogView'), 5, $item->{log_num});
  $cms->txt('view_reply', $bn->get($cf, 'LogViewReply'), 5, $item->{log_num});
  $cms->inf('view_reply_txt');
  $cms->siz('view_catalog', [$bn->get($cf, 'CatalogRow'),$bn->get($cf, 'CatalogCol')],['row','col']);
  $cms->opt('view_style_pagelink', $bn->get($sk, 'ViewStylePageLink'), ['text','form']);
  $cms->txt('view_limit_pagelink', $bn->get($sk, 'ViewLimitPageLink'), 5);

}



sub setting_menu {
  my $cms = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $args = $bn->{arg};
  my $item = $cms->{item};
  my $mode = $in->{mode};
  my $tree = $item->{'menu_' . $1} if $mode =~ /^([a-z]+)/;
  my $body = $bn->get($sk, 'TagBody');
  my $table_in = $bn->get($sk, 'TagTableIn');
  my $table_out = $bn->get($sk, 'TagTableOut');
  my $menu = $cms->{MENU} = {};
  my $menus = $cms->{MENUS} = [];
  my $list = $cms->{MENU_LIST} = [];
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};

  my $css = $bn->get($sk, 'StyleSheet');
  my @css_back = @$css;
  unshift @$css, (".ConfigHr { border-style:dotted;color:$table_out->{bgcolor} }",
   ".ConfigFld { border-color:$table_out->{bgcolor} }",
   ".ConfigBtn { border-color:$table_out->{bgcolor};color:$table_in->{bgcolor};background-color:$table_out->{bgcolor} }",
   ".ConfigCmd { border-color:$table_out->{bgcolor};color:$body->{text};background-color:$body->{bgcolor};width:100%;font-weight:900 }",
   ".ConfigArrow { background-color:$table_out->{bgcolor};color:$table_in->{bgcolor};border-style:none }",
   ".ConfigColor { font-size:12px }",
   ".ConfigMenu, .ConfigCaption { color:$table_in->{bgcolor};font-weight:bold }");
  $bn->define($sk, 'StyleSheet', undef, '@');
  map{ $bn->set($sk, 'StyleSheet', $_) }@$css;
  $bn->send_http_header->head($bn->get($cf, 'BbsTitle') . ' : '. $item->{cms_title}, 1);

  $bn->define($sk, 'StyleSheet', undef, '@');
  map{ $bn->set($sk, 'StyleSheet', $_) }@css_back;
  undef @css_back;

  $args->{SettingMenu}{fc} = 'setting';
  $args->{SettingMenuBack}{fc} = 'login';
  $args->{SettingMenu}{name} = $args->{SettingMenuBack}{name} = $in->{name};
  $args->{SettingMenu}{password} = $args->{SettingMenuBack}{password} = $in->{password};

  $menu->{view} = [qw/view_bbs view_position view_tree view_search/];
  $menu->{limit} = [qw/limit_need limit_len limit_listen limit_post limit_post_multi limit_securty limit_filter/];
  $menu->{post} = [qw/post_base post_upload post_tag post_font_color post_panic post_notice post_icon/];
  $menu->{paint} = [qw/paint_base paint_apple paint_apple_base paint_apple_spa paint_apple_sii paint_apple_spv paint_apple_pic paint_apple_pai paint_palette/];
  $menu->{user} = [qw/user_password user_profile user_names/];
  $menu->{text} = [qw/text_head text_top text_bottom text_bbs_head text_bbs_top text_bbs_bottom text_msg_top text_msg_bottom text_tree_top
  text_paint_head text_paint_top text_paint_down text_spa_head text_spa_top text_spa_down text_sii_head text_sii_top text_sii_down
  text_spv_head text_spv_top text_spv_down text_pic_head text_pic_top text_pic_down
  text_pai_head text_pai_top text_pai_down
  /];
  $menu->{skin} = [qw/skin_stylesheet skin_page skin_number skin_title skin_user skin_message skin_edit skin_form skin_catalog skin_frame skin_shipainter skin_shipaintbbs skin_pchviewer skin_picturebbs skin_dynpalette/];
  $menu->{system} = [qw/system_base system_dir system_imagetools system_log system_recover/];
  $menu->{design} = [qw/design_head design_menu design_main design_object design_foot design_post design_search design_frame design_catalog design_tree design_copyright/];

  @$menus = qw/menu_view menu_limit menu_post menu_paint menu_user menu_system menu_text menu_skin menu_design/;
  @$list = @{$menu}{qw/view limit post paint user system text skin design/};

  $bn->call( 'CMS_SettingMenu' );

  unshift @{ $menu->{skin} }, 'skin_design' if $bn->get($sk, 'Design') &&  !exists $disable_menu->{skin_design} && !exists $disable_menu->{skin_design_menu};

  $cms->load_picker;

  my $arg = $bn->set_form_arg($args->{SettingMenu});

$bn->print(<<_HTML_);
<div align="center">
<table class="ConfigTableOut" cellpadding="0" cellspacing="5" width="100%"><tr><td width="5%" style="background-color:$table_out->{bgcolor}" valign="top">
<table class="ConfigTableIn" cellpadding="1" cellspacing="2" width="100%" style="font-size:10pt">
_HTML_

  for (my $i = 0; $i <= $#{$list}; $i++ ) {
    if (exists $disable_menu->{$menus->[$i]}) {
      for (@{$list->[$i]}) { $disable_menu->{$_} = 1 }
      next;
    }
    $bn->print('<tr><form action="'.$cms->{bbsnoteurl}.'" method="post">'.$arg,
      '<td nowrap class="ConfigMenu">'.$item->{$menus->[$i]}.'</td>',
      '<td align="right"><input class="ConfigArrow" type="submit" value="'.$item->{menu_submit}.'"></td>',
      '</tr><tr><td width="100%" style="background-color:'.$table_in->{bgcolor}.'" colspan="2">',
      '<select name="mode" style="width:'.$item->{menu_width}.'" onchange="this.form.submit()">',
      (map{
      unless (exists $disable_menu->{$_}) {
      my $s = $in->{mode} eq $_ ? ' selected':'';
      '<option value="'.$_.'"'.$s.'>'.$item->{$_}.'</option>'}
      }@{$list->[$i]}),
      '</select></td></form></tr>'."\n");
  }

  unless (exists $disable_menu->{menu_archive}) {
   $bn->print('<tr><form action="'.$cms->{bbsnoteurl}.'" method="post">'.$arg.'<input type="hidden" name="mode" value="archive">',
     '<td colspan="2" nowrap><input class="ConfigCmd" type="submit" value="'.$item->{menu_archive}.'"></td></form></tr>'."\n");
  }

  $arg = $bn->set_form_arg($args->{SettingMenuBack});

  $bn->print('<tr><form action="'.$cms->{bbsnoteurl}.'" method="post">'.$arg,
    '<td colspan="2" nowrap><input class="ConfigCmd" type="submit" value="'.$item->{back}.'"></td></form></tr>'."\n",
    '</table></td>');

  $bn->{arg}{SettingMenu}{mode} = $in->{mode};
  $bn->{arg}{SettingMenu}{cmd} = 'write';
  $cms->{arg} = $bn->set_form_arg($bn->{arg}{SettingMenu});

  my $caption = $mode eq 'archive' ? $item->{archive_dirimage} : $item->{$mode};
$bn->print(<<_HTML_);
<td valign="top" width="95%"><table width="100%" cellpadding="5" cellspacing="0"><tr>
<td class="ConfigCaption" colspan="2" style="width:100%;font-size:12pt;background-color:$table_out->{bgcolor}" nowrap>
$tree : $caption </td></tr><td>
<form name="config" action="$cms->{bbsnoteurl}" method="post" target="_self" enctype="multipart/form-data">$cms->{arg}
<table width="100%" style="font-size:10pt" class="ConfigTable">
_HTML_


  if (@{ $cms->{error} }) {
    $bn->print('<tr><td colspan="2"><b>');
    for (@{ $cms->{error} }) { $bn->print("$_<br>") }
    $bn->print('</b></td></tr>');
  }

  $bn->error('ERR_FUNCTION') if exists $bn->{sys}{CMS}{DISABLE_MENU}{$mode};

}

sub hr {
  $bn->print('<tr><td colspan="2"><hr class="ConfigHr"></td></tr>');
}
sub siz {
  my( $cms, $cfg, $vals, $xcfg ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};

  $bn->print('<tr><td width="10%" align="right" nowrap>'.$cms->{item}{$cfg}.'</td><td>',
  $cms->{item}{ $cfg.'_'.$xcfg->[0] },
  ' : <input type="text" name="'.$cfg.'_'.$xcfg->[0].'" value="'.$vals->[0].'" size="4">  x ',
  $cms->{item}{ $cfg.'_'.$xcfg->[1] },
  ' : <input type="text" name="'.$cfg.'_'.$xcfg->[1].'" value="'.$vals->[1].'" size="4"></td></tr>');

}

sub txt {
  my( $cms, $cfg, $val, $size, $unit ) = @_;
  my $s = $size ? ' size=' . $size : '';
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  $val =~ s/&/&#38;/g;
  $val =~ s/</&lt;/g;
  $val =~ s/</&gt;/g;
  $val =~ s/"/&quot;/g;

$bn->print(<<_HTML_);
<tr><td width="10%" align="right" nowrap>$cms->{item}{$cfg}</td>
<td><input type="text" name="$cfg" value="$val"$s> $unit</td></tr>
_HTML_
}

sub txa {
  my( $cms, $cfg, $val, $cols, $rows ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  my $c = $cols ? ' cols=' . $cols : '';
  my $r = $rows ? ' rows=' . $rows : '';
  chomp $val;
  $val =~ s/&/&amp;/g;
  $val =~ s/</&lt;/g;
  $val =~ s/>/&gt;/g;
$bn->print(<<_HTML_);
<tr><td width="10%" align="right" nowrap>$cms->{item}{$cfg}</td>
<td><textarea name="$cfg" wrap="off"$c$r>$val</textarea>$cms->{item}{'manual_'.$cfg}</td></tr>
_HTML_
}

sub col {
  my( $cms, $cfg, $val ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  my $cp;
  if ($cms->{picker}) {
    $cp = ' onclick="PC_open(this,event)" onblur="PC_hide()" onchange="PC_set(this.value)"';
    my $tc;
    if ($val =~ /\#(\w\w)(\w\w)(\w\w)/) {
       $tc = ';color:';
       $tc .= 99 > int (((hex $1) + (hex $2) + (hex $3)) / 3) ? '#FFFFFF' : '#000000';
    }
    $cp .= " style='background-color:$val$tc'";
  }

$bn->print(<<_HTML_);
<tr><td width="10%" align="right" nowrap>$cms->{item}{$cfg}</td>
<td><input class="ConfigColor" type="text" name="$cfg" value="$val" size="9"$cp></td></tr>
_HTML_

}

sub fld_set {
  my( $cms, $cfg ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  $bn->print('<tr><td colspan="2"><fieldset class="ConfigFld"><legend>',
    $cms->{item}{$cfg},'</legend><table style="font-size:10pt"><tr>');
}

sub fld_end {
  my( $cms, $cfg ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  $bn->print('</tr></table></fieldset><br></td></tr>');
}

sub inf {
  my( $cms, $cfg, $align ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  $bn->print('<tr><td>&nbsp;</td><td', ( $align ? ' align="'.$align.'"' : '' ), '>',
  $cms->{item}{$cfg},'</td></tr>');
}
sub cap {
  my( $cms, $cfg, $info, $align1, $align2 ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  $bn->print( '<tr><td align="', ( $align1 ? $align1 : 'right' ),
   '" valign="top" nowrap>', $cms->{item}{$cfg}, '</td><td',
   ( $align2 ? ' align="'.$align2.'"' : '' ), '>',
   ( exists $cms->{item}{$info} ? $cms->{item}{$info} : '&nbsp;' ),
   '</td></tr>' );
}
sub chk {
  my( $cms, $cfg, $val ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
     return if exists $disable_menu->{$cfg};
  my $s = $val ? ' checked' : '';
$bn->print(<<_HTML_);
<tr><td align="right"><input type="checkbox" name="$cfg" id="cms_$cfg" value="1"$s></td>
<td><label for="cms_$cfg">$cms->{item}{$cfg}</label></td></tr>
_HTML_
}
sub opt {
  my( $cms, $cfg, $val, $list ) = @_;
  my $disable_menu = $bn->{sys}{CMS}{DISABLE_MENU};
    return if exists $disable_menu->{$cfg};
  my $limit_str = $bn->{sys}{CMS}{LIMIT_STR};
    $list = $limit_str->{$cfg} if exists $limit_str->{$cfg};

$bn->print(<<_HTML_);
<tr><td width="10%" align="right" nowrap>$cms->{item}{$cfg}</td>
<td><select name="$cfg">
_HTML_

  for (@$list) {
    $bn->print("<option value='$_'" . ($val eq $_ ? ' selected' : '') . '>'
    . $cms->{item}{$cfg . '_' . $_} . '</option>')}
  $bn->print('</select></td></tr>');

}

1;

__END__
