#=============================================================================
# BBS Note Comet 1.x 
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2005/10/21
# Version : 0.01
#=============================================================================
package BN::Design;
use strict;
use vars qw( $VERSION $bn );

$VERSION = '0.02.000';
sub new
{
  my $dsn = bless { }, $_[0];
  $bn = $_[1];
  $dsn;
}

# GIF摜̃J[CY
# 摜̐FFύXFɃXChB
# gif2colorize([̓pX], [o̓pX], [F], [ύXF])
sub gif2colorize {
  my( $dsn, $in_img, $out_img, $base, $color, $phase ) = @_;

  my $bin;
  my $bin1 = $bn->open($in_img);
  $bin1 or return;
  binmode $bin1;

  seek $bin1, 10, 0;
  read $bin1, $bin, 1;

  $bin = unpack 'B*', $bin;
  my $gctf = substr $bin, 0, 1;
#  my $cr = substr $bin, 1, 3;
#  my $sf = substr $bin, 4, 1;

  unless ($gctf) {
    close $bin1;
    return;
  }

  my $sgct = "0" x 29 . substr $bin, 5, 3;
  $sgct = unpack("N", pack("B32", $sgct));
  $sgct = 2 ** ($sgct + 1);

  my $bin2 = $bn->open($out_img, '>');
  $bin2 or close $bin1 and return;
  binmode $bin2;

  seek $bin1, 0, 0;
  read $bin1, $bin, 13;
  print $bin2 $bin;

  my($b_r, $b_g, $b_b) = $dsn->get_rgb($base);
  my($c_r, $c_g, $c_b) = $dsn->get_rgb($color);

  for (my $n = 0; $n < $sgct; $n++) {
    read $bin1, $bin, 1;
    my $r = unpack "C*", $bin;
    read $bin1, $bin, 1;
    my $g = unpack "C*", $bin;
    read $bin1, $bin, 1;
    my $b = unpack "C*", $bin;

    if ( ref $phase eq 'HASH' ) {
      my $hex = $dsn->rgb2color( $r, $g, $b );
      ( $r, $g, $b ) = $dsn->get_rgb( $phase->{ $hex } ) if exists $phase->{ $hex };
    } else {
      $r = $r - $b_r + $c_r;
      $g = $g - $b_g + $c_g;
      $b = $b - $b_b + $c_b;
      $r = $r < 0 ? 0 : $r > 255 ? 255 : $r;
      $g = $g < 0 ? 0 : $g > 255 ? 255 : $g;
      $b = $b < 0 ? 0 : $b > 255 ? 255 : $b;
    }

    for ($r, $g, $b) {
      print $bin2 substr(pack('I', $_), 0, 1) }
  }

  while (<$bin1>) { print $bin2 $_ }

  close $bin1;
  close $bin2;
  return 1;
}

# YUV Px␳
# yuv2colorize([F1 RGB], [F2 RGB])
# F1F2ƋPxdiffȉȂ␳B
# wiFƕFȂǂɂĂقړF̏ꍇA{ɂȂ̂ŕ␳ɗp
sub yuv2colorize {
  my( $dsn, $base, $color, $phase, $diff ) = @_;
  my @base_yuv = $dsn->rgb2yuv($dsn->get_rgb($base));
  my @phase_yuv = $dsn->rgb2yuv($dsn->get_rgb($phase->{$base}));

  my @yuv = $dsn->rgb2yuv($dsn->get_rgb($phase->{$color}));
  my @ta_yuv = $dsn->rgb2yuv($dsn->get_rgb($color));
  my @co_yuv = ($base_yuv[0] - $ta_yuv[0], $base_yuv[1] - $ta_yuv[1], $base_yuv[2] - $ta_yuv[2]);

  $diff ||= 20;

  return $phase->{$color} if abs($phase_yuv[0] - $yuv[0]) > $diff;

  '#'.$dsn->rgb2color( $dsn->yuv2rgb( $phase_yuv[0] + $co_yuv[0], $phase_yuv[1], $phase_yuv[2] ) );
}


# Px F RGB
# RGBm̋PxAFzŕԂ
sub color2yh {
  my( $dsn, $color1, $color2 ) = @_;
  my @c1 = $dsn->get_rgb( $dsn->get_color( $color1 ) );
  my @c2 = $dsn->get_rgb( $dsn->get_color( $color2 ) );
  my $y = int abs( ($dsn->rgb2yuv( @c2 ))[0] - ($dsn->rgb2yuv( @c1 ))[0] );
  my $h = abs( $c2[0] - $c1[0]) + abs( $c2[1] - $c1[1] ) + abs( $c2[2] - $c1[2] );
  return( $y, $h );
}

# F RGB
# RGBm̐FRGBl̔zŕԂ
# (Red, Green, Blue) = color2diff([FP],[FQ])
sub color2diff {
  my( $dsn, $color1, $color2 ) = @_;
  my @c1 = $dsn->get_rgb( $dsn->get_color( $color1 ) );
  my @c2 = $dsn->get_rgb( $dsn->get_color( $color2 ) );
  return( $c2[0] - $c1[0], $c2[1] - $c1[1], $c2[2] - $c1[2] );
}

# F HSB
# RGBm̐FHSVl̔zŕԂ
# (Hue, Saturation, Brightness) = hsb2diff([FP],[FQ])
sub hsb2diff {
  my( $dsn, $color1, $color2 ) = @_;
  my @h1 = $dsn->rgb2hsb( $dsn->get_rgb( $dsn->get_color( $color1 ) ) );
  my @h2 = $dsn->rgb2hsb( $dsn->get_rgb( $dsn->get_color( $color2 ) ) );
  return( $h2[0] - $h1[0], $h2[1] - $h1[1], $h2[2] - $h1[2] );
}

# Fϊ RGB->YUV
# (PxM(Y), PxMƐF̍(U), PxMƐԐF̍(V)) = rgb2yuv([Red], [Green], [Blue])
sub rgb2yuv {
  my( $dsn, $r, $g, $b ) = @_;
  my $y = ( 0.298912 * $r ) + ( 0.586611 * $g ) + ( 0.114478 * $b );
  my $u = ( -0.169 * $r ) + ( -0.331 * $g ) + ( 0.5 * $b );
  my $v = ( 0.5 * $r ) + ( -0.419 * $g ) + ( -0.081 * $b );
  return( $y, $u, $v );
}

# Fϊ YUV->RGB
# (Red, Green, Blue) = yuv2rgb([PxM(Y)], [PxMƐF̍(U)], [PxMƐԐF̍(V)])
sub yuv2rgb {
  my( $dsn, $y, $u, $v ) = @_;
  my $r = $y + ( 1.402 * $v );
  my $g = $y + ( -0.344 * $u ) + ( -0.714 * $v );
  my $b = $y + ( 1.772 * $u );
  $r = int( $r + 0.5 );
  $g = int( $g + 0.5 );
  $b = int( $b + 0.5 );
  return( $r, $g, $b );
}


# Fϊ YUV->RGB
# (Red, Green, Blue) = hsb2rbg([Hue], [Saturation], [Brightness])
sub hsb2rgb {
  my( $dsn, $h, $s, $v ) = @_;
  my( $r, $g, $b );

  $h = $h > 359 ? $h - 360 : $h < -180 ? -180 : int $h;
  $h = 360 - $h if $h < 0;
  $s = $s > 100 ? 100 : $s < 0 ? 0 : int $s;
  $v = $v > 100 ? 100 : $v < 0 ? 0 : int $v;

  my $S = $s * 2.55;
  my $V = $v * 2.55;

  if ( $S == 0 ) {
    $r = $V; $g = $V; $b = $V;
  } else {
    my $t1 = $V;
    my $t2 = ( 255 - $S ) * $V / 255;
    my $t3 = $h % 60;
    $t3 = ( $t1 - $t2 ) * $t3 / 60;
    if ( $h < 60 ) {
      $r = $t1; $b = $t2; $g = $t2 + $t3;
    } elsif( $h < 120 ) {
      $g = $t1; $b = $t2; $r = $t1 - $t3;
    } elsif( $h < 180 ) {
      $g = $t1; $r = $t2; $b = $t2 + $t3;
    } elsif( $h < 240 ) {
      $b = $t1; $r = $t2; $g = $t1 - $t3;
    } elsif( $h < 300 ) {
      $b = $t1; $g = $t2; $r = $t2 + $t3;
    } elsif( $h < 360 ) {
      $r = $t1; $g = $t2; $b = $t1 - $t3;
    } else {
      $r = $g = $b = 0;
    }
  }

  return $dsn->rgb2color( $r, $g, $b );
}

# Fϊ RGB->HSB
# (Hue, Saturation, Brightness) = rgb2hsb([Red], [Green], [Blue])
sub rgb2hsb {
  my( $dsn, $r, $g, $b ) = @_;
  my $max = $dsn->get_max( $r, $g, $b );
  my $min = $dsn->get_min( $r, $g, $b );
  my $chroma = $max - $min;
  my $hue;

  if ( $chroma == 0 ) {
    $hue = 0;
  } elsif ( $g == $max ) {
    $hue = ( $b - $r ) / $chroma * 60 + 120;
  } elsif ( $b == $max ) {
    $hue = ( $r - $g ) / $chroma * 60 + 240;
  } elsif ( $g < $b ) {
    $hue = ( $g - $b ) / $chroma * 60 + 360;
  } else {
    $hue = ( $g - $b ) / $chroma * 60;
  }

  $hue = int( $hue + 0.5 );
#  $hue = 0 if $hue == 360;

  # ʓx
  my $saturation = $chroma == 0 ? 0 : $chroma / $max * 100;
  $saturation = int( $saturation + 0.5 );

  # x
  my $brightness = $max / 255 * 100;
  $brightness = int( $brightness + 0.5 );

  return ( $hue, $saturation, $brightness );
}

# RGB`␳ RGB->#RGB
# "#RRGGBB" = rgb2color([Red], [Green], [Blue])
sub rgb2color {
  my( $dsn, @c ) = @_;
  for ( my $i = 0; $i < @c; $i++ )
   { $c[$i] = $c[$i] > 255 ? 255 : $c[$i] < 0 ? 0 : $c[$i];
    $c[$i] = int( $c[$i] + 0.5 ) }
  sprintf( "%02X%02X%02X", @c );
}

# RGB`␳ #RGB->decimal RGB
# (Red, Green, Blue) = get_rgb("#RRGGBB")
sub get_rgb {
  my( $dsn, $color ) = @_;
  return unless $color =~ /([0-9A-F].)([0-9A-F].)([0-9A-F].)/;
  my $r = hex $1;  my $g = hex $2;  my $b = hex $3;
  return( $r, $g, $b );
}

# RGBFfől擾
# Max = get_max([Red], [Green], [Blue])
sub get_max {
  my( $dsn, $r, $g, $b ) = @_;
  $r >= $g ? $r >= $b ? $r : $b : $g >= $b ? $g : $b ;
}
# RGBFfŏl擾
# Min = get_min([Red], [Green], [Blue])
sub get_min {
  my( $dsn, $r, $g, $b ) = @_;
  $r <= $g ? $r <= $b ? $r : $b : $g <= $b ? $g : $b ;
}
# RGBF 
# "RRGGBB" = set_color("RRGGBB", [Red], [Green], [Blue])
sub set_color {
  my( $dsn, $color, $r, $g, $b ) = @_;
  my $c;
  for (my $n = 0; $n < 6; $n += 2) {
    my $i = hex substr $color, $n, 2;
       if ($n == 0 && defined $r) { $i += $r }
    elsif ($n == 2 && defined $g) { $i += $g }
    elsif ($n == 4 && defined $b) { $i += $b }
    $c .= $dsn->dec2hex( $i < 0 ? 0 : $i > 255 ? 255 : $i );
  }
  $c;
}

# RRGGBB` 擾
sub get_color {
  my( $dsn, $color ) = @_;
  $color = uc $color;
  return 'FFFFFF' unless $color =~ /^#?([0-9A-F]{1,6})/;
  $color = $1;
  while( 6 > length $color ) { $color .= '0' }
  $color;
}

# 10i->16i
sub dec2hex { sprintf("%02X",int $_[1]) }

# RGB, RGB ԐF 擾
sub get_middle_color {
  my( $dsn, $color1, $color2 ) = @_;
  my @c1 = $dsn->get_rgb( $dsn->get_color( $color1 ) );
  my @c2 = $dsn->get_rgb( $dsn->get_color( $color2 ) );
  return sprintf "#%02X%02X%02X", map {
    int ( ( $c1[$_] + $c2[$_] ) >> 1 );
  } 0..2;
}
1;

__END__
