#=============================================================================
# BBS Note Comet 1.x 
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2007/07/30
# Version : 0.02.000 build 3
#=============================================================================
package BN::Log;
use strict;
use vars qw( $VERSION );
$VERSION = '0.02.000';

  # logging : cookie data
my @LOGGING_QUERY_COOKIE = qw/
  name adminuser
  /;

my @LOGGING_QUERY = (
  # Lv1 - logging : bbs
[qw/
  fc page mode log number cmd

/],

  # Lv2 - logging : bbs ex
  # Lv2 - logging : post data
  # Lv2 - logging : paint
[qw/
  sort target type search_view keyword
  name title file
  width height anime palette
/],

  # Lv3 - logging : post data
[qw/
  message icon nobr icon kaiketu font_color title_color table_color
/])
 ;

my $MAXLOGLINE = 50;

sub handler {
  my($r) = @_;
  Apache->request($r); #for Apache::CGI
  $r->send_http_header("text/html");
  my $qs = $r->args || "";
  my $v = $^V ? sprintf "v%vd", $^V : $];
  for (keys %ENV) {
    $r->print("$_ = $ENV{$_}<br>" );
  }
  $r->print( $r->dir_config('Dir') );
  $r->print( "ok $qs<br>" );
  $r->print( $v );
  1;
}

sub new {
  my $class = shift;
  my $lo = bless {}, $class;
  $lo->init(@_);
  $lo;
}

sub init {
  my( $lo, $bn ) = @_;
  my $sys = $bn->{sys};
  $lo->{bn} = $bn;

  $lo->{dir}         = $sys->{DirSys};
  $lo->{exe}         = $sys->{Exe};
  $lo->{dir_access}  = $lo->{dir} . '/access';
  $lo->{dir_log}     = $lo->{dir_access} . '/log';
  $lo->{dir_images}  = $lo->{dir_access} . '/images';
  $lo->{count_file}  = $lo->{dir_access} . '/count.' . $lo->{exe};
  $lo->{imgdat_file} = $lo->{dir_access} . '/imgdat.' . $lo->{exe};

  $lo;
}

sub menu {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};

  $bn->error('ERR_FUNCTION')
    unless $bn->get($cf, 'UseCounter') || $bn->get($cf, 'UseAccessLog');

     if ( $in->{fc} eq 'access_count' ) { $lo->count }
  elsif ( $in->{fc} eq 'access_log' )   { $lo->log   }

}

# access log analyzer
sub log {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  $bn->error('ERR_FUNCTION') unless $bn->get( $cf, 'UseAccessLog' );
  if ( $fl->{AUTHORITY} ne 'ADMIN' ) {
    my( $pwd, $usr ) = $bn->get_admin_pass;
    $bn->check_authority( $pwd ) or $bn->error( 'PASSWORD_ERROR' );
  }

  my $item = $lo->{item} = $bn->text( undef, 'log' );

  my $date = $bn->get_date();
  $date =~ m#(\d+)/(\d+)/(\d+)#;
  @{$lo}{'year', 'month', 'day'} = ( int $1, int $2, int $3 );

  ## cookie save checked
  $lo->menu_ckeck;
  $bn->send_http_header->head($bn->get($cf, 'BbsTitle').'::'.$item->{title}, 1);
  $lo->menu_log;

  if ( $in->{mode} =~ /^(?:day|week|month)$/ ) {
    my $func = 'stat_' . $in->{mode};
    if ( defined &$func ) { $lo->$func } else { $bn->error('ERR_FUNCTION') }
  } elsif ( $in->{mode} eq 'view' ) {
    $lo->view_log;
  } elsif ( $in->{mode} eq 'sea' ) {
    $lo->sea_log;
  } else {
    $lo->open_log;
  }
  $bn->foot;
  $bn->EXIT;
}

sub menu_ckeck{
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  ## cookie save checked
  my %chm;

  for ( qw/ip host agent os browse request named referer error/ )
   { $chm{$_} = 1 if $in->{$_} }

  if ( each %chm ) {
    $ck->{accesslog} = join ',', keys %chm;
    $fl->{write_cookie} = 1;
  } else {
    if ( exists $ck->{accesslog} ) {
      for ( split ',', $ck->{accesslog} )
       { $in->{$_} = 1 if /ip|host|agent|os|browse|request|named|referer|error/ }
    } else {
      # caption default check
      $in->{ip} = $in->{named} = 1;
    }
  }
}
sub _struct {
  return {
    pos => [], zone => {}, unique => {},
    ip => {}, host => {}, agent => {}, os => {},
    browse => {}, ver => {}, ref => {}, ck => {} }
}

sub stat_day {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $fl = $bn->{flag};
  my $item = $lo->{item};
  my $year = int $in->{year};
  my $month = int $in->{month};
  my $day = int $in->{day};

  ($year, $month, $day) = @{$lo}{'year', 'month', 'day'}
    unless $year || $month || $day;
  my $struct = $lo->_struct;
  my $file = sprintf "%04d%02d%02d", $year, $month, $day;
  my $path = "$lo->{dir_log}/$file.$sys->{Exe}";
  my $anzpath = "$lo->{dir_log}/$file.anz.$sys->{Exe}";
  if ( !-e $anzpath || (stat $path)[9] > (stat $anzpath)[9] ) {
    $lo->log_analyze( $file );
  }

  $lo->menu_date;
  $lo->anz_load( $anzpath, $struct );
  $lo->stat_data( $struct, $lo->stat_css );

}

sub stat_week {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $item = $lo->{item};
  my $page = int $in->{page};
  $page ||= 1;

  my( @cap_date, @a_zone, @a_unique, @total_zone, @total_unique,
    $total_zone_value, $total_unique_value, $max_zone, $max_unique );

  my $struct = $lo->_struct;
  my( $logs, $dead ) = $lo->get_log_list( $lo->init_log_date );
  shift @$logs;

  for ( my $n = 0, my $i = ($page - 1) * 7; $n < 7 && $i < @$logs; $n++, $i++ ) {
    my $file = $logs->[$i];
    $lo->cal_zone( $file, $dead, $struct, \@cap_date,
     \@total_zone, \@total_unique, \$max_zone, \$max_unique,
     \$total_zone_value, \$total_unique_value, \@a_zone, \@a_unique );
  }

  for (@a_zone) { for my $h (keys %$_) { $struct->{zone}{$h} += $_->{$h} } }

  $cap_date[0] =~ s#(\d{4})(\d.)(\d.)#$1/$2/$3#;
  $cap_date[1] =~ s#(\d{4})(\d.)(\d.)#$1/$2/$3#;

  $bn->print( '<table style="margin:10px;font-weight:900"><tr><td>',
   $item->{statistics}, ' : ', "$cap_date[0] - $cap_date[1]", '</td><td>' );
  $lo->log_page( 'pageformtop', 'week', $page, $logs );
  $bn->print( '</td></tr></table>' );

  my($txt_color, $bg_color, $midle_color1) = $lo->stat_css;

  # /* total_zone total_unique */
  $bn->print( '<hr size="1">' ) if @total_zone || @total_unique;
  $bn->print( '<table cellspacing="0"><tr valign="top"><td>' ) if @total_unique;
  my $i = 0;
  my $width = 150;
  my @week = qw/Sun Mon Tue Wed Thu Fri Sat/;
  for ( \@total_zone, \@total_unique ) {
    my $max;
    for (@$_) { $max += $_ }
    next unless $max;
    $bn->print( '</td><td>' ) if $i;
    $max = $i ? $max_unique : $max_zone;
    my $total = $i ? $total_unique_value : $total_zone_value;
    $bn->print( '<table class="AccessLogStatTable" cellpadding="1" cellspacing="0">');
    $bn->print( '<tr><th colspan="3">'.$item->{'cap_day_'.($i?'unique':'zone')}.' ('.$total.')</th></tr>');
    my $n = ($page - 1) * 7;
    $logs->[$n] =~ /(\d{4})(\d.)(\d.)/;
    my $z = $lo->zeller( $1, int $2, int $3 );
    for (@$_) {
      $logs->[$n] =~ /(\d{4})(\d.)(\d.)/;
      my $date = "$1/$2/$3";
      my $per = $max ? int ( $_ * 100 / $max ) : 0;
      my $w = int ( $width / 100 * $per );
      $bn->print( '<tr class="AccessLogStatList"><td align="right" style="padding:2px">',
        $date,'(<tt>',$item->{$week[$z]},'</tt>)</td><td align="right" style="padding:2px">',
        $_,'</td><td>',
        ( $w ? '<table cellpadding="0" cellspacing="0"><tr><td style="width:'.
        $w.'px;background-color:'.$midle_color1.';">&nbsp;</td></tr></table>' : '&nbsp;' ),
        '</td></tr>' );
      $n++; $z--; $z = 6 if $z < 0;
    }
    $bn->print( '</table>' );
    $i++; 
  }
  $bn->print( '</td></tr></table>' ) if @total_unique;
  $lo->stat_data( $struct, $txt_color, $bg_color, $midle_color1 );
}

sub stat_month {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $fl = $bn->{flag};
  my $item = $lo->{item};
  my $page = int $in->{page};
  $page ||= 1;

  my( @cap_date, @a_zone, @a_unique, @total_zone, @total_unique,
    $total_zone_value, $total_unique_value, $max_zone, $max_unique );

  my $struct = $lo->_struct;
  my( $logs, $dead ) = $lo->get_log_list( $lo->init_log_date );
  shift @$logs;

  my( @list, %mon_list );
  for ( @$logs ) { $mon_list{$1}++ if /^(\d{4}\d\d)/ }
  @list = sort { $b <=> $a } keys %mon_list;
  my $max = @list;
  { my $ta = $list[$page-1]; undef @list;
  for ( @$logs ) { unshift @list, $_ if /^$ta/ } }

  for my $file ( @list ) {
    $lo->cal_zone( $file, $dead, $struct, \@cap_date,
     \@total_zone, \@total_unique, \$max_zone, \$max_unique,
     \$total_zone_value, \$total_unique_value, \@a_zone, \@a_unique );
  }

  for (@a_zone) { for my $h (keys %$_) { $struct->{zone}{$h} += $_->{$h} } }

  my $cap = $list[$page];
  $cap =~ s#(\d{4})(\d.)\d.#$1/$2#;

  $bn->print( '<table style="margin:10px;font-weight:900"><tr><td>',
   $item->{statistics}, ' : ', "$cap", '</td><td>' );
  $lo->log_page( 'pageformtop', $max, $page, $logs );
  $bn->print( '</td></tr></table>',"\n" );

  my($txt_color, $bg_color, $midle_color1) = $lo->stat_css;

  # /* total_zone total_unique */
  $bn->print( '<hr size="1">',"\n" ) if @total_zone || @total_unique;
  my $zone_height = 100;
  my $std_day = int $1 if $list[0] =~ /(\d\d)$/;
  my $end_day = int $1 if $list[-1] =~ /(\d\d)$/;
  my $_t = \@total_zone; # \@total_unique
  my( $max, $max_zone_count );
  for (@$_t) { $max += $_; $max_zone_count = $_ if $max_zone_count < $_ }
  $bn->print(
    '<table class="AccessLogStatTable" cellpadding="1" cellspacing="0">',
    '<tr><th class="cap" colspan="',$end_day,'">'.$item->{'cap_day_zone'}.'</th></tr>',
    '<tr align="center" style="font-size:8pt;letter-spacing:-1px">',"\n"
  );
  if ( $std_day > 1 ) { for ( my $i = 1; $i < $std_day; $i++ )
      { $bn->print( '<td valign="bottom">&nbsp;</td>' ) } }

  for my $acc (@$_t) {
    my $per = $max_zone_count ? int ( $acc * 100 / $max_zone_count ) : 0;
    my $h = int ( $zone_height / 100 * $per );

    $bn->print(
      '<td valign="bottom">',
      ( $acc ? '<div align="center" style="width:17px;position:absolute">'.
      $acc.'</div><div style="height:12px"></div><table class="bar" cellspacing="0"><tr><td'.
      ' style="height:'.$h.'px">&nbsp;</td></tr></table>' : '&nbsp;'),
      '</td>',"\n"
    );
  }
  $bn->print( '</tr><tr style="font-size:10pt;background-color:', $txt_color, ';color:', $bg_color,'">' );
  my $z = $lo->zeller( split( '/', $cap, 2 ), 1 );
  for ( 1..$end_day ) {
    $bn->print( '<th', ( $z ? '' : " style=\"color:$midle_color1\"" ), '>', ( sprintf "%02d", $_ ), '</th>' );
    $z++; $z = 0 if $z > 6;
  }
  $bn->print( '</tr></table>',"\n" );
  $lo->stat_data( $struct, $txt_color, $bg_color, $midle_color1 );
}


sub stat_css {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $sk = $bn->{skin};
  my $bg_color  = $bn->get($sk, 'TagBody')->{bgcolor};
  my $txt_color  = $bn->get($sk, 'TagBody')->{text};
  my $midle_color1 = $lo->get_middle_color( $bg_color, $txt_color );
  my @table = qw/named ip host agent os browse ver referer/;
  my $zone_height = 100;
  my $item = $lo->{item};

  $bn->print( <<_HTML_ );
<style type="text/css">
.AccessLogStatList { font-size : 10pt }
.AccessLogStatList TD { border : 1px $midle_color1 solid }
.AccessLogStatTable { border : 1px $txt_color solid; margin-left : 10px }
.AccessLogStatTable TH {
  font-size : 10pt; color : $bg_color;
  text-align : left;
  padding : 2px; padding-left : 3px;
  background : $txt_color;
  border-right:1px $bg_color solid;
}
.AccessLogStatTable TH.cap { border: none }
.AccessLogStatTable .bar {
  width:15px; font-size:1px; padding: 0px;
  background : $midle_color1;
}
</style>
_HTML_

  # local anchor
  $bn->print( '<div style="margin-left:10px;margin-top:5px;font-size:10pt">[ ',
   ( join ' | ', map { '<a href="#'.$_.'">'.$item->{'cap_'.$_}.'</a>' } @table ), ' ]</div>');

  return( $txt_color, $bg_color, $midle_color1 );

}

sub stat_data {
  my( $lo, $struct, $txt_color, $bg_color, $midle_color1 ) = @_;
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $item = $lo->{item};
  my($i, $bzone, $max_zone_count, $total_zone_count, $max_unique_count, $total_unique_count);
  my $zone_height = 100;
  my @table = qw/named ip host agent os browse ver referer/;

  # /* stat of zone */
  if ( keys %{$struct->{zone}} ) {
    for (keys %{$struct->{zone}}) {
     $total_zone_count += $struct->{zone}{$_};
     $max_zone_count = $struct->{zone}{$_} if $max_zone_count < $struct->{zone}{$_} }

    $bn->print( '<hr size="1">' );
    $bn->print(
     '<table class="AccessLogStatTable" cellpadding="1" cellspacing="0" summary="access of zone">',
     '<tr><th class="cap" colspan="24">', $item->{cap_zone},' (',$total_zone_count,')','</th></tr>',
     '<tr align="center" style="font-size:8pt;letter-spacing:-1px">'
    );
    for (0..23) {
      my $k = sprintf "%02d", $_;
      my $per = $max_zone_count ? int ( $struct->{zone}{$k} * 100 / $max_zone_count ) : 0;
      my $h = int ( $zone_height / 100 * $per );
      $bn->print( '<td valign="bottom"><div align="center" style="width:17px;position:absolute">' );
      if ( exists $struct->{zone}{$k} ) {
        $bn->print( $struct->{zone}{$k},
         '</div><div style="height:12px"></div><table class="bar" cellspacing="0"><tr><td style="height:',
         $h,'px">&nbsp;</td></tr></table>' );
      }
      $bn->print('</td>' );
      $bzone .= '<th style="border-right:1px '.$bg_color.' solid">'.$k.'</th>';
    }
    $bn->print( '</tr><tr style="font-size:10pt;background-color:',
      $txt_color,';color:',$bg_color,'">', $bzone, '</tr></table>' );
    undef %{ $struct->{zone} }; undef $bzone;
  }

  # /* stat of unique */
  if ( keys %{$struct->{unique}} ) {
    for (keys %{$struct->{unique}}) {
     $total_unique_count += $struct->{unique}{$_};
     $max_unique_count = $struct->{unique}{$_} if $max_unique_count < $struct->{unique}{$_} }
    $bn->print( '<hr size="1">',
     '<table class="AccessLogStatTable" cellpadding="1" cellspacing="0" summary="access of unique">',
     '<tr><th class="cap" colspan="24">', $item->{cap_unique}, ' (',$total_unique_count,')','</th></tr>',
     '<tr align="center" style="font-size:8pt;">' );
    for (0..23) {
      my $k = sprintf "%02d", $_;
      my $per = $max_unique_count ? int ( $struct->{unique}{$k} * 100 / $max_unique_count ) : 0;
      my $h = int ( $zone_height / 100 * $per );
      $bn->print( '<td valign="bottom"><div align="center" style="width:17px;position:absolute">' );
      if ( exists $struct->{unique}{$k} ) {
        $bn->print( $struct->{unique}{$k},
         '</div><div style="height:12px"></div><table class="bar" cellspacing="0"><tr><td style="height:',
         $h,'px">&nbsp;</td></tr></table>' );
      }
      $bn->print('</td>' );
      $bzone .= '<th style="border-right:1px '.$bg_color.' solid">'.$k.'</th>';
    }
    $bn->print( '</tr><tr style="font-size:10pt;background-color:',
      $txt_color,';color:',$bg_color,'">', $bzone, '</tr></table>' );
    undef %{ $struct->{unique} }; undef $bzone;
  }

  # /* ip host agent os browse ver ref cookie */
  for ( @{$struct}{qw/ck ip host agent os browse ver ref/} ) {
    $bn->print( '<hr size="1">' ) unless $i > 4 && $i < 7;
    $bn->print( '<table cellspacing="0"><tr valign="top"><td>' ) if $i == 4;
    $bn->print( '<td>' ) if $i > 4 && $i < 7;
    my($key, $hash) = $lo->sort_table( $_ );
    my $n = 1;
    $bn->print( '<table class="AccessLogStatTable" cellpadding="3" cellspacing="0">',
      '<tr><th class="cap" colspan="3"><a name="#',$table[$i],'">'.
      $item->{'cap_'. $table[$i]}.'</a></th></tr>' );
    for (@$key) {
      my @v = split "\x08", $hash->{$_};
      if ( $in->{mode} eq 'day' && $i < 2 ) { @v = map { my $v = $_; $bn->unescape( \$v );
          '<a href="#named" onclick="Lsea(\''.$_.'\')">' . $v . '</a>' } @v }
      my $val = join ' / ', @v;
      $bn->unescape( \$val ) if $i == 3 || $i == 7 || ( $in->{mode} ne 'day' && $i == 0 );
      $bn->print( '<tr class="AccessLogStatList"><td align="right">'.
        $n.'</td><td align="right">'.$_.'</td><td>'.$val.'</td></tr>' );
      $n += @v;
      last if $n > 100;
    }
    $bn->print( '</table>' );
    $bn->print( '</td>' ) if $i > 3 && $i < 7;
    $bn->print( '</tr></table>' ) if $i == 6;
    $i++;
  }

}

sub menu_log {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $item = $lo->{item};
  my $arg = $bn->{arg};
  my $farg = $arg->{AccessLogMenuForm};
  $in->{mode} ||= 'raw';

  my $bg_color  = $bn->get($sk, 'TagBody')->{bgcolor};
  my $txt_color  = $bn->get($sk, 'TagBody')->{text};

  my $midle_color1 = $lo->get_middle_color($bg_color, $txt_color);

  my $form = '<form action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">';
  for (qw/password name fc year month day ip host
   agent os browse request named referer error/) { $farg->{$_} = $in->{$_}; }
  $farg->{mode} = '';

  $bn->print( '<table class="AccessLogMenu" width="100%" cellspacing="0" cellpadding="2"><tr>' );

  # /* read jump form <--- */
  for (qw/ip host agent os browse request named referer error/) { $farg->{$_} = $in->{$_} if $in->{$_} }
  $farg->{mode} = 'view';
  $farg->{jmp} = 0;
  $bn->print( '<form name="accesslogjmp" action="'.
    $bn->get($cf, 'BBSNoteUrl').'" method="post">' . $bn->set_form_arg($farg) . '</form>' );

  # /* css and script */
  $bn->print( <<_HTML_ );
<style type="text/css">
body { margin : 0px 0px auto }
.AccessLogMenuButton {
  color : $bg_color; cursor : pointer;
  background : $txt_color;
  border : 2px $txt_color solid;
}
.AccessLogMenu {
  background : $txt_color;
  border-top : 5px $txt_color solid;
  border-left : 5px $txt_color solid;
}
.AccessLogMenuNowMode, .AccessLogMenuNowMode INPUT {
  color : $txt_color;
  background : $bg_color;
  border : 2px $bg_color solid;
}
.AccessLogData TD, .AccessLogTrace {
  font-size : 10pt;
  padding-left : 3px;
  padding-right : 3px;
  border-bottom : 1px $txt_color solid;
  border-right : 1px $txt_color solid;
}
.AccessLogDataCaption TD, .AccessLogCaption{
  color : $bg_color; font-weight : 900;
  background-color : $txt_color;
  border : 2px $midle_color1 solid;
}
</style>
<script langage="javascript">
<!--
j=document.accesslogjmp;
function Ljmp(n){j.mode.value='view';j.jmp.value=n;j.submit()}
function Lsea(n){j.mode.value='sea';j.jmp.value=n;j.submit()}
//-->
</script>
_HTML_
  # /* css and script */

  for (qw/raw day week month/) {
    ## menu off : log limit under 7days
    next if $_ eq 'month' && $bn->get($cf, 'LimitAccessLog') <= 7;

    $farg->{mode} = $_;
    my $f = $form . $bn->set_form_arg($farg);
    $bn->print( $f, '<td', ( $in->{mode} eq $_ ? ' class="AccessLogMenuNowMode"' : '' ), '>',
      '<input class="AccessLogMenuButton" type="submit" value="',$item->{'menu_'.$_},'">',
      "</td></form>\n" );
  }

  $farg->{fc} = 'admin';
  delete $farg->{mode};
  my $f = $form . $bn->set_form_arg($farg);
  $bn->print( $f, '<td><input class="AccessLogMenuButton" type="submit" value="',
    $item->{'menu_admin'},'"></td></form>', '<td width="99%">&nbsp;</td></tr></table>' );

}

sub get_log_list {
  my( $lo, $log ) = @_;
  my( @logs, %dead );
  my $log_now = $lo->time_now;
  my @mday = (0,31,28,31,30,31,30,31,31,30,31,30,31);
  my $max = @$log;
  my $zy = $log->[0]; my $zm = $log->[1]; my $zd = $log->[2];
  my @nz = $log_now->[0,1,2];
  my $date_f = sub { int sprintf( "%04d%02d%02d", @_ ) };
  for ( my $i = 3; $i < $max; $i++ ) {
    my $n = $log->[$i];
    my $c1 = 0; my $c2 = 0;
    my $f = $i % 2;
    while( $n ) {
      my $d = &$date_f( $zy, $zm, $zd );
      unshift @logs, $d;
      $dead{ $d } = 1 if !$f;
      if ( $zd >= $mday[$zm] ) {
        $zm++; $zd = 1;
        if ( $zm > 12 ) {
          $zm =1; $zy++;
          $mday[2] = !$zy % 4 && $zy % 100 ||!$zy % 400 ? 29 : 28;
        }
      } else { $zd++ }
      $n--;
    }
  }
  return( \@logs, \%dead );
}

sub init_log_date {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $dir = $lo->{dir_log};
  my $init_date = "$dir/raw.init.$sys->{Exe}";
  my %log_array;

  if ( -e $init_date && -M $init_date < 0.8 ) {
    my $fh = $bn->open( $init_date );
    my @log_alive = split( "\t", <$fh> );
    close $fh;
    return \@log_alive;
  }

  my $dh = $bn->opendir( $dir );
  while ( $_ = readdir $dh ) {
    next if ! /^(\d{4})(\d\d)(\d\d)\.$sys->{Exe}/;
    $log_array{$1} = {} if !exists $log_array{$1};
    $log_array{$1}{$2} = {} if !exists $log_array{$1}{$2};
    $log_array{$1}{$2}{$3} = 0;
  }
  closedir $dh;

  my @year = sort keys %log_array;
  my @log_old = ( $year[0] );
  $log_old[1] = (sort keys %{$log_array{$year[0]}})[0];
  $log_old[2] = (sort keys %{$log_array{$year[0]}{$log_old[1]}})[0];
  my @log_latest = ( $year[-1] );
  $log_latest[1] = (sort keys %{$log_array{$year[-1]}})[-1];
  $log_latest[2] = (sort keys %{$log_array{$year[-1]}{$log_latest[1]}})[-1];

  my $log_now = $lo->time_now;
  my $log_alive = $lo->get_log_array( \%log_array, \@log_old, $log_now );

  my $fh = $bn->open( $init_date, '>' );
  print $fh join( "\t", @$log_alive );
  close $fh;
  return $log_alive;
}

sub cal_zone {
  my( $lo, $file, $dead, $struct, $cap_date,
   $total_zone, $total_unique, $max_zone, $max_unique,
   $total_zone_value, $total_unique_value, $a_zone, $a_unique ) = @_;
  my $bn = $lo->{bn};
  my $sys = $bn->{sys};

  $cap_date->[0] = $file unless defined $cap_date->[0];
  $cap_date->[1] = $file;
  my $path = "$lo->{dir_log}/$file.$sys->{Exe}";;
  my $anzpath = "$lo->{dir_log}/$file.anz.$sys->{Exe}";
  if( exists $dead->{$file} )
   { push @$total_zone, 0; push @$total_unique, 0; next }
  $lo->log_analyze( $file )
    if !-e $anzpath || (stat $path)[9] > (stat $anzpath)[9];

  $lo->anz_load( $anzpath, $struct );
  my %a = %{ $struct->{zone} };
  push @$a_zone, \%a;
  my %b = %{ $struct->{unique} };
  push @$a_unique, \%b;

  my( $total_zone_count, $total_unique_count ) = (0, 0);
  for (keys %{$struct->{zone}}) { $total_zone_count += $struct->{zone}{$_} }
  for (keys %{$struct->{unique}}) { $total_unique_count += $struct->{unique}{$_} }
  push @$total_zone, $total_zone_count;
  push @$total_unique, $total_unique_count;
  $$total_zone_value += $total_zone_count;
  $$total_unique_value += $total_unique_count;
  $$max_zone = $total_zone_count if $$max_zone < $total_zone_count;
  $$max_unique = $total_unique_count if $$max_unique < $total_unique_count;
  undef %{ $struct->{zone} };
  undef %{ $struct->{unique} };
}


sub menu_date {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $arg = $bn->{arg};
  my $item = $lo->{item};
  my $year = int $in->{year};
  my $month = int $in->{month};
  my $day = int $in->{day};
  my $farg = $arg->{AccessLogPageForm};
  my $bflag;

  my $log_alive = $lo->init_log_date;

  unless ( $year || $month || $day ) {
    ($year, $month, $day) = @{$lo}{'year', 'month', 'day'};
    $bflag = 1;
  }

  my @now_time = localtime( time + $bn->get( $cf, 'TimeDiffSec' ) );
  $now_time[5] += 1900; $now_time[4]++;
  my $now_year = $now_time[5];
  my $now_month = $now_time[4];
  my $now_day = $now_time[3];

  $year ||= $now_year; $month ||= $now_month; $day ||= $now_day;

  for (qw/fc password name mode jmp ip host
   agent os browse request named referer error/){ $farg->{$_} = $in->{$_} }

  $bn->print(
    '<table><form name="logdate" action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">',
    $bn->set_form_arg($farg), '<tr><td>',

    '<select name="year" class="select" onchange="rmon();rday()">',
    '</select> ',$item->{year},
    '<select name="month" class="select" onchange="rday()">',
    '</select> ',$item->{month},
    '<select name="day" class="select">',
    '</select> ',$item->{day},

    '<noscript>',
    '<input name="year" class="input" style="text-align:right" value="',$year,'" size="5" maxlength="4">',
    $item->{year},
    '<input name="month" class="input" style="text-align:right" value="',$month,'" size="3" maxlength="2">',
    $item->{month},
    '<input name="day" class="input" style="text-align:right" value="',$day,'" size="3" maxlength="2">',
    $item->{day},
    '</noscript>',

    ' <input class="button" type="submit" value="',$item->{view},'">',
    ( $in->{mode} eq 'sea' ? '</td><td class="AccessLogCaption" style="font-size:10px"> '.
      $item->{cap_trace}.' </td><td class="AccessLogTrace">'.
      ( do { my $val = $in->{jmp}; $bn->unescape( \$val ); $val } ) : '' ),
    '</td></tr></form></table>'
   );

  $bn->print( '<script langage="javascript">',"\n<!--\nvar alive=new Array(", join( ',', @$log_alive ),
    ");\nvar iz=new Array($year,$month,$day);",
    "var nz=new Array($now_year,$now_month,$now_day);\n" );

  $bn->print(<<'_HTML_');
var mday=new Array(0,31,28,31,30,31,30,31,31,30,31,30,31);
d=document.logdate;ym=d.year;ymo=ym.options;dm=d.month;dmo=dm.options;s=d.day;so=s.options
var dead= new Array();al=alive.length;fy=zy=alive[0];fm=zm=alive[1];fd=zd=alive[2];mday_g(
zy);for(i=3;i<al;i++){n=alive[i];c1=c2=0;f=i%2;while(n){if(!f){c2=date_f(zy,zm,zd);if(c1<1
)c1=c2} if(zd>=mday[zm]){zm++;zd=1; if(zm>12){zm=1;zy++;mday_g(zy)}} else {zd++}n--}if(!f)
dead.push(c1,c2)}nn=date_f(nz[0],nz[1],nz[2]);nm=date_f(zy,zm,zd);if(nn-nm>0)dead.push(nm,
nn);var m=zy-fy;for(var i=0;i<=m;i++){ymo[i]=new Option();ymo[i].value=fy+i;ymo[i].text=fy
+i}sld(ymo,iz[0]);rmon();sld(dmo,iz[1]);rday();sld(so,iz[2]);function cd_mon(y,m){mday_g(y
);f=date_f(fy,fm,fd);n1=date_f(y,m,1);n2=date_f(y,m,mday[m]);m=dead.length;for(var i=0;i<m
;i+=2){if(dead[i]<=n1&&n2<=dead[i+1]) return 1; if(n2<dead[i]) return 0} return 0}function
cd_day(y,m,d) { var n=date_f(y,m,d); f=date_f(nz[0],nz[1],nz[2]); if(f<n) return 1; var m=
dead.length;for(var i=0;i<m;i+=2){if(dead[i]<=n&&n<=dead[i+1])return 1;if(n<dead[i])return
0}return 0}function zeller(zy,zm,zd){if(zm==1||zm==2){zm+=12;zy--}zy+="";var a=zy.split(""
);var c=parseInt(a[0]+a[1]);var y=parseInt(a[2]+a[3]);return((parseInt(21*c/4)+parseInt(5*
y/4)+parseInt(26*(zm+1)/10)+(zd-1))%7)}function rmon(){var y=parseInt(ymo[ym.selectedIndex
].value); var m=parseInt(dmo.length?dmo[dm.selectedIndex].value:iz[1]);var n=0; mday_g(y);
for(var i=dmo.length;i>=0;i--) {dmo[i]=null}for(i=1;i<=12;i++){if(fy==y&&i<fm)continue;if(
nz[0] == y && nz[1] < i ) break;var f=cd_mon(y,i); if(f&&!n)continue; dmo[n]=new Option();
dmo[n].value = dmo[n].text=i; if(f)dmo[n].text='x'; n++} } function rday(){var y=parseInt(
ymo[ym.selectedIndex].value);var m=parseInt(dmo[dm.selectedIndex].value);var n=0; mday_g(y
);var w=zeller(y,m,1);for(var i=so.length;i>=0;i--){so[i]=null}for(i=1;i<=mday[iz[1]];i++)
{if(fy==y&&fm==m&&i<fd)continue;if(nz[0]==y&&nz[1]==m&&nz[2]<i)break; so[n] = new Option()
so[n].value=so[n].text=i;if(w==6){so[n].style.color='blue'} else if(!w){so[n].style.color=
'red'}w++;if(w>6)w=0;if(cd_day(y,m,i))so[n].text='x';n++}} function sld(o,v){for(var i=0;i
<=o.length;i++){if(o[i].value==v){o[i].selected=true;return}}}function mday_g(y){mday[2]=!
y%4&&y%100||!y%400?29:28} function date_f(y,m,d){return parseInt(y+''+f02d(m)+''+f02d(d))}
function f02d(n){return n>9?n:'0'+n}
//-->
</script>
_HTML_

}

sub open_log {
  my( $lo, $sea_array ) = @_;
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $arg = $bn->{arg};
  my $item = $lo->{item};
  my $year = int $in->{year};
  my $month = int $in->{month};
  my $day = int $in->{day};
  my $page = int $in->{page};

  $page ||= 1;

  ($year, $month, $day) = @{$lo}{'year', 'month', 'day'} unless $year || $month || $day;
  $lo->menu_date;

  my $file = sprintf "%04d%02d%02d", $year, $month, $day;
  my $path = "$lo->{dir_log}/$file.$sys->{Exe}";
  $bn->end( 'ERROR', $item->{ERR_LOG_DATE} ) unless -e $path;

  my $fh = $bn->open( $path ) or $bn->error( 'ERR_DATA_NOTOPEN', $path );
  my $max_line = $MAXLOGLINE;
  my $size = (stat $fh)[7];
  my $n;

  my $max_page;
  if ( ref $sea_array eq 'ARRAY' ) {
    $max_page = @$sea_array;
    $max_page = int( $max_page / $max_line ) + ( $max_page % $max_line ? 1 : 0 );
  } else {
    my $pos_array = $lo->log_analyze( $file );
    my($last_pos, $skip_cnt) = split ':', pop @$pos_array;
    $max_page = @$pos_array;
    if ( $max_page == $page ) {
      seek $fh, 0, 0;
      $n = $max_line - $skip_cnt;
    } else {
      seek $fh, int $pos_array->[$max_page - $page - 1], 0;
      for ( my $i = $skip_cnt; $i > 0; $i-- ) { <$fh> };
    }
  }

  $bn->print( '<hr size="1">' );

  # caption checkboxs
  my $farg = $arg->{AccessLogCapForm};
  for (qw/fc page password name mode jmp/) { $farg->{$_} = $in->{$_} if exists $in->{$_} }
  $farg->{year}  = $year; $farg->{month} = $month; $farg->{day}   = $day;

  $bn->print( '<table><tr><td>' );
  $lo->log_page( 'pageformtop', 'raw', $page, $max_page, $year, $month, $day );
  $bn->print( '</td><td>' );

  $bn->print(
    '<table style="font-size:10pt"><tr>',
    '<form name="accesslogcap" action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">',
    $bn->set_form_arg($farg)
  );

  for (qw/ip host agent os browse request named referer error/)
   { $bn->print( '<td><input onclick="document.accesslogcap.submit()" type="checkbox" value="1" name="',
     $_,'" id="cap_',$_,'"',($in->{$_}?' checked':''),
     '><label for="cap_',$_,'">',$item->{'cap_'.$_},'</label></td>' ) }

  $bn->print( '</form></tr></table></td></tr></table>' );

  my( @line, @sline );
  if ( ref $sea_array eq 'ARRAY' ) {
    for ( my $i = ( $page - 1 ) * $max_line ; $i < @$sea_array; $i++ ) {
      unshift @sline, $sea_array->[$i];
      seek $fh, $sea_array->[$i], 0;
      $_ = <$fh>;
      chomp; unshift @line, $_;
      $n++; last if $n >= $max_line;
    }
  } else {
    unshift @sline, tell $fh;
    while ( <$fh> ) {
      chomp; unshift @line, $_;
      $n++; last if $n >= $max_line;
      unshift @sline, tell $fh;
    }
  }
  close $fh;

  return unless $n; # no logging

  # caption
  $bn->print( "\n",'<table class="AccessLogData" summary="logging data"><tr class="AccessLogDataCaption">' );
  $in->{number} = 1; $in->{date} = 1;
  for (qw/number date ip host agent os browse request named referer error/)
   { $bn->print( '<td nowrap>',$item->{'cap_'.$_},'</td>' ) if $in->{$_} }
  $bn->print( '</tr>' );


  $n = ($page - 1) * $max_line + 1;
  my $i = 0;
  for ( @line ) {
    my $data = {};
    my $pos = $sline[$i];

    @{$data}{qw/date ip host agent os browse browsever referer method request cookie ext/} = split /\t/;
    $data->{number} = sprintf "%04d", $n;

    for (qw/agent referer/)
     { if ($in->{$_} && (my $v = $data->{$_}) ) { $bn->unescape( \$v ); $data->{$_} = $v } }

    # request
    if ( $in->{request} ) {
      my $req = $data->{request};
      $data->{request} = '';
      for ( split '&', $req ) {
        my($key, $val) = split '=', $_, 2;
        if ( $key eq 'fc' ) {
          $bn->unescape( \$val );
          $data->{request} = $val;
          last;
        }
      }
    }

    # cookie named
    for ( split '&', $data->{cookie} ) {
      my($key, $val) = split '=', $_, 2;
      if ( $key eq 'name' ) {
        $data->{named_val} = $val;
        $bn->unescape( \$val );
        $data->{named} = $val;
        last;
      }
    }

    # ext
    ($data->{error}) = split /\x08/, $data->{ext}, 2
      if $data->{ext} && $data->{ext} ne '1';

    $bn->print( '<tr><td><a href="#',$n,'" onclick="Ljmp(',$pos,')">',$data->{number},'</a></td>' );
    for (qw/date ip host agent os browse command request named referer error/) {
      next unless $in->{$_};
      if ( $_ eq 'ip' ) {
        $bn->print( '<td><a href="#ip" onclick="Lsea(\'',$data->{$_},'\')">',$data->{$_},'</a></td>' );
      } elsif ( $_ eq 'named' && $data->{$_} ) {
        $bn->print( '<td><a href="#named" onclick="Lsea(\'',$data->{named_val},'\')">',$data->{$_},'</a></td>' );
      } else {
        $bn->print( '<td>',$data->{$_}?$data->{$_}:'&nbsp;','</td>' );
      }
    }
    $bn->print( '</tr>' );
    $n++; $i++;
  }
  $bn->print( '</table>' );

}

sub view_log {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $arg = $bn->{arg};
  my $item = $lo->{item};
  my $year = int $in->{year};
  my $month = int $in->{month};
  my $day = int $in->{day};
  my $jmp = int $in->{jmp};
  $jmp ||= 0;

  ($year, $month, $day) = @{$lo}{'year', 'month', 'day'} unless $year || $month || $day;

  my $file = sprintf "%04d%02d%02d", $year, $month, $day;
  my $path = "$lo->{dir_log}/$file.$sys->{Exe}";
  $bn->end( 'ERROR', split /:/, $item->{ERR_LOG_DATE}, 2 ) unless -e $path;
  my $fh = $bn->open( $path ) or $bn->error( 'ERR_DATA_NOTOPEN', $path );
  seek $fh, $jmp, 0;
  $_ = <$fh>;
  close $fh;
  chomp;

  return unless $_;
  my $data = {};
  @{$data}{qw/date ip host agent os browse ver referer method request cookie ext/} = split /\t/;

  for (qw/agent referer/)
   { if ( (my $v = $data->{$_}) ) { $bn->unescape( \$v ); $data->{$_} = $v } }

  # cookie named
  for ( split '&', $data->{cookie} ) {
    my($key, $val) = split '=', $_, 2;
    if ( $key eq 'name' ) {
      $data->{named_val} = $val;
      $bn->unescape( \$val );
      $data->{named} = $val;
      last;
    }
  }

  # ext
  if ( $data->{ext} && $data->{ext} ne '1' ) {
    my @err = split /\x08/, $data->{ext};
    $data->{error} = $bn->text(shift @err, 'error');
    my $n;
    for ( @err ) {
      $n++;
      $data->{error} =~ s/\[_$n]/$_/g;
    }
    $bn->unescape( \$data->{error} );
  }

  $bn->print( '<table>' );
  for (qw/date ip host agent os browse named referer error/) {
    $bn->print( '<tr class="AccessLogData"><td class="AccessLogCaption" nowrap>',
      $item->{'cap_'.$_}, "</td><td>$data->{$_}</td></tr>" ) if $data->{$_};
  }

  $bn->print( '<tr class="AccessLogData"><td class="AccessLogCaption" nowrap>',
   $item->{cap_request}, '</td><td><table>' );
  my @k = split '&', $data->{request};
  my %req;
  my $printd = sub{
    my($key, $val) = @_;
    # password mask
    $val = '*' x length $val if $key =~ /pass/;
    # unescape
    $bn->unescape( \$val ) if $key =~ /^(?:name|message|title|url|icon|\w*filename|\w*Content\-Type)$/;
    # message
    if ( $key eq 'message' ) {
      $val =~ s/\n/<br>/g;
    }
    $bn->print( '<tr><td class="AccessLogCaption">', $key, '</td><td>', $val, '</td></tr>' );
    delete $req{$key};
  };

  for ( @k ) {
    my($key, $val) = split '=', $_, 2;
    next if $key eq 'submit';
    $req{$key} = $val;
  }
  delete $req{submit};
  delete $req{url} if exists $req{url} && $req{url} eq 'http%3A%2F%2F';

  if ( exists $req{fc} ) {
    if ( exists $item->{'fc_'.$req{fc}} ) {
      &$printd( $item->{fc}, $item->{'fc_'.$req{fc}} );
    } else {
      &$printd( $item->{fc}, $req{fc} );
    }
    delete $req{fc};
  }

  for (qw/title name mail url message password width height/) { &$printd($_, $req{$_}) if exists $req{$_} }
  while ( @_ = each %req ) { &$printd(@_) }

  $bn->print( '</table></td></tr>' );
  $bn->print( '</table>' );

  my $farg = $arg->{AccessLogBackForm};
  for (qw/fc password name ip host agent os browse request
   named referer error year month day page/){ $farg->{$_} = $in->{$_} }
  $farg->{mode} = 'raw';
  $bn->print(
    '<form name="logdate" action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">', $bn->set_form_arg($farg),
    '<div style="text-align:center"><input class="button" type="submit" value=" ',
    $item->{back},' "></div></form>'
  );

}

sub sea_log {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $sys = $bn->{sys};
  my $item = $lo->{item};
  my $year = int $in->{year};
  my $month = int $in->{month};
  my $day = int $in->{day};
  my $jmp = $in->{jmp};
  return unless $jmp;

  unless ( $year || $month || $day ) {
    ($year, $month, $day) = @{$lo}{'year', 'month', 'day'};
  }

  my $file = sprintf "%04d%02d%02d", $year, $month, $day;
  my $path = "$lo->{dir_log}/$file.sea.$sys->{Exe}";
  $lo->log_analyze( $file ) if !-e $path;
  my $fh = $bn->open( $path ) or $bn->error( 'ERR_SYS_NOTOPEN', $path );
  my @array;
  while ( <$fh> ) {
    my($key, $val) = split "\t", $_, 2;
    if ( $key eq $jmp ) {
      chomp $val;
      @array = split "\t", $val;
      last;
    }
  }
  close $fh;

  if ( @array ) {
    $lo->open_log(\@array);
  } else {
    $lo->menu_date;
    $bn->print( '<h2 align="center">', $item->{ERR_LOG_VALUE}, '</h2>' );
  }
}

sub log_page {
  my( $lo, $name, $mode, $now, $logs, $year, $month, $day ) = @_;
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $arg = $bn->{arg};
  my $item = $lo->{item};

  my( $max, @opt_text );
  if ( $mode eq 'week' ) {
    my $m = @$logs;
    for ( my $i = 0; $i < $m; $i+=7 ) {
      my $st = "$1/$2/$3" if $logs->[$i] =~ /(\d{4})(\d.)(\d.)/;
      my $ed = $m <= $i+6 ? $logs->[$m-1] : $logs->[$i+6];
      $ed = "$1/$2/$3" if $ed =~ /(\d{4})(\d.)(\d.)/;
      push @opt_text, $st. ' - '. $ed;
      $max++;
    }
  } elsif ( $mode eq 'raw' ) {
    $max = $logs;
    for ( 1..$max ) { push @opt_text, $_ }
  } else {
    my %list;
    for ( @$logs ) { $list{"$1 - $2"}++ if /^(\d{4})(\d.)/ }
    for ( keys %list ) { push @opt_text, $_ }
    $max = $mode;
  }

  return unless $max > 1;

  my $farg = $arg->{AccessLogForm};
  for (qw/fc password name mode jmp ip host agent os browse request named referer error/)
   { $farg->{$_} = $in->{$_} if exists $in->{$_} }
  $farg->{day}   = $day   if defined $day;
  $farg->{year}  = $year  if defined $year;
  $farg->{month} = $month if defined $month;

  $bn->print( '<table><tr>' );

  my $form = '<form action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">';
  $form .= $bn->set_form_arg($farg);
  $bn->print( $form, '<input type="hidden" name="page" value="',$now - 1,'">',
    '<td><input class="button" type="submit" value="', $item->{back}, '"></td></form>' ) if 0 < $now - 1;

  $form = '<form name="'.$name.'" action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">';
  $form .= $bn->set_form_arg($farg);

  $bn->print( $form, '<td><select class="select" name="page" onchange="document.'.$name.'.submit()">' );
  for ( my $i = 1; $i <= $max; $i++ )
   { $bn->print( '<option value="',$i,'"',$now == $i ? ' selected' : '','>',$opt_text[$i-1],'</option>' ) }
  $bn->print( '</select></td></form>' );

  $form = '<form action="'.$bn->get($cf, 'BBSNoteUrl').'" method="post">';
  $form .= $bn->set_form_arg($farg);
  $bn->print( $form, '<input type="hidden" name="page" value="',$now + 1,'">',
    '<td><input class="button" type="submit" value="', $item->{next}, '"></td></form>' ) if $max > $now;

  $bn->print( '</tr></table>',"\n" );

}

sub log_analyze {
  my( $lo, $file, $pos ) = @_;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $item = $lo->{item};

  my $path = "$lo->{dir_log}/$file.$sys->{Exe}";
  my $anzpath = "$lo->{dir_log}/$file.anz.$sys->{Exe}";
  my $seapath = "$lo->{dir_log}/$file.sea.$sys->{Exe}";

  my $flag_counter = $bn->get($cf, 'UseCounter');
  my $countup_min = $bn->get($cf, 'LimitCountUpMin');
  $countup_min = 60 if !$countup_min;
  $countup_min *= 60;# sec -> min

  $bn->end( 'ERROR', $item->{ERR_LOG_DATE} ) unless -e $path;
  my $fh = $bn->open( $path ) or $bn->error( 'ERR_DATA_NOTOPEN', $path );
  my @s = stat $fh;
  my $size = $s[7];
  my $mod = $s[9];
  my( $i, $n, $p, $t, @pos_array, %sea_hash,
    %access_zone, %access_unique, %ip_table, %host_table, %agent_table,
    %os_table, %browse_table, %ver_table, %ref_table, %ck_table );

  # rejume
  if ( -e $anzpath && $mod > (stat $anzpath)[9] ) {
    # anz
    $lo->anz_load( $anzpath, { 'pos' => \@pos_array, 'zone' => \%access_zone, 'unique' => \%access_unique,
     'ip' => \%ip_table, 'host' => \%host_table, 'agent' => \%agent_table, 'os' => \%os_table,
     'browse' => \%browse_table, 'ver' => \%ver_table, 'ref' => \%ref_table, 'ck' => \%ck_table } );

    # sea
    $lo->sea_load( $seapath, \%sea_hash ) if -e $seapath;

    # file seek rejume;
    if ( @pos_array ) {
      $_ = pop @pos_array;
      my($p, $ir) = split ':', $_;
      seek $fh, int $p, 0;
      $i = int $ir;
    }
  }
  my $anz = $bn->open( $anzpath, '>' ) or $bn->error( 'ERR_SYS_NOTWRITE', $anzpath );
  my $sea = $bn->open( $seapath, '>' ) or $bn->error( 'ERR_SYS_NOTWRITE', $seapath );

  my $maxline = $MAXLOGLINE;
  my %ip_times; ## counter off : research
  $t = tell $fh;
  while ( <$fh> ) {
    my $data = {};
    chomp;
    @{$data}{qw/date ip host agent os browse ver ref method request cookie inf/} = split /\t/;
    next unless $data->{date} =~ m#\d+/\d+/\d+ \(\w+\) (\d+):(\d+):(\d+)#;
    $access_zone{$1}++;
    if ( $data->{inf} eq '1' ) {
      $access_unique{$1}++;
    } elsif ( !$flag_counter ) {
      my $t_sec = ( $1 * 3600 ) + ( $2 * 60 ) + $3;
      if ( exists $ip_times{$data->{ip}} ) {
        $access_unique{$1}++
         if ( $ip_times{$data->{ip}} <= ( $t_sec - $countup_min ) );
      } else {
        $access_unique{$1}++;
      }
      $ip_times{$data->{ip}} = $t_sec;
    }

    for ( split /&/, $data->{cookie} ) {
      my($key, $val) = split /=/, $_, 2;
      if ( $key eq 'name' && $val ) {
        $ck_table{$val}++ if $val;
        $sea_hash{$val} = [] unless exists $sea_hash{$val};
        push @{ $sea_hash{$val} }, $t;
        last;
      }
    }

    if ( $data->{ip} ) {
      $sea_hash{$data->{ip}} = [] unless exists $sea_hash{$data->{ip}};
      push @{ $sea_hash{$data->{ip}} }, $t;
    }

    $ip_table{$data->{ip}}++ if $data->{ip};
    $host_table{$data->{host}}++ if $data->{host};
    $agent_table{$data->{agent}}++ if $data->{agent};
    $os_table{$data->{os}}++ if $data->{os};
    $browse_table{$data->{browse}}++ if $data->{browse};
    $ver_table{$data->{ver}}++ if $data->{ver};
    $ref_table{$data->{ref}}++ if $data->{ref};

    $n++;
    $i++;
    $t = tell $fh;
    if ( $i >= $maxline ) {
      $p++;
      push @pos_array, $t;
      $i = 0;
    }
  }
  # clear
  undef %ip_times;

  # /* seach hash */
  while ( my($key, $array) = each %sea_hash ) {
    print $sea (join "\t", $key, @$array), "\n";
  }

  # /* end offset */
  push @pos_array, (tell $fh) .":$i";
  close $fh;

  # /* line of seek */
  print $anz (join ',', @pos_array), "\n";

  # /* access of zone */
  my @zone;
  for (sort keys %access_zone) { push @zone, "$_:$access_zone{$_}" }
  undef %access_zone;
  print $anz (join "\t", @zone), "\n";

  # /* access of unique */
  my @unique;
  for (sort keys %access_unique) { push @unique, "$_:$access_unique{$_}" }
  undef %access_unique;
  print $anz (join "\t", @unique), "\n";

  # /* ip host agent os browse ver ref cookie */
  for (\%ip_table, \%host_table, \%agent_table, \%os_table,
       \%browse_table, \%ver_table, \%ref_table, \%ck_table )
  {
    my($key, $hash) = $lo->sort_table( $_ );
    print $anz (join "\t", map { "$_:$hash->{$_}" } @$key), "\n";
  }
  close $anz;
  close $sea;

  unshift @pos_array, 0;
  return \@pos_array;
}

sub anz_load {
  my( $lo, $path, $struct ) = @_;
  my $bn = $lo->{bn};
  my $fh = $bn->open( $path ) or $bn->error( 'ERR_SYS_NOTOPEN', $path );

  $_ = <$fh>; chomp;
  for ( split ',', $_ ) { push @{ $struct->{pos} }, $_ }

  for my $key (qw/zone unique/) {
    $_ = <$fh>; chomp;
    for ( split /\t/, $_ ) {
      my @z = split /:/;
      $struct->{$key}{$z[0]} = $z[1];
    }
  }

  for my $key (qw/ip host agent os browse ver ref ck/) {
    $_ = <$fh>; chomp;
    for ( split /\t/, $_ ) {
      my @z = split /:/, $_, 2;
      for ( split /\x08/, $z[1] ) {
        $struct->{$key}{$_} += $z[0];
      }
    }
  }

  close $fh;
}

sub sea_load {
  my( $lo, $path, $hash ) = @_;
  my $bn = $lo->{bn};
  my $fh = $bn->open( $path ) or $bn->error( 'ERR_SYS_NOTOPEN', $path );
  while ( <$fh> ) {
    chomp;
    my($key, @pos) = split "\t";
    $hash->{$key} = \@pos;
  }
  close $fh;

}

sub sort_table {
  my( $lo, $table ) = @_;
  my( $n, @list, @key, %list_sort );
  for ( keys %$table ) { push @list, (sprintf "%08d", $table->{$_}) . ":$_" }
  undef %$table;
  @list = sort { $b <=> $a } @list;
  for ( @list ) {
    my($cnt, $i) = split /:/, $_, 2;
    $cnt = int $cnt;
    push @key, $cnt unless $cnt == $n;
    $n = $cnt;
    if ( exists $list_sort{$cnt} ) {
      $list_sort{$cnt} .= "\x08$i";
    } else {
      $list_sort{$cnt} = $i;
    }
  }
  return(\@key, \%list_sort);
}

sub get_middle_color {
  my( $lo, $c1, $c2 ) = @_;
  require BN::Design;
  my $dsn = BN::Design->new( $lo->{bn} );
  return $dsn->get_middle_color( $c1, $c2 );
}

sub get_log_array {
  my( $lo, $log_array, $log_old, $log_now ) = @_;
  my @mday = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
  my ( $n, $f, $endf );
  push my @log_alive, map{int}@$log_old;
  for my $zy ( $log_old->[0]..$log_now->[0] ) {
    for my $zm ( 1..12 ) {
      my $zmf = sprintf( "%02d", $zm );
      next if $log_old->[0] == $zy && $log_old->[1] gt $zmf;
      my $zdm = $mday[$zm - 1];
      $zdm++ if $zm == 2 && ( !$zy % 4 && $zy % 100 || !$zy % 400 );
      if ( exists $log_array->{$zy}{$zmf} ) {
        for my $zd ( 1..$zdm ) {
          my $zdf = sprintf( "%02d", $zd );
          next if $log_old->[0] == $zy && $log_old->[1] == $zmf && $log_old->[2] gt $zdf;
          if ( exists $log_array->{$zy}{$zmf}{$zdf} ) {
            if ( $f ) { push @log_alive, $f; $f = 0 }
            $n++;
          } else {
            if ( $n ) { push @log_alive, $n; $n = 0 }
            $f++;
          }
          if ( $log_now->[0] == $zy && $log_now->[1] == $zm && $log_now->[2] == $zd ) {
            $endf = 1;
            last;
          }
        }
        last if $endf;
      } else {
        $f += $zdm; $n = 0;
      }
    }
  }
  push @log_alive, $f if $f;
  push @log_alive, $n if $n;
  return \@log_alive;
}

sub zeller {
  my( $lo, $zy, $zm, $zd ) = @_;
  if ( $zm == 1 || $zm == 2 ) { $zm += 12; $zy-- }
  my @ty = split "", $zy;
  my $c = $ty[0]. $ty[1];
  my $y = $ty[2]. $ty[3];
  return ( int( 21 * $c / 4 ) + int( 5 * $y / 4 ) + int( 26 * ( $zm + 1 ) / 10 ) + ( $zd - 1 ) ) % 7;
}

# image counter view only
sub count {
  my $lo = $_[0];
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my $mode = $bn->get($cf, 'UseCounter');
  $bn->error('ERR_FUNCTION') unless $mode;
  my @count;
  for ( 0..8 ) { $count[$_] = 0 }

  $bn->filelock;
  $lo->dir_init->access( \@count, 'view' );
  $bn->fileunlock;

  if ( $mode eq 'image' ) {
    if ( $bn->get($cf, 'UseGD') ) {
      $lo->view_count_gd( $count[1] );
    } elsif ( $bn->get($cf, 'UsePerlMagick') ) {
      $lo->view_count_pm( $count[1] );
    }
  } else {
    $bn->error('ERR_FUNCTION');
  }
  $bn->EXIT;
}

# called BN.pm
sub access_log {
  my( $lo, $ref_array_count ) = @_;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};

  ## not write access log function
  return if $bn->{in}{fc} eq 'access_log';

  $bn->get_user_ip;
  $bn->call( 'AccessLog' );

  for (0..8) { $ref_array_count->[$_] = 0 }
  # /* array count */
  # 0 : count up flag
  # 1 : total count
  # 2 : today count
  # 3 : yesterday count
  # 4.. history count
  $lo->dir_init->access( $ref_array_count );

  $lo;
}

sub access {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};

  $bn->get($cf, 'UseAccessLog') ? $lo->access_write( @_ ) :
  $bn->get($cf, 'UseCounter')   ? $lo->access_blank( @_ ) : return ;

}

sub access_write {
  my( $lo, $val, $view ) = @_;
  my $bn = $lo->{bn};
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $dir = $lo->{dir_log};
  my $date = $bn->get_date;
  my $file = "$dir/$1$2$3.".$lo->{exe} if $date =~ m#(\d+)/(\d+)/(\d+)#;
  my $log_rotation = !-e $file; # log rotation

  $fl->{access_write} = 1;

  my $qjoin = sub {
    my $k = shift;
    join '&', map {
      my $v = $_ eq 'file' ? length $k->{$_} : $k->{$_};
      if ( $_ eq 'fc' && $k->{$_} eq 'login' ) {
        if ( ($bn->get_admin_pass)[0] ne '' ) {
          $v = exists $fl->{AutoLogin} && $fl->{AutoLogin} ? 'login_auto' : 'login';
        } else {
          $v = 'login_entry';
        }
      }
      # name trip
      if ( $_ eq 'name' && $bn->get($cf, 'UseNameTrip') && $k->{$_} =~ /\#/ ) {
        my($user, $pass) = split /\#/, $k->{$_}, 2;
        $v = $user . '#' . $bn->crypt_string($pass, 1);
      }
      $bn->escape(\$v);
      "$_=$v"
    } grep { $k->{$_} ne '' } @_;
  };

  $bn->filelock;

  # rotation log files
  $lo->parge_access_log if $log_rotation;
  my $lv = $bn->get($cf, 'LimitAccessLogLevel');
  $lv ||= 0;
#  $lv ||= 'debug';

  my $cookie = &$qjoin( $ck, @LOGGING_QUERY_COOKIE );

  my @logging;
  if ( $lv eq 'debug' ) {
    @logging = keys %$in;
  } else {
    for ( my $i = 0; $i <= $lv && $i < 3; $i++ )
     { push @logging, @{$LOGGING_QUERY[$i]} if defined @{$LOGGING_QUERY[$i]} }
  }

  my $query = &$qjoin( $in, @logging );
  my $request_path = $ENV{HTTP_HOST}.$ENV{SCRIPT_NAME};
  $request_path =~ s#[^\\|/]+$##;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm 5;
  my $fh;
  if ( ref $val eq 'ARRAY' ) {
    $fh = $bn->open( $file, '>>' );
    $fh or $bn->error( 'ERR_SYS_NOTWRITE', $file );
    binmode $fh;

    ## disregard under from script dir
    my $ref = $ENV{HTTP_REFERER} =~ m#^\w+://$request_path# ? '' : $ENV{HTTP_REFERER};
    $bn->escape(\$ref);
    my $agent = $gl->{UserAgent};
    my $aback = $agent;
    $bn->escape(\$agent);
    $gl->{UserAgent} = $agent;
    print $fh join( "\t", map{ s/[\t\n\r]//; $_ }
      ( $date, @{$gl}{'IP','HOST','UserAgent','UserOS','UserBrowser','UserBrowserVer'},
        $ref, $ENV{REQUEST_METHOD}, $query, $cookie )
    ), "\t";
    $gl->{UserAgent} = $aback;

    # count up
    $lo->open_count( $val, $log_rotation, $view );
    print $fh 1 if $val->[0];

    # line 
    print $fh "\n";

  } else {
  # status write
    close $bn->open( $file, '>' ) unless -e $file;
    $fh = $bn->open( $file, '+<' );
    $fh or $bn->error( 'ERR_SYS_NOTWRITE', $file );
    binmode $fh;

    my $rbuf = 30;
    my( $buf, $flag, @str );
    for ( split /\x08/, $val ) { $bn->escape(\$_); push @str, $_ }
    for ( my $pos = ((stat $fh)[7]) - $rbuf - 3; !$flag; $pos -= $rbuf ) {

      $pos = 0 if $pos < 0;
      seek $fh, $pos, 0;
      read $fh, $buf, $rbuf;
      my $p = $rbuf;
      my $max = 16777215;
      while ( ( $p = rindex $buf, "\n", $p ) >= 0 || $pos == 0 ) {
        LOGLF_REDO:
        $max--;
        if ( !$max ) { $flag = 1; last }
        my $spos = $pos == 0 ? 0 : $pos + $p + 1;
        seek $fh, $spos, 0;
        my $v = <$fh>; chomp $v;
        if ( $v eq '' ) { $flag = 1; last }
        my @a = split /\t/, $v;
        if ( $gl->{IP} eq $a[1] ) {
          my @w; $a[11] = join "\b", @str;
          while ( <$fh> ) { push @w, $_ }
          seek $fh, $spos, 0;
          truncate $fh, $spos;
          print $fh ( join "\t", @a ), "\n", @w;
          $flag = 1;
          last;
        } else {
          $p++;
          goto LOGLF_REDO; # safe win old ver
        }
        $p -= 1;
      }
      last if $pos <= 0 || $flag;
    }
  }
  close $fh;
  delete $fl->{access_write};

  # alarm off
  alarm 0 };
  if ( $@ =~ /alarm\n/ ) {
    $bn->fileunlock;
    $bn->Trace('ALRM');
  }

}

sub access_blank {
  my $lo = shift;
  my($val, $view) = @_;
  my $bn = $lo->{bn};
  my $dir = $lo->{dir_log};
  my $date = $bn->get_date;
  my $file = "$dir/$1$2$3.".$lo->{exe} if $date =~ m#(\d+)/(\d+)/(\d+)#;
  my $log_rotation = !-e $file; # log rotation

  $bn->filelock;

  # parge log files
  if ( $log_rotation ) {
    my $dh = $bn->opendir( $dir );
    unless ( $bn ) {
      $lo->dir_init;
      $dh = $bn->opendir( $dir );
    }
    $dh or $bn->error( 'ERR_SYS_NOTOPEN', $dir );
    while ( $_ = readdir $dh ) {
      next if /^\.\.?/;
      unlink "$dir/$_";
    }
    closedir $dh;
  }
  # new blank file
  close $bn->open( $file, '>' );

  $lo->open_count( $val, $log_rotation, $view );
  $bn->fileunlock;
}

sub open_count {
  my $lo = shift;
  my( $array, $log_rotation, $view ) = @_;
  my $cmd = $view ? 'view_count' : 'write_count';
  $lo->$cmd( $array, $log_rotation );
  $lo;
}

sub write_count {
  my $lo = shift;
  my( $array, $log_rotation ) = @_;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};

  return unless $bn->get($cf, 'UseCounter');

  my $fh;
  ( $fh = $bn->open($lo->{count_file}) ) or
    ( $fh = $bn->open($lo->{count_file}, '>') ) or
      $bn->error('ERR_SYS_NOTOPEN', $lo->{count_file});

  my $ips_max = 5;
  my $countup_min = $bn->get($cf, 'LimitCountUpMin');
  $countup_min = 60 if $countup_min eq '';
  $countup_min *= 60;# sec -> min

  my $i = 0;
  for ( split ',', <$fh> ) { $array->[++$i] = int $_ }
  my @ips = <$fh>;
  close $fh;

  # roll count : today -> yesterday, yesterday=0
  if ( $log_rotation ) {
    for ( my $i = 8; $i > 2; $i-- ) { $array->[$i] = $array->[$i-1] }
    $array->[2] = 0;
  } else {
  # not count up
    for ( @ips ) {
      chomp;
      my( $ip, $t ) = split /,/;
      return $lo if $ip eq $bn->{global}{IP} && $t + $countup_min > time;
    }
  }

  for ( 1, 2 ) { $array->[$_]++ }
  while ( @ips >= $ips_max ) { pop @ips }

  my($fh_tmp, $tmp) = $bn->get_temp_file;
  print $fh_tmp join ',', @$array[1..8];
  print $fh_tmp "\n", $bn->{global}{IP}.','.time, "\n", join "\n", @ips;
  close $fh_tmp;
  rename $tmp, $lo->{count_file};

  $array->[0] = 1; # count up flag

  $lo;
}

sub view_count {
  my $lo = shift;
  my( $array ) = @_;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};

  return unless $bn->get($cf, 'UseCounter');

  my $fh;
  ( $fh = $bn->open($lo->{count_file}) ) or
    ( $fh = $bn->open($lo->{count_file}, '>') ) or
      $bn->error('ERR_SYS_NOTOPEN', $lo->{count_file});

  for ( my $i = 1; $i < 4; $i++ ) { $array->[$i] = int <$fh> }
  close $fh;
  $array->[0] = 0; # count up flag

  $lo;
}

sub parge_access_log {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  my $dir = $lo->{dir_log};
  my $init_date = "$dir/raw.init.$sys->{Exe}";
  unlink $init_date;

  my $max = $bn->get($cf, 'LimitAccessLog');
  $max ||= 365;

  my %list;
  my $exe = $lo->{exe};
  my $dh = $bn->opendir( $dir );
  unless ( $bn ) {
    $lo->dir_init;
    $dh = $bn->opendir( $dir ) }
    $dh or $bn->error( 'ERR_SYS_NOTOPEN', $dir );

  my $parge_date = int sprintf( '%04d%02d%02d', @{$lo->time_now( $max * -86400 )} );
  while ( $_ = readdir $dh ) {
    next if /^\.\.?/;
    $list{ $1 } = 1 if /^(\d{8}).*\.$exe$/ && int($1) < $parge_date;
  }
  closedir $dh;

  for ( sort keys %list ) {
    my $log = "$dir/$_";
    for (qw/. .anz. .sea./)
     { unlink "$log$_$exe" if -e "$log$_$exe" }
  }

}

sub dir_init {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $sys = $bn->{sys};

  # setup log directory
  unless ( -d $lo->{dir_access} ) {
    eval 'use File::Path qw( mkpath )';
    for ( $lo->{dir_access}, $lo->{dir_log}, $lo->{dir_images} )
     { mkpath($_) or $bn->error('ERR_DIR_NOTMAKE', $_) }

    my $fh = $bn->open( $lo->{count_file}, '>' );
    $fh or $bn->error('ERR_SYS_NOTWRITE', $lo->{count_file});
    print $fh "0\n" x 3;
    close $fh;
  }

  $lo;
}

sub view_count_gd {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my $count = $_[0];

  eval("use GD");
  $bn->error('ERR_MOD_NOTOPEN', 'GD') if $@;

  my $gdh = {};
  my($width, $height, $max_color);
  my $dither = $bn->get($cf,'ViewCounterImageDither');
  my $colors = $bn->get($cf,'ViewCounterImageColors');
  my $trans = $bn->get($cf,'ViewCounterImageTransparent');
  $dither ||= 0;
  $colors ||= 64;
  $colors = 256 if $colors > 256;
  $trans = -1 if $trans eq '';

  # init
  unless ( -e $lo->{imgdat_file} ) {
    $lo->view_count_gd_init( $gdh );
  }

  for ( my $i = 0; $i < length $count; $i++ ) {
    my $n = substr $count, $i, 1;
    unless ( exists $gdh->{$n} ) {
      $gdh->{$n}{fh} = GD::Image->newFromGd( $lo->{dir_images} .'/'. $n . '.gd' );
      @{$gdh->{$n}}{'width','height'} = $gdh->{$n}{fh}->getBounds();
    } else {
      $lo->view_count_gd_init( $gdh );
    }

    $width += $gdh->{$n}{'width'};
    $height = $gdh->{$n}{'height'} if $gdh->{$n}{'height'} > $height;

  }

  my $ih = GD::Image->newTrueColor( $width, $height );
#  my $ih = GD::Image->new( $width, $height );
  my $xpos = 0;
  for ( my $i = 0; $i < length $count; $i++ ) {
    my $n = substr $count, $i, 1;
    my $c = $gdh->{$n};
    my $t = $c->{fh};
    $ih->copyMergeGray( $t, $xpos, 0,
      0, 0, $c->{width}, $c->{height}, 99 );
    $xpos += $c->{width};
  }
  $ih->trueColorToPalette( $dither, $colors );
  $ih->transparent( $trans );

  $bn->send_http_header( 'image/gif' );
  $bn->print( $ih->gif );

  for my $k (0..9) { map{ delete $gdh->{$k}{$_} } keys %{$gdh->{$k}}; delete $gdh->{$k} }
  undef $gdh;

}

sub view_count_gd_init {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $gdh = $_[0];
  my $dh = $bn->opendir( $lo->{dir_images} );
  $dh or $bn->error( 'ERR_SYS_NOTOPEN', $lo->{dir_images} );
  my %file;
  while ( $_ = readdir $dh ) {
    next if /^\.\.?/;
    $file{$1} = $_ if /^(\d)/;
  }
  closedir $dh;

  for (0..9) {
    $lo->error_img_file unless exists $file{$_};
    $gdh->{$_}{fh} = GD::Image->new( $lo->{dir_images} .'/'. $file{$_} );
    $gdh->{$_}{fh} or $bn->error( 'ERR_DATA_NOTOPEN', $lo->{dir_images} .'/'. $file{$_} );
    @{$gdh->{$_}}{'width','height'} = $gdh->{$_}{fh}->getBounds;

    my $fh = $bn->open( $lo->{dir_images} . "/$_" . '.gd', '>' );
    binmode $fh;
    print $fh $gdh->{$_}{fh}->gd;
    close $fh;
  }
  my $fh = $bn->open( $lo->{imgdat_file}, '>' );
  close $fh;
}


sub view_count_pm {
  my $lo = shift;
  my $bn = $lo->{bn};
  my $count = $_[0];

  eval("require Image::Magick");
  $bn->error('ERR_MOD_NOTOPEN', 'Image::Magick') if $@;

  my $pmh = {};
  my $src = Image::Magick->new;
  my $m;
  for ( my $i = 0; $i < length $count; $i++ , $m++) {
    my $n = substr $count, $i, 1;
    my $err = $src->Read( $lo->{dir_images} .'/'. $n . '.gif' );
    $bn->warn("Image::Magick Error $err [$n.gif]") if $err;
  }

  my $image = $m > 1 ? $src->Append( stack => 'false' ) : $src;

  $bn->send_http_header( 'image/gif' );

  binmode STDOUT;

#  $image->Write("gif:-");

  #/* STDOUT Error It under code may be possible */
  $image->Write(filename => $lo->{dir_images} .'/bin.gif');
  my $fh = $bn->open($lo->{dir_images} .'/bin.gif');
  binmode $fh;
  while (<$fh>) { $bn->print( $_ ) }

  undef $src;
  undef $image;
}

sub time_now {
  my ( $lo, $offset ) = @_;
  my $bn = $lo->{bn};
  my $cf = $bn->{cfg};
  my @T = localtime( time + $bn->get( $cf, 'TimeDiffSec' ) + $offset );
  my @log_now = ( $T[5] + 1900, ++$T[4], $T[3] );
  return \@log_now;
}

sub error_img_file {
  my $lo = shift;
  my $bn = $lo->{bn};

  $bn->send_http_header( 'image/gif' );
  $bn->print( pack 'H*',
  '474946383961A3000900800000000000'.
  'FFFFFF21F90400000000002C00000000'.
  'A30009000002C68C8FA9CBED0FA39CB4',
  '3E8071C8E070AF191FD275A44786DB28'.
  '669B98BAF0BC9A9C09A2F0F8E1B2DD03'.
  'F1123E5A495744A942C7D70C473C3969'.
  'A9A76E8A3462B136257565BD6AA74D67'.
  'F2B8EC76A30BE8FADB0A9315D079DDFD'.
  '7DC7CD310D6BF824E38566B777132357'.
  'B3F4F29644E45892B676D7E242C8C576'.
  '36A6C7D8196999D526854767785537EA'.
  '2595C837AA472957D67842283BB725F6'.
  '2A0A768B3BA99AABC5094475A81908B9'.
  'D3AAB223C94CDAF377FADCE6C6C46674'.
  'AD688B6BE1FD0D1E2E3E1E5100003B'
  );

  $bn->EXIT;
}

1;

__END__
