#=============================================================================
# BBS Note Comet 1.x
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2007/05/29
# Version : 0.02.007 build 2
#=============================================================================
package BN::User;
use strict;
use vars qw( $bn $VERSION $limit_list $view_limit_list);

$VERSION = '0.02.007';
$limit_list = 100;
$view_limit_list = 10;

sub new {
  my $usr = bless {}, $_[0];
  $bn = $_[1];
  $usr;
}

sub trip {
  my $usr = shift;
  my($pname, $pflag) = @_;
  my($user, $pass) = split /\#/,$$pname,2;
  if (defined $pass) {
    my $bname = $$pname;
    $$pname = $user;
    $pflag->{user_trip} = $bn->crypt_string($pass, 1);
    $bn->set_call('WritePostLog', sub{ $bn->{cookie}{name} = $bname });
  }
  $usr;
}

sub cap {
  my $usr = shift;
  my $sys = $bn->{sys};
  my($name, $password, $pflag) = @_;
  my $path = $usr->{path} = "$bn->{sys}{DirSys}/user.$bn->{sys}{Exe}";
  $usr->{user} = $name;

  return unless -e $path;

  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_SYS_NOTOPEN', $path);
  $bn->{temp}{fh}{user} = $fh;
  my $usr_flag;
  my $guest_fnc;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};

  while (<$fh>) {
    chomp;
    my($user, $pass, $fnc) = split /\t/;
    if ($user eq $name) {
      $usr_flag = 1;
      if ($password eq $pass) {
        my $flag = $usr->fnc_flag($fnc);
        $usr->check_fnc($flag, 0);
        $pflag->{user_cap} = 1;
      } else {
        $usr->error('ERR_USER_PASS', $name);
      }
      last;
    } elsif ($user eq 'guest') {
      $guest_fnc = $fnc;
    }
  }
  delete $bn->{temp}{fh}{user};

    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;

  close $fh;
  # guest
  unless ($usr_flag) {
    my $flag = $usr->fnc_flag($guest_fnc);
    $usr->check_fnc($flag, 1);
  }

}

sub access {
  my $usr = shift;
  my $in = $bn->{in};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};

  if ( $bn->check_authority( $in->{password} ) ) {
    $fl->{ADMIN} = 1;
    $ck->{name} = $in->{name};
    $ck->{adminpass} = $in->{password};
    $fl->{write_cookie} = 1;
    return;
  }
  $usr->cap(
    exists $in->{name} ? $in->{name} : $ck->{name},
    exists $in->{password} ? $in->{password} : $ck->{password}
  );
  if (exists $in->{accept} && $in->{accept}) {
    $ck->{name} = $in->{name};
    $ck->{password} = $in->{password};
    $fl->{write_cookie} = 1;
  }

}

sub check_fnc {
  my $usr = shift;
  my $in = $bn->{in};
  my $sys_mode = $bn->{sys}{MODE};
  my($fnc, $guest_flag) = @_;
  my $paint_flag = exists $bn->{flag}{PaintWrite};

  if ($fnc->{new_post} && $sys_mode eq 'WRITE_POST' && !$paint_flag) {
    $usr->error_fnc('new_post', $guest_flag);
  } elsif ($fnc->{reply_post} && $sys_mode eq 'WRITE_REPLY') {
    $usr->error_fnc('reply_post', $guest_flag);
  } elsif ($fnc->{upload_post} && length $in->{file}) {
    $usr->error_fnc('upload_post', $guest_flag);
  } elsif ($fnc->{paint_post} && $paint_flag && !exists $in->{continue}) {
    $usr->error_fnc('paint_post', $guest_flag, 1);
  } elsif ($fnc->{paint_new_post} && $paint_flag && $in->{continue} eq 'new') {
    $usr->error_fnc('paint_new_post', $guest_flag, 1);
  } elsif ($fnc->{paint_rep_post} && $paint_flag && $in->{continue} eq 'rep') {
    $usr->error_fnc('paint_rep_post', $guest_flag, 1);
  }

  if ($fnc->{view_bbs}) {
    $usr->error_fnc('view_bbs', $guest_flag, 1);
  }

}

sub fnc_flag {
  my $usr = shift;
  my $fnc = $_[0];
  my %flag;
  for (split /,/, $fnc) {
    $flag{
    $_ eq 'V'  ? 'view_bbs':
    $_ eq 'N'  ? 'new_post':
    $_ eq 'R'  ? 'reply_post':
    $_ eq 'U'  ? 'upload_post':
    $_ eq 'P'  ? 'paint_post':
    $_ eq 'Pn' ? 'paint_new_post':
    $_ eq 'Pr' ? 'paint_rep_post':'' } = 1;
  }
  return \%flag;
}

sub error_fnc {
  my $usr = shift;
  my $in = $bn->{in};
  my($fn, $guest_flag, $login_flag) = @_;
  my $item = $usr->{item} = $bn->text(undef, 'user');
  close $bn->{temp}{fh}{user} if exists $bn->{temp}{fh}{user};
  my @arg = $guest_flag ? ('ERR_GUEST_FNC', $item->{$fn}) : ('ERR_USER_FNC', $usr->{user}, $item->{$fn});
  if ($login_flag && !exists $in->{accept}) {
    my %arg = (accept => 1);
    for (keys %$in) {
      next if /name|password/;
      $arg{$_} = $in->{$_} if exists $in->{$_} }
    $bn->fileunlock;
    my @e = split /:/,$usr->get_item(@arg),2;
    $bn->ad->login_form(\%arg, $e[1].'<br>');
  } else {
    $usr->error(@arg);
  }
}

sub get_item {
  my $usr = shift;
  my $key = shift;
  my @arg = @_;
  my $item = $usr->{item} = $bn->text(undef, 'user');
  my $n;
  my $e = $item->{$key};
  for (@arg) {
     $n++;
     $e =~ s/\[_$n]/$_/g;
  }
  return $e;
}

sub error {
  my $usr = shift;
  my $e = $usr->get_item(@_);
  close $bn->{temp}{fh}{user} if exists $bn->{temp}{fh}{user};
  $bn->end('ERROR',split /:/,$e,2);
}


sub editor {
  my $usr = shift;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $cms = $bn->{sys}{CMS};
  my $path = $usr->{path} = "$bn->{sys}{DirSys}/user.$bn->{sys}{Exe}";
  my $item = $usr->{item} = $bn->text(undef, 'user');
  my $page = $in->{page} ? abs int $in->{page} : 1 ;
  my $index = ($page - 1) * $view_limit_list;
  my $table_in = $bn->get($sk, 'TagTableIn');
  my $table_out = $bn->get($sk, 'TagTableOut');
  my $view_bbs;

  if (exists $cms->{LIMIT_NUM}{user_editor}) {
  my $ue = $cms->{LIMIT_NUM}{user_editor};
  $view_bbs = $ue->{view_bbs} if exists $ue->{view_bbs};
  $view_limit_list = $ue->{view_limit_list} if exists $ue->{view_limit_list};
  $limit_list = $ue->{limit_list} if exists $ue->{limit_list};
  }

  $usr->{error} = [];

  $page = 0 if $page < 0;

  $usr->write
  if $in->{edit} =~ /^(?:add|edit|delete)$/;

  $usr->editor_js;

  my(@list, $n, $user, $pass, $fnc, $guest_fnc, $prev_page, $next_page);
  my $fh = $bn->open($path);
  if ($fh) {

    # alarm on
    eval {
      local $SIG{ALRM} = sub{ die "alarm\n" };
      alarm $sys->{Alarm} if $sys->{Alarm};

    while (<$fh>) {
      chomp;
      my($user, $pass, $fnc) = split /\t/, $_;
      next if $user eq '';
      if ($user eq 'guest')
       { $guest_fnc = $fnc; next }
      push @list, $_ if $n >= $index && @list < $view_limit_list;
      $n++;
    }

      # alarm off
      alarm 0;
    };
    $bn->Trace('ALRM') if $@ =~ /alarm\n/;

    close $fh;
  }

  my $total = $n;
  my $max_page = int($total / $view_limit_list) + ($total % $view_limit_list ? 1 : 0);
  my $pagelink = $usr->page_link($page, $max_page);
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');
  my $arg = $bn->set_form_arg($bn->{arg}{UserEditor});
  my $form = '<form action="'.$bbsnoteurl.'" method="post">'.$arg;

  # prev
  if ($page > 1) { my $p = $page -1;
    $prev_page = $form . '<input type="hidden" name="page" value="'.$p.'"><input type="hidden" name="edit2 value="1">'.
    '<td><input class="ConfigBtn" type="submit" value="'.$item->{page_back}.'"></td></form>' }
  # next
  if ($page < $max_page) { my $p = $page + 1;
    $next_page = $form . '<input type="hidden" name="page" value="'.$p.'"><input type="hidden" name="edit" value="1">'.
    '<td><input class="ConfigBtn" type="submit" value="'.$item->{page_next}.'"></td></form>' }
  # error
  if (@{$usr->{error}}) { $bn->print('<tr><td colspan="2"><b>');
    for (@{$usr->{error}}) { $bn->print("$_<br>") }
    $bn->print('</b></td></tr>') }

  # table headers
  $bn->print(<<_HTML_);
</form><tr><td colspan="2"><table><tr>$prev_page$pagelink$next_page</tr></table>
<table><tr><td valign="top">
<table width="100%" bgcolor="$table_out->{bgcolor}" cellpadding="0" cellspacing="0"><tr><td>
<table width="100%" cellpadding="2" cellspacing="1">
<tr bgcolor="$table_in->{bgcolor}">
<th width="1%">$item->{corner_mark}</th>
<th width="45%" nowrap>$item->{user}</th>
<th width="14%" nowrap>$item->{password}</th>
<th width="10%" nowrap>$item->{fnc_disable}</th>
<th width="10%" nowrap>&nbsp;</th></tr>
_HTML_

  # user list
  $n = 1;
  for (@list) {
    my($user, $pass, $fnc) = split /\t/, $_;
    $bn->print('<tr bgcolor="'.$table_in->{bgcolor}.'" align="center">',
    '<form action="'.$bbsnoteurl.'" method="post" name="user_'.$n.'">',$arg,
    '<input type="hidden" name="orguser" value="'.$user.'">',
    '<td id="user_'.$n.'mark">&nbsp;</td>',
    '<td><input type="text" name="user" value="'.$user.'" class="input" onmouseover="view_fnc(\'user_'.$n.'\')"></td>',
    '<td><input type="text" name="pass" value="'.$pass.'" size="9" maxlength="8" class="input" onmouseover="view_fnc(\'user_'.$n.'\')"></td>',
    '<td><input type="text" name="fnc" value="'.$fnc.'" class="ConfigFnc" onmouseover="view_fnc(\'user_'.$n.'\')" readonly></td>',
    '<td nowrap><select name="edit" class="select" onmouseover="view_fnc(\'user_'.$n.'\')">',
    '<option value="edit">'.$item->{edit}.'</option><option value="delete">'.$item->{delete}.'</option>',
    '</select><input class="ConfigBtn" type="submit" value="'.$item->{submit}.'" onmouseover="view_fnc(\'user_'.$n.'\')"></td>',
    '</form></tr>');
    $n++;
  }

  # guest field
  $bn->print(<<_HTML_);
<tr bgcolor="$table_in->{bgcolor}">
<form action="$bbsnoteurl" method="post" name="guest">$arg
<input type="hidden" name="edit" value="edit">
<input type="hidden" name="user" value="guest">
<input type="hidden" name="orguser" value="guest">
<td id="guestmark">&nbsp;</td>
<td colspan="2"><b>$item->{guest}</b></td>
<td><input class="ConfigFnc" type="text" size="16" name="fnc" value="$guest_fnc" onmouseover="view_fnc('guest')" readonly></td>
<td><input class="ConfigBtn" type="submit" onmouseover="view_fnc('guest')" style="width:100%" value="$item->{guest_edit}"></td>
</form></tr>
_HTML_

  # add field
  $bn->print(<<_HTML_);
<tr bgcolor="$table_out->{bgcolor}">
<td colspan="5" style="color:$table_in->{bgcolor}"><b>&nbsp;$item->{add} : </b><small>
$item->{add_inf}</small></td></tr>

<tr bgcolor="$table_in->{bgcolor}" align="center">
<form action="$bbsnoteurl" method="post" name="add">$arg
<input type="hidden" name="edit" value="add">
<input type="hidden" name="page" value="$in->{page}">
<td id="addmark">&nbsp;</td>
<td><input class="input" type="text" name="user" style="width:100%" value="" onmouseover="view_fnc('add')"></td>
<td><input class="input" type="text" name="pass" size="9" maxlength="8" value="" onmouseover="view_fnc('add')"></td>
<td><input class="ConfigFnc" type="text" name="fnc" size="16" onmouseover="view_fnc('add')" readonly></td>
<td><input class="ConfigBtn" type="submit" style="width:100%" value="$item->{add_submit}" onmouseover="view_fnc('add')"></td>
</form></tr></table></td></tr></table>
<small><b>$item->{guest}</b>:$item->{guest_inf}</small>

</td><td valign="top">

<table bgcolor="$table_out->{bgcolor}" cellpadding="0" cellspacing="0"><tr><td>
<table cellpadding="2" cellspacing="1">
<tr bgcolor="$table_out->{bgcolor}">
<td style="color:$table_in->{bgcolor}"><b>&nbsp;$item->{fnc_disable}</b></td></tr>
<form name="fnc"><input type="hidden" name="target">
<tr bgcolor="$table_in->{bgcolor}">
<td valign="top" nowrap><small>
_HTML_

  # func flags
  my @flags = qw/new_post reply_post upload_post paint_post paint_new_post paint_rep_post/;
  unshift @flags, 'view_bbs' if $view_bbs;
  for (@flags) {
    $bn->print('<input type="checkbox" name="', $item->{$_.'_flag'},
     '" onclick="set_fnc()">(', $item->{$_.'_flag'}, ')',$item->{$_},'<br>');
  }

  # foot table
  $bn->print('</small></td></tr>',
  '</td></tr></table>' x 3 ,
  $bn->{mo}{CMS}{item}{manual_user_names_edit},
  '</td></tr>');

}

sub editor_js {
  my $sk = $bn->{skin};
  my $ast_mark = $bn->get($sk, 'TagFormAstMark');
  my $table_in = $bn->get($sk, 'TagTableIn');
  my $table_out = $bn->get($sk, 'TagTableOut');
  $bn->print(<<_JS_);
<style>
.ConfigFnc{ background:$table_in->{bgcolor}; color:$table_out->{bgcolor}; border-style:none }
</style>
<script type="text/javascript">
function view_fnc(f){
  d=document;c=d.fnc;for(i=0;i<c.length;i++){c[i].checked=0}
  if(d.forms[f]){v=d.forms[f].fnc.value;if(d.all){d=d.all}
  if(d[c.target.value+'mark']){
  d[c.target.value+'mark'].style.background='$table_in->{bgcolor}'}
  c.target.value=f;
  if(d[f+'mark']){d[f+'mark'].style.background='$ast_mark->{color}'}
  a=v.split(',');for(i=0;i<a.length;i++){if(c[a[i]]){c[a[i]].checked=1}}}
}
function set_fnc(){
  s=new Array();c=document.fnc;for(i=0;i<c.length;i++){if(c[i].checked){s.push(c[i].name)}}
  if(document.forms[c.target.value]){document.forms[c.target.value].fnc.value=s.join(',')}
}
</script>
_JS_
}

sub write {
  my $usr = shift;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $item = $usr->{item};
  my $mode = $in->{edit};

  $bn->po->escape_input(0, \$in->{user});
  $usr->err('ERR_USER_NONE') if $in->{user} eq '';
  $usr->err('ERR_USER_LEN', $bn->get($cf, 'LimitName')) if length $in->{name} > $bn->get($cf, 'LimitName');
  $usr->err('ERR_PASS_LEN') if length $in->{pass} > 8;
  $usr->err('ERR_PASS_STR', $1) if $in->{pass} =~ /(\W)/;
  $usr->err('ERR_FNC_LEN') if length $in->{fnc} > 99;
  $usr->err('ERR_FNC_STR', $1) if $in->{fnc} =~ /([^a-zA-Z,])/;
  return if @{$usr->{error}};

  my %fncf;
  map{ $fncf{$_} = 1 } split /\,/, $in->{fnc};
  my $acceptuser = 1 if $fncf{V};

  my($fh_tmp, $tmp) = $bn->get_temp_file;
  my $fh = -e $usr->{path} ? $bn->open($usr->{path}) :  $bn->open($usr->{path}, '>');
  $fh or $bn->error('ERR_SYS_NOTWRITE',$usr->{path});

  $bn->{temp}{fh}{user} = $fh;
  $in->{orguser} = $in->{user} unless exists $in->{orguser};
  my $format = "$in->{user}\t$in->{pass}\t$in->{fnc}\n";
  my $u = $mode eq 'add' ? $in->{user} : $in->{orguser};
  my($n, $flag);
  if ($mode eq 'add') { print $fh_tmp $format; $n++ }
  while (<$fh>) {
    chomp;
    my($user, $pass, $fnc) = split /\t/;
    if ($user eq $u) {
      if ($mode eq 'add') {
        close $fh_tmp; close $fh; delete $bn->{temp}{fh}{user};
        unlink $tmp;
        $usr->err('ERR_USER_COMP');
        return
      } elsif ($mode eq 'edit') {
        print $fh_tmp $format unless $flag;
        $flag = 1;
        next
      } elsif ($mode eq 'delete') { next }
    }
    my %fncf;
    map{ $fncf{$_} = 1 } split /\,/, $fnc;
    $acceptuser = 1 if $fncf{V};
    print $fh_tmp "$_\n";
    $n++;
  }

  if ($mode eq 'edit' && !$flag) { print $fh_tmp $format; $n++ }
  close $fh_tmp; close $fh; delete $bn->{temp}{fh}{user};

  if ($n > $limit_list) {
    $usr->err('ERR_LIST',$limit_list);
    unlink $tmp; return;
  }

  rename $tmp, $usr->{path}
    or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $usr->{path});
  if ($bn->get($cf, 'LimitSecurityAcceptUser') != $acceptuser) {
    $in->{acceptuser} = $acceptuser ? 1 : 0;
    require BN::Write;
    my $wri = BN::Write->new($bn);
    $wri->write(
    [
      [ 'LimitSecurityAcceptUser', 'acceptuser', '$' ],
    ]
    );
  }

}

sub page_link {
  my $usr = shift;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my($page, $max_page) = @_;

  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');
  $bn->{arg}{UserEditor}{fc} = $in->{fc};
  $bn->{arg}{UserEditor}{name} = $in->{name};
  $bn->{arg}{UserEditor}{password} = $in->{password};
  $bn->{arg}{UserEditor}{mode} = $in->{mode};
  $bn->{arg}{UserEditor}{edit} = 1;
  my $arg = $bn->set_form_arg($bn->{arg}{UserEditor});
  delete $bn->{arg}{UserEditor}{edit};
  return if $max_page < 2;

  my $l = '<form name="pagelink" action="'.$bbsnoteurl.'" method="post">'.$arg.
    '<td><select class="select" name="page" onchange="document.pagelink.submit()">';

  for (my $n=1; $n <= $max_page; $n++) {
    my $sl = $n == $page ? ' selected' : '';
    $l .= '<option value="'.$n.'"'.$sl.'>'.$n.'</option>'."\n";
  }
  $l .= '</select></td></form>';

}
sub err {
  my $usr = shift;
  my $key = shift;
  my @arg = @_;
  my $t = $usr->{item}{$key};
  my $n = 1;
  for (@arg){
    $t =~ s/\[_$n\]/$arg[$n-1]/g;
    $n++;
  }
  push @{ $usr->{error} }, $t;
  $usr;
}

1;

__END__
