#=============================================================================
# BBS Note Comet 1.x 
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2008/02/20
# Version : 0.40.000 build 2
#=============================================================================
package BN::Write;
use strict;
use vars qw( $bn $VERSION );

$VERSION = '0.40.000';

sub new {
  my $wri = bless { }, $_[0];
  $bn = $_[1];
  $wri->init;
  $wri;
}

sub lib { $bn = $_[1]; bless { }, $_[0] }

sub init {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $mode = $in->{mode};
  $bn->{mo}{Write} = $wri;
  $wri->{cms} = $bn->{mo}{CMS};
  $wri->{disable} = $bn->{sys}{CMS}{DISABLE_MENU};
  $wri->{limit_len} = $bn->{sys}{CMS}{LIMIT_LENGTH};
  $wri->{limit_num} = $bn->{sys}{CMS}{LIMIT_NUM};
  $wri->{limit_str} = $bn->{sys}{CMS}{LIMIT_STR};
  $wri->{limit_sty} = $bn->{sys}{CMS}{LIMIT_STY};
  $wri->{limit_esc} = $bn->{sys}{CMS}{LIMIT_ESC};

  return if exists $wri->{disable}{$mode};

  $bn->call( 'CMS_WriteInit' );

  my $menu = "set_$mode";
     if ( defined &$menu )            { $wri->$menu                      }
  elsif ($mode eq 'text_head')        { $wri->set_text('TxtHead')        }
  elsif ($mode eq 'text_top')         { $wri->set_text('TxtTop')         }
  elsif ($mode eq 'text_bottom')      { $wri->set_text('TxtBottom')      }
  elsif ($mode eq 'text_bbs_head')    { $wri->set_text('TxtBbsHead')     }
  elsif ($mode eq 'text_bbs_top')     { $wri->set_text('TxtBbsTop')      }
  elsif ($mode eq 'text_bbs_bottom')  { $wri->set_text('TxtBbsBottom')   }
  elsif ($mode eq 'text_msg_top')     { $wri->set_text('TxtMsgTop')      }
  elsif ($mode eq 'text_msg_bottom')  { $wri->set_text('TxtMsgBottom')   }
  elsif ($mode eq 'text_tree_top')    { $wri->set_text('TxtTreeTop')     }
  elsif ($mode eq 'text_paint_head')  { $wri->set_text('TxtPaintHead')   }
  elsif ($mode eq 'text_paint_top')   { $wri->set_text('TxtPaintTop')    }
  elsif ($mode eq 'text_paint_down')  { $wri->set_text('TxtPaintBottom') }
  elsif ($mode eq 'text_spa_head')    { $wri->set_text('TxtSpaHead')     }
  elsif ($mode eq 'text_spa_top')     { $wri->set_text('TxtSpaTop')      }
  elsif ($mode eq 'text_spa_down')    { $wri->set_text('TxtSpaBottom')   }
  elsif ($mode eq 'text_sii_head')    { $wri->set_text('TxtSiiHead')     }
  elsif ($mode eq 'text_sii_top')     { $wri->set_text('TxtSiiTop')      }
  elsif ($mode eq 'text_sii_down')    { $wri->set_text('TxtSiiBottom')   }
  elsif ($mode eq 'text_spv_head')    { $wri->set_text('TxtSpvHead')     }
  elsif ($mode eq 'text_spv_top')     { $wri->set_text('TxtSpvTop')      }
  elsif ($mode eq 'text_spv_down')    { $wri->set_text('TxtSpvBottom')   }
  elsif ($mode eq 'text_pic_head')    { $wri->set_text('TxtPicHead')     }
  elsif ($mode eq 'text_pic_top')     { $wri->set_text('TxtPicTop')      }
  elsif ($mode eq 'text_pic_down')    { $wri->set_text('TxtPicBottom')   }
  elsif ($mode eq 'text_pai_head')    { $wri->set_text('TxtPaiHead')     }
  elsif ($mode eq 'text_pai_top')     { $wri->set_text('TxtPaiTop')      }
  elsif ($mode eq 'text_pai_down')    { $wri->set_text('TxtPaiBottom')   }

  elsif ($mode eq 'skin_stylesheet')  { $wri->set_css }
  elsif ($mode eq 'skin_number')      { $wri->set_skin_tags('Number')  }
  elsif ($mode eq 'skin_title')       { $wri->set_skin_tags('Title')   }
  elsif ($mode eq 'skin_user')        { $wri->set_skin_tags('User')    }
  elsif ($mode eq 'skin_message')     { $wri->set_skin_tags('Message') }
  elsif ($mode eq 'skin_edit')        { $wri->set_skin_tags('Edit')    }

  elsif ($mode eq 'skin_shipainter')  { $wri->set_skin_shi('AppletColorShiPainter')  }
  elsif ($mode eq 'skin_shipaintbbs') { $wri->set_skin_shi('AppletColorShiPaintBBS') }

  elsif ($mode eq 'design_head')      { $wri->set_design('HEAD')    }
  elsif ($mode eq 'design_menu')      { $wri->set_design('MENU')    }
  elsif ($mode eq 'design_main')      { $wri->set_design('MAIN')    }
  elsif ($mode eq 'design_object')    { $wri->set_design('OBJECT')  }
  elsif ($mode eq 'design_catalog')   { $wri->set_design('CATALOG') }
  elsif ($mode eq 'design_tree')      { $wri->set_design('TREE')    }
  elsif ($mode eq 'design_frame')     { $wri->set_design('FRAME')   }
  elsif ($mode eq 'design_foot')      { $wri->set_design('FOOT')    }
  elsif ($mode eq 'design_post')      { $wri->set_design('POST')    }
  elsif ($mode eq 'design_search')    { $wri->set_design('SEARCH')  }

  $bn->call( 'CMS_WriteMode' );

}

sub esc_tag {
  my( $wri, $cfg, $err_handle ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  $err_handle ||= $wri;

  my $ok_tags = $bn->{sys}{CMS}{LIMIT_TAG};
  my $s = $in->{$cfg};

  $s =~ s/\x0D\x0A/\n/go;
  $s =~ s/\x0D/\n/go;
  $s =~ s/\x0A/\n/go;

  my $pos = 0;
  my $line = 1;
  my $line_pos = 1;
  my $end_pos = length $s;
  my $debug = sub{ push @{ $wri->{cms}{error} }, $_[0] };
  my ($tag_str, $flag_tag_area, @tag_layer,@pos_tag_start);

  for ( my $pos = 0; $pos <= $end_pos ; $pos++ , $line_pos++)
  {

  my $chr = substr $s, $pos, 1;

  if ($chr eq '<') {
    if ($flag_tag_area) {
      $err_handle->err('ERR_BRACKET', $pos_tag_start[1], $pos_tag_start[2], $cfg);
      return;
    }
    $flag_tag_area = 1;
    @pos_tag_start = ($pos, $line, $line_pos);

  } elsif ($chr eq '>') {

    if ($flag_tag_area) {
      $tag_str =~ s/\n|\t/ /g;
      $tag_str =~ s/ +/ /g;
      my $close = 1 if $tag_str =~ s/^\///;

      unless ($tag_str =~ /(\S+)/) {
        $err_handle->err('ERR_TAGNAME', $pos_tag_start[1], $pos_tag_start[2], $cfg);
        return;
      }
      my $tag = $1;
      my $uc_tag = uc $tag;

      unless (exists $ok_tags->{$uc_tag}) {
        $err_handle->err('ERR_TAG', $tag, $pos_tag_start[1], $pos_tag_start[2], $cfg);
        return;
      }

      my $tag_end = (length $tag_str) + 1;
      my %elmt;
      my $elmt_name;
      my $elmt_val;
      my $elmt_flag;
      my $apot_flag;
      my $apot_str;
      my $add_elmt = sub {
        return if $elmt_name eq '';
        $elmt{$elmt_name} = $elmt_val;
        undef $elmt_name;
        undef $elmt_val;
        undef $elmt_flag;
        undef $apot_str;
      };
      for (my $tag_pos = length $tag; $tag_pos <= $tag_end; $tag_pos++)
      {
      my $chr = substr $tag_str, $tag_pos, 1;
        if ($chr eq ' ' || $chr eq '' || $tag_pos == $tag_end) {
          if ($apot_str) {
            $elmt_val .= $chr;
          } else {
            undef $apot_flag;
            &$add_elmt;
          }
        } elsif ($elmt_flag && ( $chr eq '"' || $chr eq "'" ) ) {
          if ($apot_str) {
            if ($chr eq $apot_str) {
              &$add_elmt;
              $apot_flag = 1;
            } else {
              $elmt_val .= $chr;
            }
          } else {
            $apot_str = $chr;
          }
        } elsif ($chr eq '=') {
          if ($elmt_flag) {
            # err
          } else {
            $elmt_flag = 1;
          }
        } else {
          if ($elmt_flag) {
            $elmt_val .= $chr;
          } elsif ($apot_flag) {
            $err_handle->err('ERR_ELMMARGIN', $tag, $pos_tag_start[1], $pos_tag_start[2], $cfg);
            return;
          } else {
            $elmt_name .= $chr;
            undef $apot_flag;
          }
        }
      }

      my @ok_elmt = @{ $ok_tags->{$uc_tag} };
      for my $e (keys %elmt) {
        my $ok;
        my $el = lc $e;
        for (@ok_elmt) {
          if ($el eq $_) {
            $ok = 1;
            last;
          }
        }
        unless ($ok) {
          $err_handle->err('ERR_ELM', $tag, $e, $pos_tag_start[1], $pos_tag_start[2], $cfg);
          return;
        }
        if ($elmt{$e} =~ /javascript(\s+)?(:|\&\#58)/) {
          $err_handle->err('ERR_ELMJAVA', $tag, $e, $pos_tag_start[1], $pos_tag_start[2], $cfg);
          return;
        }
        if ($close) {
          $err_handle->err('ERR_TAGCLSELM', $tag, $pos_tag_start[1], $pos_tag_start[2], $cfg);
          return;
        }
      }

      for ('href', 'src', 'background') {
        if (exists $elmt{$_}) {
          if ($elmt{$_} =~ /[^-.!~*'()\w;\/?:\@&=+\$,%#]/) {
            $err_handle->err('ERR_ELMSRC', $tag, $_, $pos_tag_start[1], $pos_tag_start[2], $cfg);
            return;
          }
          if ($elmt{$_} !~ m#^(?:https?://|images/)([-.!~*'()\w;/?:\@&=+\$,%\#]+)#) {
            $err_handle->err('ERR_ELMHREF', $tag, $_, $pos_tag_start[1], $pos_tag_start[2], $cfg);
            return;
          } elsif (/src|background/ && $elmt{$_} !~ /\.(jpe?g|gif|png)$/) {
            $err_handle->err('ERR_ELMSRCEXE', $tag, $_, $pos_tag_start[1], $pos_tag_start[2], $cfg);
            return;
          }
        }
      }

      for (keys %elmt) {
        next if /href/;
        while ($elmt{$_} =~ m#(https?://[\w\ \,\.\'\"\(\)\[\]\\\^\~\{\}\`\&\?\%\#\$\=\-\+\*\@\!\/\:\;]+)#g) {
          my $r = index $1, '?';
          last if $r == -1;
          $err_handle->err('ERR_ELMSRCARG', $tag, $_, $pos_tag_start[1], $pos_tag_start[2], $cfg);
          return;
        }
      }

      if ($close) {
        if (@tag_layer && $tag_layer[-1]->{tag} eq $uc_tag) {
          pop @tag_layer;
        } else {
          # err
          $err_handle->err('ERR_TAGNST', $tag, $pos_tag_start[1], $pos_tag_start[2], $cfg);
          return;
        }
      } else {
        unless ($uc_tag =~ /AREA|BASEFONT|BR|COL|COLGROUP|HR|IMG|WBR/) {
          push @tag_layer,
          { tag => $uc_tag, pos => $pos_tag_start[2], line => $pos_tag_start[1] };
        }
      }

      undef $tag_str;
      undef @pos_tag_start;
    } else {
      # none err
    }
      $flag_tag_area = 0;

  } else {
    if ($flag_tag_area) {
      $tag_str .= $chr;
    } elsif ($chr eq "\n") {
      $line++; $line_pos = 0; next;
    }
  }

  }

  if ($flag_tag_area) {
    $err_handle->err('ERR_BRACLS', $pos_tag_start[1], $pos_tag_start[2], $cfg);
  }

  for (@tag_layer) {
    $err_handle->err('ERR_TAGCLS', @{$_}{'tag','line','pos'}, $cfg);
  }



  $wri;
}

sub num {
  my( $wri, $cfg, $min, $max, $need, $percent ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  my $n = $wri->{limit_num};
  if (exists $n->{$cfg}) {
    $n = $n->{$cfg};
    $min = $n->{min} if exists $n->{min};
    $max = $n->{max} if exists $n->{max};
    $need = $n->{need} if exists $n->{need};
  }

  if ($need && $in->{$cfg} eq '') {
    $wri->err('ERR_NEED', $wri->{cms}{item}{$cfg});
    return $wri;
  } elsif (!$need && $in->{$cfg} eq '') {
    return $wri;
  }

  if ($percent && $in->{$cfg} =~ /(\d+)(\%)?/) {
    $in->{$cfg} = (int $1) . $2;
  } else {
    $in->{$cfg} = int $in->{$cfg};
  }
  return $wri if $in->{$cfg} >= $min && $in->{$cfg} <= $max;
  $wri->err('ERR_NUM', $wri->{cms}{item}{$cfg}, $min, $max);
}

sub str {
  my( $wri, $cfg, @list ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};
  my $s = $wri->{limit_str};
  @list = @{$s->{$cfg}} if exists $s->{$cfg};

  for (@list) { return $wri if $in->{$cfg} eq $_ }
  $wri->err('ERR_STR', $wri->{cms}{item}{$cfg}, join ', ', @list);
}

sub color {
  my( $wri, $cfg ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  return $wri if $in->{$cfg} eq '';
  if ($in->{$cfg} =~ /\#?([\da-fA-F]+)/) {
    $in->{$cfg} = sprintf "#%06s", $1;
    return $wri;
  }
  $wri->err('ERR_COLOR', $wri->{cms}{item}{$cfg});
}

sub len {
  my( $wri, $cfg, $len ) = @_;
  my $in = $bn->{in};
  my $l = $wri->{limit_len};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  $len = $l->{$cfg} if exists $l->{$cfg};
  return $wri unless defined $len;
  $wri->err('ERR_LENGTH', $wri->{cms}{item}{$cfg}, $len .'/'. length $in->{$cfg})
    if length $in->{$cfg} > $len;
  $wri;
}

sub esc {
  my( $wri, $cfg, $flag_line ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  chomp $in->{$cfg};
  $in->{$cfg} =~ s/&/&amp;/g;
  $in->{$cfg} =~ s/</&lt;/g;
  $in->{$cfg} =~ s/>/&gt;/g;
  $in->{$cfg} =~ s/\x0D\x0A/\n/go;
  $in->{$cfg} =~ s/\x0D/\n/go;
  $in->{$cfg} =~ s/\x0A/\n/go;

  unless ($flag_line) {
    $in->{$cfg} =~ s/\n|\t//g;
  }
  $wri;
}

# esc -> len
sub esc_len {
  my( $wri, $cfg, $len, $flag_line ) = @_;
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};
  $wri->esc($cfg, $flag_line)->len($cfg, $len);
}

sub sty {
  my( $wri, $cfg, $len, $flag_line ) = @_;
  my $in = $bn->{in};
  my $d = $wri->{disable};
    return $wri if exists $d->{$cfg};

  $in->{$cfg} =~ s/[^0-9a-zA-Z\-:;,.]//g;
  my @a = split /;/, $in->{$cfg};
  my @b;
  for (@a) {
    my ($s, $v) = split /:/;
    if (exists $wri->{limit_sty}{$cfg} && $s !~ /$wri->{limit_sty}{$cfg}/) {
      next;
    }
    push @b, "$s:$v";
  }
  $in->{$cfg} = join ';',@b;
  $wri->esc($cfg, $flag_line)->len($cfg, $len);
}

sub err {
  my $wri = shift;
  my $key = shift;
  my @arg = @_;
  my $t = $wri->{cms}{item}{$key};
  my $n = 1;
  for (@arg){
    $t =~ s/\[_$n\]/$arg[$n-1]/g;
    $n++;
  }
  push @{ $wri->{cms}{error} }, $t;
  $wri;
}

sub set_hash {
  my( $wri, $in, $cfg, $in_key, $cfg_key ) = @_;
  my $d = $wri->{disable};
    return $wri if exists $d->{$in_key};
#  $in->{$in_key} eq '' ? delete $cfg->{$cfg_key} :
#   $cfg->{$cfg_key} = $in->{$in_key} if exists $in->{$in_key};
  if ( exists $in->{$in_key} ) {
    if ( $in->{$in_key} eq '' ) {
      delete $cfg->{$cfg_key};
    } else {
      $cfg->{$cfg_key} = $in->{$in_key};
    }
  }

  $wri;
}

sub write {
  my( $wri, $vars, $svars) = @_;

  return if exists $wri->{cms}{error} && @{ $wri->{cms}{error} };
  return if exists $wri->{disable}{$bn->{in}{mode}};

  $bn->filelock;
  $wri->write_cfg(
    'cfg', $bn->{sys}{Config}, $vars) if defined $vars;
  $wri->write_cfg(
    'skin', $bn->{sys}{SkinCfg}, $svars) if defined $svars;
  unlink "$bn->{sys}{DirSys}/boot.$bn->{sys}{Exe}";
  $bn->fileunlock;

}

sub write_cfg {
  my( $wri, $cfg, $path, $vars ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{$cfg};
  my $sys = $bn->{sys};
  my %flag;

  for (my $n = $#$vars; $n >= 0; $n--) {
  splice @$vars, $n, 1
    if exists $wri->{disable}{ $vars->[$n][1] };
  }

  local($_, $/);
  $/ = "\n";
  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_CONFIG_NOTFOUND', $path);
  my($fh_tmp, $tmp) = $bn->get_temp_file;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};

  while (<$fh>) {
  my $org = $_;
  if (!/\S/ || /^#/)
   { print $fh_tmp $org; next }
  chomp;
  my($var, $val) = $_ =~ /^\s*(\S+)(?: +|\t+)(.*)?$/;
  my $dtype = $1 if $var =~ s/^(\$|\%|\@)//;
  my $f;
  for (my $n = 0; $n <= $#$vars; $n++) {
   my $ovar = $vars->[$n][0];
   my $type = $1 if $ovar =~ s/^(\$|\%|\@)//;
   if ($var eq $ovar) {
    $f = 1;
    # skip
    if (exists $flag{$var}) {
      last;
    # simple var
    } elsif ($vars->[$n][2] eq '$') {
      print $fh_tmp "$type$var\t$in->{$vars->[$n][1]}\n" if $in->{$vars->[$n][1]} ne '';
      $bn->set($cf, $var, $in->{$vars->[$n][1]});
      $flag{$var} = 1;
      last;
    # array var
    } elsif ($vars->[$n][2] eq '@') {
      my @a = ref $in->{$vars->[$n][1]} eq 'ARRAY' ?
        @{$in->{$vars->[$n][1]}} : split /\n/, $in->{$vars->[$n][1]};
      $bn->define($cf, $var, undef, '@');
#      $a[0] = '' unless @a;
      for (@a) {
        print $fh_tmp "$type$var\t$_\n";
        $bn->set($cf, $var, $_, '@');
      }
      $flag{$var} = 1;
      last;
    # hash var
    } elsif ($vars->[$n][2] eq '%') {
      if (ref $in->{$vars->[$n][1]} eq 'HASH') {
        $bn->define($cf, $var, undef, '%');
        my $vf;
        for (keys %{ $in->{$vars->[$n][1]} }) {
          next if $in->{$vars->[$n][1]}{$_} eq '';
          print $fh_tmp "$type$var\t\t$_\t\t$in->{$vars->[$n][1]}{$_}\n";
          $bn->set($cf, $var, "$_\t$in->{$vars->[$n][1]}{$_}", '%');
          $vf++;
        }
#        print $fh_tmp "$type$var\t\n" unless $vf;
        $flag{$var} = 1;
        last;
      }
    }
   }
  }
  print $fh_tmp $org unless $f;
  }

  for (my $n = 0; $n <= $#$vars; $n++) {
    my $var = $vars->[$n][0];
    my $type = $1 if $var =~ s/^(\$|\%|\@)//;
    next if exists $flag{$var};
    # simple var
    if ($vars->[$n][2] eq '$') {
      print $fh_tmp "$type$var\t$in->{$vars->[$n][1]}\n" if $in->{$vars->[$n][1]} ne '';
      $bn->set($cf, $var, $in->{$vars->[$n][1]});
      $flag{$var} = 1;
    # array var
    } elsif ($vars->[$n][2] eq '@') {
      my @a = ref $in->{$vars->[$n][1]} eq 'ARRAY' ?
      @{$in->{$vars->[$n][1]}} : split /\n/, $in->{$vars->[$n][1]};
      $bn->define($cf, $var, undef, '@');
#      $a[0] = '' unless @a;
      for (@a) {
        print $fh_tmp "$type$var\t$_\n";
        $bn->set($cf, $var, $_, '@');
      }
      $flag{$var} = 1;
    # hash var
    } elsif ($vars->[$n][2] eq '%') {
      if (ref $in->{$vars->[$n][1]} eq 'HASH') {
        $bn->define($cf, $var, undef, '%');
        my $vf;
        for (keys %{ $in->{$vars->[$n][1]} }) {
          next if $in->{$vars->[$n][1]}{$_} eq '';
          print $fh_tmp "$type$var\t\t$_\t\t$in->{$vars->[$n][1]}{$_}\n";
          $bn->set($cf, $var, "$_\t$in->{$vars->[$n][1]}{$_}", '%');
          $vf++;
        }
#        print $fh_tmp "$type$var\t\n" unless $vf;
        $flag{$var} = 1;
      }
    }
  }

    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;

  close $fh;
  close $fh_tmp;

  rename $tmp, $path
  or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $path);

}

sub set_archive {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $opt = $in->{opt};
  my $mode = $in->{mode};
  my $dir = $bn->{sys}{DirImages};
  return if exists $wri->{disable}{$mode};

  my $limit_list = 100;
  my $limit_file = 1024000;
  my $limit_mime = '.*';

  if (exists $wri->{limit_num}{archive}){
    $limit_list = $wri->{limit_num}{archive}{list};
    $limit_file = $wri->{limit_num}{archive}{file};
    $limit_mime = $wri->{limit_num}{archive}{mime};
    $dir = $wri->{limit_num}{archive}{dir} if exists $wri->{limit_num}{archive}{dir};
  }

  $in->{to} =~ s/[^\w\.\_\+\-\=\~\']//g if exists $in->{to};
  $in->{from} =~ s/[^\w\.\_\+\-\=\~\']//g if exists $in->{from};
  $in->{filename} =~ s/[^\w\.\_\+\-\=\~\']//g if exists $in->{filename};

  if ($opt eq 'rename') {
    return if $in->{from} eq $in->{to};
    unless (-f "$dir/$in->{from}"){$wri->err('ERR_NOTFOUND',$in->{from});return}
    if ($in->{to} =~ /\.$limit_mime$/) {
      $wri->err('ERR_RENAME',"$in->{from} > $in->{to}")
        unless $in->{to} !~ /^\./ && rename "$dir/$in->{from}", "$dir/$in->{to}";
    } else {
      $wri->err('ERR_MIME',$in->{to});
    }
  } elsif ($opt eq 'delete') {
    unlink "$dir/$in->{filename}"
      or $wri->err('ERR_DELETE',$in->{from});
  } elsif ($opt eq 'upload') {
    my $fn = $in->{file_filename};
    $fn = $1 if $fn =~ /([^\/\\]+)$/;

    $wri->err('ERR_FILENAME', $fn) and return if $fn =~ /[^\w\.\_\+\-\=\~\']/;

    return unless length $in->{file};

    if (length $in->{file} >= $limit_file) {
      $wri->err('ERR_DATAOVER', length $in->{file}, $limit_file);
      return;
    }

    $wri->err('ERR_MIME',$fn) and return
      unless ($fn =~ /\.$limit_mime$/);

    my $cnt;
    my $dh = $bn->opendir($dir);
    $dh or $bn->error('ERR_DATA_NOTOPEN', $dir);
    while ($_ = readdir $dh) {
      next if /^\.\.?/;
      $cnt++;
    }
    close $dh;

    if ($cnt >= $limit_list) {
      $wri->err('ERR_LISTOVER', $limit_list);
      return;
    }

    $wri->err('ERR_OVERWRITE', $fn)if -f "$dir/$fn";
    my $fh = $bn->open("$dir/$fn", '>');
    $fh or $wri->err('ERR_NOTWRITE', $fn) and return;
    binmode $fh;
    print $fh $in->{file};
    close $fh;
  }

}

sub set_design_copyright {
  my( $wri, $target ) = @_;
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $mode = $in->{mode};
  my($label, $tmpl, %flag);

  return if exists $wri->{disable}{$mode};

  my @cc = qw/skin_name skin_ver skin_lastmodify skin_copy skin_auth skin_authurl/;
  my @sc = qw/SkinName SkinVersion SkinLastmodify SkinCopyright SkinAuthor SkinAuthorSiteURL/;
  for (@cc) { $wri->esc_len($_, 100) }

  my $path = "$bn->{sys}{DirCfgSys}/$bn->{sys}{Skin}.$bn->{sys}{Exe}";

  local($_, $/);
  $/ = "\n";
  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_SKIN_NOTFOUND', $path);

  my($fh_tmp, $tmp) = $bn->get_temp_file;

  while (<$fh>) {
    my $f;
    if ( /^#\s*(\w+):(\w+)/ ) {
      if ( $1 eq 'CONFIG' ) {
        $label = $2 eq 'END' ? 0 : $1;
      }
    }
    if( $label eq 'CONFIG' ){
      if (!/\S/ || /^#/) { print $fh_tmp $_; next }
      my($var, $val) = $_ =~ /^\s*(\S+)\s+(.+)$/;
      my $n;
      for (@sc) {
        if ($var eq $_) {
          unless ($flag{$_}) {
            my $s = 25 - length $_;
            $s = ' ' x $s if $s > 0;
            print $fh_tmp "$_$s".$in->{$cc[$n]}."\n";
            $flag{$_} = 1;
          }
          $f = 1;
          last;
        }
        $n++;
      }
    }
    print $fh_tmp $_ unless $f;
  }

  close $fh;
  close $fh_tmp;

  rename $tmp, $path
    or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $path);
  unlink "$bn->{sys}{DirSys}/boot.$bn->{sys}{Exe}";

  $bn->app->reflesh;
}


sub set_design {
  my( $wri, $target ) = @_;
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $mode = $in->{mode};
  my( $label, $tmpl, $flag );

  return if exists $wri->{disable}{$mode};

  $wri->len($mode, 999999);
  $in->{$mode} =~ s/\x0D\x0A/\n/go;
  $in->{$mode} =~ s/\x0D/\n/go;
  $in->{$mode} =~ s/\x0A/\n/go;
  # error handler
  $bn->{template} = { error => sub {
    delete $in->{cmd};
    my $err = $bn->text( 'ERR_SKIN_TAG', 'error' );
    for ( my $n = 1; $n <= @_; $n++ ) { $err =~ s/\[_$n]/$_[$n - 1]/g }
    $err =~ s/\n/<br>/g;
    push @{ $wri->{cms}{error} }, $err;
    $wri->{cms}->config;
    $bn->EXIT } };
  $bn->template_trans( { data => $in->{$mode} } );

  my $path = "$bn->{sys}{DirCfgSys}/$bn->{sys}{Skin}.$bn->{sys}{Exe}";

  local($_, $/);
  $/ = "\n";
  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_SKIN_NOTFOUND', $path);

  my($fh_tmp, $tmp) = $bn->get_temp_file;

  while (<$fh>) {
    if ( /^#\s*(\w+):(\w+)/ ) {
      if ( $1 eq 'TEMPLATE' ) {
        $label = $2 eq 'END' ? 0 : $1;
        next if $label eq 'END';
        $tmpl = $2;
        print $fh_tmp $_;
        next;
      }
    }
    if ( $label eq 'TEMPLATE' && $target eq $tmpl) {
      next if $flag;
      print $fh_tmp $in->{$mode}, "\n";
      $flag = 1;
      next;
    }
    print $fh_tmp $_;
  }

  close $fh;
  close $fh_tmp;

  rename $tmp, $path
    or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $path);
  unlink "$bn->{sys}{DirSys}/boot.$bn->{sys}{Exe}";

  $bn->app->reflesh;
}

sub set_skin_dynpalette {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $app = 'AppletColorDynPalette';
  my $cfg = $bn->get($sk, $app);
  for (qw/ palette_border palette_back palette_font
  gradation_border gradation_back gradation_font /)
   { $wri->color($_); $cfg->{$_} = $in->{$_} }

  return if @{ $wri->{cms}{error} };

  $in->{$app} = join ',', map { "$_:$cfg->{$_}" } keys %$cfg;
  $bn->define($sk, $app, undef, '$');

  $wri->write( [],[  [ $app, $app, '$' ] ] );
  $bn->cfg2hash($sk, $app);

}

sub set_skin_picturebbs {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $app = 'AppletColorPictureBBS';
  my $cfg = $bn->get($sk, $app);
  for (qw/ bgcolor window_color caption_color /)
   { $wri->color( $_ )->set_hash( $in, $cfg, $_, $_ ) }

  return if @{ $wri->{cms}{error} };

  $in->{$app} = join ',', map { "$_:$cfg->{$_}" } keys %$cfg;
  $bn->define($sk, $app, undef, '$');

  $wri->write( [],[  [ $app, $app, '$' ] ] );
  $bn->cfg2hash($sk, $app);

}

sub set_skin_pchviewer {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $app = 'AppletColorShiPCHViewer';
  my $cfg = $bn->get($sk, $app);
  for (qw/ color_text color_back color_icon color_iconselect
    color_bar color_frame color_bar_select /)
  { $wri->color( $_ )->set_hash( $in, $cfg, $_, $_ ) }

  return if @{ $wri->{cms}{error} };

  $in->{$app} = join ',', map { "$_:$cfg->{$_}" } keys %$cfg;
  $bn->define($sk, $app, undef, '$');

  $wri->write( [],[  [ $app, $app, '$' ] ] );
  $bn->cfg2hash($sk, $app);

}

sub set_skin_shi {
  my( $wri, $app ) = @_;
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $cfg = $bn->get($sk, $app);

  for (qw/spa_color_text spa_color_icon spa_color_iconselect spa_color_bar spa_color_bar_hl/) {
    if ( exists $in->{$_} && $in->{$_} ne '' ) { my $a = delete $in->{$_};
     my $s = $_; $s =~ s/^spa_//; $in->{$s} = $a }
  }

  for (qw/color_text color_bk color_bk2 color_icon color_frame
    color_iconselect color_bar color_bar_frame_hl color_bar_hl
    color_bar color_bar_shadow color_frame color_icon
    tool_color_text tool_color_bk tool_color_frame tool_color_button tool_color_button2
    tool_color_bar tool_color_button_hl tool_color_button_dk
    bar_color_text bar_color_bk bar_color_frame bar_color_off bar_color_off_hl bar_color_off_dk
    bar_color_on bar_color_on_hl bar_color_on_dk
    window_color_text window_color_bk window_color_frame window_color_bar_text window_color_bar window_color_bar_hl
    pro_menu_color_text pro_menu_color_on pro_menu_color_on_hl pro_menu_color_on_dk
    pro_menu_color_off pro_menu_color_off_dk pro_menu_color_off_hl
    l_m_color l_m_color_text dlg_color_bk dlg_color_text/)
  { $wri->color($_)->set_hash($in, $cfg, $_, $_) }

  return if @{ $wri->{cms}{error} };

  my @config;
  for ( keys %$cfg ) { push @config, "$_:$cfg->{$_}" if $cfg->{$_} ne '' }
  $in->{$app} = join ',', @config;
  $bn->define($sk, $app, undef, '$');

  $wri->write( [],[  [ $app, $app, '$' ] ] );
  $bn->cfg2hash($sk, $app);
}

sub set_skin_frame {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $item = $wri->{cms}{item};
  my %cfg;

  for my $cf ('out','in') {
    $cfg{$cf} = $bn->get($sk, 'TagTable' . ucfirst $cf);
    for ('bgcolor','width','cellpadding','cellspacing') {
      $item->{$cf . '_' . $_} = "$item->{'skin_catalog_'.$cf}:$item->{$_}";
    }

    $wri->color($cf.'_bgcolor');
      $cfg{$cf}->{bgcolor} = $in->{$cf.'_bgcolor'};
    $wri->len($cf.'_width', 9);
      $cfg{$cf}->{width} = $in->{$cf.'_width'};
    $wri->num($cf.'_cellpadding', 0, 999, 0);
      $cfg{$cf}->{cellpadding} = $in->{$cf.'_cellpadding'};
    $wri->num($cf.'_cellspacing', 0, 999, 0);
      $cfg{$cf}->{cellspacing} = $in->{$cf.'_cellspacing'};
  }

  my @b = @{$item}{'font_color','font_size'};
  $item->{'font_color'} = "$item->{'skin_frame_font'}:$item->{font_color}";
  $item->{'font_size'} = "$item->{'skin_frame_font'}:$item->{font_color}";

  $wri->color('font_color');
    $cfg{font}->{color} = $in->{'font_color'};
  $wri->num('font_size', 1, 7, 0);
    $cfg{font}->{size} = $in->{'font_size'};

  return if @{ $wri->{cms}{error} };

  @{$item}{'font_color','font_size'} = @b;

  $in->{'TagTableOut'} = $cfg{out};
  $in->{'TagTableIn'} = $cfg{in};
  $in->{'TagTableFont'} = $cfg{font};

  $wri->write(
    [],[
      [ '%TagTableOut', 'TagTableOut', '%' ],
      [ '%TagTableIn', 'TagTableIn', '%' ],
      [ '%TagTableFont', 'TagTableFont', '%' ],
    ]
  );

  $bn->app->reflesh;
}
sub set_skin_catalog {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $item = $wri->{cms}{item};
  my %cfg;

  for my $cf ('out','in') {
    $cfg{$cf} = $bn->get($sk, 'TagCatalogTable' . ucfirst $cf);
    for ('bgcolor','width','cellpadding','cellspacing') {
      $item->{$cf . '_' . $_} = "$item->{'skin_catalog_'.$cf}:$item->{$_}";
    }

    $wri->color($cf.'_bgcolor');
      $cfg{$cf}->{bgcolor} = $in->{$cf.'_bgcolor'};
    $wri->len($cf.'_width', 9);
      $cfg{$cf}->{width} = $in->{$cf.'_width'};
    $wri->num($cf.'_cellpadding', 0, 999, 0);
      $cfg{$cf}->{cellpadding} = $in->{$cf.'_cellpadding'};
    $wri->num($cf.'_cellspacing', 0, 999, 0);
      $cfg{$cf}->{cellspacing} = $in->{$cf.'_cellspacing'};
  }

  return if @{ $wri->{cms}{error} };

  $in->{'TagCatalogTableOut'} = $cfg{out};
  $in->{'TagCatalogTableIn'} = $cfg{in};

  $wri->write(
    [],[
      [ '%TagCatalogTableOut', 'TagCatalogTableOut', '%' ],
      [ '%TagCatalogTableIn',  'TagCatalogTableIn',  '%' ],
    ]
  );

  $bn->app->reflesh;
}
sub set_skin_form {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $item = $wri->{cms}{item};
  my $cfg_form = $bn->get($sk, 'TagFormItems');
  my $cfg_ast = $bn->get($sk, 'TagFormAstMark');

  for ('string','font_color','font_size') {
    $item->{'skin_form_ast_' . $_} = "$item->{skin_form_ast}:$item->{$_}";
  }
  for ('color','style','bold','italic','under','teletype') {
    $item->{'item_' . $_} = "$item->{skin_form}:$item->{$_}";
  }
  $item->{'item_size'} = "$item->{skin_form}:$item->{font_size}";
  $item->{'item_face'} = "$item->{skin_form}:$item->{font_face}";

  $wri->color('item_color');
    $cfg_form->{color} = $in->{'item_color'};
  $wri->num('item_size', 1, 7, 0);
    $cfg_form->{size} = $in->{'item_size'};
  $wri->esc_len('item_face', 999);
    $cfg_form->{face} = $in->{'item_face'};
  $wri->sty('item_style',99);
    $cfg_form->{style} = $in->{'item_style'};

  for ('bold','italic','under','teletype') {
    $wri->str('item_' . $_, '', '1');
      $cfg_form->{$_} = $in->{'item_' . $_};
  }

  $wri->len('skin_form_ast_string', 999);
    $cfg_ast->{string} = $in->{'skin_form_ast_string'};
  $wri->color('skin_form_ast_font_color');
    $cfg_ast->{color} = $in->{'skin_form_ast_font_color'};
  $wri->num('skin_form_ast_font_size', 1, 7, 0);
    $cfg_ast->{size} = $in->{'skin_form_ast_font_size'};

  return if @{ $wri->{cms}{error} };

  $in->{'TagFormItems'} = $cfg_form;
  $in->{'TagFormAstMark'} = $cfg_ast;

  $wri->write(
    [],[
      [ '%TagFormItems',   'TagFormItems',   '%' ],
      [ '%TagFormAstMark', 'TagFormAstMark', '%' ],
    ]
  );

  $bn->app->reflesh;
}


sub set_skin_tags {
  my( $wri, $cfg ) = @_;
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $mode = $in->{mode};
  my $item = $wri->{cms}{item};

  my %cf;
  $cf{main} = $bn->get($sk, 'TagMain' . $cfg);
  $cf{res} = $bn->get($sk, 'TagRes' . $cfg);

  for ('color','style','bold','italic','under','teletype') {
    $item->{$mode . '_main_' . $_} = "$item->{$mode}:$item->{skin_main}$item->{$_}";
    $item->{$mode . '_res_' . $_} = "$item->{$mode}:$item->{skin_res}$item->{$_}";
  }

  for ('main' , 'res') {
    $cf{$_} = {} if ref $cf{$_} ne 'HASH';
    $item->{$mode.'_'.$_.'_size'} = "$item->{$mode}:$item->{'skin_'.$_}$item->{font_size}";
    $item->{$mode.'_'.$_.'_face'} = "$item->{$mode}:$item->{'skin_'.$_}$item->{font_face}";
    $in->{$mode.'_'.$_.'_color'} = $in->{$_.'_color'};
    delete $in->{$_.'_color'};
    $wri->color($mode.'_'.$_.'_color');
      $cf{$_}->{color} = $in->{$mode.'_'.$_.'_color'};

    $in->{$mode.'_'.$_.'_size'} = $in->{$_.'_size'};
    delete $in->{$_.'_size'};
    $wri->num($mode.'_'.$_.'_size', 0, 99, 0);
      $cf{$_}->{size} = $in->{$mode.'_'.$_.'_size'};

    $in->{$mode.'_'.$_.'_face'} = $in->{$_.'_face'};
    delete $in->{$_.'_face'};
    $wri->esc_len($mode.'_'.$_.'_face',99);
      $cf{$_}->{face} = $in->{$mode.'_'.$_.'_face'};

    $in->{$mode.'_'.$_.'_style'} = $in->{$_.'_style'};
    delete $in->{$_.'_style'};
    $wri->sty($mode.'_'.$_.'_style',99);
      $cf{$_}->{style} = $in->{$mode.'_'.$_.'_style'};

    $in->{$mode.'_'.$_.'_bold'} = $in->{$_.'_bold'};
    delete $in->{$_.'_bold'};
    $wri->str($mode.'_'.$_.'_bold', '', '1');
      $cf{$_}->{bold} = $in->{$mode.'_'.$_.'_bold'};

    $in->{$mode.'_'.$_.'_italic'} = $in->{$_.'_italic'};
    delete $in->{$_.'_italic'};
    $wri->str($mode.'_'.$_.'_italic', '', '1');
      $cf{$_}->{italic} = $in->{$mode.'_'.$_.'_italic'};

    $in->{$mode.'_'.$_.'_under'} = $in->{$_.'_under'};
    delete $in->{$_.'_under'};
    $wri->str($mode.'_'.$_.'_under', '', '1');
      $cf{$_}->{under} = $in->{$mode.'_'.$_.'_under'};

    $in->{$mode.'_'.$_.'_teletype'} = $in->{$_.'_teletype'};
    delete $in->{$_.'_teletype'};
    $wri->str($mode.'_'.$_.'_teletype', '', '1');
      $cf{$_}->{teletype} = $in->{$mode.'_'.$_.'_teletype'};

  }

  return if @{ $wri->{cms}{error} };

  $in->{'TagMain' . $cfg} = $cf{main};
  $in->{'TagRes' . $cfg} = $cf{res};

  $wri->write(
    [],[
      [ '%TagMain' . $cfg, 'TagMain' . $cfg, '%' ],
      [ '%TagRes'  . $cfg, 'TagRes'  . $cfg, '%' ],
    ]
  );

  $bn->app->reflesh;
}

sub set_skin_page {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $cfg = $bn->get($sk, 'TagBody');
  my $item = $wri->{cms}{item};

  for ('bgcolor','text','link','alink','vlink','background',
  'leftmargin','topmargin','marginwidth','marginheight') {
    $item->{'skin_page_' . $_} = "$item->{skin_page}:$item->{$_}";
  }

  $wri->color('skin_page_bgcolor');
    $cfg->{bgcolor} = $in->{skin_page_bgcolor};
  $wri->color('skin_page_text');
    $cfg->{text} = $in->{skin_page_text};
  $wri->color('skin_page_link');
    $cfg->{link} = $in->{skin_page_link};
  $wri->color('skin_page_alink');
    $cfg->{alink} = $in->{skin_page_alink};
  $wri->color('skin_page_vlink');
    $cfg->{vlink} = $in->{skin_page_vlink};
  $wri->len('skin_page_background', 99);
    $cfg->{background} = $in->{skin_page_background};
  $wri->num('skin_page_leftmargin', 0, 99, 0);
    $cfg->{leftmargin} = $in->{skin_page_leftmargin};
  $wri->num('skin_page_topmargin', 0, 99, 0);
    $cfg->{topmargin} = $in->{skin_page_topmargin};
  $wri->num('skin_page_marginwidth', 0, 99, 0);
    $cfg->{marginwidth} = $in->{skin_page_marginwidth};
  $wri->num('skin_page_marginheight', 0, 99, 0);
    $cfg->{marginheight} = $in->{skin_page_marginheight};

  return if @{ $wri->{cms}{error} };
  $in->{TagBody} = $cfg;

  $wri->write(
    [],[
      [ '%TagBody', 'TagBody', '%' ],
    ]
  );

  $bn->app->reflesh;
}

sub set_css {
  my( $wri, $cfg ) = @_;
  my $in = $bn->{in};
  my $mode = $in->{mode};

  $wri->esc_len($mode, 9999, 1);

  $bn->call('CMS_SetCSS');

  my @a = split /\r?\n/, $in->{$mode};
  $in->{$mode} = \@a;

  return if @{ $wri->{cms}{error} };

  $wri->write(
    [],[
      [ 'StyleSheet', $mode, '@' ],
    ]
  );
  $bn->app->reflesh;

}
sub set_text {
  my( $wri, $cfg ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $mode = $in->{mode};
  my $limit = $wri->{limit_esc};
  $limit->{$mode} = $limit->{menu_txt} if exists $limit->{menu_txt};

  $wri->len($mode, 9999);

  if ($limit->{$mode} == 1) {
    $wri->esc($mode, 1);
  } elsif ($limit->{$mode} == 2) {
    $wri->esc_tag($mode);
  }

  $bn->call('CMS_SetText');

  return if @{ $wri->{cms}{error} };
  $in->{$mode} =~ s/\r\n/\n/g;
  my @a = split /\n/, $in->{$mode};
  my @m = split /_/, $mode;
  $m[0] = 'txt';
  my $s = join '',map ucfirst,@m;
  $s =~ s/Down/Bottom/;
  $bn->define($cf, $s, '', '@');
  for (@a) { $bn->set($cf, $s, $_) }
  $in->{$mode} = \@a;

  $wri->write(
    [
      [ $cfg, $mode, '@' ],
    ]
  );

  $bn->app->reflesh;

}

sub set_system_counter_tmpl_image {
  my $wri = $_[0];
  my $d = $wri->{disable};
    return $wri if exists $d->{system_counter_tmpl} || exists $d->{system_counter_tmpl_image};
  $wri->set_text('TxtCounterImage')
}

sub set_system_counter_tmpl_text {
  my $wri = $_[0];
  my $d = $wri->{disable};
    return $wri if exists $d->{system_counter_tmpl} || exists $d->{system_counter_tmpl_text};
  $wri->set_text('TxtCounterText')
}

sub set_system_counter_value {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $gl = $bn->{global};
  my $d = $wri->{disable};
    return $wri if exists $d->{system_log};
  $wri->num('total', 0, 999999999);
  $wri->num('today', 0, 999999999);
  $wri->num('yesterday', 0, 999999999);
  my $path = $bn->{sys}{DirSys}.'/access/count.cgi';
  my $fh = $bn->open( $path, '>' );
  $fh or $bn->error('ERR_SYS_NOTWRITE', $path);
  print $fh (join ',', @{$in}{'total','today','yesterday'}, @{$gl->{accesslog}}[3..8]), "\n";
  close $fh;
  $gl->{accesslog} = [0,@{$in}{'total','today','yesterday'}];
  $in->{mode} = 'system_log';
}
sub set_system_counter_image_config {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $limit = $wri->{limit_num};
  my $d = $wri->{disable};
    return $wri if exists $d->{system_log};
  $wri->num('counter_image_trans', -1, 255);
  $wri->num('counter_image_colors', 1, 256);
  $wri->str('counter_image_dither', '','1');
  $in->{counter_image_dither} ||= 0;
  $in->{num} ||= 0;

  # file upload
  my $len = length $in->{file};
  if ( $len ) {
    my $dir = $bn->{sys}{DirSys}.'/access/';
    my $h = {};
    my $l = exists $limit->{counter_image_file} ? $limit->{counter_image_file} : { size => 10240, width => 100, height => 100 };
    @{$h}{'mime', 'width', 'height', 'size'} = $bn->po->file_analyze(\$in->{file});
    my $fh = $bn->open($dir."images/$in->{num}.gif", '>');
    binmode $fh;
    print $fh $in->{file};
    close $fh;
    unlink $dir.'imgdat.cgi';
    $wri->err('ERR_EXE') if $h->{mime} ne 'gif';
    $wri->err('ERR_UP_LENGTH', $h->{size}, $l->{size}) if $h->{size} > $l->{size};
    $wri->err('ERR_UP_SIZE', $h->{width}, $l->{width}, $h->{height}, $l->{height}) if $h->{width} > $l->{width} || $h->{height} > $l->{height} ;
  }

  $wri->write(
    [
      [ 'ViewCounterImageTransparent', 'counter_image_trans',  '$' ],
      [ 'ViewCounterImageColors',      'counter_image_colors', '$' ],
      [ 'ViewCounterImageDither',      'counter_image_dither', '$' ],
    ]
  );
}
sub set_system_log {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('use_access_log', '','1');
  $wri->num('limit_access_log', 1, 999999);
  $wri->str('limit_access_log_lv', '0','1','2','debug');
  $wri->str('use_counter', '0','text', 'image');
  $wri->str('view_counter_pos', 'skin','bbstop','bbsbottom','msgtop','msgbottom');
  $wri->str('view_counter_mode', 'add','insert');
  $wri->num('limit_countup_min', 0, 999999);

  $in->{use_access_log} ||= 0;
  $bn->lo->dir_init if $in->{use_access_log};

  $wri->write(
    [
      [ 'UseAccessLog',        'use_access_log',       '$' ],
      [ 'LimitAccessLog',      'limit_access_log',     '$' ],
      [ 'LimitAccessLogLevel', 'limit_access_log_lv',  '$' ],
      [ 'UseCounter',          'use_counter',          '$' ],
      [ 'ViewCounterPos',      'view_counter_pos',     '$' ],
      [ 'ViewCounterMode',     'view_counter_mode',    '$' ],
      [ 'LimitCountUpMin',     'limit_countup_min',    '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_system_imagetools {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('use_gd', '','1');
  $wri->str('use_perlmagick', '','1');
  $wri->str('use_imagemagick', '','1');
  $wri->esc_len('imagemagick_path', 999);

  for ( qw/gd perlmagick imagemagick/ ) { $in->{'use_'.$_} ||= 0 }

  $wri->write(
    [
      [ 'UseGD',           'use_gd',           '$' ],
      [ 'UsePerlMagick',   'use_perlmagick',   '$' ],
      [ 'UseImageMagick',  'use_imagemagick',  '$' ],
      [ 'ImageMagickPath', 'imagemagick_path', '$' ],
    ]
  );

}
sub set_system_dir {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};

  $wri->esc_len('dir_bbsnote_url', 9999);
  $wri->esc_len('dir_images_url', 9999);
  $wri->esc_len('dir_images_path', 9999);
  $wri->esc_len('dir_data_url', 9999);
  $wri->esc_len('dir_data_path', 9999);
  $wri->esc_len('dir_index_path', 9999);
  $wri->esc_len('dir_sendmail_path', 9999);

  $wri->esc_len('dir_head_logs', 9);
  $wri->esc_len('dir_head_image', 9);
  $wri->esc_len('dir_head_anime', 9);
  $wri->esc_len('dir_head_thumb', 9);
  $wri->esc_len('dir_head_object', 9);


  for (qw/logs images anime thumb object/) {
    if ($bn->get($cf, 'FileHead' . ucfirst $_) ne $in->{'dir_head_'.$_}) {
      my $count = {};
      $bn->po->get_count($count);
      $wri->err('ERR_HEADS', $wri->{cms}{item}{dir_head}.':'.$wri->{cms}{item}{'dir_head_'.$_})
        unless $count->{log} =~ /^(?:$|0)/ && $count->{num} =~ /^(?:$|0)/;
    }
  }

  $wri->write(
    [
      [ 'BBSNoteUrl',     'dir_bbsnote_url',   '$' ],
      [ 'DirImagesUrl',   'dir_images_url',    '$' ],
      [ 'DirImagesPath',  'dir_images_path',   '$' ],
      [ 'DirFileUrl',     'dir_data_url',      '$' ],
      [ 'DirFilePath',    'dir_data_path',     '$' ],
      [ 'IndexFilePath',  'dir_index_path',    '$' ],
      [ 'SendMailPath',   'dir_sendmail_path', '$' ],
      [ 'FileHeadLogs',   'dir_head_logs',     '$' ],
      [ 'FileHeadImage',  'dir_head_image',    '$' ],
      [ 'FileHeadAnime',  'dir_head_anime',    '$' ],
      [ 'FileHeadThumb',  'dir_head_thumb',    '$' ],
      [ 'FileHeadObject', 'dir_head_object',   '$' ],
    ]
  );

  $bn->app->reflesh;
}
sub set_system_base {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sys = $bn->{sys};

  $wri->str('system_crypt_password', '','1');
  $wri->str('system_crypt_mail', '','1');
  $wri->esc_len('system_crypt_id', 2);
  $wri->str('system_meta_cookie', '','1');
  $wri->str('system_pragma', '','1');
  $wri->num('system_time_diff', -99999, 999999);

  for ( qw/crypt_password crypt_mail meta_cookie pragma/ ) {
    $in->{'system_'.$_} = 0 if $in->{'system_'.$_} eq '';
  }

  if ($in->{system_permission} =~ /0([0-7]{3})/) {
    my $a = $1;
    if ($a =~ /[2367]/) {
      $in->{system_permission} = $a;
    } else {
      $wri->err('ERR_PM', $wri->{cms}{item}{system_permission});
    }
  }

  $wri->write(
    [
      [ 'UseCrypt',      'system_crypt_password', '$' ],
      [ 'UseCryptKey',   'system_crypt_id',       '$' ],
      [ 'UseMailCrypt',  'system_crypt_mail',     '$' ],
      [ 'UseMetaCookie', 'system_meta_cookie',    '$' ],
      [ 'HttpPragma',    'system_pragma',         '$' ],
      [ 'TimeDiffSec',   'system_time_diff',      '$' ],
      [ 'Permission',    'system_permission',     '$' ],
    ]
  );

  $bn->cfg_init->boot_init(
    $sys->{Config},
    "$sys->{DirCfgSys}/$sys->{Skin}.$sys->{Exe}",
    $sys->{SkinCfg});
  unlink "$sys->{DirCfgSys}/boot.$sys->{Exe}";

  $bn->app->reflesh;
}

sub set_user_names {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $sk = $bn->{skin};
  my $mode = $in->{mode};
  my $item = $wri->{cms}{item};

  $wri->str('user_names_cap', '','1');
  $wri->str('user_names_trip', '','1');

  for ( qw/cap trip/ ) { $in->{'user_names_'.$_} ||= 0 }

  my %cf;
  $cf{cap} = $bn->get($sk, 'TagNameCap');
  $cf{trip} = $bn->get($sk, 'TagNameTrip');

  for ('string','color','style','bold','italic','under','teletype') {
    $item->{'cap_' . $_} = "$item->{user_names_cap}:$item->{$_}";
    $item->{'trip_' . $_} = "$item->{user_names_trip}:$item->{$_}";
  }

  for ('cap' , 'trip') {
    $item->{$_.'_size'} = "$item->{'user_names_'.$_}:$item->{font_size}";
    $item->{$_.'_face'} = "$item->{'user_names_'.$_}:$item->{font_face}";

    $wri->err('ERR_STR_NULL', $item->{$_.'_string'})
      if length $in->{$_.'_string'} < 1;

    $wri->esc_len($_.'_string', 99);
      $cf{$_}->{string} = $in->{$_.'_string'};

    $wri->color($_.'_color');
      $cf{$_}->{color} = $in->{$_.'_color'};

    $wri->num($_.'_size', 0, 99, 0);
      $cf{$_}->{size} = $in->{$_.'_size'};

    $wri->esc_len($_.'_face',99);
      $cf{$_}->{face} = $in->{$_.'_face'};

    $wri->sty($_.'_style',99);
      $cf{$_}->{style} = $in->{$_.'_style'};

    $wri->str($_.'_bold', '', '1');
      $cf{$_}->{bold} = $in->{$_.'_bold'};

    $wri->str($_.'_italic', '', '1');
      $cf{$_}->{italic} = $in->{$_.'_italic'};

    $wri->str($_.'_under', '', '1');
      $cf{$_}->{under} = $in->{$_.'_under'};

    $wri->str($_.'_teletype', '', '1');
      $cf{$_}->{teletype} = $in->{$_.'_teletype'};

  }

  return if @{ $wri->{cms}{error} };

  $in->{'TagNameCap'} = $cf{cap};
  $in->{'TagNameTrip'} = $cf{trip};

  $wri->write(
    [
      [ 'UseNameCap',  'user_names_cap',  '$' ],
      [ 'UseNameTrip', 'user_names_trip', '$' ],
    ],[
      [ 'TagNameCap',  'TagNameCap',    '%' ],
      [ 'TagNameTrip', 'TagNameTrip',   '%' ],
    ]
  );

  $bn->app->reflesh;
}

sub set_user_profile {
  my $wri = $_[0];

  $wri->esc_len('user_admin_name', 9999);
  $wri->esc_len('user_admin_mail', 9999);
  $wri->esc_len('user_admin_site', 9999);
  $wri->esc_len('user_admin_url', 9999);
  $wri->esc_len('user_admin_comment', 9999);
  $wri->str('user_admin_anonymous', '', '1');

  $wri->write(
    [
      [ 'AdminName',      'user_admin_name',      '$' ],
      [ 'AdminMail',      'user_admin_mail',      '$' ],
      [ 'AdminSiteName',  'user_admin_site',      '$' ],
      [ 'AdminSiteUrl',   'user_admin_url',       '$' ],
      [ 'AdminComment',   'user_admin_comment',   '$' ],
      [ 'AdminAnonymous', 'user_admin_anonymous', '$' ],
    ]
  );

}
sub set_user_password {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};

  $item->{ad_pass} = $item->{password_admin};
  $item->{sb_pass} = $item->{password_sub};
  $wri->len('ad_pass', 9999);
  $wri->len('sb_pass', 9999);

  my %ap;
  my @a;
  for (split /(\r|\n|,)+/, $in->{ad_pass}) {
    chomp;
    next if $_ eq '';
    my($pass, $name) = split /:/,$_, 2;
    $pass =~ s/\W//g;
    $pass = sprintf "%.8s", $pass;
    push @a, ($name eq '' ? $pass : "$pass:$name")
  }
  $wri->err('ERR_NEED', $wri->{cms}{item}{password_admin}) unless @a;
  $in->{AdminPassword} = \@a;
  my @b;
  for (split /(\r|\n|,)+/, $in->{sb_pass}) {
    chomp;
    next if $_ eq '';
    my($pass, $name) = split /:/,$_, 2;
    $pass =~ s/\W//g;
    $pass = sprintf "%.8s", $pass;
    push @b, ($name eq '' ? $pass : "$pass:$name")
  }
  $in->{SubPassword} = \@b;

  $wri->write(
    [
      [ 'AdminPassword', 'AdminPassword', '@' ],
      [ 'SubPassword',   'SubPassword',   '@' ],
    ]
  );

   unless ($bn->check_authority($in->{password})) {
     my $ck = $bn->{cookie};
     my @b = split /:/, $bn->get($cf, 'AdminPassword')->[0], 2;
     $in->{password} = $ck->{adminpass} = $b[0];
     $in->{name} = $ck->{adminuser} = $b[1] if defined $b[1];
     $bn->write_cookie($ck);
   }

}

sub set_paint_palette {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};

  my $path = "$bn->{sys}{DirSys}/palette.$bn->{sys}{Exe}";
  my($fh_tmp, $tmp) = $bn->get_temp_file;

  for (split /\n+/, $in->{palettes}) {
    my($key, $name, @colors) = split /\t/;
    $key =~ s/\W//g;
    $name =~ s/,//g;
    @colors = map { $_ = sprintf "%06s", $1 if /\#?([\da-fA-F]+)/ } @colors;
    print $fh_tmp (join ',', $key,$name,@colors) . "\n";
  }

  close $fh_tmp;

  rename $tmp, $path
    or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $path);

  $bn->app->reflesh;
}

sub set_paint_apple_pai {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_pai = $bn->get($cf, 'ConfigBBSPainter');

  $wri->len('applet_archive', 999)->set_hash(
    $in, $cfg_pai, 'applet_archive', 'archive');
  $wri->len('applet_code', 999)->set_hash(
    $in, $cfg_pai, 'applet_code', 'code');
  $wri->len('applet_codebase', 999)->set_hash(
    $in, $cfg_pai, 'applet_codebase', 'applet_codebase');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigBBSPainter} = join ',', map { "$_:$cfg_pai->{$_}" } keys %$cfg_pai;
  $bn->define($cf, 'ConfigBBSPainter', undef, '$');

  $wri->write(
    [
      [ 'ConfigBBSPainter', 'ConfigBBSPainter', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigBBSPainter');

  $bn->app->reflesh;
}
sub set_paint_apple_pic {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_pic = $bn->get($cf, 'ConfigPictureBBS');

  $wri->len('applet_archive', 999)->set_hash(
    $in, $cfg_pic, 'applet_archive', 'archive');
  $wri->len('applet_code', 999)->set_hash(
    $in, $cfg_pic, 'applet_code', 'code');
  $wri->len('applet_codebase', 999)->set_hash(
    $in, $cfg_pic, 'applet_codebase', 'applet_codebase');

  $wri->color('applet_gradation_begin')->set_hash(
    $in, $cfg_pic, 'applet_gradation_begin', 'gradation_begin');
  $wri->color('applet_gradation_end')->set_hash(
    $in, $cfg_pic, 'applet_gradation_end', 'gradation_end');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigPictureBBS} = join ',', map { "$_:$cfg_pic->{$_}" } keys %$cfg_pic;
  $bn->define($cf, 'ConfigPictureBBS', undef, '$');

  $wri->write(
    [
      [ 'ConfigPictureBBS', 'ConfigPictureBBS', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigPictureBBS');

  $bn->app->reflesh;
}

sub set_paint_apple_spv {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_spv = $bn->get($cf, 'ConfigShiPCHViewer');

  $wri->len('applet_archive', 999)->set_hash(
    $in, $cfg_spv, 'applet_archive', 'archive');
  $wri->len('applet_code', 999)->set_hash(
    $in, $cfg_spv, 'applet_code', 'code');
  $wri->len('applet_codebase', 999)->set_hash(
    $in, $cfg_spv, 'applet_codebase', 'applet_codebase');
  $wri->len('applet_resource', 999)->set_hash(
    $in, $cfg_spv, 'applet_resource', 'resource');
  $wri->len('applet_texture', 999)->set_hash(
    $in, $cfg_spv, 'applet_texture', 'texture');
  $wri->len('applet_res', 999)->set_hash(
    $in, $cfg_spv, 'applet_res', 'res');
  $wri->num('applet_layer_count', 1, 999, 1)->set_hash(
    $in, $cfg_spv, 'applet_layer_count', 'layer_count');
  $wri->num('applet_tt_size', 1, 999, 1)->set_hash(
    $in, $cfg_spv, 'applet_tt_size', 'tt_size');
  $in->{applet_run} ||= 0;
  $wri->str('applet_run', '0', '1')->set_hash(
    $in, $cfg_spv, 'applet_run', 'run');
  $wri->num('applet_speed', -1, 99999)->set_hash(
    $in, $cfg_spv, 'applet_speed', 'speed');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigShiPCHViewer} = join ',', map { "$_:$cfg_spv->{$_}" } keys %$cfg_spv;
  $bn->define($cf, 'ConfigShiPCHViewer', undef, '$');

  $wri->write(
    [
      [ 'ConfigShiPCHViewer', 'ConfigShiPCHViewer', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigShiPCHViewer');

}

sub set_paint_apple_sii {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_shi = $bn->get($cf, 'ConfigShiPaintBBS');

  $wri->len('applet_archive', 999)->set_hash(
    $in, $cfg_shi, 'applet_archive', 'archive');
  $wri->len('applet_code', 999)->set_hash(
    $in, $cfg_shi, 'applet_code', 'code');
    $cfg_shi->{code} = $in->{applet_code};
  $wri->len('applet_codebase', 999)->set_hash(
    $in, $cfg_shi, 'applet_codebase', 'applet_codebase');
  $wri->len('applet_anime', 999)->set_hash(
    $in, $cfg_shi, 'applet_anime', 'anime_ext');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigShiPaintBBS} = join ',', map { "$_:$cfg_shi->{$_}" } keys %$cfg_shi;
  $bn->define($cf, 'ConfigShiPaintBBS', undef, '$');

  $wri->write(
    [
      [ 'ConfigShiPaintBBS', 'ConfigShiPaintBBS', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigShiPaintBBS');

  $bn->app->reflesh;
}

sub set_paint_apple_spa {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_spa = $bn->get($cf, 'ConfigShiPainter');

  $wri->len('applet_archive', 999)->set_hash(
    $in, $cfg_spa, 'applet_archive', 'archive');
  $wri->len('applet_code', 999)->set_hash(
    $in, $cfg_spa, 'applet_code', 'code');
  $wri->len('applet_codebase', 999)->set_hash(
    $in, $cfg_spa, 'applet_codebase', 'applet_codebase');
  $wri->len('applet_anime', 999)->set_hash(
    $in, $cfg_spa, 'applet_anime', 'anime_ext');
  $wri->len('applet_resource', 999)->set_hash(
    $in, $cfg_spa, 'applet_resource', 'resource');
  $wri->len('applet_texture', 999)->set_hash(
    $in, $cfg_spa, 'applet_texture', 'texture');
  $wri->len('applet_pro', 999)->set_hash(
    $in, $cfg_spa, 'applet_pro', 'pro');
  $wri->len('applet_normal', 999)->set_hash(
    $in, $cfg_spa, 'applet_normal', 'normal');
  $wri->num('applet_layer_count', 1, 999, 1)->set_hash(
    $in, $cfg_spa, 'applet_layer_count', 'layer_count');
  $wri->len('applet_quality', 1, 99, 1)->set_hash(
    $in, $cfg_spa, 'applet_quality', 'quality');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigShiPainter} = join ',', map { "$_:$cfg_spa->{$_}" } keys %$cfg_spa;
  $bn->define($cf, 'ConfigShiPainter', undef, '$');

  $wri->write(
    [
      [ 'ConfigShiPainter', 'ConfigShiPainter', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigShiPainter');

  $bn->app->reflesh;
}


sub set_paint_apple_base {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_shi = $bn->get($cf, 'ConfigShiApplet');

  for (qw/image_jpeg send_header_timer send_header_count
    tool_advance jp dyn_color send_advance dyn_palette/)
      { $in->{'applet_' . $_ } ||= 0 }

  $wri->num('applet_image_size', 0, 999, 1)->set_hash(
    $in, $cfg_shi, 'applet_image_size', 'image_size');
  $wri->num('applet_compress_level', 1, 999, 1)->set_hash(
    $in, $cfg_shi, 'applet_compress_level', 'compress_level');
  $wri->num('applet_thumbnail_compress_level', 1, 999, 1)->set_hash(
    $in, $cfg_shi, 'applet_thumbnail_compress_level', 'thumbnail_compress_level');
  $wri->color('applet_image_bkcolor')->set_hash(
    $in, $cfg_shi, 'applet_image_bkcolor', 'image_bkcolor');
  $wri->num('applet_undo', 1, 999, 1)->set_hash(
    $in, $cfg_shi, 'applet_undo', 'undo');
  $wri->num('applet_undo_in_mg', 1, 999, 1)->set_hash(
    $in, $cfg_shi, 'applet_undo_in_mg', 'undo_in_mg');
  $wri->str('applet_image_jpeg', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_image_jpeg', 'image_jpeg');
  $wri->str('applet_send_header_timer', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_send_header_timer', 'send_header_timer');
  $wri->str('applet_send_header_count', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_send_header_count', 'send_header_count');
  $wri->str('applet_tool_advance', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_tool_advance', 'tool_advance');
  $wri->str('applet_jp', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_jp', 'jp');
  $wri->str('applet_dyn_palette', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_dyn_palette', 'dynamic_palette');
  $wri->str('applet_dyn_color', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_dyn_color', 'dynamic_pcolor');
  $wri->str('applet_send_advance', '0','1')->set_hash(
    $in, $cfg_shi, 'applet_send_advance', 'send_advance');
  $wri->num('applet_security_click', 0, 9999, 1)->set_hash(
    $in, $cfg_shi, 'applet_security_click', 'security_click');
  $wri->num('applet_security_timer', 0, 999999, 1)->set_hash(
    $in, $cfg_shi, 'applet_security_timer', 'security_timer');
  $wri->len('applet_security_url', 999)->set_hash(
    $in, $cfg_shi, 'applet_security_url', 'security_url');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigShiApplet} = join ',', map { "$_:$cfg_shi->{$_}" } keys %$cfg_shi;
  $bn->define($cf, 'ConfigShiApplet', undef, '$');

  $wri->write(
    [
      [ 'ConfigShiApplet', 'ConfigShiApplet', '$' ],
    ]
  );
  $bn->cfg2hash($cf, 'ConfigShiApplet');
  $bn->app->reflesh;

}

sub set_paint_apple {
  my $wri = $_[0];
  my $cf = $bn->{cfg};
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $cfg_app = $bn->get($cf, 'ConfigApplet');

  $wri->len('applet_use', 999);
  $wri->len('applet_setting', 9999);
  $in->{applet_setting} =~ s/(\r|\n)+/,/g;

  my(@key, @cfg);
  for (split /;/, $in->{applet_setting}) {
    if (s/(\w+)=//) {
      my @var;
      $in->{'AppletSetting:' . $1} = $_;
      push @key, 'AppletSetting:' . $1;
      push @var, 'AppletSetting:' . $1;
      push @var, 'AppletSetting:' . $1;
      push @var, '$';
      push @cfg, \@var;
    }
  }

  $in->{applet_cgi_stream} ||= 0;
  $wri->str('applet_cgi_stream', '0','1')->set_hash(
    $in, $cfg_app, 'applet_cgi_stream', 'cgi_stream');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigApplet} = join ',', map { "$_:$cfg_app->{$_}" } keys %$cfg_app;
  $bn->define($cf, 'UseApplet', undef, '$');
  $bn->define($cf, 'ConfigApplet', undef, '$');
  map {$bn->define($cf, $_, undef, '$')} @key;

  $wri->write(
    [
      [ 'UseApplet',  'applet_use',   '$' ],
      [ 'ConfigApplet', 'ConfigApplet', '$' ],
      @cfg
    ]
  );

  $bn->cfg2hash($cf, 'ConfigApplet');
  map {$bn->cfg2hash($cf, $_)} @key;
  $bn->define($cf, 'UseApplet', undef, '@');
  for (split /,/, $in->{applet_use}) { $bn->set($cf, 'UseApplet', $_) }

  $bn->app->reflesh;
}
sub set_paint_base {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $item = $wri->{cms}{item};
  my $cfg_shi = $bn->get($cf, 'ConfigShiApplet');

  for ('width','height') {
    $item->{'paint_canvace_default_' . $_} = "$item->{paint_canvace_default}:$item->{$_}";
    $item->{'paint_canvace_select_min_' . $_} = "$item->{paint_canvace_select_min}:$item->{$_}";
    $item->{'paint_canvace_select_max_' . $_} = "$item->{paint_canvace_select_max}:$item->{$_}";
    $item->{'paint_canvace_select_var_' . $_} = "$item->{paint_canvace_select_var}:$item->{$_}";
  }

  if ($wri->{disable}{paint_pool}) {
    $wri->{disable}{paint_pool_max} = 1;
    $wri->{disable}{paint_pool_time} = 1;
  }

  $wri->str('paint_canvace_mode', 'text','fix','select');
  $wri->num('paint_canvace_default_width', 1, 9999, 1);
  $wri->num('paint_canvace_default_height', 1, 9999, 1);
  $wri->num('paint_canvace_select_min_width', 1, 9999, 1);
  $wri->num('paint_canvace_select_min_height', 1, 9999, 1);
  $wri->num('paint_canvace_select_max_width', 1, 9999, 1);
  $wri->num('paint_canvace_select_max_height', 1, 9999, 1);
  $wri->num('paint_canvace_select_var_width', 1, 9999, 1);
  $wri->num('paint_canvace_select_var_height', 1, 9999, 1);
  $wri->str('paint_anime', 'user','true','false')->set_hash(
    $in, $cfg_shi, 'paint_anime', 'animation');
  $in->{paint_anime_default} ||= 0;
  $wri->str('paint_anime_default', '0','1')->set_hash(
    $in, $cfg_shi, 'paint_anime_default', 'default_anime');
  $wri->str('paint_anime_dl_type', 'auto','get','post','cgi')->set_hash(
    $in, $cfg_shi, 'paint_anime_dl_type', 'anime_dl_type');
  $wri->str('paint_anime_limit', '0','1','2');
  $wri->str('paint_continue', '0','1','2','3');
  $wri->str('paint_continue_pass', '0','1','2');
  $in->{paint_continue_history} ||= 0;
  $wri->str('paint_continue_history', '0','1','2');
  $in->{paint_password_auth} ||= 0;
  $wri->str('paint_password_auth', '0','1');
  $wri->str('paint_continue_conv', '','1');
  $wri->str('paint_data_over_conv', '','1');
  $wri->num('paint_pool_max', 1, 999, 1);
  $wri->num('paint_pool_time', 1, 99999, 1);
  $wri->num('paint_palette_view', 1, 999, 1);
  $wri->str('paint_applet_fit', '','1');

  return if @{ $wri->{cms}{error} };

  $in->{ConfigShiApplet} = join ',', map { "$_:$cfg_shi->{$_}" } keys %$cfg_shi;
  $bn->define($cf, 'ConfigShiApplet', undef, '$');

  for ( qw/anime_default continue_pass continue_conv data_over_conv applet_fit/ ) {
    $in->{'paint_'.$_} ||= 0;
  }

  $wri->write(
    [
      [ 'CanvasMode',               'paint_canvace_mode',              '$' ],
      [ 'DefaultCanvasWidth',       'paint_canvace_default_width',     '$' ],
      [ 'DefaultCanvasHeight',      'paint_canvace_default_height',    '$' ],
      [ 'CanvasSelectMinWidth',     'paint_canvace_select_min_width',  '$' ],
      [ 'CanvasSelectMinHeight',    'paint_canvace_select_min_height', '$' ],
      [ 'CanvasSelectMaxWidth',     'paint_canvace_select_max_width',  '$' ],
      [ 'CanvasSelectMaxHeight',    'paint_canvace_select_max_height', '$' ],
      [ 'CanvasSelectVarWidth',     'paint_canvace_select_var_width',  '$' ],
      [ 'CanvasSelectVarHeight',    'paint_canvace_select_var_height', '$' ],
      [ 'ConfigShiApplet',          'ConfigShiApplet',                 '$' ],
      [ 'UsePaintContinue',         'paint_continue',                  '$' ],
      [ 'UsePaintContinuePassword', 'paint_continue_pass',             '$' ],
      [ 'UsePaintContinueConvert',  'paint_continue_conv',             '$' ],
      [ 'UsePaintDataOverConvert',  'paint_data_over_conv',            '$' ],
      [ 'PaintPoolMaxData',         'paint_pool_max',                  '$' ],
      [ 'PaintPoolMaxTime',         'paint_pool_time',                 '$' ],
      [ 'LimitViewPaintAnime',      'paint_anime_limit',               '$' ],
      [ 'ViewLimitPaletteList',     'paint_palette_view',              '$' ],
      [ 'ViewPaintContinueHistory', 'paint_continue_history',          '$' ],
      [ 'PaintPasswordAuth',        'paint_password_auth',             '$' ],
      [ 'UseAppletFit',             'paint_applet_fit',                '$' ],
    ]
  );

  $bn->cfg2hash($cf, 'ConfigShiApplet');
  $bn->app->reflesh;

}

sub set_post_icon {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};

  for ('width','height') {
    $item->{'post_icon_limit_size_' . $_} = "$item->{post_icon_limit_size}:$item->{$_}";
  }

  if (exists $wri->{disable}{post_icon_limit_size}) {
    $wri->{disable}{post_icon_limit_size_width} = 1;
    $wri->{disable}{post_icon_limit_size_height} = 1;
  }

  $in->{post_icon_window} =~ s/[^\w\,\=]//g;

  $wri->str('post_icon_use', '0','1','2','3');
  $wri->str('post_icon_ex', '','1');
  $wri->str('post_icon_regist_allow', '0','1','2');
  $wri->str('post_icon_preview', '','1');
  $wri->num('post_icon_limit_post', 1, 9999, 1);
  $wri->num('post_icon_limit_size_width', 1, 999, 1);
  $wri->num('post_icon_limit_size_height', 1, 999, 1);
  $wri->num('post_icon_limit_data', 1, 9999999, 1);
  $wri->num('post_icon_limit_msg', 1, 9999, 1);
  $wri->len('post_icon_window', 999);

  for ( qw/ex preview/ ) { $in->{'post_icon_'.$_} ||= 0 }

  $wri->write(
    [
      [ 'UseIcon',           'post_icon_use',               '$' ],
      [ 'UseIconEx',         'post_icon_ex',                '$' ],
      [ 'UseIconPreview',    'post_icon_preview',           '$' ],
      [ 'IconViewForce',     'post_icon_view_force',        '$' ],
      [ 'IconRegist',        'post_icon_regist_allow',      '$' ],
      [ 'LimitIcon',         'post_icon_limit_post',        '$' ],
      [ 'LimitIconWidth',    'post_icon_limit_size_width',  '$' ],
      [ 'LimitIconHeight',   'post_icon_limit_size_height', '$' ],
      [ 'LimitIconSize',     'post_icon_limit_data',        '$' ],
      [ 'LimitIconComment',  'post_icon_limit_msg',         '$' ],
      [ 'IconPreviewWindow', 'post_icon_window',            '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_post_notice {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('post_notice_use', '','1');
  $wri->str('post_notice_all', '','1');
  $wri->len('post_notice_format', 9999);

  my @a = split /\r?\n/, $in->{post_notice_format};
  $in->{post_notice_format} = \@a;
  for ( qw/use all/ ) { $in->{'post_notice_'.$_} ||= 0 }
  $wri->write(
    [
      [ 'UseNoticeMail',  'post_notice_use',  '$' ],
      [ 'UseAllPostNotice', 'post_notice_all',  '$' ],
      [ 'NoticeMailFormat', 'post_notice_format', '@' ],
    ]
  );
}

sub set_post_panic {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('post_panic_use', '0','1','2');
  $wri->len('post_panic_color', 385);
  my @a; map { chomp;
   if( /\#?([\da-fA-F]+)/){ my $c = $1;
    for (my $n = length $c; $n < 6; $n++) { $c .= '0' }
    push @a, sprintf "#%06s", $c }
  } split /\s/, $in->{post_panic_color};
  $in->{post_panic_color} = join ',', @a;
  $in->{post_panic_use} ||= 0;

  $wri->write(
    [
      [ 'UsePanic',   'post_panic_use',   '$' ],
      [ 'PanicColor', 'post_panic_color', '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_post_font_color {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('post_font_color_use', '','1');
  $wri->len('post_font_color_font', 385);
  $wri->len('post_font_color_title', 385);
  $wri->len('post_font_color_table', 385);

  my @a; map { chomp;
   if( /\#?([\da-fA-F]+)/){ my $c = $1;
    for (my $n = length $c; $n < 6; $n++) { $c .= '0' }
    push @a, sprintf "#%06s", $c }
  } split /\s/, $in->{post_font_color_font};
  $in->{post_font_color_font} = join ',', @a;

  undef @a; map { chomp;
   if( /\#?([\da-fA-F]+)/){ my $c = $1;
    for (my $n = length $c; $n < 6; $n++) { $c .= '0' }
    push @a, sprintf "#%06s", $c }
  } split /\s/, $in->{post_font_color_title};
  $in->{post_font_color_title} = join ',', @a;

  undef @a; map { chomp;
   if( /\#?([\da-fA-F]+)/){ my $c = $1;
    for (my $n = length $c; $n < 6; $n++) { $c .= '0' }
    push @a, sprintf "#%06s", $c }
  } split /\s/, $in->{post_font_color_table};
  $in->{post_font_color_table} = join ',', @a;

  $in->{post_font_color_use} ||= 0;

  $wri->write(
    [
      [ 'UseColor',                 'post_font_color_use',   '$' ],
      [ 'ColorSetting:font_color',  'post_font_color_font',  '$' ],
      [ 'ColorSetting:title_color', 'post_font_color_title', '$' ],
      [ 'ColorSetting:table_color', 'post_font_color_table', '$' ],
    ]
  );
  $bn->app->reflesh;
}
sub set_post_tag {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('post_tag_use', '','1');
  $wri->str('post_tag_all', '0','1','2','3');
  $wri->len('post_tag_ok', 999);
  my @a; map { chomp; s/[^a-zA-Z:]//g; push @a, $_ if $_ } split /\n/, $in->{post_tag_ok};
  $in->{post_tag_ok} = \@a;
  $in->{post_tag_use} ||= 0;

  $wri->write(
    [
      [ 'UseTag',  'post_tag_use', '$' ],
      [ 'OkTag',   'post_tag_ok',  '@' ],
      [ 'UseAllTag', 'post_tag_all', '$' ],
    ]
  );
}

sub set_post_upload {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  if (exists $wri->{disable}{post_file_ex}) {
    for (qw/post_file_text post_file_bmp post_file_midi
      post_file_swf post_file_zip post_file_lzh/)
      { $wri->{disable}{$_} = 1 }
  }
  if (exists $wri->{disable}{post_thumbnail_use}) {
    for (qw/post_thumbnail_size_width post_thumbnail_size_height/)
      { $wri->{disable}{$_} = 1 }
  }
  for ('width','height') {
    $item->{'post_thumbnail_size_' . $_} = "$item->{limit_image_size}:$item->{$_}";
  }

  $wri->str('post_upload_new', '0','1','2','3');
  $wri->str('post_upload_reply', '0','1','2','3');
#  $wri->str('post_upload_ex', '0','1','2','3');
#  $wri->str('post_upload_local', '0','1','2','3');

  $wri->str('post_thumbnail_use', '','1');
  $wri->num('post_thumbnail_size_width', 1, 9999, 1);
  $wri->num('post_thumbnail_size_height', 1, 9999, 1);

  $wri->str('post_file_text', '','1');
  $wri->str('post_file_bmp', '','1');
  $wri->str('post_file_midi', '','1');
  $wri->str('post_file_swf', '','1');
  $wri->str('post_file_zip', '','1');
  $wri->str('post_file_lzh', '','1');

  $in->{post_thumbnail_use} ||= 0;

  for (qw/text bmp midi swf zip lzh/) {
    $bn->{in}{'post_file_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'UseUpload',       'post_upload_new',            '$' ],
      [ 'UseReplyUpload',  'post_upload_reply',          '$' ],
      [ 'UseThumbnail',    'post_thumbnail_use',         '$' ],
      [ 'ThumbnailWidth',  'post_thumbnail_size_width',  '$' ],
      [ 'ThumbnailHeight', 'post_thumbnail_size_height', '$' ],
      [ 'UseFileExText',   'post_file_text',             '$' ],
      [ 'UseFileExBmp',    'post_file_bmp',              '$' ],
      [ 'UseFileExMidi',   'post_file_midi',             '$' ],
      [ 'UseFileExSwf',    'post_file_swf',              '$' ],
      [ 'UseFileExZip',    'post_file_zip',              '$' ],
      [ 'UseFileExLzh',    'post_file_lzh',              '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_post_base {
  my $wri = $_[0];

  $wri->str('post_auto_link', '','1');
  $wri->str('post_auto_mail', '','1');
  $wri->str('post_quotation', '','1');
  $wri->str('post_index', '','1');
  $wri->str('post_res_title', '','1');
  $wri->str('post_edit_date', '0','1','2','3');

  for (qw/auto_link auto_mail quotation index res_title post_keep_date/) {
    $bn->{in}{'post_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'UseAutoLinkUrl',  'post_auto_link', '$' ],
      [ 'UseAutoLinkMail', 'post_auto_mail', '$' ],
      [ 'UseQuotation',  'post_quotation', '$' ],
      [ 'UseRefleshIndex', 'post_index',   '$' ],
      [ 'UseReplyTitle',   'post_res_title', '$' ],
      [ 'UseEditDate',   'post_edit_date', '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_limit_filter {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->len('limit_filter_string', 9999);
  $wri->str('limit_filter_address', '','1');

  my @a = grep{ $_ ? $_ : '' } split /\r?\n/, $in->{limit_filter_string};
  
  $in->{limit_filter_string} = \@a;
  $in->{limit_filter_spam} ||= 0;
  $in->{limit_filter_en_chr} ||= 0;
  $in->{limit_filter_address} ||= 0;

  $wri->write(
    [
      [ 'LimitFilterString',  'limit_filter_string',  '@' ],
      [ 'LimitFilterSpam',    'limit_filter_spam',    '$' ],
      [ 'LimitFilterEnChr',   'limit_filter_en_chr',  '$' ],
      [ 'LimitFilterAddress', 'limit_filter_address', '$' ],
    ]
  );

}

sub set_limit_securty {
  my $wri = $_[0];
  my $in = $bn->{in};

  $wri->str('limit_security_proxy', '1','2','3','4');

  for ('limit_security_deny_ip', 'limit_security_deny_host',
     'limit_security_accept_ip', 'limit_security_accept_host',) {
    $in->{$_} =~ s/\/|\\|\;|\$\#//g;
    $wri->len($_,  999);
  }

  $wri->esc_len('limit_security_accept_ref', 999);
  $wri->str('limit_security_deny_raw_ip', '','1');
  $wri->str('limit_security_bookmark', '','1');
  $wri->str('limit_no_robots', '','1');

  for ( qw/deny_raw_ip bookmark/ )
   { $bn->{in}{'limit_security_' . $_} ||= 0 }
  $bn->{in}{limit_no_robots} ||= 0;

  $wri->write(
    [
      [ 'LimitSecurityProxy',      'limit_security_proxy',       '$' ],
      [ 'LimitSecurityDenyIp',     'limit_security_deny_ip',     '$' ],
      [ 'LimitSecurityDenyHost',   'limit_security_deny_host',   '$' ],
      [ 'LimitSecurityAcceptIp',   'limit_security_accept_ip',   '$' ],
      [ 'LimitSecurityAcceptHost', 'limit_security_accept_host', '$' ],
      [ 'LimitSecurityAcceptRef',  'limit_security_accept_ref',  '$' ],
      [ 'LimitSecurityDenyRawIp',  'limit_security_deny_raw_ip', '$' ],
      [ 'LimitSecurityBookMark',   'limit_security_bookmark',    '$' ],
      [ 'LimitNoRobots',           'limit_no_robots',            '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_limit_post_multi {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $m = 'limit_post_multi_';
  my $r = 'limit_post_range_';

  my $n = 0;
  my( @vars, @add_vars );
  my @c = ( 'new', 'reply', 'edit', 'paint' );
  my @e = ( 'ip', 'name', 'message', 'title', 'file_size' );
  for my $a ( @c ) {
    for ( @e ) { $wri->str($m.$a.'_'.$_,'','1');
      $item->{$m.$a.'_'.$_} = $item->{$m.$a}.':'.$item->{$m.$_};
      $in->{$m.$a.'_'.$_} ||= 0 }
    $wri->num($m.$a.'_time', 0, 999999, 0);
    $item->{$m.$a.'_time'.$_} = $item->{$m.$a}.':'.$item->{$m.'time'};
    if ( $a eq 'new' ) { push @e, 'num' }
      else{ $item->{$m.$a.'_num'.$_} = $item->{$m.$a}.':'.$item->{$m.'num'} }
  }

  $wri->num($m.'history', 1, 999, 1);
  $wri->num($m.'log', 1, 999, 1);

  if (exists $wri->{limit_num}{limit_post_multi}) {
    my $l = $wri->{limit_num}{limit_post_multi};
    for my $mode (qw/new reply edit paint/) {
      my $flag;
      for (qw/ip name message title file_size num/) {
        $flag++ if $in->{$m.$mode.'_'.$_} <= $l->{$_} }
      $flag++ if $in->{$m.$mode.'_time'} < $l->{'time'};
      $wri->err( 'ERR_POST_MULTI', $item->{$m.$mode}, $l->{'time'} ) if $flag > 6;
    }
  }

  @e = qw/WRITE_POST WRITE_REPLY EDIT WRITE_PAINT_CAPTOR/;
  for my $a ( @c ) {
    my @val; push @val, 'mode:'.$e[$n];
    for ('ip','name','message','title','file_size','num','time' ) {
      push @val, $_.':'.$in->{$m.$a.'_'.$_} if $in->{$m.$a.'_'.$_} }
    push @vars, join ';', @val;
    $n++;
  }
  $in->{LimitPostMulti} = \@vars;

  my $l = $r; chop $l;
  my @cfg;
  for ('new','reply') {
    $item->{$r.$_.'_day'}   = $item->{$l}.':'.$item->{$m.$_}.'('.$item->{$r.'time'}.' '.$item->{day}.')';
    $item->{$r.$_.'_hour'}  = $item->{$l}.':'.$item->{$m.$_}.'('.$item->{$r.'time'}.' '.$item->{hour}.')';
    $item->{$r.$_.'_log'}   = $item->{$l}.':'.$item->{$m.$_}.'('.$item->{$r.'count'}.')';
    $item->{$r.$_.'_guest'} = $item->{$l}.':'.$item->{$m.$_}.'('.$item->{$r.'guest'}.')';
    delete $in->{$r.$_.'_day'} if $in->{$r.$_.'_day'} == 0;
    delete $in->{$r.$_.'_hour'} if $in->{$r.$_.'_hour'} == 0;
    delete $in->{$r.$_.'_log'} if $in->{$r.$_.'_log'} == 0;
    $wri->err( 'ERR_POST_RANGE', $item->{$l}.':'.$item->{$m.$_} ) if $in->{$r.$_.'_day'} && $in->{$r.$_.'_hour'};
    $wri->num( $r.$_.'_day',   1, 30 );
    $wri->num( $r.$_.'_hour',  1, 168 );
    $wri->num( $r.$_.'_log',   1, 30 );
    $wri->str( $r.$_.'_guest','', '1');
    my @c; for my $b ( 'day', 'hour', 'log', 'guest' )
      { push @c, "$b=".$in->{$r.$_.'_'.$b} if $in->{$r.$_.'_'.$b} }
    push @cfg, $_.':'. join ',', @c if @c;
  }
  if ( @cfg ) {
    $in->{'LimitPostRange'} = join ';', @cfg;
    push @add_vars, [ 'LimitPostRange', 'LimitPostRange', '$' ];
  }

  $wri->write(
    [
      [ 'LimitPostMultiHistory', $m.'history',     '$' ],
      [ 'LimitPostMultiLog',     $m.'log',         '$' ],
      [ 'LimitPostMulti',        'LimitPostMulti', '@' ],
      @add_vars
    ]
  );
}
sub set_limit_post {
  my $wri = $_[0];

  $wri->str('limit_post_new', '0','1','2');
  $wri->str('limit_post_reply', '0','1','2');
  $wri->str('limit_post_edit', '0','1','2');
  $wri->str('limit_post_delete', '0','1','2');
  $wri->str('limit_post_paint', '0','1','2');
  $wri->str('limit_view_post_new', '0','1','2');
  $wri->str('limit_view_post_reply', '0','1','2');
  $wri->str('limit_view_post_edit', '0','1','2');
  $wri->str('limit_view_post_delete', '0','1','2');
  $wri->str('limit_view_post_paint', '0','1','2');
  $wri->str('limit_view_post_upload', '0','2');

  $wri->write(
    [
      [ 'LimitPostNew',        'limit_post_new',         '$' ],
      [ 'LimitPostReply',      'limit_post_reply',       '$' ],
      [ 'LimitPostEdit',       'limit_post_edit',        '$' ],
      [ 'LimitPostDelete',     'limit_post_delete',      '$' ],
      [ 'LimitPostPaint',      'limit_post_paint',       '$' ],
      [ 'LimitViewPostNew',    'limit_view_post_new',    '$' ],
      [ 'LimitViewPostReply',  'limit_view_post_reply',  '$' ],
      [ 'LimitViewPostEdit',   'limit_view_post_edit',   '$' ],
      [ 'LimitViewPostDelete', 'limit_view_post_delete', '$' ],
      [ 'LimitViewPostPaint',  'limit_view_post_paint',  '$' ],
      [ 'LimitViewPostUpload', 'limit_view_post_upload', '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_limit_listen {
  my $wri = $_[0];
  my $item = $wri->{cms}{item};

  for ('width','height') {
    $item->{'limit_image_size_' . $_} = "$item->{limit_image_size}:$item->{$_}";
    $item->{'limit_image_view_' . $_} = "$item->{limit_image_view}:$item->{$_}";
  }

  $wri->num('limit_log', 1, 30000, 1);
  $wri->num('limit_log_size', 1, 9999999, 1);
  $wri->num('limit_res_total', 0, 9999999, 1);
  $wri->str('limit_res', '0','1','2','3');
  $wri->str('limit_log_removed', '0','1');
  $wri->num('limit_image_size_width', 1, 9999, 1);
  $wri->num('limit_image_size_height', 1, 9999, 1);
  $wri->num('limit_image_view_width', 1, 9999, 1);
  $wri->num('limit_image_view_height', 1, 9999, 1);
  $wri->num('limit_data_upload', 1, 99999999, 1);
  $wri->num('limit_data_paint', 1, 9999999, 1);
  $wri->num('limit_data_anime', 1, 9999999, 1);
  $wri->num('limit_data_thumbnail', 1, 999999, 1);
  $wri->str('limit_thread_lock', '','1');

  $bn->{in}{'limit_thread_lock'} ||= 0;

  $wri->write(
    [
      [ 'LogMax',               'limit_log',               '$' ],
      [ 'LimitResLogSize',      'limit_log_size',          '$' ],
      [ 'LimitResTotal',        'limit_res_total',         '$' ],
      [ 'LimitResType',         'limit_res',               '$' ],
      [ 'LimitLogRemoved',      'limit_log_removed',       '$' ],
      [ 'LimitImageWidth',      'limit_image_size_width',  '$' ],
      [ 'LimitImageHeight',     'limit_image_size_height', '$' ],
      [ 'ViewLimitImageWidth',  'limit_image_view_width',  '$' ],
      [ 'ViewLimitImageHeight', 'limit_image_view_height', '$' ],
      [ 'LimitUpLoadData',      'limit_data_upload',       '$' ],
      [ 'LimitPaintData',       'limit_data_paint',        '$' ],
      [ 'LimitAnimeData',       'limit_data_anime',        '$' ],
      [ 'LimitThumbnailData',   'limit_data_thumbnail',    '$' ],
      [ 'UseThreadLock',        'limit_thread_lock',       '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_limit_len {
  my $wri = $_[0];

  $wri->num('limit_len_title', 1, 9999, 1);
  $wri->num('limit_len_name', 1, 9999, 1);
  $wri->num('limit_len_mail', 1, 9999, 1);
  $wri->num('limit_len_url', 1, 9999, 1);
  $wri->num('limit_len_message', 1, 999999, 1);
  $wri->num('limit_len_message_line', 1, 9999, 1);

  $wri->write(
    [
      [ 'LimitTitle',   'limit_len_title',        '$' ],
      [ 'LimitName',    'limit_len_name',         '$' ],
      [ 'LimitMail',    'limit_len_mail',         '$' ],
      [ 'LimitUrl',     'limit_len_url',          '$' ],
      [ 'LimitMessage', 'limit_len_message',      '$' ],
      [ 'LimitLine',    'limit_len_message_line', '$' ],
    ]
  );
}


sub set_limit_need {
  my $wri = $_[0];

  $wri->str('limit_need_title', '','1');
  $wri->str('limit_need_name', '','1');
  $wri->str('limit_need_mail', '','1');
  $wri->str('limit_need_url', '','1');
  $wri->str('limit_need_message', '','1');
  $wri->str('limit_need_password', '','1');
  $wri->str('limit_need_data', '','1');

  for (qw/title name mail url message password data/) {
    $bn->{in}{'limit_need_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'NeedTitle',    'limit_need_title',    '$' ],
      [ 'NeedName',     'limit_need_name',     '$' ],
      [ 'NeedMail',     'limit_need_mail',     '$' ],
      [ 'NeedUrl',      'limit_need_url',      '$' ],
      [ 'NeedMessage',  'limit_need_message',  '$' ],
      [ 'NeedPassword', 'limit_need_password', '$' ],
      [ 'NeedData',     'limit_need_data',     '$' ],
    ]
  );
}

sub set_view_search {
  my $wri = $_[0];

  $wri->str('view_search_use', '','1');
  $wri->num('view_search_default_view', 1, 999, 1);
  $wri->num('view_search_limit_min', 1, 999, 1);
  $wri->num('view_search_limit_max', 1, 999, 1);
  $wri->num('view_search_limit_var', 1, 100, 1);
  $wri->num('view_search_limit_log', 1, 9999, 1);
  $wri->str('view_search_form_ex', '','1');

  for (qw/use form_ex/) {
    $bn->{in}{'view_search_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'UseSearch',          'view_search_use',          '$' ],
      [ 'ViewSearchDefault',  'view_search_default_view', '$' ],
      [ 'ViewSearchLimitMin', 'view_search_limit_min',    '$' ],
      [ 'ViewSearchLimitMax', 'view_search_limit_max',    '$' ],
      [ 'ViewSearchLimitVar', 'view_search_limit_var',    '$' ],
      [ 'ViewSearchLimitLog', 'view_search_limit_log',    '$' ],
      [ 'ViewSearchFormEx',   'view_search_form_ex',      '$' ],
    ]
  );
  $bn->app->reflesh;
}

sub set_view_tree {
  my $wri = shift;

  $wri->str('view_tree_use', '','1');
  $wri->str('view_tree_frame', '','1');
  $wri->num('view_tree_new_log', 0, 999, 1);
  $wri->num('view_max_title', 0, 999, 1);
  $wri->num('view_tree_child_space', 0, 999, 1);
  $wri->str('view_tree_spp_check', '','1');

  for (qw/use frame spp_check/) {
    $bn->{in}{'view_tree_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'UseTree',            'view_tree_use',         '$' ],
      [ 'UseTreeFrame',       'view_tree_frame',       '$' ],
      [ 'ViewTreeNewLog',     'view_tree_new_log',     '$' ],
      [ 'ViewMaxTitle',       'view_max_title',        '$' ],
      [ 'ViewTreeChildSpace', 'view_tree_child_space', '$' ],
      [ 'UseTreeSppCheck',    'view_tree_spp_check',   '$' ],
    ]
  );

  $bn->app->reflesh;
}

sub set_view_rotation {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my @ro = split ',', $bn->get( $cf, 'ViewRotation' );
  my $p = int $in->{pos};
  my $s = splice @ro, $p, 1;
  splice @ro, ( $in->{move} eq 'down' ? $p + 1 : $p - 1 ), 0, $s;
  $in->{view_rotation} = join ',', @ro;
  $in->{mode} = 'view_position';
  $bn->set( $cf, 'ViewRotation', $in->{view_rotation} );
  $wri->write( [ [ 'ViewRotation', 'view_rotation', '$' ] ] );
  $bn->app->reflesh;
}

sub set_view_position {
  my $wri = $_[0];

  $wri->str('view_sort', '0','1');
  $wri->str('view_number', 'real','virtual','off');
  $wri->str('view_paint_time', '','1');
  $wri->str('view_agent', '','1');
  $wri->str('view_edit', '','1');
  $wri->str('view_id', '','1');
  $wri->str('view_first_date', '','1');
  $wri->str('view_image_pos', 'left','right','center');
  $wri->str('view_reply_image_pos', 'left','right','center');
  $wri->str('view_form_type', 'normal','thread','layer','line');
  $wri->str('view_top_form', '','1');

  for (qw/paint_time agent edit id first_date top_form/) {
    $bn->{in}{'view_' . $_} ||= 0;
  }

  $wri->write(
    [
      [ 'ViewSortType',      'view_sort',            '$' ],
      [ 'ViewPaintTime',     'view_paint_time',      '$' ],
      [ 'ViewId',            'view_id',              '$' ],
      [ 'ViewFirstDate',     'view_first_date',      '$' ],
    ],[
      [ 'ViewAgent',         'view_agent',           '$' ],
      [ 'ViewEditCount',     'view_edit',            '$' ],
      [ 'ViewThreadNumber',  'view_number',          '$' ],
      [ 'ViewImagePos',      'view_image_pos',       '$' ],
      [ 'ViewReplyImagePos', 'view_reply_image_pos', '$' ],
      [ 'FormType',          'view_form_type',       '$' ],
      [ 'ViewTopForm',       'view_top_form',        '$' ],
    ]
  );
  $bn->app->reflesh;

}


sub set_view_bbs {
  my $wri = $_[0];
  my $in = $bn->{in};
  my $item = $wri->{cms}{item};
  my $limit = $wri->{limit_esc};
  my @b = @{$item}{'view_catalog_row', 'view_catalog_col'};
  $item->{view_catalog_row} = "$item->{view_catalog}:$item->{view_catalog_row}";
  $item->{view_catalog_col} = "$item->{view_catalog}:$item->{view_catalog_col}";

  $wri->esc_len('view_bbs_title', 100);
  $wri->esc_len('view_homepage_url', 300);
  $wri->esc_len('view_homepage_url_ta', 100);
  $wri->num('view_log_view', 1, 255, 1);
  $wri->num('view_reply', 0, 999, 1);
  $wri->num('view_catalog_row', 1, 999, 1);
  $wri->num('view_catalog_col', 1, 999, 1);
  $wri->str('view_style_pagelink', 'text', 'form');
  $in->{view_limit_pagelink} ||= 0;
  $wri->num('view_limit_pagelink', 0, 999, 1);

  if ($limit->{view_homepage_url} == 2) {
    $wri->err('ERR_JAVASCRIPT',$item->{view_homepage_url})
    if $bn->{in}{view_homepage_url} =~ /javascript(\s+)?(:|\&\#58)/i;
  }

  @{$item}{'view_catalog_row', 'view_catalog_col'} = @b;

  $wri->write(
    [
      [ 'BbsTitle',          'view_bbs_title',       '$' ],
      [ 'HomePageUrl',       'view_homepage_url',    '$' ],
      [ 'HomePageUrlTarget', 'view_homepage_url_ta', '$' ],
      [ 'LogView',           'view_log_view',        '$' ],
      [ 'LogViewReply',      'view_reply',           '$' ],
      [ 'CatalogRow',        'view_catalog_row',     '$' ],
      [ 'CatalogCol',        'view_catalog_col',     '$' ],
    ],[
      [ 'ViewStylePageLink', 'view_style_pagelink',  '$' ],
      [ 'ViewLimitPageLink', 'view_limit_pagelink',  '$' ],
    ]
  );
  $bn->app->reflesh;

}

1;

__END__
