#=============================================================================
# BBS Note Comet 1.x
# Copyright(C) WonderCatStudio. All rights reserved.
# This code cannot be redistributed without
# permission from wondercatstudio.com
# LastModify : 2011/01/04
# Version    : 1.00 build 1
#=============================================================================
package BN;
use strict;

use FileHandle;

use vars qw( $VERSION );

$VERSION = '1.00';

sub version_slug {
  my $v = $VERSION =~ /(\d+.\d+)/ ? $1 : '';
  return '<a title="Powered by WonderCatStudio" href="http://wondercatstudio.com" target="_blank">BBSNote Comet ' . $v . '</a>';
}

sub new {
  my $bn = bless {}, shift;
  $bn->init(@_)->bn_init or
    return $bn->error('ERR_BOOT_INIT',__PACKAGE__);
  $bn;
}

sub lib { my $bn = bless {}, shift; $bn->init(@_) }

sub init {
  my $bn = shift;
  my %param = @_;
  my $mo   = $bn->{module} = {};
  my $in   = $bn->{in}     = {};
  my $fl   = $bn->{flag}   = {};
  my $gl   = $bn->{global} = {};
  my $data = $bn->{data}   = {};
  my $item = $bn->{item}   = {};
  my $cf   = $bn->{cfg}    = {};
  my $sk   = $bn->{skin}   = {};
  $bn->{param} = {%param};
  $cf->{__var} = {}; $cf->{__type}   = {};
  $sk->{__var} = {}; $sk->{__type}   = {};
  $sk->{flag}  = {}; $sk->{form}     = {};
  $sk->{menu}  = {}; $sk->{template} = {};
  $bn->{mo}    = {}; $bn->{arg}      = {};
  $bn->{temp}  = {}; $bn->{handler}  = {};
  $bn->{error} = []; $gl->{FormInfo} = [];

  $bn->{bn_dir} = $param{Directory} ? $param{Directory} : './';

  my $sys = $bn->{sys} = {
    CaptorFormat => 'jpg,png,gif',
    Exe          => (exists $param{Exe}          ? $param{Exe}          : 'cgi'),
    Language     => (exists $param{Language}     ? $param{Language}     : 'jp'),
    Convert      => (exists $param{Convert}      ? $param{Convert}      : ''),
    Charset      => (exists $param{Charset}      ? $param{Charset}      : 'shift_jis'),
    DirSys       => (exists $param{DirSys}       ? $param{DirSys}       : $bn->{bn_dir} . 'sys'),
    DirCfgSys    => (exists $param{DirCfgSys}    ? $param{DirCfgSys}    : $bn->{bn_dir} . 'sys'),
    Skin         => (exists $param{Skin}         ? $param{Skin}         : 'skin'),
    SkinConfig   => (exists $param{SkinConfig}   ? $param{SkinConfig}   : 'skincfg'),
    RecoverSleep => (exists $param{RecoverSleep} ? $param{RecoverSleep} : 1),
    LockSleep    => (exists $param{LockSleep}    ? $param{LockSleep}    : 1),
    LockSec      => (exists $param{LockSec}      ? $param{LockSec}      : 60),
    LockTry      => (exists $param{LockTry}      ? $param{LockTry}      : 5),

    Alarm        => int $param{Alarm},
    CFG          => $param{CFG},
    CMS          => $param{CMS},
    Comet        => $param{Comet},
  };


  if ( exists $param{Benchmark} ) {
    eval( 'use Benchmark;$gl->{Benchmark} = Benchmark->new' );
  }

  $SIG{TERM} = sub { $bn->Trace("TERM") };

  $sys->{LockDir} = exists $param{LockDir} ? $param{LockDir} : $sys->{DirSys} . '/lock';

  if (exists $ENV{MOD_PERL}) {
    eval "require mod_perl";
    if (defined $mod_perl::VERSION) {
      if ($mod_perl::VERSION >= 1.99) {
        $bn->{MOD_PERL} = 2;
        require Apache2::Connection;
        require Apache2::RequestUtil;
        $bn->{apache} = $param{Apache} || Apache2::RequestUtil->request;
      } else {
        $bn->{MOD_PERL} = 1;
        require Apache;
        $bn->{apache} = $param{Apache} || Apache->request;
      }
    }
  }
  $bn;
}
sub bn_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};

    $sys->{Config}     = "$sys->{DirCfgSys}/config.$sys->{Exe}";
    $sys->{SkinCfg}    = "$sys->{DirCfgSys}/$sys->{SkinConfig}.$sys->{Exe}";
    $sys->{SkinCgi}    = "$sys->{DirCfgSys}/$sys->{Skin}.$sys->{Exe}";
    $sys->{BootCgi}    = "$sys->{DirCfgSys}/boot.$sys->{Exe}";
    $sys->{PaletteCgi} = "$sys->{DirCfgSys}/palette.$sys->{Exe}";

    $bn->cfg_init;

    unless ( $bn->boot_strap( $sys->{BootCgi}, $sys->{Config}, $sys->{SkinCfg}, $sys->{SkinCgi}, $sys->{PaletteCgi} ) ) {
      $bn->boot_init( $sys->{Config}, $sys->{SkinCgi}, $sys->{SkinCfg} );
    }

    for ( qw/File Images/ ) {
      $sys->{'Dir'.$_} = $bn->get($cf, 'Dir'.$_.'Path') =~ m#^/|(?:\w[\|\:][\\\/])# ?
       $bn->get($cf, 'Dir'.$_.'Path') : $bn->{bn_dir} . $bn->get($cf, 'Dir'.$_.'Path');
    }

    $sys->{DirData} = delete $sys->{DirFile};
    $bn->{http_pragma} = $bn->get($cf, 'HttpPragma');

    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;
  $bn
}
sub cfg_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};

  $bn->define($cf, 'HomePageUrl',   '');
  $bn->define($cf, 'BbsTitle',      'BBS Note');
  $bn->define($cf, 'BBSNoteUrl',    $ENV{SCRIPT_NAME} );

  $bn->define($cf, 'SkinDir',       '');

  $bn->define($cf, 'DirImagesUrl',  'images');
  $bn->define($cf, 'DirImagesPath', 'images');
  $bn->define($cf, 'DirFileUrl',    'data');
  $bn->define($cf, 'DirFilePath',   'data');

  $bn->define($cf, 'IndexFilePath', 'index.html');
  $bn->define($cf, 'FileHeadLogs',  'LOG');
  $bn->define($cf, 'FileHeadImage', 'IMG');
  $bn->define($cf, 'FileHeadAnime', 'ANM');
  $bn->define($cf, 'FileHeadThumb', 'THM');

  $bn->define($cf, 'Permission',    '0606');

  $bn->define($cf, 'LogView',          10);
  $bn->define($cf, 'LogMax',           50);
  $bn->define($cf, 'CatalogRow',       4);
  $bn->define($cf, 'CatalogCol',       5);

  $bn->define($cf, 'LimitName',        30);
  $bn->define($cf, 'LimitMail',        30);
  $bn->define($cf, 'LimitUrl',         150);
  $bn->define($cf, 'LimitTitle',       80);
  $bn->define($cf, 'LimitMessage',     3000);
  $bn->define($cf, 'LimitLine',        50);

  $bn->define($cf, 'LimitResType',     2);
  $bn->define($cf, 'LimitResTotal',    30);
  $bn->define($cf, 'LimitResLogSize',  6000);

  $bn->define($cf, 'LimitImageWidth',  600);
  $bn->define($cf, 'LimitImageHeight', 600);
  $bn->define($cf, 'LimitUpLoadData',  102400);
  $bn->define($cf, 'PaintPoolMaxAnime',5242880);

  $bn->define($cf, 'LimitSecurityProxy', 4);
  $bn->define($cf, 'LimitPostMultiLog', 10);
  $bn->define($cf, 'ViewMaxTitle',      20);
  $bn->define($cf, 'ViewRotation', 'normal,catalog,thumbnail,noimage');

  $bn->define($cf, 'UseApplet', undef, '$');
  $bn->define($cf, 'UseCryptKey',     'ID');

  $bn->define($cf, 'ImageMagickPath',      'convert');
  $bn->define($cf, 'ImageMagickFormat',    'jpg');
  $bn->define($cf, 'ImageMagickGeometory', '50%');
  $bn->define($cf, 'ImageMagickOption');

  ## define skin cfg
  $bn->define($sk, 'StyleSheet',         undef, '@');
  $bn->define($sk, 'ViewImagePos',      'left', '$');
  $bn->define($sk, 'ViewReplyImagePos', 'left', '$');
  for ( qw/FormName FormTable TagBody TagTableIn TagTableOut TagTableFont
    TagFormItems TagFormAstMark TagCatalogTableIn TagCatalogTableOut/ )
   { $bn->define($sk, $_, undef, '%') }

  ## define 0 value
  for ( qw/ViewSortType ViewFirstDate LimitLogRemoved
    NeedTitle NeedMail NeedUrl NeedPassword NeedData IconViewForce
    UseTag UseReplyUpload UseMetaCookie UseImageMagick TimeDiffSec/)
   { $bn->define($cf, $_, 0) }

  ## define 1 value
  for ( qw/NeedName NeedMessage UseAutoLinkUrl UseAutoLinkMail UseQuotation
    UseRefleshIndex UseUpload UseCrypt UseMailCrypt HttpPragma
     LimitPostMultiHistory LimitFilterSpam PaintPasswordAuth/)
   { $bn->define($cf, $_, 1) }

  ## define array
  for ( qw/AdminPassword SubPassword LimitPostMulti LimitFilterString
    OkTag NoticeMailFormat TxtHead TxtTop TxtBottom TxtBbsHead
    TxtBbsTop TxtBbsBottom TxtMsgTop TxtMsgBottom TxtTreeTop
    TxtPaintHead TxtPaintTop TxtPaintBottom TxtSpaHead TxtSpaTop
    TxtSpaBottom TxtSiiHead TxtSiiTop TxtSiiBottom TxtSpvHead
    TxtSpvTop TxtSpvBottom TxtPicHead TxtPicTop TxtPicBottom
    TxtPaiHead TxtPaiTop TxtPaiBottom TxtCounterText TxtCounterImage/ )
   { $bn->define($cf, $_, undef, '@') }

  $bn->call( 'CfgInit' );

  $bn;
}

sub cfg_over_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sys = $bn->{sys};
  for ( keys %{ $sys->{CFG} } ) {
    if (ref $sys->{CFG}{$_} eq 'HASH') {
      $bn->define($cf, $_, undef, '%') if ref $bn->get($cf, $_) ne 'HASH';
      for my $key (keys %{ $sys->{CFG}{$_} })
       { $bn->set($cf, $_, $key."\t".$sys->{CFG}{$_}{$key}) }
    } elsif (ref $sys->{CFG}{$_} eq 'ARRAY') {
      $bn->define($cf, $_, undef, '@');
      for my $val (@{$sys->{CFG}{$_}}) { $bn->set($cf, $_, $val) }
    } else {
      $bn->define($cf, $_, undef);
      $bn->set($cf, $_, $sys->{CFG}{$_});
    }
  }
}

sub boot_init {
  my( $bn, $cfg_file, $skin_file, $skin_cfg ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};

  $cfg_file = $sys->{Config} if $cfg_file eq '';
  $skin_file = $sys->{SkinCgi} if $skin_file eq '';
  $skin_cfg = $sys->{SkinCfg} if $skin_cfg eq '';
  eval{ delete $INC{ $sys->{BootCgi} } };

  $bn->filelock;
  $bn->cfg_init;
  $bn->read_config($cf, $cfg_file);
  $bn->read_skin($skin_file);
  $bn->read_config($sk, $skin_cfg);

  $bn->template_init;
  $bn->exfile_init;
  $bn->paint_init;
  $bn->bootcfg_init;
  $bn->cfg_over_init;
  $bn->cash_boot;
  $bn->fileunlock;
}

sub boot_strap {
  my( $bn, $boot_file, @files ) = @_;
  my $in = $bn->{in};
  my $gl = $bn->{global};
  my $sys = $bn->{sys};
  my $boot_mod = $gl->{boot_mtime} = (stat $boot_file)[9];

  for (@files) {
    if ($boot_mod < (stat $_)[9]) {
      $gl->{boot_mtime} = time;
      return 0;
    }
  }


  $bn->cfg_init;

  delete $INC{$boot_file};
  eval { require $boot_file;
      BN::boot->new( $bn )->boot };
  return 0 if $@;

  $bn->exfile_init;
  $bn->paint_init or return 0;
  $bn->bootcfg_init or return 0;
  $bn->cfg_over_init;

  return 1;
}

sub exfile_init {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $fo = \$bn->{sys}{CaptorFormat};

  $$fo .= ',txt' if $bn->get($cf, 'UseFileExText');
  $$fo .= ',bmp' if $bn->get($cf, 'UseFileExBmp');
  $$fo .= ',mid' if $bn->get($cf, 'UseFileExMidi');
  $$fo .= ',swf' if $bn->get($cf, 'UseFileExSwf');
  $$fo .= ',zip' if $bn->get($cf, 'UseFileExZip');
  $$fo .= ',lzh' if $bn->get($cf, 'UseFileExLzh');

  unless (ref $bn->get($cf, 'FILE_EX') eq 'HASH') {
    $bn->set($cf, 'FILE_EX', {
       txt => $bn->text('FILE_TXT'),
       bmp => $bn->text('FILE_BMP'),
       mid => $bn->text('FILE_MID'),
       swf => $bn->text('FILE_SWF'),
       zip => $bn->text('FILE_ZIP'),
       lzh => $bn->text('FILE_LZH'),
    });
  }

}

sub bootcfg_init {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $result = 1;

  if (ref $bn->get($sk, 'FormName') ne 'HASH') {
    $bn->define($sk, 'FormName', undef, '%');
    $result = 0;
  }

  for my $m (qw/Main Reply/) {
    for (qw/Number Title User Message Edit/) {
      if (ref $bn->get($sk, 'Tag' . $m . $_,) ne 'HASH') {
        $bn->define($sk, 'Tag' . $m . $_, undef, '%');
        $result = 0;
      }
    }
  }

  if (ref $bn->get($sk, 'Week') ne 'HASH') {
    $bn->define($sk, 'Week', undef, '%');
    my $w = $bn->text('WEEK');
    map{ $bn->set($sk, 'Week', "$_ $w->{$_}") }keys %$w;
    $result = 0;
  }

  if ($bn->get($cf, 'UseIcon') && ref $bn->get($cf, 'IconList') ne 'ARRAY') {
    my @list;
    $bn->ic->open_data(\@list);
    my $item = $bn->{mo}{Icon}{item};
    $bn->define($cf, 'IconListString', $item->{icon_list});
    $bn->define($cf, 'IconRegistString', $item->{icon_regist});
    $bn->define($cf, 'IconExclusive', $item->{icon_exclusive});
    $bn->define($cf, 'IconList', undef, '@');
    map{ $bn->set($cf, 'IconList', { applet => $_->{applet},
        title => $_->{title}, file_name => $_->{file_name} })
    } @list;
    $result = 0;
  }

  return $result;
}

sub paint_init {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $mo = $bn->{module};
  my $pm = $mo->{Paint} = {};

  my $paint_applet = $bn->get($cf, 'UseApplet');

  if (ref $paint_applet ne 'ARRAY') {
    $bn->define($cf, 'UseApplet', undef, '@');
    for (split /,/, $paint_applet) { $bn->set($cf, 'UseApplet', $_) }
    $paint_applet = $bn->get($cf, 'UseApplet');
  }

  for (@$paint_applet) { $bn->cfg2hash($cf, 'AppletSetting:' . $_) }
  for (qw/Applet ShiApplet ShiPainter ShiPaintBBS ShiPCHViewer PictureBBS BBSPainter/)
   { $bn->cfg2hash($cf, 'Config' . $_) }
  for (qw/ShiPainter ShiPaintBBS ShiPCHViewer PictureBBS BBSPainter DynPalette/)
   { $bn->cfg2hash($sk, 'AppletColor' . $_) }

  unless (@$paint_applet) { delete $mo->{Paint}; return 1 }

  $pm->{applets} = ();
  my $ly = $pm->{layout} = {};
  my $spa = $bn->get($cf, 'ConfigShiPainter');
  my $sii = $bn->get($cf, 'ConfigShiPaintBBS');
  my $flag = 1;
  my %copy;

  $bn->call('PaintInitBegin');

  for (@$paint_applet) {
    my %cfg = %{$bn->get($cf, 'AppletSetting:' . $_)};
    $cfg{key} = $_;
    if ($cfg{applet} eq 'spapro' || $cfg{applet} eq 'spanormal') {
      $copy{SPA} = 1;
      $bn->set_app_cfg('ConfigShiPainter', $cf, \%cfg);
      $bn->set_app_cfg('ConfigShiApplet', $cf, \%cfg);
      $cfg{anime} = $copy{spch} = 'spa' if !exists $cfg{anime};
      $cfg{palette} = ($cfg{applet} eq 'spapro' ? 20 : 14) unless exists $cfg{palette};
    } elsif($cfg{applet} eq 'sii') {
      $copy{SII} = 1;
      $bn->set_app_cfg('ConfigShiPaintBBS', $cf, \%cfg);
      $bn->set_app_cfg('ConfigShiApplet', $cf, \%cfg);
      $cfg{anime} = $copy{pch} = 'sii' if !exists $cfg{anime};
      $cfg{palette} = 14 unless exists $cfg{palette};
    } elsif($cfg{applet} eq 'pic') {
      $copy{PIC} = 1;
      $bn->set_app_cfg('ConfigPictureBBS', $cf, \%cfg);
      $cfg{anime} = 0 unless exists $cfg{anime};
      $cfg{palette} = 16 unless exists $cfg{palette};
    } elsif($cfg{applet} eq 'pai') {
      $copy{PAI} = 1;
      $bn->set_app_cfg('ConfigBBSPainter', $cf, \%cfg);
      $cfg{anime} = 0  unless exists $cfg{anime};
      delete $cfg{palette};
    }
    push @{$pm->{applets}}, \%cfg;
  }

  @{$ly}{'palette', 'anime'} = (-1, -1);
  for (@{$pm->{applets}}) {
    $ly->{anime} = $_->{anime} && $_->{animation} ne 'false' ? $ly->{anime} == 0 || $ly->{anime} == 2 ? 2 : 1 : $ly->{anime} > 0 ? 2 : 0;
    $ly->{palette} = $_->{palette} ? $ly->{palette} == 0 || $ly->{palette} == 2 ? 2 : 1 : $ly->{palette} > 0 ? 2 : 0;
    $ly->{continue} .= $_->{applet} . ',' if $_->{applet} =~ /^(sii|spa)/;
  }
  $ly->{complex} = $ly->{palette} == 2 || $ly->{anime} == 2 ? 1 : 0;

  my @anime_ext;
  push @anime_ext, $spa->{anime_ext} if exists $copy{spch};
  push @anime_ext, $sii->{anime_ext} if exists $copy{pch};
  $bn->{sys}{CaptorFormat} .= ','. join ',', @anime_ext if @anime_ext;

  if (ref $bn->get($cf, 'PaletteNames') ne 'ARRAY') {
    my $palette_name = ();
    my $item = $pm->{item} = $bn->text('','paint');
    push @$palette_name, "\t$item->{default}";
    $bn->get_palette('', $palette_name);
    $bn->define($cf, 'PaletteNames', undef, '@');
    map{ $bn->set($cf, 'PaletteNames', $_) }@$palette_name;
    $flag = 0;
  }
  unless ($bn->get($cf, 'PaintAppCopy')) {
    $bn->define($cf, 'PaintAppCopy', undef, '$');
    my @s;
    if ($copy{SPA} || $copy{SII}) {
      my $spa = 'ShiPainter' if $copy{SPA};
      my $sii = 'PaintBBS' if $copy{SII};
      my $spc = ', ' if $copy{SPA} && $copy{SII};
      push @s, "$spa$spc$sii Program " . $bn->text('APP_COPY_SPA');
    }
    push @s, 'PictureBBS Program ' . $bn->text('APP_COPY_PIC') if $copy{PIC};
    push @s, 'BBSPainter Program ' . $bn->text('APP_COPY_PAI') if $copy{PAI};
    $bn->set($cf, 'PaintAppCopy', join '<BR>', @s);
    $flag = 0;
  }

  $bn->set($cf, 'ViewPaintContinueHistory', 1)
    unless defined $bn->get($cf, 'ViewPaintContinueHistory');


  $bn->set_call('OpenPageBegin', sub{$bn->check_pool});
  $bn->set_call('OpenAdminMenu', sub{$bn->pm->admin_menu});
  $bn->set_call('SkinHeadFormat', sub{$bn->set_applet_form_script});
  $bn->set_call('CheckInputFile', sub{$bn->pm->check_file});
  $bn->set_call('FileAnalyzeInit', sub{$bn->pm->anime_analyze});

  $bn->call('PaintInitEnd');
  return $flag;
}

sub run {
  my $bn = $_[0];
  my $cf = $bn->{cfg};

  @{$bn->{global}}{'UserAgent','UserOS','UserBrowser','UserBrowserVer'} = $bn->get_useragent;

  $bn->{cookie} = $bn->get_cookie;
  my $in = $bn->{in} = $bn->get_in;

  $bn->call( 'Run' );

  $bn->path_info;
  $bn->access_log;
  $bn->check_access;

  my $fc = $in->{fc};
     if ( $fc eq '' || $fc eq 'page' ) { $bn->open_page }
  elsif ( $fc eq 'post'         ) { $bn->open_post }
  elsif ( $fc eq 'edit'         ) { $bn->open_edit }
  elsif ( $fc eq 'help'         ) { $bn->open_help }
  elsif ( $fc eq 'write'        ) { $bn->open_write }
  elsif ( $fc eq 'login'        ) { $bn->open_login }
  elsif ( $fc eq 'admin'        ) { $bn->open_admin }
  elsif ( $fc eq 'anime'        ) { $bn->open_paint }
  elsif ( $fc eq 'continue'     ) { $bn->open_paint }
  elsif ( $fc eq 'stream'       ) { $bn->open_paint }
  elsif ( $fc eq 'search'       ) { $bn->open_search }
  elsif ( $fc eq 'delete'       ) { $bn->open_delete }
  elsif ( $fc eq 'thread'       ) { $bn->open_thread }
  elsif ( $fc eq 'setting'      ) { $bn->open_setting }
  elsif ( $fc =~ /^access/      ) { $bn->open_access }
  elsif ( $fc =~ /^paint/       ) { $bn->open_paint }
  elsif ( $fc =~ /^icon/        ) { $bn->open_icon }
  elsif ( $fc =~ /^(?:rss|xsl)/ ) { $bn->open_xml }
  else { $bn->call( 'RunOther' ); $bn->open_page }

  if ( exists $bn->{global}{Benchmark} )
   { $bn->print( timestr( timediff( Benchmark->new, $bn->{global}{Benchmark} )) ) }

  $bn->EXIT;
}

sub path_info {
  my $bn = $_[0];
  return if !exists $ENV{PATH_INFO};
  my $r = "http://$ENV{HTTP_HOST}$ENV{REQUEST_URI}";
  $r =~ s/$ENV{PATH_INFO}$//;
  if ( $ENV{MOD_PERL} ) {
    $bn->{apache}->status( 301 );
    $bn->{apache}->send_cgi_header( "Location: $r\n\n" );
  } else {
    $bn->print( "Location: $r\n\n" );
  }
  $bn->EXIT;
}

sub open_page {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  my $gl = $bn->{global};

  $bn->open_thread if $ENV{QUERY_STRING} =~ /^[\d\-,:]+$/;


  $fl->{header_modified} = $sys->{MODE} eq 'ADMIN' ? 0 : 1;

  my @ro = split ',', $bn->get( $cf, 'ViewRotation' );
  $in->{mode} = $ro[0] if !exists $in->{mode};

  $bn->call('OpenPageBegin');

  if ($bn->get($cf, 'UseTree') && $bn->get($cf, 'UseTreeFrame')) {
    $bn->{flag}{use_tree_frame} = 1;
      my $ta = $bn->get($cf, 'HomePageUrlTarget');
      $bn->set($cf, 'HomePageUrlTarget', '_parent') if $ta =~ /^($|_self)/i;
    if (!exists $in->{frame}) {
      my $type = exists $in->{frametype} ? $in->{frametype} : exists $ck->{frametype} ? $ck->{frametype} : 1 ;
      $in->{frametype} = $ck->{frametype} = $type;
      $fl->{write_cookie} = 1;
      $fl->{header_modified} = 0;
    }
  }

  $sys->{sortmode} = $in->{sort} eq '' ?
    $bn->get($cf, 'ViewSortType') : $in->{sort} ? 1 : 0;
  my $log_view = $in->{mode} eq 'catalog' ?
    $bn->get($cf, 'CatalogCol') * $bn->get($cf, 'CatalogRow') : $bn->get($cf, 'LogView');
  $bn->set($cf, 'LogView', $log_view);
  $gl->{Page} = $in->{page} ? abs int $in->{page} : 1 ;
  $gl->{Index} = ($gl->{Page} - 1) * $log_view;
  $bn->{flag}{SkinMenu} = 1 if ($bn->{flag}{use_tree_frame} && $in->{frame} eq '1');

  $bn->get_mtime;
  $bn->send_http_header(undef, $fl->{header_modified});
  $bn->mail_crypt->head(undef, undef, 1)->tree or return;
  $bn->open_index;
  $gl->{Index} = $gl->{Page} = 0 if $gl->{Index} >= $gl->{LogTotal} || $gl->{Index} < 0;
  $gl->{IndexPos} = 0;
  my( @indexs, @threads );

  my $open_log = sub{
    my $index_seek = $_[0];
    $bn->open_index unless exists $bn->{temp}{fh}{index};
    for ( my $index_seek = $gl->{Index};
      $gl->{IndexPos} < $log_view && $index_seek < $gl->{LogTotal};
      $gl->{IndexPos}++, $index_seek++
    ) {
      push @indexs, $bn->read_index($index_seek, $sys->{sortmode});
      push @threads, ($sys->{sortmode} ? $bn->read_index($index_seek, 2) : $gl->{LogTotal} - $index_seek);
    }
    $bn->close_index;
    for ( my $i = $index_seek, $gl->{IndexPos} = 0;
      $i <= $#indexs;
      $gl->{IndexPos}++, $gl->{Index}++, $i++
    ) {
      $gl->{IndexNumber} = $indexs[$i];
      $gl->{ThreadNumber} = $threads[$i];
      $bn->open_log($gl->{IndexNumber});
    }
  };

  $bn->menu;
  $bn->call('OpenPageMenu');
  &$open_log();
  &$open_log( $gl->{Index} ) if $gl->{IndexPos} <= $log_view;

  $bn->call('OpenPageFoot');
  $bn->foot(undef, 'VIEW_COUNTER');
}

sub open_post {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $sys = $bn->{sys};

  $bn->send_http_header(undef, $fl->{reply_mode});

  if (exists $in->{log}) {
    $bn->mail_crypt;
    $sys->{MODE} = exists $in->{edit} ? 'EDIT' : 'REPLY';
    $fl->{POST_FORM_TITLE} = 1 if $bn->get($cf, 'UseReplyTitle');
    $fl->{UPLOAD_ENABLE} = $bn->check_limit_up_view('Reply');
  } else {
    $sys->{MODE} = 'POST' unless $sys->{MODE};
    $fl->{POST_FORM_TITLE} = 1;
    $fl->{UPLOAD_ENABLE} = $bn->check_limit_up_view;
    if (!$bn->get($cf, 'UseUpload') && $bn->get($cf, 'NeedData') && $sys->{MODE} ne 'PAINT') {
      $bn->error('ERR_FUNCTION');
    }
  }

  $bn->call('OpenPostInit');
  $bn->app->check_post_range->post_form;
  $bn->foot('NotPageLink');
  $bn;
}

sub open_edit {
  my $bn = $_[0];
  my $in = $bn->{in};
  $bn->{sys}{MODE} = 'EDIT';
  $bn->call('OpenEdit');
  unless ( exists $in->{number} ) { $bn->open_page; return }
  $bn->app->edit;
}

sub open_delete {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $sys = $bn->{sys};
  my $f;
  $sys->{MODE} = $sys->{MODE} . 'DELETE';

  $bn->call('OpenDelete');
  for (keys %{$in}) { if ( /^log([\d\-]+)/ ) { $f = 1 ;last } }
  unless ( $in->{number} ne '' || $f ) {
    return if $sys->{MODE} =~ /ADMIN/;
    $bn->open_page;
    return;
  }

  $bn->app->delete;
}

sub open_thread {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  $bn->call('OpenThreadBegin');

  if ($ENV{QUERY_STRING} =~ /^([\d\-,:]+)$/) {
    $_ = $1; $in->{log} = /([\d\-,]+):([\d\-,]+)/?($in->{res} = $2)?$1:$1:$1;
  }

  $gl->{LogTotal} = $bn->{sys}{index}{total} = (-s "$bn->{sys}{DirSys}/index.$bn->{sys}{Exe}") >> 1;
  $gl->{IndexNumber} = $gl->{ThreadNumber} = $in->{log};
  $gl->{ReplyNumber} = $in->{res};

  $bn->{sys}{MODE} = 'THREAD';
  if    (exists $in->{delete}) { $bn->{sys}{MODE} = 'DELETE' }
  elsif (exists $in->{edit})   { $bn->{sys}{MODE} = 'EDIT' }
  elsif (exists $in->{admin})  { $bn->ad->menu }

  $in->{no} = $in->{log} unless $in->{no};

  if ($in->{reply} && exists $in->{log}) {
    $fl->{reply_mode} = 1;
    $bn->open_post->EXIT;
  }

  my $mtime;
  unless ($gl->{IndexNumber}=~/\d+(-|,)\d+/) {
    $bn->get_mtime($bn->filelog($gl->{IndexNumber}));
    $mtime = 1;
  }
  $bn->send_http_header(undef, $mtime)->mail_crypt->head->menu;

  $bn->call('OpenThreadMenu');

  $fl->{thread_view} = 1;
  my $log_view = $bn->get($cf, 'LogView');
  my $loop_open = sub {
    my $n;
    for (@_) {
      last if $n >= $log_view;
      $gl->{IndexNumber} = $_;
      $bn->open_log($_);
      $n++;
    }
  };

  if ($gl->{IndexNumber}=~/(\d+)(-|,)(\d+)/) {
    my($st, $l, $ed) = ($1, $2, $3);
    if ($l eq ',') {
      &$loop_open(split /,/, $gl->{IndexNumber}, $log_view + 1);
    } else {
      my($bs,$be,@l);
      $bs = $st < $ed ? $st : $ed;
      $be = $st < $ed ? $ed : $st;

      if ($st < $ed) {
        $be = $bs + ($log_view - 1) if $bs + ($log_view - 1) < $be;
      } else {
        $bs = $be - ($log_view - 1) if $be - ($log_view - 1) > $bs;
      }

      @l = $bs..$be;
      @l = reverse @l if $st > $ed;
      &$loop_open(@l);
    }
  } else {
    $bn->open_log($gl->{IndexNumber});
  }

  $bn->call('OpenThreadEnd');

  delete $gl->{LogTotal};

  $bn->foot('NotPageLink');

  $bn->sea->keyword if exists $in->{keyword};

  $bn->EXIT;
}

sub open_help { $_[0]->hp }

sub open_write { $_[0]->po->write }

sub open_login { $_[0]->ad->login }

sub open_search { $_[0]->sea->search }

sub open_icon { $_[0]->ic->menu }

sub open_access { $_[0]->lo->menu }

sub open_admin { $_[0]->ad->menu }

sub open_setting { $_[0]->cms->menu }

sub open_xml {
  my $bn = $_[0];
  my $in = $bn->{in};
  $in->{fc} eq 'rss1' ? $bn->xml->rss1 :
  $in->{fc} eq 'rss2' ? $bn->xml->rss2 :
  $in->{fc} eq 'xsl1' ? $bn->xml->xsl1 :
#  $in->{fc} eq 'xsl2' ? $bn->xml->xsl2 :
  $bn->xml->rss1 ;
}

sub open_paint {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $mo = $bn->{module};

  $bn->error('ERR_FUNCTION') unless exists $mo->{Paint};

  my $pm = $bn->pm;
  if ($in->{fc} eq 'anime') {
    $pm->open_anime;
  } elsif($in->{fc} eq 'continue') {
    $pm->open_continue;
  } elsif($in->{fc} eq 'stream') {
    $pm->stream;
  } elsif($in->{fc} eq 'paint_write') {
    $pm->paint_write;
  } elsif($in->{fc} eq 'paint_regist') {
    $pm->post_regist
  } elsif($in->{fc} eq 'paint_anime_size') {
    $pm->open_anime_size
  }

  for my $c (@{$mo->{Paint}{applets}}) {
    $pm->paint($c) if $in->{fc} eq 'paint_' . $c->{key};
  }

}

sub open_log {
  my( $bn, $log ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $sys = $bn->{sys};
  my $path = $bn->filelog( $log );
  return -1 unless -e $path;

  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_DATA_NOTOPEN', $path);
  my @data;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};
    while ( <$fh> ) { push @data, $_ if /^\d+/ }
    close $fh;
    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;

  $gl->{LogSize} = (stat $fh)[7];
  $gl->{LogReply} = $#data;

  if ($bn->get($cf, 'UseThreadLock')) {
    my $val = $gl->{LogValue} = {};
    my $ex = $gl->{LogExValue} = {};
    $bn->get_log_main_value($data[0], $val, $ex);
    $fl->{thread_lock} = $ex->{thread_lock} ? 1 : '';
    if (exists $ex->{thread_view} && $ex->{thread_view} == 1 && !$fl->{thread_view})
     { $gl->{IndexPos}--; return }
  }

  # reply enable or disable check
  $fl->{reply_enable} = $bn->check_reply_enable($gl->{LogReply}, $gl->{LogSize});
  $fl->{reply_disable} = 0;

  $bn->call('OpenLogManual');

  if ( !$bn->{flag}{manual_log_open} ) {
    if ($in->{res} =~ /^(\d+)/ && $1 > 0) {
      $bn->{flag}{view_local} = 1;
      $bn->reply(\@data);
    } else {
      $bn->main(\@data);
    }
  }
  undef @data;
  $bn;
}

sub head {
  my( $bn, $title, $txt_flag, $tree_flag ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $arg = $bn->{arg};
  my $data = $bn->{data} = {};
  my $use_extform = $bn->get($sk, 'FormType');

  $data->{title} = $title;

  $bn->call('SkinHeadBegin');

  return $bn if $bn->{flag}{SkinHead};
  $bn->{flag}{SkinHead} = 1;

  $bn->bsys_init($data)->tag_init($data);

  $data->{pagelink} = sub{ $bn->page_link_init($data) };
  $data->{encoding} = $data->{charset} = $sys->{Charset};
  $data->{title} = $bn->get($cf, 'BbsTitle') unless $data->{title};
  my $rss_title = $bn->get($cf, 'BbsTitle');
  $rss_title =~ s/"/&#34;/g;

  my $rss_url = $data->{bbsnoteurl}. '?fc=rss';
  if ( $bn->get( $cf, 'UseRssIndex' ) )
   { $rss_url = $data->{bbsnoteurl}; $rss_url =~ s/[^\\\/]+$/index.rdf/ }

  my $meta = $sk->{META} = [
    { TAG => 'META', 'http-equiv' => 'Content-Type', content => 'text/html; charset=' . $sys->{Charset} },
    { TAG => 'META', 'http-equiv' => 'Content-Script-Type', content => 'text/javascript' },
    { TAG => 'META', 'http-equiv' => 'Content-Style-Type', content => 'text/css' },
    { TAG => 'LINK', rel => 'alternate', type => 'application/rss+xml', title => $rss_title, href => $rss_url },
  ];

  unshift @{ $meta },{ TAG => 'META', name => 'ROBOTS', content => 'NOINDEX,NOFOLLOW,NOARCHIVE,NOIMAGEINDEX' }
    if $bn->get($cf, 'LimitNoRobots');

  if (exists $gl->{mtime}) {
    unshift @{ $meta },{ TAG => 'META', 'http-equiv' => 'Last-Modified',
      content => $gl->{mtime} }
  }

  $data->{txthead} = $bn->cfg2join($cf, 'TxtHead');
  $data->{txthead} .= $bn->cfg2join($cf, 'TxtBbsHead') unless $txt_flag;
  $data->{txthead} = $bn->trans_values($data->{txthead}, $data);
  $data->{txtbbstop} .= $bn->cfg2join($cf, 'TxtTop');
  $data->{txtbbstop} .= $bn->cfg2join($cf, 'TxtBbsTop') unless $txt_flag;
  $data->{stylesheet} = $bn->trans_values($bn->cfg2join($sk, 'StyleSheet'), $data);
  $data->{getagent} = sub{ $bn->get_javascript('GetAgent',\$data->{javascript}); return };
  $data->{no_body} = sub{ $gl->{NO_BODY} = 1; return };
  $data->{mm_findobj} = sub{ $bn->get_javascript('MM_findObj',\$data->{javascript}); return };
  $data->{mm_swapimage} = sub{ $bn->get_javascript('MM_swapImage',\$data->{javascript}); return };
  $data->{mm_reloadpage} = sub{ $bn->get_javascript('MM_reloadPage',\$data->{javascript}); return };
  $data->{mm_preloadimages} = sub{ $bn->get_javascript('MM_preloadImages',\$data->{javascript}); return };
  $data->{mm_showhidelayers} = sub{ $bn->get_javascript('MM_showHideLayers',\$data->{javascript}); return };
  $data->{mm_swapimgrestore} = sub{ $bn->get_javascript('MM_swapImgRestore',\$data->{javascript}); return };

  if ($use_extform eq 'layer' && $sys->{MODE} !~ /POST|REPLY/) {
    $bn->get_javascript('LayerFormScript', \$data->{javascript}, \$data->{bodyprint}, $bn->reply_form( $use_extform ));
  }

  if ($bn->get($cf, 'UseIcon') && $bn->get($cf, 'UseIconPreview')) {
    $bn->get_javascript('IM_Preview', \$data->{javascript});
  }

  $bn->call('SkinHeadFormat');

  # META
  undef $data->{meta};
  for (@{ $meta }) { $data->{meta} .= $bn->set_tag($_)."\n" }

  $bn->set($sk, 'TagBody', 'TAG BODY');
  my $tag_body = $bn->get($sk, 'TagBody');
  $tag_body->{background} =~ s/^images\//$data->{images}\// if ref $tag_body eq 'HASH';
  my $body = $bn->set_tag( $bn->get($sk, 'TagBody') );
  $body =~ s/<\w+//; chop $body;
  $data->{bodyattribute} = $body;

  $bn->call('SkinHeadTranslate');

  $bn->translate($sk->{template}{HEAD}, $data);

  $bn->call('SkinHeadEnd');

  if ($tree_flag) {
    $bn->count('bbstop');
  } else {
    $bn->print( $bn->set_tag( $bn->get($sk, 'TagBody') ) ) if !exists $gl->{NO_BODY};
    $bn->print( $bn->trans_values($data->{txtbbstop}, $data) ) if defined $data->{txtbbstop};
    $bn->print( "\n$data->{bodyprint}" ) if defined $data->{bodyprint};
    $bn->check_access_alert;
  }

  # color
  if ($bn->get($cf, 'UseColor') && !$gl->{UseColorFlag}) {
    $gl->{UseColorFlag} = 1;
    my @tags = qw/Message Title Table/;
    my $set_color = sub {
      my $t = $_[0];
      my $data = $bn->{data};
      my @c = qw/font_color title_color table_color/;
      for (my $n = 0; $n < 3; $n++) {
        next unless $data->{$c[$n]};
        $bn->define( $sk, 'Tag'.$t.$tags[$n], undef, '%' ) if ref $bn->get( $sk, 'Tag'.$t.$tags[$n] ) ne 'HASH';
        $gl->{ 'Tag'.$t.$tags[$n].'Color' } = $bn->get( $sk, 'Tag'.$t.$tags[$n] )->{color};
        $bn->get($sk, 'Tag'.$t.$tags[$n] )->{color} = $data->{$c[$n]};
      }
    };
    my $rev_color = sub { my $t = $_[0];
      for (@tags) {
        if (exists $gl->{'Tag'.$t.$_.'Color'})
          { $bn->get( $sk, 'Tag'.$t.$_ )->{color} = $gl->{'Tag'.$t.$_.'Color'};
          delete $gl->{'Tag'.$t.$_.'Color'} }
      }
    };
    $bn->set_call('SkinMainBegin',  sub{ &$set_color('Main') });
    $bn->set_call('SkinMainEnd',    sub{ &$rev_color('Main') });
    $bn->set_call('SkinReplyValue', sub{ &$set_color('Res') });
    $bn->set_call('SkinReplyEnd',   sub{ &$rev_color('Res') });
  }

  $bn;
}

sub tree {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $data = $bn->{data};

  if ($bn->get($cf, 'UseTree') && $bn->get($cf, 'UseTreeFrame')) {
    my $in = $bn->{in};
    my $arg = $bn->{arg};
    if (!exists $in->{frame} && $in->{frametype} ne '0' && $bn->{sys}{MODE} ne 'ADMIN') {
      $in->{frametype} = 1 unless exists $in->{frametype};
      $arg->{FrameNormalLink}{frame} = 1;
      $arg->{FrameTreeLink}{frame} = 2;
      $arg->{FrameTreeLink}{fc} = exists $in->{edit} ? 'edit' : $in->{fc} eq 'delete' ? 'delete' : '';
      $arg->{FrameTreeLink}{frametype} = $in->{frametype};
      my $tree = $data->{bbsnoteurl} . '?' . $bn->set_text_arg($arg->{FrameTreeLink});
      my $main = $data->{bbsnoteurl} . '?' . $bn->set_text_arg($arg->{FrameNormalLink});
      my @fn = $in->{frametype} % 2 ? ( 'tree', $tree, 'main', $main ) : ( 'main', $main, 'tree', $tree );
      my $div = $in->{frametype} < 3 ? 'cols' : 'rows';
      my @divarea = split ',', ( $bn->get( $sk, 'TreeFrame'. ucfirst $div ) or '*,*' );
      for ( 0..1 ) { my $k = 'TreeFrame'. ( ucfirst $div ). ( ucfirst $fn[$_] );
        $divarea[$_] = $bn->get( $sk, $k ) if $bn->get( $sk, $k ) }
      $bn->print( '<frameset ', $div, '="', ( join ',', @divarea ), '" border="5" class="frameset">', "\n",
        '<frame name="bn_', $fn[0], '" src="', $fn[1], '">', "\n",
        '<frame name="bn_', $fn[2], '" src="', $fn[3], '">', "\n",
        '</frameset><noframes>'."\n$in->{frame}",
        $bn->set_tag( $bn->get($sk, 'TagBody') ),
        '<h2><a href="', $main, '">Click Here</a></h2></body></noframes></html>'
      );
      return 0;
    } elsif ($in->{frame} eq '1') {
      $bn->{sys}{MODE} = 'view_main';
      $arg->{Menu}{frame} = 1;
      $arg->{PageLink}{frame} = 1;
    } elsif ($in->{frame} eq '2') {
      $arg->{Menu}{frame} = 2;
      $arg->{PageLink}{frame} = 2;
      $arg->{aticle_link}{frame} = 1;
      $arg->{thread_link}{frame} = 1;
      $arg->{tree_link_target} = 'bn_main';
      $arg->{applet_form_target} = '_parent' if $bn->get($cf, 'HomePageUrlTarget') =~ /^($|_self|_parent)/i;
    }

  }
  $bn->print($bn->set_tag( $bn->get($sk, 'TagBody') )) unless exists $bn->{global}{NO_BODY};
  $bn->print($bn->trans_values($data->{txtbbstop}, $data)) if defined $data->{txtbbstop};
  $bn->print("\n$data->{bodyprint}") if defined $data->{bodyprint};
  $bn->check_access_alert;
  $bn;
}

sub tree_link {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $data = $bn->{data};
  my $ck = $bn->{cookie};
  if ($bn->get($cf, 'UseTree')){
    if (
      $bn->{sys}{MODE} =~ /^(?:$|EDIT|DELETE)/ &&
      $in->{mode} =~ /^(?:$|normal)/
    ) {
      my $i = int($bn->{in}{frametype}) + 1;
      $i = 1 if $i > 4;
      $data->{treereloadlink} = $data->{bbsnoteurl} . '?' . $bn->set_text_arg($bn->{arg}{TreeReloadLink},{frametype => $in->{frametype}});
      $data->{treeframelink} = $data->{bbsnoteurl} . '?' . $bn->set_text_arg($bn->{arg}{TreeFrameLink},{frametype => $i});
      $data->{treenoframelink} = $data->{bbsnoteurl} . '?' . $bn->set_text_arg($bn->{arg}{TreeNoFrameLink},{frametype => 0});
      $data->{txttreetop} = $bn->trans_values($bn->cfg2join($cf, 'TxtTreeTop'), $data);
      $data->{usetreeframe} = $bn->get($cf, 'UseTreeFrame');
    }
    if ($ck->{frametype} ne '0') {
      $data->{menu_write_target} = ' target="_parent"';
      $data->{menu_login_target} = ' target="_parent"';
      $data->{menu_help_target} = ' target="_parent"';
    }
  }

}
sub foot {
  my( $bn, $flag_pagelink, $flag_counter ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $menu = $bn->{menu};
  my $tmpl = $sk->{template};
  my $data = $bn->{data} = {};


  map { $data->{$_}=$menu->{$_} } keys %$menu if ref $menu eq 'HASH';
  $bn->bsys_init($data)->tag_init($data);

  $data->{pagelink} = sub{ $bn->page_link_init($data) } unless $flag_pagelink;

  $bn->call('SkinFootBegin');

  $bn->catalog_init($data);

  if ($data->{mode_catalog}) {
    my $row = $bn->get($cf, 'CatalogRow');
    my $sp_row = ($gl->{Index}) % $row;
    if ($gl->{IndexPos} > $row && $sp_row && $gl->{Index} > $row) {
      $data->{catalog_span_col} = $row - $sp_row;
    }
  }

  $data->{menu_form_end} = 1  if $bn->{flag}{MENUFORM};
  $data->{menu} = 1  if $bn->{flag}{MENU};

  $data->{txtmsgbottom} .= $bn->trans_values( $bn->cfg2join($cf, 'TxtBottom') . $bn->cfg2join($cf, 'TxtMsgBottom'), $data );
  $data->{txtbbsbottom} .= $bn->trans_values( $bn->cfg2join($cf, 'TxtBbsBottom'), $data );

  $bn->count('msgbottom')->count('bbsbottom') if $flag_counter;

  $bn->tree_link;

  $bn->call('SkinFootTranslate');

  $bn->translate($sk->{template}{FOOT}, $data);

  $bn->call('SkinFootEnd');

  $bn;
}
sub menu {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  my $gl = $bn->{global};
  my $tmpl = $sk->{template};
  my $data = $bn->{data} = {};
  my $menu = $bn->{menu} = {};

  return if $bn->{flag}{SkinMenu};
  $bn->{flag}{MENU} = 1;

  $bn->menu_init($menu)->catalog_init($menu);
  map { $data->{$_}=$menu->{$_} } keys %$menu;

  $data->{menu_write} = 0 if $bn->check_limit_post_view('New');
  $data->{menu_edit} = 0 if $bn->check_limit_post_view('Edit');
  $data->{menu_delete} = 0 if $bn->check_limit_post_view('Delete');
  $data->{title} = $bn->get($cf, 'BbsTitle');

  $bn->bsys_init($data)->tag_init($data);

  $bn->call('SkinMenuBegin');

  $data->{txtmsgtop} .= $bn->trans_values( $bn->cfg2join( $cf, 'TxtMsgTop' ), $data );

  $bn->count('msgtop');

  unless ($bn->check_limit_post_view('Paint')) {
    $data->{applet_form} = $bn->applet_form($data);
    $data->{applet_form_target} = $bn->{arg}{applet_form_target};
    $data->{applet_form_target} ||= '_self';
  }

  if ( defined $bn->get($cf, 'HomePageUrlTarget') ) {
    my $ta = $bn->get($cf, 'HomePageUrlTarget');
    $bn->escape_quote( \$ta );
    $data->{menu_top_target} = ' target="' . $ta . '"';
  }

  if ($bn->get($cf, 'UseSearch')) {
    $data->{use_search} = 1;
    $data->{search_form} = sub{ $bn->search_form_init($data) }
  }

  $bn->tree_link;

  if ($bn->{sys}{MODE} eq 'DELETE') {
    $data->{top_form} = sub{ $bn->delete_form; return };
  } elsif ($bn->{sys}{MODE} eq 'EDIT') {
    $data->{top_form} = sub{ $bn->edit_form; return };
  } elsif ($bn->get($sk, 'ViewTopForm')) {
    $data->{top_form} = sub{
      my $f = \$bn->{flag}{POST_FORM_TITLE};
      my $b = $$f; $$f = 1;
      $fl->{ViewTopForm} = 1;
      $fl->{UPLOAD_ENABLE} = $bn->check_limit_up_view;
      $bn->print($bn->create_post_form);
      delete $fl->{ViewTopForm};
      $$f = $b; return;
    };
  }

  $bn->call('SkinMenuTranslate');

  $bn->translate($tmpl->{MENU}, $data);

  $bn->call('SkinMenuEnd');

  $bn;
}

sub main {
  my( $bn, $ref_data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $sys = $bn->{sys};
  my $gl = $bn->{global};
  my $tmpl = $sk->{template};
  my $data = $bn->{data} = {};

  my $val = $gl->{LogValue} = {};
  my $ex = $gl->{LogExValue} = {};

  $bn->get_log_main_value($$ref_data[0], $val, $ex);
  $bn->get_src_value($data, $val, $ex);
  my @edit_record = split ',',$ex->{edit};

  $bn->call('SkinMainInit');

  if ($fl->{ MENUFORM }) {
    $data->{mode_button} = '<input name="log' . ($sys->{MODE} eq 'EDIT' ? '" type="radio" class="radio"' :
     $gl->{IndexNumber} . '" type="checkbox" class="checkbox"').' value="'.$gl->{IndexNumber}.'" onclick="document.menuform.number.value=this.value" onkeypress="document.menuform.submit()">';
  }

  $gl->{title} = $val->{title} if $bn->get($cf, 'UseTree') && $bn->get($cf, 'UseReplyTitle');

  undef $gl->{LogReplyCount};
  $gl->{LogNumber} = $bn->get($sk, 'ViewThreadNumber') eq 'virtual' ? $gl->{ThreadNumber} : $gl->{IndexNumber};

  my $ed_cnt;
  $data->{ip} = $val->{ip};
  $data->{host} = $val->{host};
  $data->{ip_host} = $sys->{MODE} eq 'ADMIN' && $bn->{in}{mode} ne 'delete' ? @edit_record ?
    "$val->{ip}($val->{host})" . '<table cellpadding="0" cellspacing="0"><tr><form><td>'.$bn->set_tag($bn->get($sk, 'TagMainUser'),{string=>'<b>NO. - DATE - IP(HOST)</b> : '}).'<select class="select">'.
    (join '',map{;sprintf("<option>%02d - %s -  %s</option>",(@edit_record-($ed_cnt++)),split '@')}
    reverse(@edit_record,"$val->{date}\@$val->{ip}($val->{host})")).'</select></td></form></tr></table>':
    "$val->{ip}($val->{host})":'';

  # view first date
  $val->{date} = (split /\@/,$edit_record[0],2)[0] if $bn->get($cf, 'ViewFirstDate') && @edit_record;

  $bn->bsys_init($data)->data_init($data)->tag_init($data);

  $data->{'image_align_' . lc $bn->get($sk, 'ViewImagePos')} = 1;
  $data->{agent} = $val->{agent} if $bn->get($sk, 'ViewAgent');
  $data->{edit} = @edit_record if $bn->get($sk, 'ViewEditCount') && @edit_record;

  $bn->src_init;

  if ($fl->{reply_enable} && $sys->{MODE} ne 'REPLY'){
    $data->{reply_form} = $bn->reply_form( $bn->get($sk, 'FormType'), $data);
    $data->{reply_form_init} = sub{ $bn->reply_form_init(@_); '' };
  } elsif (!$fl->{reply_enable} && !$fl->{limit_post_view} && $sys->{MODE} !~ /EDIT|DELETE/ &&
   !($sys->{MODE} eq 'ADMIN' && exists $bn->{in}{cmd} && $bn->{in}{cmd} eq 'delete_wide') ) {
    $data->{reply_disable} = 1;
  }

  # catalog mode
  if($bn->{in}{mode} eq 'catalog') { $bn->catalog($data); return }

  $bn->call('SkinMainBegin');

  my $templ = 'MAIN';

  if (
    $bn->get($cf, 'UseTree') &&
    $sys->{MODE} =~ /^(?:$|EDIT|DELETE)/ &&
    $bn->{in}{mode} =~ /^(?:$|normal)/ &&
    $bn->{in}{frame} ne '1'
  ) {
    unless ($sys->{DATESTR}) {
      $sys->{DATESTR} = $bn->get_date( (time +$bn->get($cf, 'TimeDiffSec')) - ($bn->get($cf, 'ViewTreeNewLog') * 3600) );
      $sys->{DATESTR} =~ s/\D//g;
      $sys->{DATESTR} = int $sys->{DATESTR};
    }
    my $logdate = int join '',@$data{qw/year month day hour min sec/};
    $data->{view_new_aticle} = 1 if $sys->{DATESTR} < $logdate;

    # cut title
    my $maxt = $bn->get($cf, 'ViewMaxTitle');
    if ($maxt && $maxt < length $data->{title})
     { $data->{title} = substr( $data->{title}, 0, $maxt ). '..' }

    $bn->{arg}{aticle_link} = { fc => 'thread', res => 0, reply => 1,
      log => $data->{index_number}, no => $data->{log_number},
      page => $bn->{in}{page}, mode => $bn->{in}{mode}, sort => $bn->{in}{sort} };
    $bn->{arg}{thread_link} = { fc => 'thread', reply => 1,
      log => $data->{index_number}, no => $data->{log_number},
      page => $bn->{in}{page}, mode => $bn->{in}{mode}, sort => $bn->{in}{sort} };
    $data->{tree_link_target} = $bn->{arg}{tree_link_target};
    $data->{aticle_link} = $bn->get($cf, 'BBSNoteUrl') . '?' . $bn->set_text_arg($bn->{arg}{aticle_link});
    $data->{thread_link} = $bn->get($cf, 'BBSNoteUrl') . '?' . $bn->set_text_arg($bn->{arg}{thread_link});
    $sys->{tree_no}{ $val->{num} } = 0;
    $templ = 'TREE';
  }
  $data->{main} = 1;
  $data->{object} = sub{$bn->translate($sk->{template}{OBJECT}, $data, 'Main');return};
  $bn->translate($sk->{template}{$templ . '_BEGIN'}, $data, 'Main');
  $bn->reply($ref_data);
  $data->{reply_count} = $gl->{LogReplyCount};
  $bn->translate($sk->{template}{$templ . '_END'}, $data, 'Main');

  $bn->call('SkinMainEnd');
  $bn;
}

sub reply {
  my( $bn, $ref_data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $sys = $bn->{sys};
  my $gl = $bn->{global};
  my $res_total = $gl->{LogReply};
  my $res_view_end = $#$ref_data;
  my $res_view_begin = 1;
  my $view_limit_reply = $fl->{use_tree_frame} && $bn->{in}{frame} eq '2' ? 0 : $bn->get($cf, 'LogViewReply');
  my %omission;
  my $margin = 1;
  $bn->call('SkinReplyInit');

  # number reply
  if ($bn->{in}{res} =~ /(\d+)\-?(\d+)?/) {
    $res_view_begin  = $1 < $res_total ? $1 : $res_total;
    $res_view_end  = $2 ? $2 < $res_total ? $2 : $res_total : $res_view_begin ;
    return if $res_view_begin == 0 && $res_view_end == 0;
    $res_view_begin = 1 if $1 < 1;

  # limit view reply
  } elsif($view_limit_reply && $view_limit_reply < $res_total) {
    $res_view_begin = $res_total - $view_limit_reply;
    # thread args
    $bn->{arg}{Arg} = { fc => 'thread', log => $gl->{IndexNumber}, res => "0-$res_total",
      no   => ($sys->{MODE} eq "REPLY" ? $gl->{LogNumber}:''),
      edit   => ($sys->{MODE} eq "EDIT" ? 1:''),
      admin  => ($sys->{MODE} eq "ADMIN" ? 1:''),
      reply  => ($sys->{MODE} eq "REPLY" ? 1:''),
      delete => ($sys->{MODE} eq "DELETE" ? 1:'')
    };
    $bn->call('SkinReplyTreadArg');
    $omission{link} = $bn->get($cf, 'BBSNoteUrl') . '?' . $bn->set_text_arg($bn->{arg}{Arg});
    $omission{log} = $res_view_begin;
    $res_view_begin++;
  }

  for (my $number = $res_view_begin, $gl->{LogReplyCount} = 0;
     $number <= $res_view_end; $number++, $gl->{LogReplyCount}++) {

     my $data = $bn->{data} = {};
     my $val = $gl->{LogValue} = {};
     my $ex = $gl->{LogExValue} = {};

     $bn->get_log_reply_value($ref_data->[$number], $val, $ex);
     $bn->get_src_value($data, $val, $ex);

     $data->{'image_align_' . lc $bn->get($sk, 'ViewReplyImagePos')} = 1;
     $data->{omission} = $omission{link};
     $data->{omissionlog} = $omission{log};

     # edit count
     my @edit_record = split ',', $ex->{edit};

     $bn->call('SkinReplyBegin');

     # delete mode : view radio bottun
     if ($fl->{MENUFORM}) {
       $data->{mode_button} = '<input name="log' . ($sys->{MODE} eq 'EDIT' ? '" type="radio" class="radio"' :
      $gl->{IndexNumber}.'-'.$val->{num}.'" type="checkbox" class="checkbox"') . ' value="'.
      $gl->{IndexNumber}.'-'.$val->{num}.'" onclick="document.menuform.number.value=this.value"'.
      ' onkeypress="document.menuform.submit()">';
       $bn->set($sk, 'ViewThreadNumber', 'real');
     }
     if (!exists $gl->{title} && exists $ex->{title} && $bn->get($cf, 'UseTree') && $bn->get($cf, 'UseReplyTitle')) {
      $gl->{title} = $ex->{title};
     }

     # view first date
     $val->{date} = (split /\@/,$edit_record[0],2)[0] if $bn->get($cf, 'ViewFirstDate') && @edit_record;

     $bn->bsys_init($data)->data_init($data)->tag_init($data);

     # view local reply disable form
     if ($fl->{reply_mode} && $fl->{view_local} &&
       !$fl->{reply_enable} && !$fl->{limit_post_view} &&
       $number == $res_view_end && $sys->{MODE} ne 'EDIT')
     {
       $data->{reply_disable} = 1;
     }

     $data->{reply_view_begin} = $res_view_begin;
     $data->{reply_view_end} = $res_view_end;
     $data->{reply_number} = $number;
     $data->{agent} = "$val->{browser} / $val->{os}" if $bn->get($sk, 'ViewAgent');
     $data->{edit} = @edit_record if $bn->get($sk, 'ViewEditCount') && @edit_record;
     my $ed_cnt;
     $data->{ip} = $val->{ip};
     if ( $val->{iphost} =~ /([\d\.]+)\(([^\)]+)/ ) { $data->{ip} = $1; $data->{host} = $2 }
     $data->{ip_host} = $sys->{MODE} eq 'ADMIN' && $bn->{in}{mode} ne 'delete' ? @edit_record ?
       $val->{iphost} . '<table cellpadding="0" cellspacing="0"><tr><form><td>' . $bn->set_tag($bn->get($sk, 'TagResUser'), {string=>'<b>NO. - DATE - IP(HOST)</b> : '}) .
       '<select class="select">' . (join '', map{sprintf("<option>%02d - %s -  %s</option>",(@edit_record-($ed_cnt++)),split '@')}
       reverse(@edit_record,"$val->{date}\@$val->{iphost})")) . '</select></td></form></tr></table>' : $val->{iphost}:'';

     $bn->src_init;
     my $templ = 'REPLY';

     if ( $bn->get($cf, 'UseTree') && $sys->{MODE} =~ /^(?:$|EDIT|DELETE)/ &&
          $bn->{in}{mode} =~ /^(?:$|normal)/ && $bn->{in}{frame} ne '1'
     ) {
       unless ($sys->{DATESTR}) {
         $sys->{DATESTR} = $bn->get_date( (time +$bn->get($cf, 'TimeDiffSec')) - ($bn->get($cf, 'ViewTreeNewLog') * 3600) );
         $sys->{DATESTR} =~ s/\D//g;
         $sys->{DATESTR} = int $sys->{DATESTR};
       }
       my $logdate = int join '',@$data{qw/year month day hour min sec/};
       $data->{view_new_aticle} = 1 if $sys->{DATESTR} < $logdate;

       # cut title
       my $maxt = $bn->get($cf, 'ViewMaxTitle');
       if ($maxt && $maxt < length $data->{title}) {
         $data->{title} = substr $data->{title}, 0, $maxt;
         $data->{title} .= '..';
       }

       $bn->{arg}{aticle_link} = { fc => 'thread', log => $data->{index_number},
                                   no => $data->{log_number}, page => $bn->{in}{page}, mode => $bn->{in}{mode},
                                   sort => $bn->{in}{sort}, res => $number, reply => 1
                                 };

       $data->{tree_link_target} = $bn->{arg}{tree_link_target};
       $data->{aticle_link} = $bn->get($cf, 'BBSNoteUrl') . '?' . $bn->set_text_arg($bn->{arg}{aticle_link});
       if ($ex->{tree}) {
         $sys->{tree_no}{ $val->{num} } = $sys->{tree_no}{ $ex->{tree} } + 1;
         $margin = $sys->{tree_no}{ $val->{num} } + 1;
       } else {
         $sys->{tree_no}{ $val->{num} } = $margin;
         $margin++;
       }
       $data->{tree_child_space} = $bn->get($cf, 'ViewTreeChildSpace') * $sys->{tree_no}{ $val->{num} };

       $templ = 'TREE_REPLY';

     }
     $data->{reply} = 1;
     $data->{object} = sub{$bn->translate($sk->{template}{OBJECT}, $data, 'Res');return};
     $bn->call('SkinReplyValue');
     $bn->translate($sk->{template}{$templ}, $data, 'Res');
     $bn->call('SkinReplyEnd');

     undef %omission;

  }
  $bn;
}

sub frame {
  my( $bn, $caption, $contents ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $tmpl = $sk->{template};
  my $data = $bn->{data} = {};

  $data->{caption} = $caption;
  $data->{contents} = $contents;
  $bn->call('SkinFrameInit');
  $bn->bsys_init($data)->tag_init($data);
  $data->{pagelink} = sub{ $bn->page_link_init($data) };
  $bn->call('SkinFrameBegin');
  $data->{contents} = $bn->set_tag($bn->get($sk, 'TagTableFont'), { string => $data->{contents}});
  $bn->call('SkinFrameTranslate');
  $bn->translate($sk->{template}{FRAME}, $data);
  $bn->call('SkinFrameEnd');
  $bn;
}

sub catalog {
  my( $bn, $data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $bg = $bn->get($sk, 'TagCatalogTableIn');

  if ( ($gl->{IndexPos} + 1) % $bn->get($cf, 'CatalogRow') == 0 &&
     ($gl->{IndexPos} + 1) < $bn->get($cf, 'LogView') )
  { $data->{catalog_next_row} = 1 }
  $bn->catalog_init($data)->translate( $sk->{template}{CATALOG}, $data );
  $bn;
}

sub count {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $data = $bn->{data};
  my $cnt = $bn->get($cf, 'UseCounter');
  return $bn unless $cnt;
  my $pos = $bn->get($cf, 'ViewCounterPos');
  my $mode = $bn->get($cf, 'ViewCounterMode');
  $pos ||= 'msgbottom';
  $mode ||= 'add';
  return $bn if $pos ne $_[1];
  if ( $cnt eq 'text' ) {
    my $val;
    my $hash = { data => $bn->cfg2join($cf, 'TxtCounterText') };
    $hash->{data} ||= '<div class="AccessCount">$AccessTotal</div>';
    $bn->template_trans($hash)->translate($hash, $data, undef, \$val);
    $data->{'txt'.$pos} = $mode eq 'add' ? $data->{'txt'.$pos} . $val : $val . $data->{'txt'.$pos};
  } elsif ( $cnt eq 'image' ) {
    my $val;
    my $hash = { data => $bn->cfg2join($cf, 'TxtCounterImage') };
    $hash->{data} ||= '<div class="AccessCount">$AccessCounterImage</div>';
    $bn->template_trans($hash)->translate($hash, $data, undef, \$val);
    $data->{'txt'.$pos} = $mode eq 'add' ? $data->{'txt'.$pos} . $val : $val . $data->{'txt'.$pos};
  }
  $bn;
}

sub page_link {
  my( $bn, $data, $hash ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $style = $bn->get($sk, 'ViewStylePageLink');
  my $log_view = $bn->get($cf, 'LogView');
  my $view_limit = $bn->get($sk, 'ViewLimitPageLink');
  $data->{bbsnoteurl} = $bn->get($cf, 'BBSNoteUrl');

  $style = 'form' if($bn->{sys}{MODE} =~ /ADMIN/);

  my( $linktext, $linkform );
  my( $fc, $mode, $page, $sort, $pl ) = ( @{$bn->{in}}{'fc', 'mode', 'page', 'sort'}, $view_limit );

  $fc = $mode = '' if $bn->{flag}{RefleshIndex};
  my $maxpage = int( $gl->{LogTotal} / $log_view) + ($gl->{LogTotal} % $log_view ? 1 : 0 );
  return if $maxpage < 2;

  $fc = 'page' if $fc eq '' || $fc eq 'write';

  $page = int $page;
  $page = 1 unless $page > 0;

  $bn->{arg}{PageLink}{fc} = $fc;
  $bn->{arg}{PageLink}{mode} = $mode;
  $bn->{arg}{PageLink}{sort} = $sort;

  $bn->call('CreatePageLinkInit');

  my $arg = $bn->set_text_arg($bn->{arg}{PageLink});

  if ($style eq 'form') {
    my $form_begin = {data => $hash->{form_begin}};
    my $form_option = {data => $hash->{form_option}};
    my $form_end = {data => $hash->{form_end}};
    $bn->template_trans($form_begin, $form_option, $form_end);

    $bn->translate($form_begin, $data);
    my $n;
    while ( $maxpage > 0 ) { $n++; $maxpage--;
      $data->{select} = $n == $page ? ' selected' : '';
      $data->{page} = $n;
      $data->{formpage} = "$data->{bbsnoteurl}?$arg&page=$n";
      $bn->translate($form_option, $data);
    }
    $bn->translate($form_end, $data);
    delete $data->{pagelink_formpage};

  } else {
    my $page_here = {data => $hash->{page_here}};
    my $page_link = {data => $hash->{page_link}};
    my $next_block = $hash->{page_link};
    my $back_block = $hash->{page_link};
    $next_block =~ s/(link=["']?textpage["']?)/$1 text='&lt\;&lt\;'/;
    $back_block =~ s/(link=["']?textpage["']?)/$1 text='&gt\;&gt\;'/;
    $next_block = {data => $next_block};
    $back_block = {data => $back_block};
    $bn->template_trans($page_here, $page_link, $next_block, $back_block);

    my($i, $n, $mblock, $pblock, $pos);
    if ($pl) {
      $mblock = int($maxpage / $pl);
      $pblock = int(($page-1) / $pl);
      $pos = $pblock * $pl;
    }

    if ($pl && $pblock) {
      $n = $pblock * $pl;
      $data->{page} = $n;
      $data->{textpage} = "$data->{bbsnoteurl}?$arg&page=$n";
      $bn->translate($next_block, $data);
    }

    for ( $i = 0, $n = $pos; $i < $maxpage-$pos; $i++ ) {
      $n++;
      if ($n == $page) {
        $data->{page} = $n;
        $data->{textpage} = "$data->{bbsnoteurl}?$arg&page=$n";
        $bn->translate($page_here, $data);
        next;
      }
      if ($pl && $i >= $pl) {
        $data->{page} = $n;
        $data->{textpage} = "$data->{bbsnoteurl}?$arg&page=$n";
        $bn->translate($back_block, $data);
        last;
      }
      $data->{page} = $n;
      $data->{textpage} = "$data->{bbsnoteurl}?$arg&page=$n";
      $bn->translate($page_link, $data);
    }
    delete $data->{pagelink_textpage};
   }
   return;
}

sub back_button {
  my( $bn, $mode ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $in = $bn->{in};
  my $fr = $sk->{form} = {};
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  $fr->{form} = { TAG => 'FORM', action => $bn->get($cf, 'BBSNoteUrl'),
    method => 'post', target => '_self' };

  $fr->{FORMS} = [
  [[
    [ { TAG => 'INPUT', type => 'hidden', name => 'page', value => $in->{page} } ],
    [ { TAG => 'INPUT', type => 'hidden', name => 'mode', value => $in->{mode} } ],
    [ { TAG => 'INPUT', type => 'hidden', name => 'sort', value => $in->{sort} } ]
  ]],[[
    [ { TAG => 'TD', align => 'center' } ],
    [ { TAG => 'INPUT', id => 'BackButton', type => 'submit', class => 'back', value => $item->{back} } ]
  ]]];

  $bn->call('SkinBackButtonInit');
  $bn->set_init_arg($fr->{FORMS}, $bn->{arg}{BackButton});
  return $bn->create_form($fr->{FORMS}, $fr->{form});
}

sub create_form {
  my( $bn, $a, $e, $t ) = @_;
  my( $form, $c, $h );
  my( $mr, $mc ) = $bn->get_form($a);
  my $r = 0;

  $t ||= { TAG => 'TABLE' };

  for ( @$a ) { $c = 0;
    my $f1 = 1; my $fs;
    for ( @$_ ) {
      my $f2 = 1; my $fv;
      $fs.="<td";
      $bn->end('DEFF_FORM_ARRAY_TYPE_ROW') if ref ne 'ARRAY';
      for ( @$_ ) {
        my @array; my $cref = ref;
        $bn->get_ref( $_, \@array );
        for my $rf ( @array ) {
          if ($cref eq 'ARRAY' && $$rf{TAG} eq 'TD') {
            delete $$rf{TAG};
            $$rf{rowspan}=$mr if $$rf{rowspan} eq '*';
            $$rf{colspan}=$mc if $$rf{colspan} eq '*';
            delete $$rf{rowspan} if $$rf{rowspan}<2;
            delete $$rf{colspan} if $$rf{colspan}<2;
            for (keys %$rf) { $fs .= $_ eq 'nowrap'?" $_":qq/ $_="$$rf{$_}"/ }
            undef %$rf;
            next;
          }
          if ($f2) { $fs.=">"; $f2 = 0 }
          $f1 = 0 if $$rf{type} eq 'hidden';
        }
        unless ($f1) { $h .= "\n". $bn->set_tag(@array) }
        elsif (@array) { $fv .= $bn->set_tag(@array) }
      }
      $f1 = 0 if (!$fv);
      $fs .= $fv."</td>\n";
    }
    $form.="<tr>\n$fs</tr>\n" if $f1 && $fs;
  }
  return $bn->set_tag( $t,{string=>$bn->set_tag($e,{string=>$h.$form})} )."\n";
}

sub create_menu_form {
  my( $bn, $fc, $string, $arg ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $in = $bn->{in};
  my $ck = $bn->{cookie};
  my $fr = $sk->{form};
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  my $tag_form_items = $bn->get($sk, 'TagFormItems');

  my $target = '_self';

  if ( $bn->get($cf, 'UseTree') && $bn->get($cf, 'UseTreeFrame') && exists $in->{frame} ) {
    if ($bn->{sys}{MODE} eq 'DELETE') {
      $target = '_parent';
    } elsif ($bn->{sys}{MODE} eq 'EDIT') {
      $target = 'bn_main';
      $bn->{arg}{$arg}{frame} = 1;
    }
  }

  $fr->{form} = { TAG => 'FORM', action => $bn->get($cf, 'BBSNoteUrl'),
    name => 'menuform', method => 'post', target => $target };

  my $hash = ref $bn->get($sk, 'TagTableFont') eq 'HASH' ? $bn->get($sk, 'TagTableFont') : {};
  my $style = 'text-align:center;ime-mode:disabled' if $bn->{global}{UserBrowserVer} ne 'NN 4';
  $fr->{FORMS} = [[[
    [ $fc ne 'login' ? { TAG => 'INPUT', type => 'hidden', name => 'name', value => $ck->{name} }:'' ]
  ]],[[
    [ { TAG => 'TD', align => 'center' } ],
    [ $hash, {  string => $string } ],
    [ $hash, { string => ($fc =~ /login/ ? $item->{name} : $item->{number}) . ' ' } ],
    [ { TAG => 'INPUT', type => 'text', name => ($fc =~ /login/ ? 'name' : 'number'), class => 'input', style => ($fc ne 'login' ? $style : ''), size => '10', value => ($fc ne 'login' ? '' : $ck->{name}) } ],
    [ $hash, { string => " $item->{password} " } ],
    [ { TAG  => 'INPUT', type => 'password', name => 'password', class => 'input', style => $style, size => '10', value => $in->{password} ? $in->{password} : $ck->{password} } ],
    [ { string => " " } ],
    [ { TAG => 'INPUT', type => 'submit', class => 'submit', value => $item->{$fc} } ]
  ]]];

  my $h = $bn->{arg}{$arg};
  for (qw/fc mode sort page/) {
    $h->{$_} = $in->{$_} if !exists $h->{$_} && exists $in->{$_};
  }

  $bn->call('SkinCreateMenuFormInit');
  $bn->set_init_arg($fr->{FORMS},$h);
  my $form = $bn->create_form($fr->{FORMS}, $fr->{form});
  $form =~ s#</form>## if $bn->{sys}{MODE} =~ /^(?:DELETE|EDIT)$/;

  $bn->frame('', $form);
}

sub create_applet_form {
  my( $bn, $data ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $ck = $bn->{cookie};
  my $mo = $bn->{module};
  return unless exists $mo->{Paint};
  my $pm = $mo->{Paint};
  my $app = $pm->{applets};
  my $ly = $pm->{layout};
  $bn->call('CreateAppletFormBegin');

  $data->{applet_form_begin} = $bn->set_tag({ TAG => 'FORM', name => 'appform', method => 'post',
                                              action => $bn->get($cf, 'BBSNoteUrl'), noclose => 1 });
  # canvas
  if($bn->get($cf, 'CanvasMode') eq 'text'){
    $data->{applet_form_canvas_text} = 1;
    $data->{applet_form_canvas_width} = $ck->{pmcw} ? $ck->{pmcw} : $bn->get($cf, 'DefaultCanvasWidth');
    $data->{applet_form_canvas_height} = $ck->{pmch} ? $ck->{pmch} : $bn->get($cf, 'DefaultCanvasHeight');
  } elsif($bn->get($cf, 'CanvasMode') eq 'select') {
    my $min_w = $bn->get($cf, 'CanvasSelectMinWidth');
    my $min_h = $bn->get($cf, 'CanvasSelectMinHeight');
    my $max_w = $bn->get($cf, 'CanvasSelectMaxWidth');
    my $max_h = $bn->get($cf, 'CanvasSelectMaxHeight');
    my $var_w = $bn->get($cf, 'CanvasSelectVarWidth');
    my $var_h = $bn->get($cf, 'CanvasSelectVarHeight');

    $min_w ||= 100; $min_h ||= 100;
    $max_w ||= 400; $max_h ||= 400;
    $var_w ||= 50;  $var_h ||= 50;
    my $val;

    my $w .= '<select id="AppletCanvasWidthSelect" class="select" name="width">';
    for ($val = $min_w; $val <= $max_w; $val += $var_w) {
      $w .= '<option value="'.$val.'"' .
      ($val == ($ck->{pmcw} ? $ck->{pmcw} : $bn->get($cf, 'DefaultCanvasWidth')) ? ' selected' : '') . ">$val</option>";
    }
    $w .= '</select>';
    my $h = '<select id="AppletCanvasHeightSelect" class="select" name="height">';
    for ($val = $min_h; $val <= $max_h; $val += $var_h) {
      $h .= '<option value="'.$val.'"' .
      ($val == ($ck->{pmch} ? $ck->{pmch} : $bn->get($cf, 'DefaultCanvasHeight')) ?' selected':'') . ">$val</option>";
    }
    $h .= '</select>';
    $data->{applet_form_canvas_select} = 1;
    $data->{applet_form_canvas_width} = $w;
    $data->{applet_form_canvas_height} = $h;

  } elsif($bn->get($cf, 'CanvasMode') eq 'fix') {
    $data->{applet_form_canvas_fix} = 1;
  }

  $data->{applet_form_submit_onclick} = $ly->{complex} ? ' onclick="PM_getParam()"' :'';

  # palette
  if ($ly->{palette}) {
    my($palette_name, $layer_palette, $layer_anime);
    my $names = $bn->get($cf, 'PaletteNames');
    if (ref $names eq 'ARRAY') {
      $palette_name = $names;
    } else {
      my $item = $pm->{item} = $bn->text('','paint');
      push @$palette_name, "\t$item->{default}";
      $bn->get_palette('', $palette_name);
    }
    $data->{applet_form_palettename} = $palette_name;

    $data->{applet_form_palette_select} ='<select id="AppletPaletteSelect" name="'.($ly->{palette}==2?'val':'palette').'" class="select">';
    for (@$palette_name) {
      my($key, $name) = split "\t";
      $data->{applet_form_palette_select} .= '<option value="'.$key.'"'.($key eq $ck->{pmp}?' selected':'').">$name</option>";
    }
    $data->{applet_form_palette_select} .= '</select>';

    $data->{ $ly->{anime} == 2 ?
       'applet_form_palette_span' : 'applet_form_palette_fix' } = 1;
  }


  # anime checkbox
  if ($ly->{anime} && $bn->get($cf, 'ConfigShiApplet')->{animation} eq 'user') {
    my $ch = ' checked';
    if (exists $ck->{pma}) {
      $ch = '' unless $ck->{pma};
    } elsif(!$bn->get($cf, 'ConfigShiApplet')->{default_anime}) {
      $ch = '';
    }
    $data->{applet_form_anime_check} = $ch;
    $data->{ $ly->{anime} == 2 ?
       'applet_form_anime_span' : 'applet_form_anime_fix' } = 1;
  }

  $bn->call('CreateAppletFormEnableApp');

  my $arg = $bn->{arg}{PaintAppForm};
  return unless ref $pm->{applets} eq 'ARRAY';

  if (@{$pm->{applets}} > 1) {
    my $s;
    for (@{$pm->{applets}}) {
      $s .= '<option value="paint_' . $_->{key} . '"' .
      ($_->{key} eq $ck->{pmapp} ? ' selected' : '') . '>' .
      $_->{name} . '</option>';
    }
    my $form_app = [[
      { string => '<select id="AppletToolSelect" name="fc" class="select"'.
      ($ly->{complex} ? ' onchange="PM_changeApp()"':'').
      '>'.$s.'</select>' }
    ]];
    $data->{applet_form_tool_select} = $bn->set_tag(\$form_app);

  } elsif(@{$pm->{applets}} == 1) {
    $arg->{fc} = 'paint_' . ${$pm->{applets}}[0]->{key};
  } else {
    return;
  }

  my $name = exists $in->{name} ? $in->{name} : exists $ck->{name} ? $ck->{name} : exists $ck->{adminuser} ? $ck->{adminuser} : undef;
  my $pass = exists $in->{password} ? $in->{password} : exists $ck->{password} ? $ck->{password} : exists $ck->{adminpass} ? $ck->{adminpass} : undef;
  $arg->{name} = $name if defined $name;
  $arg->{password} = $pass if defined $pass;
  $arg->{anime} = '' if $ly->{anime} == 2;
  $arg->{palette} = '' if $ly->{palette} == 2;

  $data->{applet_form_arg} = $bn->set_form_arg($arg);

  $data->{complex} = $ly->{complex};
  if ($ly->{complex}) {
    $bn->set_call('SkinFootTranslate', sub{ $bn->print('<script type="text/javascript">PM_changeApp()</script>') if $bn->{module}{Paint}{app_script} });
  }

}

sub create_post_form {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $in = $bn->{in};
  my $ck = $bn->{cookie};
  my $fr = $sk->{form} = {};
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my( $info_size, $info_exe, $info_pass );

  $bn->post_form_init({
    fc       => 'write',
    mode     => $in->{mode},
    sort     => $in->{sort},
    page     => $in->{page},
    title    => $in->{title},
    message  => $in->{message},
    url      => $in->{url} ? $in->{url} : $ck->{url},
    name     => $in->{name} ? $in->{name} : $ck->{name},
    mail     => $in->{mail} ? $in->{mail} : $ck->{mail},
    password => $in->{password} ? $in->{password} :$ck->{password},
  });

  $bn->{flag}{FormInfo} = 1;
  ($info_pass) = $bn->info('POST_PASSWORDINFO');
  if ($bn->{flag}{UPLOAD_ENABLE}) {
    ($info_size) = $bn->info('POST_UPLOADSIZEINFO',$bn->get($cf, 'LimitUpLoadData') > 1024 ? int($bn->get($cf, 'LimitUpLoadData') / 1024) : 1, $bn->get($cf, 'LimitImageWidth'), $bn->get($cf, 'LimitImageHeight'));
    ($info_exe) = $bn->info('POST_UPLOADEXEINFO',(join 'E',(map uc "<nobr>$_</nobr>",split ',',$bn->{sys}{CaptorFormat})));
  }
  $fr->{info} = [[
    [ { TAG => 'TD', colspan => '*', id => 'PostFormInfo' } ],
    [ $tag_form_items, { string => join '',map{"* $_<br>" if $_} $info_size, $info_exe,@{ $bn->{global}{FormInfo} }, $info_pass }]]];

  my $parent = 1 if (
    ( $bn->get($cf, 'UseTree') && $bn->get($cf, 'UseTreeFrame') ) &&
    (
      ( $in->{fc} eq 'post' && $ck->{frametype} ne '0') ||
      ( $bn->{sys}{MODE} eq 'REPLY' && $in->{fc} eq 'thread' && $ck->{frametype} ne '0') ||
      ( $bn->{sys}{MODE} eq 'EDIT' &&  $in->{frame})
    )
  );

  $bn->call('SkinCreateFormInfo');

  $bn->set_form($fr->{FORMS}, $fr->{info}, 'add') if $bn->{flag}{FormInfo};
  $bn->set_init_arg($fr->{FORMS},$bn->{arg}{PostForm});
  $fr->{table} = ref $bn->get($sk, 'FormTable') eq 'HASH' && keys %{ $bn->get($sk, 'FormTable')}
    ? $bn->get($sk, 'FormTable') : { id => 'PostFormTable', width => '400', cellpadding => '2', cellspacing => '2' };

  $fr->{table}->{TAG} = 'TABLE';
  $fr->{form} = { TAG => 'FORM', action => $bn->get($cf, 'BBSNoteUrl'), name => 'cgiform', method => 'post', enctype => 'multipart/form-data',
     target => ($parent ? '_parent' : '_self') };

  $bn->call('SkinCreateFormEnd');

  return $bn->create_form($fr->{FORMS}, $fr->{form}, $fr->{table});

}

sub reply_form_init {
  my ( $bn, $data, $hash ) = @_;
  my $sk = $bn->{skin};
  if (exists $hash->{form_submit}) {
    my $submit = { data => $hash->{form_submit} };
    $bn->template_trans($submit);
    $bn->translate($submit, $data, undef, \$data->{reply_form_submit});
    $data->{reply_form} = $bn->reply_form($bn->get($sk, 'FormType'), $data);
    delete $data->{reply_form_submit};
  }
}

sub reply_form {
  my( $bn, $mode, $data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $fr = $sk->{form} = {};
  my $item = $bn->{item} = $bn->get($sk, 'FormName');

  my $arg = $bn->{arg}{ReplyForm} = {};
  $arg->{mode} = $bn->{in}{mode};
  $arg->{sort} = $bn->{in}{sort};
  $arg->{page} = $bn->{in}{page};
  $arg->{log} = $gl->{IndexNumber};
  $arg->{no} = $gl->{LogNumber};

  my %values = ( name => $ck->{name}, mail => $ck->{mail},
                 url => $ck->{url}, password => $ck->{password} );

  $fr->{form} = { TAG => 'FORM', method => 'post',
                  enctype => 'multipart/form-data', action => $bn->get($cf, 'BBSNoteUrl') };

  $bn->{flag}{UPLOAD_ENABLE} = $bn->check_limit_up_view('Reply');

  $bn->call('ReplyFromBegin');

  if ($mode eq 'thread') {
    $arg->{fc} = 'write';
    $bn->thread_form_init(\%values);
  } elsif ($mode eq 'line') {
    $arg->{fc} = 'write';
    $bn->line_form_init(\%values);
  } elsif ($mode eq 'layer') {
    if ((caller 1)[3]=~/head/) {
      $arg->{fc} = 'write';
      $arg->{log} = '';
      $fr->{form}{name} = 'layerform';
      $bn->layer_form_init(\%values);

      my $table_in = $bn->get($sk, 'TagTableIn');
      my $table_out = $bn->get($sk, 'TagTableOut');
      $bn->set_init_arg($fr->{FORMS}, $arg);

      $fr->{table} = { TAG => 'TABLE', id => 'LayerFormTable',
                       width => '400', border => '0',
                       cellpadding => '2', cellspacing => '0',
                       bgcolor => $table_in->{bgcolor} };

      my $form = $bn->create_form($fr->{FORMS}, $fr->{form}, $fr->{table});
      $form = "<tr><td>$form</td></tr>";
      $form = $bn->set_tag({ TAG => 'TABLE', width => '400', border => '0',
                             cellpadding => '1', cellspacing => '0',
                             bgcolor => $table_out->{bgcolor}, string => $form });
      return $bn->set_tag({ TAG => 'DIV', id => 'Comment', string => $form,
        style  => 'position:absolute; width:410px; height:250px; z-index:1; left: -1000px; top: 0px; visibility: hidden'
        }),$item->{submit};
    } else {
      my $t = $gl->{LogValue}{title}?$gl->{LogValue}{title}:$item->{notitle};
      $t =~ s/(['&])/\\$1/g;
      my @s = ('<table class="'. 'ReplyForm' . ucfirst $mode. '">','</table>') unless exists $data->{reply_form_submit};
      return $s[0].'<form><tr><td><font size="2"><input class="reply" type="button" value="'.
      $item->{reply}.'" onclick="LF_open('.$gl->{IndexNumber}.','.$gl->{ThreadNumber} .',event,\''.$t.'\')"></font></td></tr></form>'.$s[1];
    }
  } else {
    $arg->{fc} = 'post';
    $fr->{FORMS} = [[[]]];

    if (exists $data->{reply_form_submit}) {
      $fr->{submit} = [[[ {string=>$data->{reply_form_submit}} ]]];
    } else {
      $fr->{submit} = [[[ { TAG => 'INPUT', type => 'submit', class => 'reply', value => $item->{reply} } ]]];
    }
    $bn->set_form($fr->{FORMS}, $fr->{submit}, 'add');
  }

  $bn->set_init_arg($fr->{FORMS}, $arg);
  my $div = {};
  my $form_table = {};
  unless (exists $data->{reply_form_submit}) {
    $div = { TAG => 'DIV', align => 'right', class => 'ReplyForm' . ucfirst $mode };
    $form_table = { TAG => 'TABLE', cellspacing => 0};
  }
  $bn->call('ReplyFromEnd');
  return $bn->set_tag($div,{ string => $bn->create_form($fr->{FORMS}, $fr->{form}, $form_table) });
}

sub applet_form {
  my( $bn, $data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $mo = $bn->{module};
  return unless exists $mo->{Paint};

  $bn->call('SkinAppletFormBegin');
  $bn->create_applet_form($data);
  $bn->call('SkinAppletFormEnd');
  return 1;
}

sub delete_form {
  my $bn = $_[0];
  $bn->{flag}{MENUFORM} = $bn->{sys}{MODE};
  $bn->call('SkinDeleteFormInit');
  $bn->create_menu_form('delete',$bn->info('SKIN_FORM_DELETE'),'DeleteForm');
}

sub edit_form {
  my $bn = $_[0];
  $bn->{flag}{MENUFORM} = $bn->{sys}{MODE};
  $bn->call('SkinEditFormInit');
  $bn->create_menu_form('edit',$bn->info('SKIN_FORM_EDIT'),'EditForm');
}

sub read_config {
  my( $bn, $cf, $cfg_file ) = @_;
  return unless defined $cfg_file;
  local( $_, $/ );
  $/ = "\n";
  my $fh = $bn->open($cfg_file);
    $fh or $bn->error('ERR_CONFIG_NOTFOUND', $cfg_file);
  while (<$fh>) {
    chomp;
    next if !/\S/ || /^#/;
    my($var, $val) = $_ =~ /^\s*(\S+)(?: +|\t+)(.+)$/;
    $val =~ s/\s*$//;
    next unless $var ne '' && defined $val;

    $bn->set($cf, $var, $val);
  }
  close $fh;
  $bn;
}

sub read_skin {
  my( $bn, $skin_file ) = @_;
  my $sk = $bn->{skin};
  my( $label, $tmpl );

  local($_, $/);
  $/ = "\n";

  my $fh = $bn->open($skin_file);
  $fh or $bn->error('ERR_SKIN_NOTFOUND',$skin_file);

  while (<$fh>) {
    if ( /^#\s*(\w+):(\w+)/ ) {
      if ( $1 eq 'CONFIG' ) {
          $label = $2 eq 'END' ? 0 : $1;
      } elsif ( $1 eq 'TEMPLATE' ) {
          $label = $2 eq 'END' ? 0 : $1;
          next if $label eq 'END';
          $sk->{template}{ $2 } = {};
          $tmpl = $2;
          next;
      }
    }
    if( $label eq 'CONFIG' ){
      chomp;
      next if !/\S/ || /^#/;
      my($var, $val) = $_ =~ /^\s*(\S+)\s+(.+)$/;
      $val =~ s/\s*$//;
      next unless $var ne '' && defined $val;
      $bn->set($sk, $var, $val);
    } elsif ( $label eq 'TEMPLATE' ){
      $sk->{template}{ $tmpl }{data} .= $_;
    }
  }
  close $fh;
  for ( keys %{$sk->{template}} ) { chomp $sk->{template}{$_}{data} };
  $bn;
}

sub tag_init {
  my( $bn, $data ) = @_;
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $val = $gl->{LogValue};
  my $ex = $gl->{LogExValue};
  my $vars = $sk->{__var};

  for ( keys %$vars ) {
    next unless /^Tag/;
    my $t = lc $_; my $v = $bn->get( $sk, $_ );
    next if ref $v ne 'HASH';
    for ( keys %$v ) { $data->{$t . lc $_} = $v->{$_} }
  }
  $bn;
}

sub bsys_init {
  my( $bn, $data ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $sys = $bn->{sys};
  my $cw = $sys->{Comet};

  if ( $fl->{RefleshIndex} ) {
    $data->{mode_reflesh} = 1;
  } elsif ( $fl->{CreateHtml} ) {
    $data->{mode_html} = 1;
  } else {
   for (keys %$ck) { $data->{'cookie'.$_} = $ck->{$_} }
   $data->{admin} = $bn->{flag}{ADMIN};
   $data->{fc} = lc $in->{fc};
   $data->{'fc_' . lc $in->{fc}} = 1;
   $data->{mode} = $sys->{MODE};
   $data->{'mode_' . lc $sys->{MODE}} = 1;
   $data->{view} = $in->{mode};
   $data->{sort} = $in->{sort};
   $data->{page} = $in->{page};
   $data->{useragent} = $gl->{UserAgent};
   $data->{useros} = $gl->{UserOS};
   $data->{userbrowser} = $gl->{UserBrowser};
   $data->{userbrowserver} = $gl->{UserBrowserVer};
   $data->{userip} = $ENV{REMOTE_ADDR};
   $data->{userhost} = $ENV{REMOTE_HOST};
  }

  $data->{lang} = $sys->{Language};
  $data->{log} = $in->{log} if exists $in->{log};
  $data->{res} = $in->{res} if exists $in->{res};
  $data->{data} = $bn->get($cf, 'DirFileUrl');
  $data->{images} = $bn->get($cf, 'DirImagesUrl');
  $data->{bbsnoteurl} = $bn->get($cf, 'BBSNoteUrl');
  $data->{skindesign} = $bn->get($cf, 'SkinDesign');
  $data->{skindesignpattern} = 'rgb'
   if ( defined $bn->get($sk, 'SkinDesignPattern') ? $bn->get($sk, 'SkinDesignPattern') :
    defined $bn->get($cf, 'SkinDesignPattern') ? $bn->get($cf, 'SkinDesignPattern') : undef ) eq '0';
  $data->{skindesignpattern} = 'hsb'
   if ( defined $bn->get($sk, 'SkinDesignPattern') ? $bn->get($sk, 'SkinDesignPattern') :
    defined $bn->get($cf, 'SkinDesignPattern') ? $bn->get($cf, 'SkinDesignPattern') : undef ) eq '1';
  $data->{skinname} = $bn->get($sk, 'SkinName');
  $data->{skinversion} = $bn->get($sk, 'SkinVersion');
  $data->{skinlastmodify} = $bn->get($sk, 'SkinLastmodify');
  $data->{skincopyright} = $bn->get($sk, 'SkinCopyright');
  $data->{skinauthor} = $bn->get($sk, 'SkinAuthor');
  $data->{skinauthorsiteurl} = $bn->get($sk, 'SkinAuthorSiteURL');
  $data->{clientcopyright} = $data->{toolscopyright} = $bn->get($cf, 'PaintAppCopy');

  $data->{bbscopyright} = $bn->version_slug;

  $data->{comet} = exists $cw->{comet} ? $cw->{comet} : 'BBS NOTE Comet';
  $data->{cometcopyright} = exists $cw->{cometcopyright} ? $cw->{cometcopyright} : 'Powered by WonderCatStudio';
  $data->{cometcopyrightsite} = exists $cw->{cometcopyrightsite} ? $cw->{cometcopyrightsite} : 'http://wondercatstudio.com/';

  if ( $bn->get($cf, 'UseCounter') ) {
    $data->{viewcounter} = $bn->get($sk, 'ViewCounter');
    $gl->{accesslog} = [] if ref $gl->{accesslog} ne 'ARRAY';
    $data->{accesscounterimage} = '<img src="'.$data->{bbsnoteurl}.'?fc=access_count">';
    @$data{qw/accessunique accesstotal accesstoday accessyesterday
      accesshistory1 accesshistory2 accesshistory3 accesshistory4 accesshistory5/} = @{$gl->{accesslog}}
  }

  if ( $VERSION =~ /(\d+)\.(\d+)\.(\w+)/ ) {
    $data->{cometversion} = "$1.$2";
    $data->{cometversionmejor} = $1;
    $data->{cometversionminer} = $2;
    $data->{cometversionbuild} = $3;
  }
  $bn;
}

sub date_init {
  my( $bn, $data, $date ) = @_;
  my $sk = $bn->{skin};
  if ( $date =~ /(\d*)\/(\d\d)\/(\d\d) \((\w*)\) (\d\d):(\d\d):?(\d\d)?/ ) {
    @$data{qw/year month day week hour min sec/} = ($1,$2,$3,$4,$5,$6,$7);
    @$data{qw/int_month int_day int_hour int_min int_sec/} = (int $2,int $3,int $5,int $6,int $7);
    my @em = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
    my $jw = $bn->get($sk, 'Week'); $jw ||= {};
    $data->{jp_week} = $jw->{$data->{week}};
    $data->{en_month} = $em[$data->{int_month}-1];
    $data->{short_year} = $1 if $data->{year} =~ /(\d\d)$/;
  }
  $bn;
}
sub data_init {
  my( $bn, $data, $val, $ex ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  $val = $gl->{LogValue} unless defined $val;
  $ex = $gl->{LogExValue} unless defined $ex;

  $bn->date_init( $data, $val->{date} );

  # reply total
  $data->{reply_total} = $gl->{LogReply};
  $data->{reply_count} = $gl->{LogReplyCount};

  # view limit
  $data->{view_limit_image_width} = $bn->get($cf, 'ViewLimitImageWidth');
  $data->{view_limit_image_height} = $bn->get($cf, 'ViewLimitImageHeight');

  # form ex : skin config
  $data->{'form_type_' . $bn->get($sk, 'FormType')} = 1;

  # flag : reply form
  $data->{reply_enable} = $fl->{reply_enable};
  $data->{reply_disable} = $fl->{reply_disable};

  # flag : local view of the reply
  $data->{view_local} = $fl->{view_local};
#  # flag : view reply mode
#  $data->{view_reply_mode} = $fl->{view_reply_mode};
  # flag : spport checked
  $data->{view_kaiketu} = $ex->{kaiketu} if $bn->get($cf, 'UseTreeSppCheck');
  # real or virtual log number : view config
  $data->{number} = $gl->{LogNumber} if $bn->get($sk, 'ViewThreadNumber') ne 'off';
  # index position
  $data->{index} = $gl->{Index};
  # index position
  $data->{index_pos} = $gl->{IndexPos};
  # real or virtual log number
  $data->{log_number} = $gl->{LogNumber};
  # real log number
  $data->{index_number} = $gl->{IndexNumber};
  # virtual log number
  $data->{thread_number} = $gl->{ThreadNumber};
  # this thread link
  $data->{thread_link} = $data->{bbsnoteurl} . '?' . $bn->set_text_arg(
    { fc => 'thread', log => $data->{index_number}, no => $data->{log_number}, page => $bn->{in}{page}});

  # message number
  $data->{num} = $gl->{num} = $val->{num};
  $data->{date} = $val->{date};
  $data->{title} = $val->{title} ne '' ? $val->{title} : exists $ex->{title} && $bn->get($cf, 'UseReplyTitle') ? $ex->{title} : undef;
  $data->{name} = $val->{name};
  $data->{mail} = $val->{mail};
  $data->{url} = $val->{url};
  $data->{message} = $val->{message};

  # escape
  for ( qw/title name mail url/ )
   { $bn->escape_quote( \$data->{$_} ) }

  # id
  $data->{id} = $bn->set_id( $val ) if $bn->get( $cf, 'ViewId' );

  # icon
  if ( $bn->get($cf, 'UseIcon') && $ex->{icon} ) {
    ## icon view force
    unless ( $data->{data_enable} && !$bn->get($cf, 'IconViewForce') ) {
      $data->{icon} = "$data->{data}/icon/" unless $ex->{icon} =~ m#^https?://#;
      $data->{icon}.= $ex->{icon};
    }
  }

  # thread lock
  if ($bn->get($cf, 'UseThreadLock')) {
    if ($fl->{thread_lock} || $ex->{thread_lock}) {
      $fl->{thread_lock} = 0;
      $fl->{reply_enable} = 0;
      $data->{thread_lock} = 1;
    }
    $data->{thread_view} = $ex->{thread_view} if exists $ex->{thread_view};
  }

  # color
  if ($bn->get($cf, 'UseColor')) {
    $data->{font_color} = $ex->{font_color};
    $data->{title_color} = $ex->{title_color};
    $data->{table_color} = $ex->{table_color};
  }

  # user : name cap
  if ($bn->get($cf, 'UseNameCap') && $ex->{user_cap}) {
    $data->{name} .= $bn->set_tag($bn->get($sk, 'TagNameCap'));
  }
  # user : name trip
  if ($bn->get($cf, 'UseNameTrip') && $ex->{user_trip}) {
    my $tag = $bn->get($sk, 'TagNameTrip');
    $data->{name} .= $bn->set_tag($tag, {string => $tag->{string}.$ex->{user_trip}});
  }

  # paint time
  $data->{paint_time} = $val->{paint_time} ? $val->{paint_time} : $ex->{paint_time} if $bn->get($cf, 'ViewPaintTime');

  $data->{file_name} = $val->{file_name} ne '' ? $val->{file_name} : $ex->{file_name} ne '' ? $ex->{file_name} : $ex->{filename};
  $data->{file_width} = $val->{file_width} ne '' ? $val->{file_width} : $ex->{file_width} ne '' ? $ex->{file_width} : $ex->{width};
  $data->{file_height} = $val->{file_height} ne '' ? $val->{file_height} : $ex->{file_height} ne '' ? $ex->{file_height} : $ex->{height};
  $data->{file_length} = $val->{file_length} ne '' ? $val->{file_length} : $ex->{file_length} ne '' ? $ex->{file_length} : $ex->{size};

  exists $ex->{nobr} && $ex->{nobr} ?
    $data->{message} =~ s/<\!br>/\n/g :
    $data->{message} =~ s/(<[^>]+>)|(\&\w+;)|([-.!~*'()\w;\/?:\@=+\$,%#\[\]\^`"{|}]{10})/($3 eq '')?(($2 eq '')?$1:"<wbr>$2"):"$3<wbr>"/eg;

  # object length KB
  $data->{object_length} = $data->{object_src_length} > 1000 ? int $data->{object_src_length}/1000:1;

  $bn;
}

sub src_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $val = $gl->{LogValue};
  my $ex = $gl->{LogExValue};
  my $data = $bn->{data};

  if ($data->{image_src}) {
    # useing applet
    undef $data->{applet_info};
    $data->{applet_info_raw} = $val->{applet};
    $data->{applet_key} = (split '\+',$val->{applet})[0];

    if (exists $bn->{module}{Paint}) {
     for (@{ $bn->{module}{Paint}{applets} })
      { $data->{applet_info} = $_->{name} if $data->{applet_key} eq $_->{key} } }

    $data->{applet_info} = $data->{applet_key} eq 'Up' || $val->{applet} eq '' ?
      'Upload' : 'Unknown' unless $data->{applet_info};

    if (
      $bn->{in}{mode} eq 'thumbnail'  ||
      $bn->{in}{mode} eq 'catalog'  ||
      $bn->{in}{mode} eq 'noimage'  ||
      $data->{file_width} > $data->{view_limit_image_width} ||
      $data->{file_height} > $data->{view_limit_image_height}
    ) {
      $data->{ 'view_'. ( $data->{thumbnail_src} eq '' || $bn->{in}{mode} eq 'noimage' ?
       'noimage' : 'thumbnail' ) } = 1;
    } else {
      $data->{view_image} = 1;
    }
  }

  # object
  if ($data->{object_src}) {
    $data->{view_object} = 1;
    $data->{object_type} = $bn->get($cf, 'FILE_EX')->{$1} if $data->{object_src} =~ /\.(\w+)$/;
    $data->{'object_type_'.$1} = 1;
    $data->{object_info} = $ex->{file_info};
  }

  # continue link
  if(
    ($data->{anime_src} || $data->{image_src} =~ /\.(jpg|gif)$/i ||
     ($bn->get($cf, 'UsePaintContinueConvert') && $data->{image_src} =~ /\.png$/i &&
       ( $bn->get($cf, 'UseGD') || $bn->get($cf, 'UsePerlMagick') || $bn->get($cf, 'UseImageMagick') )
     )
    ) &&
    exists $bn->{module}{Paint} &&
    $bn->get($cf, 'UsePaintContinue') &&
    !$bn->check_limit_post_view('Paint') &&
    exists $bn->{module}{Paint}{layout}{continue}
  ){
    $bn->{arg}{continue_link} = { fc => 'continue', log => $gl->{IndexNumber} };
    $bn->call('SkinMainCountinueArg');
    $data->{continue_link} = $bn->get($cf, 'BBSNoteUrl') . '?'. $bn->set_text_arg($bn->{arg}{continue_link}),
  }

  # anime link
  if( $data->{anime_src} && exists $bn->{module}{Paint} &&
    $bn->{module}{Paint}{layout}{anime} && $bn->check_limit_config( 'LimitViewPaintAnime' ) )
  {
    $bn->{arg}{anime_link} = { fc => 'anime', log  => $data->{index_number},
                               page => $bn->{in}{page}, mode => $bn->{in}{mode},
                               sort => $bn->{in}{sort} };
    $data->{anime_link} = $bn->get($cf, 'BBSNoteUrl') . '?' . $bn->set_text_arg($bn->{arg}{anime_link});
  }
  $bn;
}


sub menu_init {
  my( $bn, $data ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $menu = $sk->{menu} = {};

  my @menu_array = qw/top reload write delete edit sort_post sort_reply
    mode_normal mode_catalog mode_thumbnail mode_noimage login help/;

  map{ $menu->{$_} = { url =>  $bn->get($cf, 'BBSNoteUrl'), arg => {},
                       expansion => $bn->{arg}{Menu}, enable => 1 } } @menu_array;

  my $p = $in->{page};
  my $s = $in->{sort};
  my $m = $in->{mode};
  my $top = $bn->get($cf, 'HomePageUrl');
  $bn->escape_quote( \$top );
  $menu->{top}{url} = $top;
  $menu->{help}{arg}{fc} = 'help';
  $menu->{login}{arg}{fc} = 'login';
  $menu->{write}{arg}{fc} = 'post';
  $menu->{delete}{arg} = { fc => 'delete', mode => $m, sort => $s, page => $p };
  $menu->{edit}{arg} = { fc => 'edit', mode => $m, sort => $s, page => $p };

  $menu->{sort_post}{arg} = { fc => 'page', sort => '0' };
  $menu->{sort_reply}{arg} = { fc => 'page', sort => '1' };

  $menu->{mode_normal}{arg} = { fc => 'page', mode => 'normal', sort => $s, page => $p };
  $menu->{mode_catalog}{arg} = { fc => 'page', mode => 'catalog', sort => $s, page => $p };
  $menu->{mode_noimage}{arg} = { fc => 'page', mode => 'noimage', sort => $s, page => $p };
  $menu->{mode_thumbnail}{arg} = { fc => 'page', mode => 'thumbnail', sort => $s, page => $p };

  for (keys %{ $menu }) {
    if($menu->{$_}{url}){
      my $arg = $bn->set_text_arg($menu->{$_}{arg}, $menu->{$_}{expansion}) if $_ ne 'top';
      $menu->{$_}{url} .= '?' . $arg if $arg;
      $data->{'menu_' . $_ . '_url'} = $menu->{$_}{url};
    }
  }

  # sort mode rotation
  my $i;
  my @ro = split ',', $bn->get( $cf, 'ViewRotation' );
  $m ||= $ro[0] or 'normal';
  for ( @ro ) { $i++;
    next if $m ne $_;
    $m = $i < @ro ? $ro[$i++] : $ro[0];
    last;
  }

  $menu->{mode} = $menu->{'mode_' . $m};
  $menu->{sort} = $bn->{sys}{sortmode} ? $menu->{sort_post} : $menu->{sort_reply};

  $data->{menu_top} = $menu->{top}{url} eq '' ? 0 : 1;
  $data->{menu_write} = 1;
  $data->{menu_reload} = 1;
  $data->{menu_delete} = 1;
  $data->{menu_edit} = 1;
  $data->{menu_login} = 1;
  $data->{menu_help} = 1;
  $data->{menu_mode} = $m;
  $data->{'menu_mode_' . $m} = 1;

  $data->{menu_sort} = 1;
  $data->{'menu_sort_post'} = $bn->{sys}{sortmode};

  $data->{menu_mode_url} = $menu->{mode}{url};
  $data->{menu_sort_url} = $menu->{sort}{url};
  $data->{menu_top_url_target} = $bn->get( $cf, 'HomePageUrlTarget' );

  $bn->escape_quote( \$data->{menu_top_url_target} );

  if (!$bn->get($cf, 'UseUpload') && $bn->get($cf, 'NeedData')) {
    $data->{menu_write} = 0;
  }

  delete $sk->{menu};
  $bn;
}

sub line_form_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fr = $sk->{form};
  my $val = $bn->{global}{value} = $_[1];
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  my $tag_ast_mark = $bn->get($sk, 'TagFormAstMark');
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $item_size = $bn->form_size_init('thread');

  $bn->call('SkinLineFormInitBegin');
  $fr->{FORMS} = [[[[{ }]]],
  [[
    [ { TAG => 'TD', align => 'right', nowrap=>1 } ],
    [ ($bn->get($cf, 'NeedName') ? $tag_ast_mark : '') ],
    [ $tag_form_items, {  string => " $item->{name} " } ],
    [ { TAG => 'INPUT', name => 'name', type => 'text', class => 'input',
      size => $item_size->{name}, value => $val->{name} } ],
    [ $tag_form_items, {  string => " $item->{message} " } ],
    [ { TAG => 'INPUT', name => 'message', class => 'input', size => $item_size->{message},
      value => $val->{message} } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', type => 'submit', name => 'submit', class => 'reply', value => $item->{reply} } ]
  ]]];
  $bn->call('SkinLineFormInitEnd');
}

sub thread_form_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fr = $sk->{form};
  my $val = $bn->{global}{value} = $_[1];
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  my $tag_ast_mark = $bn->get($sk, 'TagFormAstMark');
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $item_size = $bn->form_size_init('thread');
  $val->{url} =~ s#^http://##;

  $bn->call('SkinThreadFormInitBegin');
  $fr->{FORMS} = [[[[{ }]]],
  [[
    [ { TAG => 'TD', align => 'right', nowrap => 1 } ],
    [ ($bn->get($cf, 'NeedName') ? $tag_ast_mark : '') ],
    [ $tag_form_items, {  string => " $item->{name} " } ],
    [ { TAG => 'INPUT', name => 'name', type => 'text', class => 'input',
      size => $item_size->{name}, value => $val->{name} } ]
  ],[
    [ { TAG => 'TD', rowspan => '2' } ],
    [ { TAG => 'TEXTAREA', name => 'message', class => 'input', wrap => 'soft',
      rows => '3', cols => $item_size->{message}, string => $val->{message} } ]
  ]],
  [[
    [ { TAG => 'TD', align => 'right', nowrap => 1 } ],
    [ ($bn->get($cf, 'NeedMail') ? $tag_ast_mark : '') ],
    [ $tag_form_items, { string => " $item->{mail} " } ],
    [ { TAG => 'INPUT', name => 'mail', type => 'text', class => 'input',
      size => $item_size->{name}, value => $val->{mail} } ]
  ]],
  [[
    [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
    [ ($bn->get($cf, 'NeedUrl') ? $tag_ast_mark : '') ],
    [ $tag_form_items, { string => " $item->{url} " } ],
    [ { TAG => 'INPUT', name => 'url', type => 'text', class => 'input',
      size => $item_size->{url}, value => 'http://'.$val->{url} } ]
  ]]
  ];
  if ($bn->get($cf, 'UseReplyTitle')) {
    $fr->{title} = [[
      [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
      [ ($bn->get($cf, 'NeedTitle') ? $tag_ast_mark : '') ],
      [ $tag_form_items, { string => " $item->{title} " } ],
      [ { string => ' ' } ],
      [ { TAG => 'INPUT', type => 'text', name => 'title', class => 'input',
        size => $item_size->{title}, value => $val->{title} } ]
    ]];
    $bn->set_form($fr->{FORMS}, $fr->{title}, 'add', 'message');
  }

  if ($bn->{flag}{UPLOAD_ENABLE}) {
    $fr->{upload} = ([[
      [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
      [ $tag_form_items, { string => " $item->{upload} " } ],
      [ { string => ' ' } ],
      [ { TAG => 'INPUT', name => 'file', type => 'file', class => 'input',
         size => $item_size->{upload} } ]
    ]]);
    $bn->set_form($fr->{FORMS}, $fr->{upload}, 'add', 'message');
  }

  $fr->{submit} = [[
    [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
    [ ($bn->get($cf, 'NeedPassword') ? $tag_ast_mark : '') ],
    [ $tag_form_items, { string => " $item->{password} " } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', type => 'password', name => 'password', class => 'input',
      size => $item_size->{password}, value => $val->{password} } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', type => 'submit', name => 'submit', class => 'reply', value => $item->{reply} } ]
  ]];
  $bn->set_form($fr->{FORMS}, $fr->{submit}, 'add');
  $bn->set_color_form( 'insert', 'password' );
  $bn->set_icon_form( 'insert', 'password' );
  $bn->set_panic_form;
  $bn->set_notice_form(1);

  $bn->call('SkinThreadFormInitEnd');
}

sub layer_form_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fr = $sk->{form};
  my $val = $bn->{global}{value} = $_[1];
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  my $table_in = $bn->get($sk, 'TagTableIn');
  my $table_out = $bn->get($sk, 'TagTableOut');
  my $tag_ast_mark = $bn->get($sk, 'TagFormAstMark');
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $item_size = $bn->form_size_init('layer');

  $bn->call('SkinLayerFormInitBegin');

  $fr->{FORMS} = [
  [[[{ }]]],[[
    [ { TAG => 'TD', colspan => '*', bgcolor => $table_out->{bgcolor} } ],
    [ { string => '<table width="100%" cellspacing="0" cellpadding="0"><tr><td>' .
      '<font size="2"><input id="LayerFormBar" name="layerbar" value="&nbsp;" type="button" width="379" style="width:378;background:'.
      $table_out->{bgcolor} . ';color:' . $table_in->{bgcolor} . ';font-size:10pt;border:0'.
      '" onmousedown="LF_drag(event)" onmouseup="LF_drop()"></td><td>$close</td></tr></table>'
    } ]
  ]],[[
    [ { TAG => 'TD', width => '50%' } ],
    [ $tag_form_items, { id => 'LayerFormName', string => " $item->{name} " } ],
    [ ($bn->get($cf, 'NeedName') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'LayerFormNameInput', name => 'name', type => 'text', class => 'input', style => 'width:100%',
      size => $item_size->{name}, value => $val->{name} } ]
  ],[
    [ $tag_form_items, { id => 'LayerFormMail', string => " $item->{mail} " } ],
    [ ($bn->get($cf, 'NeedMail') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'LayerFormMailInput', name => 'mail', type => 'text', class => 'input', style => 'width:100%',
      size => $item_size->{name}, value => $val->{mail} } ]
  ]],[[
    [ { TAG => 'TD', colspan => '*' } ],
    [ $tag_form_items, { id => 'LayerFormUrl', string => " $item->{url} " } ],
    [ ($bn->get($cf, 'NeedUrl') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'LayerFormUrlInput', name => 'url', type => 'text', class => 'input', style => 'width:100%',
      size => $item_size->{url}, value => 'http://'.$val->{url} } ]
  ]],[[
    [ { TAG => 'TD', colspan => '*' } ],
    [ $tag_form_items, { id => 'LayerFormMessage', string => " $item->{message} " } ],
    [ ($bn->get($cf, 'NeedMessage') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'TEXTAREA', id => 'LayerFormMessageTextarea', name => 'message', class => 'input', style => 'width:100%',
      wrap => 'soft', rows => '8', cols => $item_size->{message}, string => $val->{message} } ]
  ]]];
  if ($bn->get($cf, 'UseReplyTitle')) {
    $fr->{title} = [[
      [ { TAG => 'TD', colspan => '*' } ],
      [ $tag_form_items, { id => 'LayerFormTitle', string => " $item->{title} " } ],
      [ ($bn->get($cf, 'NeedTitle') ? $tag_ast_mark : '') ],
      [ { string => '<br>' } ],
      [ { TAG => 'INPUT', id => 'LayerFormTitleInput', type => 'text', name => 'title', class => 'input', style => 'width:100%',
        size => $item_size->{title}, value => $val->{title} } ]
    ]];
    $bn->set_form($fr->{FORMS}, $fr->{title}, 'insert', 'name');
  }


  $fr->{submit} = [[
    [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
    [ $tag_form_items, { id => 'LayerFormPassword', string => " $item->{password} " } ],
    [ $bn->get($cf, 'NeedPassword') ? $tag_ast_mark : { string => ' ' } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', id => 'LayerFormPasswordInput', type => 'password', size => '10',
      name => 'password', class => 'input', value => $val->{password} } ],
    [ { string => ' $submit' } ],
  ]];

  $bn->set_color_form;
  $bn->set_icon_form;
  $bn->set_form($fr->{FORMS}, $fr->{submit}, 'add');
  $bn->set_notice_form(1);
  $bn->set_panic_form('left');

  $bn->call('SkinLayerFormInitEnd');

}

sub post_form_init {
  my $bn = $_[0];
  return if $bn->{flag}{PostFormInit};
  $bn->{flag}{PostFormInit} = 1;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fr = $sk->{form};
  my $ck = $bn->{cookie};
  my $gl = $bn->{global};
  my $val = $gl->{value} = $_[1];
  my $arg = $bn->{arg};
  my $item = $bn->{item} = $bn->get($sk, 'FormName');
  my $item_size = $bn->{item_size} = $bn->form_size_init('thread');
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $tag_ast_mark = $bn->get($sk, 'TagFormAstMark');
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');

  $bn->call('PostFormInitBegin');

  $fr->{FORMS} = [
  [[
    [ { TAG => 'INPUT', type => 'hidden', name => 'fc',   value => $val->{fc}   } ],
    [ { TAG => 'INPUT', type => 'hidden', name => 'mode', value => $val->{mode} } ],
    [ { TAG => 'INPUT', type => 'hidden', name => 'sort', value => $val->{sort} } ],
    [ { TAG => 'INPUT', type => 'hidden', name => 'page', value => $val->{page} } ]
  ]],[[
    [ { TAG => 'TD', width => '50%' } ],
    [ $tag_form_items, { id => 'PostFormName', string => " $item->{name} " } ],
    [ ($bn->get($cf, 'NeedName') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'PostFormNameInput', name => 'name', type => 'text', class => 'input',
      style => 'width:100%', size => $item_size->{name}, value => $val->{name} } ]
  ],[
    [ $tag_form_items, { id => 'PostFormMail', string => " $item->{mail} " } ],
    [ ($bn->get($cf, 'NeedMail') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'PostFormMailInput', name => 'mail', type => 'text', class => 'input',
       style => 'width:100%', size  => $item_size->{name}, value => $val->{mail} } ]
  ]],[[
    [ { TAG => 'TD', colspan => '*' } ],
    [ $tag_form_items, { id => 'PostFormUrl', string  => " $item->{url} " } ],
    [ ($bn->get($cf, 'NeedUrl') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'INPUT', id => 'PostFormUrlInput', name => 'url', type => 'text', class => 'input',
      style => 'width:100%', size => $item_size->{url}, value => 'http://'.$val->{url} } ]
  ]],[[
    [ { TAG => 'TD', colspan => '*' } ],
    [ $tag_form_items, { id => 'PostFormMessage', string => " $item->{message} " } ],
    [ ($bn->get($cf, 'NeedMessage') ? $tag_ast_mark : '') ],
    [ { string => '<br>' } ],
    [ { TAG => 'TEXTAREA', id => 'PostFormMessageTextarea', name => 'message', class => 'input', style => 'width:100%', string => $val->{message},
      wrap => $bn->get($sk, 'FormTextAreaWrap') ? $bn->get($sk, 'FormTextAreaWrap') : 'soft',
      rows => $bn->get($sk, 'FormTextAreaRows') ? $bn->get($sk, 'FormTextAreaRows') : '8',
      cols => $bn->get($sk, 'FormTextAreaCols') ? $bn->get($sk, 'FormTextAreaCols') : $item_size->{message}} ]
  ]]];

  $bn->set_color_form if $bn->{sys}{MODE} !~ /ICON/;
  $bn->set_icon_form;

  if ($bn->po->check_auth($bn->get($cf, 'UseAllTag'))) {
    $fr->{nobr} = [[[
      [ { TAG => 'TD', colspan => '*' } ],
      [ { TAG => 'INPUT', id => 'PostFormNobrCheck',name => 'nobr', type => 'checkbox', class => 'check', checked => exists $val->{nobr} ? $val->{nobr} ? 1 : '' : $ck->{nobr} ? 1 : '' } ],
      [ $tag_form_items, { id => 'PostFormNobr', string => $item->{nobr} } ],
    ]]];
    $bn->set_form($fr->{FORMS}, $fr->{nobr}, 'add', 'message');
  }

  if ($bn->{flag}{UPLOAD_ENABLE}) {
    $fr->{upload} = ([[
      [ { TAG => 'TD', colspan => '*' } ],
      [ $tag_form_items, { id => 'PostFormUpload', string => " $item->{upload} " } ],
      [ ($bn->get($cf, 'NeedData') ? $tag_ast_mark : '') ],
      [ { string => '<br>' } ],
      [ { TAG => 'INPUT', id => 'PostFormUploadInput',name => 'file', type => 'file', class => 'input',
        style => 'width:100%', size => $item_size->{upload} } ]
    ]]);
    $bn->set_form($fr->{FORMS}, $fr->{upload}, 'insert', 'message');
  }

  $fr->{submit} = [[
    [ { TAG => 'TD', colspan => '*', align => 'right', nowrap => 1 } ],
    [ $tag_form_items, { id => 'PostFormPassword', string => " $item->{password} " } ],
    [ ($bn->get($cf, 'NeedPassword') ? $tag_ast_mark : '') ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', id => 'PostFormPasswordInput', type => 'password', name => 'password', class => 'input',
      size => $item_size->{password}, value => $val->{password} } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', id => 'PostFormSubmit', type => 'submit', name => 'submit', class => 'post', value => $item->{submit} } ],
    [ { string => ' ' } ],
    [ { TAG => 'INPUT', id => 'PostFormReset', type => 'reset', name => 'reset', class => 'reset', value => $item->{reset} } ]
  ]];

  $bn->set_form($fr->{FORMS}, $fr->{submit}, 'add');

  if ($bn->{sys}{MODE} !~ /ICON/) {
#    $bn->set_color_form;
    $bn->set_panic_form;
    $bn->set_notice_form;
  }

  $arg->{PostForm}{log} = $in->{log} if $in->{log};

  # tree
  if ($bn->get($cf, 'UseTree')) {
    $arg->{PostForm}{num} = $gl->{num} if exists $in->{res};
    if (!defined $val->{title} && $bn->get($cf, 'UseReplyTitle') && $bn->{sys}{MODE} eq 'REPLY') {
      my $n =  $gl->{title} =~ s/(Re\[)(\d+)(\]:)(\s+)?// ? $2 : 0 ;
      $val->{title} = 'Re['.++$n."]: $gl->{title}";
    }
  }

  # tree
  if ($bn->get($cf, 'UseTreeSppCheck')) {
    $fr->{check} = [[[
      [{ TAG => 'TD', colspan => '*', nowrap => 1 }],
      [{ TAG => 'INPUT', id => 'PostFormSupportCheck', type => 'checkbox', name => 'kaiketu', checked => $val->{kaiketu}?1:'', value => 1, }],
      [$tag_ast_mark, { id => 'PostFormSupport', string => '<label for="PostFormSupportCheck">' . $item->{spportcheck} . '</label>' } ]
    ]]];
    $bn->set_form($fr->{FORMS}, $fr->{check}, 'insert', 'submit');
  }

  if ($bn->{flag}{POST_FORM_TITLE}) {
    $fr->{title} = [
    [
      [ { TAG => 'TD', colspan => '*' } ],
      [ $tag_form_items, { id => 'PostFormTitle', string => " $item->{title} " } ],
      [ ($bn->get($cf, 'NeedTitle') ? $tag_ast_mark : '') ],
      [ { string => '<br>' } ],
      [ { TAG => 'INPUT', id => 'PostFormTitleInput', name => 'title', type => 'text', class => 'input',
        size => $item_size->{title}, style => 'width:100%', value => $val->{title} } ]
    ]];
    $bn->set_form($fr->{FORMS}, $fr->{title}, 'insert', 'url');
  }

  $bn->call('PostFormInitEnd');
}

sub search_form_init {
  my( $bn, $data ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};

  my( $ta, %target );
  for ( sort grep { /^target(?:\d+)?/ ? $_ : undef } keys %$in ) {
    map { my @a = split '=', $_, 2;
      $target{$a[0]} = @a > 1 ? $a[1] : 1 } split ',', $in->{$_} }
  for ( sort keys %target ) {
    $data->{'search_target_'. $_} = $target{$_};
    /^(\w)/; $ta .= $1 }

  $data->{"search_target_$ta"} = ' selected';
  $data->{'search_type_' . $in->{type}} = ' selected';
  $data->{search_form_ex} = $bn->get($cf, 'ViewSearchFormEx');

  $bn->{arg}{SearchForm}{fc} = 'search';
  $data->{search_form_arg} = $bn->set_form_arg($bn->{arg}{SearchForm});
  $data->{search_keyword} = $in->{keyword};

  # options loop
  $data->{search_form_view_options} = sub{
    my($data, $hash) = @_;
    my $min = $bn->get($cf, 'ViewSearchLimitMin');
    my $max = $bn->get($cf, 'ViewSearchLimitMax');
    my $var = $bn->get($cf, 'ViewSearchLimitVar');
    my $form_option = {data => $hash->{form_option}};
    $bn->template_trans($form_option);
    my $view = $in->{search_view};
    $view ||= $bn->get($cf, 'ViewSearchDefault');
    for (my $i = $min; $i <= $max; $i+=$var) {
      $data->{value} = $i;
      $data->{selected} = $i eq $view ? ' selected' : '';
      $bn->translate($form_option, $data);
    }
    return
  };

  $bn->translate($sk->{template}{SEARCH}, $data);

  return;
}


sub set_icon_form {
  my( $bn, @set_target ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $val = $gl->{value};
  my $fr = $sk->{form};
  my $item = $bn->{item};
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');

  # icon
  if ( $bn->get( $cf, 'UseIcon' ) ) {
    $gl->{icon_preview_num}++;
    my $list = $bn->get( $cf, 'IconList' );
    my $now_icon = $bn->get($cf, 'DirImagesUrl') . '/lf_close.gif';
    my( $fn, $option );
    for my $h ( @$list ) {
      if ( $h->{title} ne '' ) {
        my $exc = $h->{name} . $bn->get($cf, 'IconExclusive') if $h->{applet};
        my $f;
        if( ($bn->{sys}{MODE} =~ /^(?:$|POST|REPLY)/ && $h->{file_name} eq $ck->{icon} ) ||
            ($bn->{sys}{MODE} =~ /EDIT/ && $h->{file_name} eq $val->{icon})
        ){ $f = $fn = 1;
          $now_icon = $h->{file_name} =~ m#https?://# ? $h->{file_name} : $bn->get($cf, 'DirFileUrl') . "/icon/$h->{file_name}" if $h->{file_name}
        }
        $option .= '<option value="'.$h->{file_name}.'"'.
         ( $f || $h->{file_name} eq '' && !$fn ? ' selected' : '' ).
         ">$h->{title} $exc</option>\n";
      }
    }
    my $wind = $bn->get($cf, 'IconPreviewWindow');
    $bn->escape_quote( \$wind );
    $fr->{icon} = [[
    [ { TAG => 'TD', colspan => '*', nowrap => 1 } ],
    [ $tag_form_items, { id => 'PostFormIcon', string => " $item->{icon} "} ],
    [ { TAG => 'SELECT', id => 'PostFormIconSelect', name => 'icon', class => 'select', string => $option,
      onchange => ( $bn->get($cf, 'UseIconPreview') ? "IM_Preview(this,'icon_pre_" . $gl->{icon_preview_num} . "')" :'') } ],
    [ { string => ' ' } ],
    [ $tag_form_items, ($wind eq '0' ?
        { TAG => 'A', id => 'PostFormIconListAnchor', string => $bn->get($cf, 'IconListString'),
          target => ($wind ? '_blank' : ''), href => $bbsnoteurl . '?fc=icon_list' }:
        { TAG => 'SPAN', id => 'PostFormIconListSpan', style => 'cursor:hand', string  => $bn->get($cf, 'IconListString'),
          onclick => "window.open('" . $bbsnoteurl . "?fc=icon_list','IconList','$wind')" }
        ) ],
    [ { string => ' ' } ],
    [ $tag_form_items, ($wind eq '0' ?
        { TAG => 'A', id => 'PostFormIconRegistAnchor',
          target => ($wind ? '_blank' : ''), href => $bbsnoteurl . '?fc=icon_regist',
          string => $bn->get($cf, 'IconRegistString')}:
        { TAG => 'SPAN', id => 'PostFormIconRegistSpan', style => 'cursor:hand',
          onclick => "window.open('$bbsnoteurl?fc=icon_regist','IconList','$wind')",
          string => $bn->get($cf, 'IconRegistString')}
         ) ],
    [ { string => $bn->get($cf, 'UseIconPreview') ?
      ' <img name="icon_pre_' . $gl->{icon_preview_num} . '" align="top" src="'.$now_icon.'">':''  }
    ]]];
    $set_target[0] ||= 'add';
    $set_target[1] ||= 'message';
    $bn->set_form( $fr->{FORMS}, $fr->{icon}, @set_target );

  }

}

sub set_panic_form {
  my( $bn, $position ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $val = $gl->{value};
  my $fr = $sk->{form};
  my $item = $bn->{item};
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');

  # panic
  if ($bn->get($cf, 'UsePanic') eq '1' && $bn->{in}{fc} !~ /^icon/) {
    my $c = $bn->{sys}{MODE} =~ /EDIT/ ? $gl->{LogExValue}{panic} : $ck->{panic};
    $fr->{panic} = [[
      [ { TAG => 'INPUT', id => 'PostFormPanicCheck', type => 'checkbox', name => 'panic', value => 1, checked => $c ? 1 : ''} ],
      [ $tag_form_items, { id => 'PostFormPanic', string => '<label for="PostFormPanicCheck">'."$item->{flowerpanic}</label> " } ],
    ]];
    $bn->set_form($fr->{FORMS}, $fr->{panic}, ($position ? $position : 'right'), 'password');
  }

}

sub set_color_form {
  my( $bn, @set_target ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $val = $gl->{value};
  my $fr = $sk->{form};
  my $item = $bn->{item};
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');

  return if $bn->{in}{fc} =~ /^icon/;
  # color
  return if $bn->get($cf, 'UseColor') ne '1';
  for ('table_color','title_color','font_color') {
    next unless exists $item->{$_};

    my $c = $bn->{sys}{MODE} =~ /EDIT/ ? $gl->{LogExValue}{$_} : $ck->{$_};
    my @s = split /,/, $bn->get($cf, 'ColorSetting:' . $_);
    my $list;
    for(my $n = 0; $n < @s; $n++) {
      $list .= '<nobr><input type="radio" name="'.$_.'" value="'.$s[$n].'"' . ($c eq $s[$n] ? ' checked':'').
        '><font color="'.$s[$n].'">'.$item->{color_sample}.'</font></nobr>';
    }
    $list .= '<nobr><input type="radio" name="' . $_ . '" value=""' . ($c eq '' ? ' checked':'').
      '>'.$bn->set_tag($tag_form_items, {string => $item->{color_none}}).'</nobr>';
    my $class = ucfirst $1 if /([^_]+)/;
    $fr->{$_} = [[
      [ { TAG => 'TD', align => 'right', colspan => '*' } ],
      [ { string => '<table><tr><td width="50" nowrap>' } ],
      [ $tag_form_items, { id => 'PostForm'.$class.'Color', string  => $item->{$_} } ],
      [ { string => '</td><td id="PostForm'.$class.'ColorList" width="350">'.$list.'</td></tr></table>' } ]
    ]];
    $set_target[0] ||= 'add';
    $set_target[1] ||= 'message';
    $bn->set_form($fr->{FORMS}, $fr->{$_}, @set_target );
  }

}
sub set_notice_form {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $val = $gl->{value};
  my $fr = $sk->{form};
  my $item = $bn->{item};
  my $tag_form_items = $bn->get($sk, 'TagFormItems');
  my $bbsnoteurl = $bn->get($cf, 'BBSNoteUrl');

  if ($bn->get($cf, 'UseNoticeMail')) {
    my $c = $bn->{sys}{MODE} =~ /EDIT/ ? $gl->{LogExValue}{mail_notice} : $ck->{mail_notice};
    $fr->{notice} = [[[
    [ { TAG => 'TD', colspan => '*', align => $_[0] ? 'right' : '' } ],
    [ { TAG => 'INPUT', id => 'PostFormNoticeCheck', type => 'checkbox', name => 'mail_notice', value => 1, checked => $c ? 1 : ''} ],
    [ $tag_form_items, { id => 'PostFormNotice', string => "$item->{noticecheck} " } ] ]]];
    $bn->set_form($fr->{FORMS}, $fr->{notice}, $_[0] ?  'add' : 'insert', 'password');

  }

}

sub page_link_init {
  my( $bn, $data ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $gl = $bn->{global};
  my $style = $bn->get($sk, 'ViewStylePageLink');
  my $log_view = $bn->get($cf, 'LogView');
  my $view_limit = $bn->get($sk, 'ViewLimitPageLink');

  $style = 'form' if($bn->{sys}{MODE} =~ /ADMIN/);

  my( $linktext, $linkform );
  my( $fc, $mode, $page, $sort, $pl ) = (@{$bn->{in}}{'fc', 'mode', 'page', 'sort'}, $view_limit);

  $fc = $mode = '' if $bn->{flag}{RefleshIndex};
  $fc = 'page' if $fc eq '' || $fc eq 'write';

  return if $gl->{LogTotal} < 1 || $log_view < 1;
  my $maxpage = int($gl->{LogTotal} / $log_view);
  $maxpage++ if $gl->{LogTotal} - ($maxpage * $log_view) > 0;
  return if $maxpage < 2;

  $page = int $page;
  $page = 1 unless $page > 0;

  $bn->{arg}{PageLink}{fc} = $fc;
  $bn->{arg}{PageLink}{mode} = $mode;
  $bn->{arg}{PageLink}{sort} = $sort;

  $bn->call('CreatePageLinkInit');

  my( $i, $n, $mblock, $pblock, $pos );
  $data->{bbsnoteurl} = $bn->get($cf, 'BBSNoteUrl');
  $data->{pages} = sub{ $bn->page_link(@_) };

  if ( $style eq 'form' ) {
    $data->{pagelink_form} = 1;

    # admin modeFadd password / don't reflesh index
    @{$bn->{arg}{PageLink}}{'password', 'name'} = $bn->get_admin_pass
      if $bn->{sys}{MODE} eq 'ADMIN' && ! $bn->{flag}{RefleshIndex};

    $data->{arg} = $bn->set_form_arg($bn->{arg}{PageLink});

    for ( $i = $maxpage, $n = 0; $i > 0; $i-- ) {
      $n++;
      if ( $n == $page ) {
        if ( $n - 1 > 0 ) {
          $data->{page} = $n - 1;
          $data->{next} = $n - 1;
          $data->{formnext} = $data->{bbsnoteurl} . '?' .  $data->{arg} . '&page=' . ($n - 1);
        }
        if ( $n < $maxpage ) {
          $data->{page} = $n + 1;
          $data->{back} = $n + 1;
          $data->{formback} = $data->{bbsnoteurl} . '?' .  $data->{arg} . '&page=' . ($n + 1);
        }
        last;
      }
    }

  } else {
    $data->{pagelink_text} = 1;

    if ($pl) {
      $mblock = int($maxpage / $pl);
      $pblock = int(($page-1) / $pl);
      $pos = $pblock * $pl;
    }
    my $arg = $bn->set_text_arg($bn->{arg}{PageLink});

    for ( $i = 0; $i <= $maxpage; $i++ ) {
      $n++;
      if ($n == $page) {
        if ($n - 1 > 0) {
          $data->{next} = '1';
          $data->{textnext} = $data->{bbsnoteurl} . '?' . $arg . '&page=' . ($n-1);
        }
        if ($n < $maxpage) {
          $data->{back} = '1';
          $data->{textback} = $data->{bbsnoteurl} . '?' . $arg . '&page=' . ($n+1);
        }
        last;
      }
    }
  }
  return;
}


sub catalog_init {
  my( $bn, $data ) = @_;
  my $sk = $bn->{skin};
  my $m = lc $bn->{in}{mode};
  $m ||= 'normal';

  $data->{mode} = $m;
  $data->{'mode_' . $m } = 1;

  if ( $data->{mode_catalog} ) {
    my $t = $bn->get($sk, 'TagCatalogTableIn');
    for ( keys %$t ) { $data->{'table_in_' . $_} = $t->{$_} }
    $t = $bn->get($sk, 'TagCatalogTableOut');
    for ( keys %$t ) { $data->{'table_out_' . $_} = $t->{$_} }
  }
  $bn;
}

sub form_size_init {
  my( $bn, $type ) = @_;
  my( $os, $ag, $av ) = @{$bn->{global}}{'UserOS', 'UserBrowser', 'UserBrowserVer'};
  my %h;

  if ( $type eq 'layer' ) {
    if ( $os =~ /Win/ ) {
         if ( $ag eq 'MSIE' ) { %h = ( name => 35, url => 75, message => 54 ) }
      elsif ( $ag =~ /Opera/ ){ %h = ( name => 26, url => 55, message => 54 ) }
      elsif ( $av =~ /NN 4/ ) { %h = ( name => 20, url => 43, message => 42 ) }
      else                    { %h = ( name => 35, url => 75, message => 52 ) }
    } else {
      if ( $ag eq 'MSIE' )    { %h = ( name => 26, url => 55, message => 53 ) }
      else                    { %h = ( name => 26, url => 49, message => 53 ) }
    }
  } elsif ( $type eq 'thread' ) {
    if ( $ag eq 'MSIE' )      { %h = ( name => 15, url => 70, message => 50, password => 10, title => 70, upload => 60 ) }
    elsif ( $av =~ /NN 4/ )   { %h = ( name => 10, url => 40, message => 40, password => 7,  title => 40, upload => 40 ) }
    else                      { %h = ( name => 15, url => 60, message => 40, password => 10, title => 60, upload => 40) }
  } else {
    if ( $ag eq 'MSIE' )      { %h = ( name => 30, url => 50, message => 53, password => 10, title => 40, upload => 50 ) }
    elsif( $av  = ~ /NN 4/ )  { %h = ( name => 28, url => 60, message => 60, password => 7,  title => 60, upload => 50 ) }
    else                      { %h = ( name => 30, url => 50, message => 53, password => 10, title => 40, upload => 50 ) }
  }
  return \%h;
}


sub template_init {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  return if $bn->{flag}{template_init};
  $bn->{template} = { import => 0 };
  my $sk = $bn->{skin};
  my $tmpl = $sk->{template};

  $sk->{flag}{template_init} = 1;

  $tmpl->{MAIN_BEGIN} = {};
  $tmpl->{MAIN_END} = {};
  $tmpl->{TREE_BEGIN} = {};
  $tmpl->{TREE_END} = {};
  $tmpl->{TREE_REPLY} = {};
  $tmpl->{REPLY} = {};

  $tmpl->{MAIN}{data} =~ s!(.*)<REPLY>!!is
    and $tmpl->{MAIN_BEGIN}{data} = $1;
  $tmpl->{MAIN}{data} =~ s!(.*)</REPLY>!!is
    and $tmpl->{REPLY}{data} = $1;
  $tmpl->{MAIN_END}{data} = $tmpl->{MAIN}{data};
  delete $tmpl->{MAIN};

  $tmpl->{TREE}{data} =~ s!(.*)<REPLY>!!is
    and $tmpl->{TREE_BEGIN}{data} = $1;
  $tmpl->{TREE}{data} =~ s!(.*)</REPLY>!!is
    and $tmpl->{TREE_REPLY}{data} = $1;
  $tmpl->{TREE_END}{data} = $tmpl->{TREE}{data};
  delete $tmpl->{TREE};

  $tmpl->{MAIN_BEGIN}{data} =~ s!<USER_INFO>(.*?)</USER_INFO>!$bn->set_tag($bn->get($sk, 'TagMainUser'),{string=>$1})!iseg;
  $tmpl->{MAIN_END}{data} =~ s!<USER_INFO>(.*?)</USER_INFO>!$bn->set_tag($bn->get($sk, 'TagMainUser'),{string=>$1})!iseg;
  $tmpl->{REPLY}{data} =~ s!<USER_INFO>(.*?)</USER_INFO>!$bn->set_tag($bn->get($sk, 'TagResUser'),{string=>$1})!iseg;

  $bn->call('TemplateInit');

  $bn->template_trans(
    $tmpl->{MAIN_BEGIN}, $tmpl->{REPLY}, $tmpl->{MAIN_END},
    $tmpl->{TREE_BEGIN}, $tmpl->{TREE_REPLY}, $tmpl->{TREE_END},
    $tmpl->{CATALOG}, $tmpl->{HEAD}, $tmpl->{FOOT}, $tmpl->{MENU},
    $tmpl->{OBJECT}, $tmpl->{POST}, $tmpl->{FRAME}, $tmpl->{SEARCH}
  );

  delete $bn->{template};
  $bn;
}

sub template_trans {
  my $bn = shift;
  my $tgl = $bn->{template};

  for my $tmpl ( @_ ) {
    next if ref $tmpl ne 'HASH';

    unless ( $tgl->{nest} ) {
      $tgl->{data} = $tmpl->{data};
      $tgl->{point} = 0;
    }

    my $data_len = length $tmpl->{data};
    my( @trans, @trans_data );
    while ( $tmpl->{data} =~ /(<SKIN\w*|\$\w+)/ig ) {
      my($i, $var, $skin_tag);
      my $str = $1;
      my $tlen = length $str;


      if ( $str =~ /<SKIN_(\w+)/i ) { $skin_tag = lc $1 }
      elsif ( $str =~ /^\$(\w+)/ ) { $var = lc $1 }
      my $s = pos $tmpl->{data};
      if ( defined $var ) {
        $i = $s;
        push @trans_data, \$var;
      } else {
        ( my $elmt_hash, $i ) = $bn->_tmpl_element( \$tmpl->{data} );
        if ( $skin_tag eq 'include' || $skin_tag eq 'require' ) {
          if ( $bn->{param}{ 'AllowTemplate'. ucfirst $skin_tag } ) {
            if ( $skin_tag eq 'include' ) {
              $s -= $tlen;
              my $src = $bn->_tmpl_get_src( $elmt_hash->{name} );
              $tgl->{data} = ( substr $tgl->{data}, 0, $tgl->{point} + $s )
                .$src. ( substr $tgl->{data}, $tgl->{point} + $i );
              $tmpl->{data} = ( substr $tmpl->{data}, 0, $s ). $src. ( substr $tmpl->{data}, $i );
              next;
            } else {
              $elmt_hash->{__tag} = $skin_tag;
              $tmpl->{data} = ( substr $tmpl->{data}, 0, $s ). ( substr $tmpl->{data}, $i );
              push @trans_data, [[$elmt_hash]];
              $i = $s;
            }
          }
        } elsif ( $skin_tag && $skin_tag ne 'else' ) {
          my @data_array = ();
          pos $tmpl->{data} = $i;
          $elmt_hash->{__tag} = $skin_tag;
          push @data_array, [$elmt_hash];
          $tgl->{nest}++;
          $i = $bn->_tmpl_nest( \$tmpl->{data}, $skin_tag, \@data_array );
          $tgl->{nest}--;
          push @trans_data, \@data_array;
        } else {
          push @trans_data, $elmt_hash;
        }
      }
      push @trans, substr $tmpl->{data}, 0, $s - $tlen;
      $tmpl->{data} = substr $tmpl->{data}, $i;
      $tgl->{point} += $i;
      pos $tmpl->{data} = 0;

    }
    push @trans, $tmpl->{data};
    $tmpl->{__trans} = \@trans;
    $tmpl->{__data} = \@trans_data;
    delete $tmpl->{data};
  }
  $bn;
}

sub _tmpl_nest {
  my( $bn, $data, $catch, $array ) = @_;
  my $data_len = length $$data;
  my $tgl = $bn->{template};
  my $nest = 0;
  my $st = pos $$data;
  my $match = $catch =~ /if|unless/ ? 'if|unless|else' : $catch;
  while ( $$data =~ /(<\/?SKIN_)($match)/ig ) {
    my $str = $1;
    my $skin_tag = lc $2;
    my $end = 1 if $str =~ /^<\//;
    my $tlen = length $str;
    $tlen += length $skin_tag;
    if ( $end ) {
      unless ( $nest ) {
        my $s = pos $$data;
        my $val = substr $$data, $st, $s - $st - $tlen;
        my $p = $tgl->{point};
        $tgl->{point} += $st;
        my $hash = { data => $val };
        $tgl->{nest}++;
        $bn->template_trans($hash);
        $tgl->{nest}--;
        push @{$array->[-1]}, $hash;
        $tgl->{point} = $p;
        my( $elmt_hash, $i ) = $bn->_tmpl_element( $data );
        return $i;
      } else {
        $nest--;
      }
    } else {
      if ( $skin_tag =~ /if|unless|val/ ) {
        $nest++;
      } elsif ( !$nest ) {
        my $s = pos $$data;
        my $val = substr $$data, $st, $s - $st - $tlen;
        my $hash = { data => $val };
        $tgl->{nest}++;
        $bn->template_trans($hash);
        $tgl->{nest}--;
        my( $elmt_hash, $i ) = $bn->_tmpl_element( $data );
        pos $$data = $st = $i;
        push @{$array->[-1]}, $hash;
        $elmt_hash->{__tag} = $skin_tag;
        push @$array, [$elmt_hash];
      }
    }
  }
  $bn->_tmpl_error( $st );
  $bn->EXIT;
}

sub _tmpl_error {
  my( $bn, $end_offset ) = @_;
  my $tgl = $bn->{template};
  my $len = length $tgl->{data};
  my $o;;
  for ( $o = $end_offset + $tgl->{point}; $o > 0; $o-- )
   { last if ( substr $tgl->{data}, $o, 6 ) =~ /<SKIN_/i }
  my $s = substr $tgl->{data}, 0, $o;
  my $line = 1; while ( $s =~ /\n/g ) { $line++ }
  my $r = $o - rindex $s, "\n";
  my( $at, $i ) = $bn->_tmpl_attribute( \$tgl->{data}, $o );

  $s = substr $tgl->{data}, $o, $i - $o;
  $tgl->{data} = ( substr $tgl->{data}, 0, $o ) . "\x08\x07". $s. "\x08\x07". substr $tgl->{data}, $i;

  $s =~ s/</&lt;/g;$s =~ s/>/&gt;/g;

  $tgl->{data} =~ s/</&lt;/g;$tgl->{data} =~ s/>/&gt;/g;$tgl->{data} =~ s/ /&nbsp;/g;$tgl->{data} =~ s/\t/'&nbsp;' x 8/eg;
  my $n = 0;
  $tgl->{data} =~ s#^(.*)$#$n++;my $c = $n == $line ? ' style="color:red"':'';
    sprintf "<span$c>%03d</span> : %s", $n, $1#omeg;
  $tgl->{data} = '<div style="width:100%;text-align:left;font-size:10pt">'. $tgl->{data}. '</div>';
  $tgl->{data} =~ s#\x08\x07([^\x08]*)\x08\x07#<b style="color:red">$1</b>#;

  if ( exists $tgl->{error} ) {
    &{$tgl->{error}}( $line, $r, $s, $tgl->{data} );
  } else {
    $bn->error( 'ERR_SKIN_TAG', $line, $r, $s, $tgl->{data} );
  }
  goto EXIT;
}

sub _tmpl_get_src {
  my( $bn, $src ) = @_;
  return unless defined $src;
  $bn->{template}{import}++;
  return "[ error : import( $src ) unable over limit import ]"
   if ( $bn->{template}{import} > 20 );

  my $type = $src =~ /^\/|[a-zA-Z]:/ ? 'absolute' :
    $src =~ m#^https?://# ? 'http' : 'relative';
  my $data;

  if ( $type eq 'http' && $bn->{param}{AllowTemplateHttp} ) {
    require LWP::UserAgent;
    require HTTP::Request;
    my $ua = LWP::UserAgent->new;
    $ua->agent( '' ); $ua->timeout( 10 );
    my $req = HTTP::Request->new( 'GET', $src ) or return undef;
    my $rc = $ua->request( $req );
    return '[ error : import( '. $src. ' ) status line = '. $rc->status_line. ' ]'
     if $rc->is_error( $rc );
    $data = $rc->content;
  } else {
    $src =~ s#[^\w\.\-]##g unless $bn->{param}{AllowTemplatePathAbsolute};
    my $path = $type eq 'absolute' ? $src :
     ( exists $bn->{param}{TemplatePath} ? $bn->{param}{TemplatePath} : $bn->{sys}{DirSys} ). '/'. $src;
    my $fh = $bn->open( $path );
    return "[ error : import( $path ) ]" unless $fh;
    while ( <$fh> ) { $data .= $_ };
    close $fh;
  }

  return $data;
}

sub _tmpl_attribute {
  my( $bn, $data, $pos ) = @_;
  my $data_len = length $$data;
  my ( @at, $a, $b, $c, $f, $p, $i );
  for ( $i = $pos ; $i < $data_len; $i++ ) {
    $_ = substr $$data, $i, 1;
    if ($a == 0 && /\s/) {
      push @at, $b if $b ne '' && !$f;
      undef $b; $f = 1;
    } elsif (!$p && />/) {
      push @at, $b if $b ne '' && !$f;
      $i++; $f = 1; undef $b; last;
    } else {
      if (/$p/ && $b !~ /\\$/) { $a = 0; undef $p; next }
      if ($c == 1) {
        $c = 0;
        if ( /('|")/ && $b !~ /\\$/ ) { $p = $1; next }
        $a = 0; $b .= $_;
      } else {
        $b .= $_; $f = 0;
        if ( /=/ && !$a ) { $a = 1; $c = 1 }
      }
    }
  }
  return( \@at, $i );
}

sub _tmpl_element {
  my( $bn, $data ) = @_;
  my $tgl = $bn->{template};
  my( $a, $b, $c, $p, $f, %tag );
  my( $at, $i ) = $bn->_tmpl_attribute( $data, pos $$data );

  for ( @$at ) {
    my($name, $val) = split /=/,$_,2;
    $name = lc $name;
    if ($name =~ /^(?:value|text)$/) {
      my $hash = { data => $val };
      $tgl->{nest}++;
      $bn->template_trans($hash);
      $tgl->{nest}--;
      $tag{$name} = $hash;
    } elsif ($name eq '__tag') {
      next;
    } elsif($name eq 'name' && defined $val) {
      $tag{name} = lc $val;
    } elsif($name eq 'if') {
      $val =~ s/\n//g;
      $val =~ s#([^ \*\$\%\(\)\[\]\-/=|+^~!&?><]+)#'$1'#g;
      $val =~ s#\w?/([^/]+)/#my @r;
        for (split /\|/,$1) { push @r, $1 if /^'([^']*)'$/ }
        '/' . (join '|', @r) . '/'#ge;
      $val =~ s/'(\d+)'/$1/g;
      $val =~ s/\$'(\w+)'/\$data->{$1}/g;
      $val =~ s/'(eq|ne|lt|gt|le|ge|not|and|or)'/$1/g;
      $val =~ s/(=|&)+/$1$1/g;
      $val =~ s/(?:==~)/=~/g;
      $val =~ s/([><])==/$1=/g;
      $tag{$name} = $val;
    } elsif( !exists $tag{name} && !defined $val ) {
      $tag{name} = $name;
    } else {
      $val =~ s/\\("|'|\\)/$1/g;
      $tag{$name} = $val;
    }
  }
  return (\%tag, $i);
}

sub translate {
  my( $bn, $tl, $data, $tag, $buffer ) = @_;
  my $gl = $bn->{global};
  my $t = $tl->{__trans};
  my $d = $tl->{__data};
  my $flag = 1 if ref $buffer;
  for (my $i = 0; $i <= $#{$t}; $i++) {
    $flag ? $$buffer .= $t->[$i] : ref $t->[$i] eq 'CODE' ? &{$t->[$i]} : $bn->print($t->[$i]);
    if (ref $d->[$i] eq 'SCALAR') {
      $flag ? $$buffer .= $data->{ ${$d->[$i]} } :
        ref $data->{ ${$d->[$i]} } eq 'CODE' ? &{ $data->{ ${$d->[$i]} } } :
        $bn->print($data->{ ${$d->[$i]} });
      next;
    } elsif (ref $d->[$i] eq 'ARRAY') {
      my($f, $h);
      for (my $n = 0; $n <= $#{ $d->[$i] }; $n++) {
        $h = $d->[$i][$n];
        if ($h->[0]{__tag} eq 'val') {
          $f = 1 if $data->{$h->[0]{name}};
        } elsif (exists $h->[0]{if}) {
          $f = eval("($h->[0]{if})?1:0");
          $bn->print("Error : $@ <br>") if $@;
        } elsif ( $h->[0]{__tag} eq 'require' && $bn->{param}{AllowTemplateRequire} ) {
          my $tmpl = { data => $bn->_tmpl_get_src( $h->[0]{name} ) };
          my $b;
          if ( exists $bn->{template} ) {
            $b = $bn->{template};
            $bn->{template} = { import => $b->{import} };
          }
          $bn->template_trans( $tmpl )->translate( $tmpl, $data, $tag );
          $bn->{template} = $b if $b;
        } elsif (defined $h->[0]{name}) {
          $f = 1 if $data->{ $h->[0]{name} } || ( $h->[0]{name} eq 'name' && $data->{ $h->[0]{name} } ne '' );
        } elsif ($h->[0]{__tag} eq 'else') {
          $f = 1;
        }

        if ($h->[0]{__tag} eq 'unless') { $f = $f ? 0 : 1 }
        if ($f) {
          my $data_back;
          if (defined $h->[0]{target}) {
            if (defined $h->[0]{text}) {
              $data_back = $data->{$h->[0]{target}};
              if (ref $h->[0]{text} eq 'HASH') {
                undef $data->{$h->[0]{target}};
                $bn->translate($h->[0]{text}, $data, $tag, \$data->{$h->[0]{target}});
              } else {
                $data->{$h->[0]{target}} = $h->[0]{text};
              }
            }
            if (defined $h->[0]{link}) {
              $data_back = $data->{$h->[0]{target}} unless $data_back;
              $data->{$h->[0]{target}} = $bn->_trans_link($data->{$h->[0]{target}}, $data, $h->[0]{link});
            }
          }
          $bn->translate($h->[1], $data, $tag, $flag ? $buffer : undef );
          $data->{$h->[0]{target}} = $data_back if $data && $h->[0]{target};
          last;
        }
      }
      next;
    }

    $bn->_trans_elmt($d->[$i], $data, $tag, $buffer);
  }

  $bn;
}

sub trans_values {
  my( $bn, $tmpl, $data, $block ) = @_;
  while ($tmpl =~ /\$(\w+)/g) {
    my $val = lc $1;
    if ($val !~ /^(?:txt|stylesheet)/ && exists $data->{$val}) {
      my $s = pos $tmpl;
      my $len = (length $val) + 1;
      $tmpl = (substr $tmpl, 0, $s - $len) . $data->{$val} . substr $tmpl, $s;
      pos $tmpl = $s - $len + length $data->{$val};
    }
  }
  return $tmpl;
}

sub _trans_elmt {
  my( $bn, $elmt, $data, $tag, $buffer ) = @_;
  my $flag = 1 if ref $buffer;
  my $f = defined $data->{$elmt->{name}} ? 1 : 0;
  if ($elmt->{__tag} eq 'unless') { $f = $f ? 0 : 1 }
  if ($f) { my $str = ''; my $data_back;
    if ($elmt->{target} ne '') {
      if ($elmt->{text} ne '') {
        $data_back = $data->{$elmt->{target}};
        if (ref $elmt->{text} eq 'HASH') {
          $bn->translate($elmt->{text}, $data, $tag, \$data->{$elmt->{target}});
        } else {
          $data->{$elmt->{target}} = $elmt->{text};
        }
      }
      if ($elmt->{link} ne '') {
        $data_back = $data->{$elmt->{target}} unless $data_back;
        $data->{$elmt->{target}} = $bn->_trans_link($data->{$elmt->{target}}, $data, $elmt->{link});
      }
    }
    if ($elmt->{value}) {
      if (ref $elmt->{value} eq 'HASH') {
        $bn->translate($elmt->{value}, $data, $tag, \$str);
      } else {
        $str = $elmt->{value};
      }
    } else {
      $str = $data->{$elmt->{name}};
    }

    $str = &$str($data, $elmt) if ref $str eq 'CODE';
    $data->{$elmt->{target}} = $data_back if $data && $elmt->{target};
    if ($elmt->{target} eq '' && $elmt->{link} ne '') {
      $str = $bn->_trans_link($str, $data, $elmt->{link});
    }
    if ($tag) {
      my $h = $bn->get( $bn->{skin}, 'Tag' . $tag . ucfirst lc $elmt->{name} );
      $str = $bn->set_tag( $h, { string => $str } ) if $h;
    }
    $flag ? $$buffer .= $str : $bn->print($str) ;
  }

}

sub _trans_link {
  my( $bn, $str, $data, $link ) = @_;
  if ( $link eq 'mail' ) {
    if ( $bn->get( $bn->{cfg}, 'UseMailCrypt' ) ) {
      my @c = split /,/,$bn->{flag}{mailcrypt};
      my $s = $data->{mail};
      my( $a, $i, $n, $r, $l );
      $l = length $s;
      while ($n < $l) {
        my $b = substr $s, $n, 1;
        for ($i = 0; $i < 52; $i++) { last if $c[$i] eq $b }
        if (0 <= $i && $i < 26) { $r .= chr $i + 65 }
        elsif (25 < $i && $i < 51) { $r .= chr $i + 71 }
        else { $r .= $b }
        $n++;
      }
      $str =~ s/([^\x81-\x9F])@/$1\x81\x97/g;
      $str = $bn->set_tag({ TAG => 'A', href => 'mailto:', rel => 'nofollow',
      onmouseover => "this.href=BN_mailcrypt('$r')",
      string => $str }) if $str && $data->{mail};
    } else {
      $str = $bn->set_tag({ TAG => 'A', href => 'mailto:' . $data->{mail},
      string => $str }) if $str && $data->{mail};
    }
  } elsif ( $link eq 'url' ) {
      $str = $bn->set_tag({ TAG => 'A', href => 'http://'.$data->{url},
      target => '_blank', string => $str }) if $str && $data->{url};
  } else {
      $str = $bn->set_tag({ TAG => 'A', href => $data->{$link}, string => $str }) if $str && $data->{$link};
  }
  return $str;
}

sub access_log {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  $bn->call('AccessLogInit');
  if ( $bn->get($cf, 'UseCounter') || $bn->get($cf, 'UseAccessLog') )
   { $bn->lo->access_log( $bn->{global}{accesslog} = [] ) }
  $bn;
}

sub access_write {
  my $bn = $_[0];
  $bn->call('AccessWriteInit');
  if ( $bn->get($bn->{cfg}, 'UseAccessLog') && !exists $bn->{flag}{access_write} ) {
    $bn->filelock;
    $bn->lo->access_write( $_[1] );
    $bn->fileunlock;
  }
  $bn;
}

sub reset_javascript {
  my $bn = $_[0];
  my $h = $bn->{flag}{JavaScript};
  if ( $_[1] ) { undef $h->{$_[1]} } else { undef %$h }
}

sub set_javascript {
  my( $bn, $js, $src ) = @_;
  $bn->{sys}{JavaScript}{$js} = $src;
}

sub get_javascript {
  my( $bn, $js, $ref, @arg ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $mo = $bn->{module};
  my $f = \$bn->{flag}{JavaScript}{$js};
  my $h = $bn->{sys}{JavaScript};
  return 0 if $$f;

  if ($h->{$js}) {
    $bn->{flag}{JavaScript}{$js} = 1;
    $$ref .= $h->{$js};
    return 1;
  }

## GetAgent
if ( 'GetAgent' eq $js ) { $$f = 1;
$$ref .= <<SRC;
var ua,an,type,nn,ie,op,mac,win;
ua = navigator.userAgent.toUpperCase();an = navigator.appName.toUpperCase();av = navigator.appVersion;
function GetAgent(){
  if(an.indexOf("NETSCAPE")>=0){s=av.indexOf(" ",0);type=eval(parseInt(av.substring(0,s)))}
  else if(an.indexOf("MICROSOFT")>=0){s=av.indexOf("MSIE ",0)+5;e=av.indexOf(";",s);type=eval(parseInt(av.substring(s,e))-4)}
  if(ua.indexOf("MAC")>=0){mac=1}else if(ua.indexOf("WIN")>=0){win=1}
  if(ua.indexOf("OPERA")>=0||window.opera){op=1}else if(an.indexOf("NETSCAPE")>=0){nn=1}else if(an.indexOf("MICROSOFT")>=0){ie=1}
} GetAgent();

SRC
  return 1;
}

## MM_showHideLayers
if ( 'MM_showHideLayers' eq $js ) { $$f = 1;
$$ref .= <<SRC;
function MM_showHideLayers() { //v3.0
  var i,p,v,obj,args=MM_showHideLayers.arguments;
  for (i=0; i<(args.length-2); i+=3) if ((obj=MM_findObj(args[i]))!=null) { v=args[i+2];
  if (obj.style) { obj=obj.style; v=(v=='show')?'visible':(v='hide')?'hidden':v; }
  obj.visibility=v; }
}

SRC
  return 1;
}

## MM_findObj
if ( 'MM_findObj' eq $js ) { $$f = 1;
$$ref.=<<SRC;
function MM_findObj(n, d) { //v4.0
  var p,i,x;  if(!d) d=document; if((p=n.indexOf("?"))>0&&parent.frames.length) {
  d=parent.frames[n.substring(p+1)].document; n=n.substring(0,p);}
  if(!(x=d[n])&&d.all) x=d.all[n]; for (i=0;!x&&i<d.forms.length;i++) x=d.forms[i][n];
  for(i=0;!x&&d.layers&&i<d.layers.length;i++) x=MM_findObj(n,d.layers[i].document);
  if(!x && d.getElementById) x=d.getElementById(n); return x;
}

SRC
  return 1;
}

## MM_swapImage
if ( 'MM_swapImage' eq $js ) { $$f = 1;
$$ref.=<<SRC;
function MM_swapImage() { //v3.0
  var i,j=0,x,a=MM_swapImage.arguments; document.MM_sr=new Array; for(i=0;i<(a.length-2);i+=3)
   if ((x=MM_findObj(a[i]))!=null){document.MM_sr[j++]=x; if(!x.oSrc) x.oSrc=x.src; x.src=a[i+2];}
}

SRC
  return 1;
}

## MM_swapImgRestore
if ( 'MM_swapImgRestore' eq $js ) { $$f = 1;
$$ref.=<<SRC;
function MM_swapImgRestore() { //v3.0
  var i,x,a=document.MM_sr; for(i=0;a&&i<a.length&&(x=a[i])&&x.oSrc;i++) x.src=x.oSrc;
}

SRC
  return 1;
}

## MM_preloadImages
if ( 'MM_preloadImages' eq $js ) { $$f = 1;
$$ref.=<<SRC;
function MM_preloadImages() { //v3.0
 var d=document; if(d.images){ if(!d.MM_p) d.MM_p=new Array();
   var i,j=d.MM_p.length,a=MM_preloadImages.arguments; for(i=0; i<a.length; i++)
   if (a[i].indexOf("#")!=0){ d.MM_p[j]=new Image; d.MM_p[j++].src=a[i];}}
}

SRC
  return 1;
}

## MM_reloadPage
if ( 'MM_reloadPage' eq $js ) { $$f = 1;
$$ref.=<<SRC;
function MM_reloadPage(init) {  //reloads the window if Nav4 resized
  if (init==true) with (navigator) {if ((appName=="Netscape")&&(parseInt(appVersion)==4)) {
  document.MM_pgW=innerWidth; document.MM_pgH=innerHeight; onresize=MM_reloadPage; }}
  else if (innerWidth!=document.MM_pgW || innerHeight!=document.MM_pgH) location.reload();
} MM_reloadPage(true);

SRC
  return 1;
}

## BN_mailcrypt
if ( 'BN_mailcrypt' eq $js ) {
  $$f = 1;
  my @a = ('A'..'Z','a'..'z');
  my($s,$i1,$i2);
  srand;
  for(@a){
    $i1 = int rand 51; $i2 = int rand 51;
    $s = $a[$i1];
    $a[$i1] = $a[$i2];
    $a[$i2] = $s;
  }
  $bn->{flag}{mailcrypt} = join ',',@a;
  my $b = join ',',map{"\'$_\'"}@a;

$$ref.=<<SRC;
function BN_mailcrypt(str){
  var len = str.length;var a = new Array($b);var n = 0;var r = ''
  while(n<len){ s=str.substr(n,1);c=s.charCodeAt(0);if(c>64&&91>c){r+=a[c-65]}else if(c>96&&123>c){r+=a[c-71]}else{r+=s}n++
  }return 'mailto:' + r
}

SRC
  return 1;
}

## PM_appForm
if ( 'PM_appForm' eq $js ) { $$f = 1;
  my $pm = $mo->{Paint};
  $pm->{app_script} = 1;
  my(@enable_anime, @enable_palette);
  for (@{$pm->{applets}}) {
    push @enable_anime, $_->{anime} && $_->{animation} ne 'false' ? 1 : 0;
    push @enable_palette, $_->{palette} ? 1 : 0 ;
  }
  my $js_anime = join ',', @enable_anime;
  $js_anime = "new Array($js_anime)" if @enable_anime > 1;
  $js_anime = 'anime = ' . $js_anime;

  my $js_palette = join ',', @enable_palette;
  $js_palette = "new Array($js_palette)" if @enable_palette > 1;
  $js_palette = 'palette = ' . $js_palette;

$$ref .= <<SRC;
var palette, anime
function PM_showlayer(t,f){if(f){MM_showHideLayers('l_'+t,'','show')}else{MM_showHideLayers('l_'+t,'','hide')}}
function PM_changeApp(){if(!document.appform) return;var d = document.appform;t = d.fc
  $js_anime
  $js_palette
  num = parseInt(t.selectedIndex)
  if(d.palette && MM_findObj('l_palette')){MM_findObj('l_palette').style.position=''}
  if(!isNaN(num)){PM_showlayer('anime',anime[num]);PM_showlayer('palette',palette[num])}else{PM_showlayer('anime',anime);PM_showlayer('palette',palette)}
}
function PM_getParam(){
  var d = document.appform;
  if(d.anime && MM_findObj('ch_anime')){d.anime.value = MM_findObj('ch_anime').val.checked?1:0;}
  if(d.palette && MM_findObj('ch_palette')){d.palette.value = MM_findObj('ch_palette').val.options[(MM_findObj('ch_palette').val.selectedIndex)].value};
}
SRC

  $bn->get_javascript('MM_findObj',$ref);
  $bn->get_javascript('MM_showHideLayers',$ref);

return 1;
}

## PM_cookieVal
if ( 'PM_cookieVal' eq $js ) { $$f = 1;
$$ref .= <<'SRC';
function PM_cookieVal(){
  var d = document;if(! d.appform)return;
  var a = d.appform; var c = d.cookie
  n = c.match('pmcw:(\\d+)');if(n!=null) a.width.value=n[1];
  n = c.match('pmch:(\\d+)');if(n!=null) a.height.value=n[1]
  n = c.match('pmapp:(\\w+)')
  if(n!=null && a.fc.options){for(i=0;i<a.fc.options.length;i++){
  if(a.fc.options[i].value=='paint_'+n[1]){a.fc.options[i].selected=1;break}}}
  n = c.match('pma:(\\d)');if(n!=null){var obj;if(MM_findObj('ch_anime')){obj=MM_findObj('ch_anime').val}
  else if(MM_findObj('anime')){obj=MM_findObj('anime')}obj.checked=n!=null&&n[1]=='1'?1:0}
  n = c.match('pmp:(\\w+)');if(n!=null){var obj;if(MM_findObj('ch_palette')){obj=MM_findObj('ch_palette').val}
  else if(MM_findObj('palette')){obj=MM_findObj('palette')}
  for(i=0;i<obj.options.length;i++){if(obj.options[i].value == n[1]){obj.options[i].selected=1;break}}}
}

SRC
  $bn->get_javascript('PM_appForm',$ref);

  return 1;
}

## IM_Preview
if ( 'IM_Preview' eq $js ) { $$f = 1;
  my $url = $bn->get($cf, 'DirFileUrl');
  my $images = $bn->get($cf, 'DirImagesUrl');
$$ref.=<<SRC;
function IM_Preview(el,ta){
  var file = el.options[el.selectedIndex].value
  MM_findObj(ta).src = file ? file.indexOf('http',0) == 0 ? file : '$url/icon/' + file : '$images/lf_close.gif';
}

SRC
  $bn->get_javascript('MM_findObj',$ref);

  return 1;
}

## LayerForm
if ( 'LayerForm' eq $js ) { $$f = 1;
  my($close,$submit);
  undef $bn->{sys}{JavaScript}{NNSendScript};
  my $logno = $bn->get($sk, 'ViewThreadNumber') eq 'virtual' ? 'no' : 'log';
  my $arg = $bn->set_text_arg($bn->{arg}{LayerFormOut});
  $arg = "+\'&$arg\'" if $arg;
  my @text = $bn->info('LAYERFORM_TEXT');
  my $bbsurl = $bn->get($cf, 'BBSNoteUrl');
  my $mode = $bn->{in}{mode};
  my $sort = $bn->{in}{sort};
  my $page = $bn->{in}{page};

if($bn->{global}{UserBrowserVer} =~ /NN 4/){

  $bn->call('GetJavaScriptLayerFormNN4');

  $close = '<font size="2"><input type="button" value="w" onclick="MM_showHideLayers(\'Comment\',\'\',\'hide\')" width="20" height="20">';
  $submit = '<input type="button" value="'.$arg[2].'" onclick="LF_nnsend()">';
  my $farg = $bn->set_form_arg($bn->{arg}{ReplyForm});

${$arg[0]}.=<<SRC;
<form method="post" name="nnsend" action="$bbsurl">
<input type="hidden" name="fc" value="write"><input type="hidden" name="log">
<input type="hidden" name="name"><input type="hidden" name="mail">
<input type="hidden" name="url"><input type="hidden" name="message">
<input type="hidden" name="mode" value="$mode">
<input type="hidden" name="sort" value="$sort">
<input type="hidden" name="page" value="$page">
<input type="hidden" name="password">
$farg</form>

SRC
}else{
  my $img_dir = $bn->get($cf, 'DirImagesUrl');
  $close = '<a onclick="MM_showHideLayers(\'Comment\',\'\',\'hide\')"><img src="'.$img_dir.'/lf_close.gif" alt="'.$text[0].'"></a>';
  $submit = '<input class="reply" type="submit" value="'.$arg[2].'">';
}

  my $tag = $arg[1];
  $tag =~ s/\$close/$close/;
  $tag =~ s/\$submit/$submit/;
  ${$arg[0]}.=$tag;

  my $NNSendScript = $bn->{sys}{JavaScript}{NNSendScript};

  # $arg[2]; submit text

$$ref .= <<SRC;
var d = document;
var log,no,t,d_d,p,lfb,lfx,iex,gX,gY;
function LF_open(lrn,lvn,e,m){var d = document;log = lrn;no = lvn;d_d = 0;window.onerror=LF_out;
  if(ie&&type>=0){x=e.clientX;y=e.clientY}else if(nn&&type==5){x=e.pageX;y=e.pageY}else if(op||(nn&&type==4)){x=gX;y=gY}else{LF_out()}
  LF_title(m);lfb = MM_findObj("layerbar");lfb.value=t;p=MM_findObj("Comment");
  if((!op&&type<4)||(op&&type!=1)){if(document.body.scrollTop) y+=document.body.scrollTop;if(document.body.scrollLeft) x+=document.body.scrollLeft}
  if(p.style){p=p.style}p.left=x-370;iex=parseInt(p.left);
  if(ie){p.top=y+15;p.filter="Alpha(Opacity=100,FinishOpacity=85,Style=1,StartX=1,StartY=1,FinishX=30,FinishY=30)";
  }else if(op||(nn&&type>3)){p.top=y+18}MM_showHideLayers('Comment','','show');if(type==4){d.nnsend.log.value=log}else{d.layerform.log.value=log}
}var d = document;if(d.layers){d.captureEvents(Event.MOUSEMOVE)}d.onmousemove=LF_capture;
function LF_out() {window.location=unescape('$bbsurl?fc=post&log='+log+'&no='+no$arg)}
function LF_title(m){a="";len=0;tlen = m.length+(log+"").length;limit = win?type>=4?55:60:35;
  for(i=0;i<tlen&&len<limit;i++,len++){c=escape(m.charAt(i));if(c.length>3)len++}
  if(len>=limit){m=m.substring(0,i)+'...';}else{if(op){limit-=2}else{limit+=2}for(i=len;i<limit;i+=2){a+="@"}}
  t="No."+$logno+"$text[1]"+m+"$text[2]$text[3]"+a}
function LF_drag(event){d_d=d_d?0:1;if(event.clientX){lfx=event.clientX}lfb.value="DRAG";if((op||type==4||(mac&&ie))&&!d_d){lfb.value=t}}
function LF_drop(){if(op||type==4){return}if(mac&&ie){iex=parseInt(p.left);return}d_d=0;lfb.value=t;if(d.all&&type){iex=parseInt(p.left)}}
function LF_move(gX,gY){if(ie|(op&&type>1)){p.visibility="hidden";p.left=gX-(lfx-iex);p.top=gY+document.body.scrollTop-15;p.visibility="visible"}
  else{p.left=gX-200;p.top=gY-20}}
function LF_capture(e){var p=MM_findObj("Comment");if(document.all){gX=event.clientX;gY=event.clientY}
  else if(op){gX=e.clientX;gY=e.clientY}else if(nn){gX=e.pageX;gY=e.pageY}if(d_d){LF_move(gX,gY)}}
function LF_nnsend(){lff=MM_findObj("layerform");s=MM_findObj('nnsend');s.name.value=lff.name.value;s.mail.value=lff.mail.value;s.url.value=lff.url.value;
  s.message.value=lff.message.value;s.password.value=lff.password.value;$NNSendScript
  s.submit()}

SRC

  return 1;
}

## LayerFormScript
if ( 'LayerFormScript' eq $js ) { $$f = 1;
  $bn->get_javascript('GetAgent',$ref);
  $bn->get_javascript('LayerForm',$ref,@arg);
  $bn->get_javascript('MM_findObj',$ref);
  $bn->get_javascript('MM_showHideLayers',$ref);
  $bn->get_javascript('MM_reloadPage',$ref);
  return 1;
}
  return -1;
}

sub set_applet_form_script {
  my $bn = $_[0];
  my $mo = $bn->{module};
  return unless exists $mo->{Paint};
  my $pm = $mo->{Paint};
  my $flag = $mo->{flag};
  my $js = \$bn->{data}{javascript};

  $flag->{set_applet_form_script} = 1;

  $bn->call('SetAppletFormScriptBegin');

  return unless $flag->{set_applet_form_script};

  if ($bn->{flag}{RefleshIndex}) {
    $bn->get_javascript('PM_cookieVal', $js);
    $bn->set_call('SkinFootTranslate',sub{$bn->print('<SCRIPT>PM_cookieVal()</SCRIPT>') if $bn->{flag}{RefleshIndex}});
  }

  return unless $pm->{layout}{complex};

  $bn->get_javascript('PM_appForm', $js);

}

sub get { $_[1]->{__var}{ $_[2] } }

sub set {
  my $bn = shift;
  my $sk = $bn->{skin};
  my($cf, $var, $val) = @_;
  my $type = $var =~ s/^($|%|@)// ? $1 : $var =~ /^Tag/ ? '%' : '$';

  unless (exists $cf->{__type}{$var}) {
     $bn->define($cf, $var, undef, $type);
  }

  if ($cf->{__type}{$var} eq '@') {
    push @{ $cf->{__var}{$var} }, $val;
  } elsif ($cf->{__type}{$var} eq '%') {
    my($n, $v) = split /\s+/, $val, 2;
    $cf->{__var}{$var}{$n} = $v;
  } else {
    $cf->{__type}{$var} = '$';
    $cf->{__var}{$var} = $val;
  }
}

sub define {
  my $bn = shift;
  my($cf, $var, $val, $type) = @_;
  $type = $type ? $type : $cf->{__type}{$var} ? $cf->{__type}{$var} : '$';
  undef $cf->{__var}{$var};
  $cf->{__type}{$var} = $type;

  if ($cf->{__type}{$var} eq '@') {
     $cf->{__var}{$var} = [];
  } elsif ($cf->{__type}{$var} eq '%') {
     $cf->{__var}{$var} = {};
  } else {
     $cf->{__var}{$var} = undef;
  }
  $bn->set($cf, $var, $val) if defined $val;
}

sub cfg2join {
  my $bn = shift;
  return join "\n",@{ $bn->get(@_) } if ref $bn->get(@_) eq 'ARRAY';
}

sub cfg2hash {
  my( $bn, $cf, $cfg ) = @_;
  my $val = $bn->get( $cf, $cfg );
  $bn->define( $cf, $cfg, undef, '%' );
  if ( ref $val eq 'HASH' ) {
    for ( keys %$val ) { $bn->set( $cf, $cfg, "$_\t$val->{$_}" ) }
  } elsif ( !ref $val ) {
    for ( split /,/, $val ) {
      my( $name, $val ) = split /:/, $_, 2;
      $bn->set( $cf, $cfg, "$name\t$val" );
    }
  }
  $bn;
}

sub mail_crypt {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  if ($bn->get($cf, 'UseMailCrypt')) {
    $bn->set_call('SkinHeadFormat',
        sub{ $bn->get_javascript('BN_mailcrypt', \$bn->{data}{javascript}) } );
  }
  $bn;
}

sub get_form {
  my( $bn, $a, $name ) = @_;
  my( $r, $c, $d, $e );
  $r = 0;
  $d = 0;
  for (@$a) {
    $c = 0;
    for (@$_) {
      my @array;
      $e = 0;
      $bn->get_ref($_, \@array);
        for (@array) {
          return ++$r,$c,$e,$_ if ($name && $$_{name} eq $name);
          $e++;
        }
      $c++ if @array;
    }
    $d = $c if $d < $c;
    $r++ if $c;
  }

  return $r,$d unless $name;
  return -1,-1;
}

sub set_form {
  my( $bn, $a, $e, $option, $target ) = @_;
  my( $row, $col, $el, $ref_name ) = split ',',$target;
  my $name = $row if $col eq '';
  if ($option eq 'add' && !$name) {
    $row = $#$a;
  } elsif($name) {
    ($row,$col,$el,$ref_name) = $bn->get_form($a,$name);
  }

  return -1,-1 if($row<0 || $col<0);

  if ($option eq 'add' || $option eq 'insert') {
    my @ref = ref $e eq 'REF' ? $$e : $e;
    if(ref $ref[0] eq 'ARRAY' && ref $ref[0][0] eq 'ARRAY'){
      if($option eq 'add')    {  $row++    }
      elsif($option eq 'insert')  {  $row--    }
      $row = 0 if $row < 0;
      splice @$a,$row,0,@ref if ref $a eq 'ARRAY';
      return $row,0;
    }
    $bn->end('DEFF_FORM_ARRAY_TYPE_ROW');
  }

  $row--;
  if ($option eq 'delete') {
    my $ref = $$a[$row][$col];
    return $row,$col if (splice @$ref,$el,1)
  }
  if ($option eq 'replace') {
    my @ref = $$e;
    if (ref $$e eq 'ARRAY') {
      if(ref $ref[0] eq 'ARRAY'){
        if(ref $ref[0][0] eq 'ARRAY'){
          splice @$a,$row,1,@ref;
          return $row,0;
        }elsif(ref $ref[0][0] eq 'HASH'){
          my $ref = $$a[$row][$col];
          my $r = $$e;
          @$ref = @$r;
          return $row,$col;
        }
      }
    } elsif(ref $$e eq 'HASH') {
      my $ref = $$e;
      %$ref_name = %$ref;
      return $row,$col;
    }
    $bn->end('DEFF_FORM_ARRAY_TYPE_ROW');
  }
  if ($option eq 'left') {
    my $ref = $$a[$row][$col];
    if ($el>0) {
      splice @$ref,$el,0,ref $e ? $e : $$e;
    } else {
      unshift @$ref,ref $e ? $e : $$e;
    }
    return $row,$col,$el;
  }
  if ($option eq 'right') {
    my $ref = $$a[$row][$col];
    $el+=2;
    splice @$ref,$el,0,ref $e ? $e : $$e;
    return $row,$col;
  }

  return -1,-1;
}

sub get_temp_file {
  my $bn = $_[0];
  my $tmp = $bn->{temp};
  my $dir = $bn->{sys}{DirSys};
  srand;
  for (my $i = 0; $i < 100; $i++) {
    my $f = "$dir/$ENV{'REMOTE_ADDR'}_".(int(rand(999999))).".tmp";
    unless (-e $f) {
      my $fh = $bn->open($f, '+>');
      $tmp->{num}++;
      $tmp->{fh}{$tmp->{num}} = $fh;
      $tmp->{file}{$tmp->{num}} = $f;
      return ($fh, $f);
    }
  }
  $bn->error('ERR_TMP_FILENAME');
}

sub purge_temp_file {
  my( $bn, $flag ) = @_;
  my $tmp = $bn->{temp};

  if ($flag) {
    if ( my $dh = $bn->opendir($bn->{sys}{DirSys}) ) {
      while( my $path = readdir $dh ){
         unlink "$bn->{sys}{DirSys}/$path" if $path =~ /\.tmp$/ }
      closedir $dh;
    }
  }

  for (keys %{$tmp->{fh}}) {
    close $tmp->{fh}{$_} if fileno $tmp->{fh}{$_};
    unlink $tmp->{file}{$_};
    delete $tmp->{fh}{$_};
    delete $tmp->{file}{$_};
  }
}

sub get_useragent {
  my $bn = $_[0];
  my( $a, $c, $o );

  my $b = $bn->{MOD_PERL} ? $bn->{apache}->headers_in->{'User-Agent'} : $ENV{'HTTP_USER_AGENT'} ;

  $b =~ s/</&lt;/g; $b =~ s/>/&gt;/g; $b =~ s/\n|\t//g;

  if($b=~ /Opera\/? ?([\w\.]+)/) { $a = 'Opera'; $c = "Opera $1" }
  elsif($b=~ /MSIE ([\w\.]+)/) { $a = 'MSIE'; $c = "MSIE $1"; $c =~ s/\.0$//; }
  elsif($b=~ /DreamPassport/) { $a = 'DC'; $c = 'DreamPassport' }
  elsif($b=~ /iCab \w\/?([\w\.]*)/) { $a = 'iCab'; $c = "iCab $1" }
  elsif($b=~ /Firefox\/?([\w\.]*)/) { $a = 'Firefox'; $c = "Firefox $1" }
  elsif($b=~ /Safari\/?([\w\.]*)/) { $a = 'Safari'; $c = "Safari $1" }
  elsif($b=~ /Mozilla\/5/) {
    if ($b =~ /Netscape\/?([\w\.]+)/) { $a = 'NN'; $c = "NN $1" }
    else {$a = 'Mozilla'; $c = "Mozilla" . ($b =~ /rv:([\d\.]+)/?" $1":'')} }
  elsif($b=~ /Mozilla\/(4|3)/ && $b!~ /Compatible/) { $a = 'NN'; $c = "NN $1" }
  elsif($b=~ /MSIE/) { $a = 'MSIE'; $c = 'MSIE' }
  else { $a = 'OTHER'; $c = 'OTHER' }
  if ( $a eq 'MSIE' && $b =~ /Netscape\/?([\w\.]+)/ ) { $c .= "(NN $1)" } ## NN8 on IE engine

  my @hash = (
  'Windows Vista:WinVista', 'Windows NT 6.0:WinVista', 'Windows NT 6:WinVista',
  'Windows XP:WinXP', 'Windows NT 5.1:WinXP', 'Windows NT 5:Win2000',
  'Windows NT:WinNT', 'Windows 2000:Win2000', 'Win2000:Win2000', 'WinNT:WinNT',
  'Win95:Win95', 'Windows 95:Win95', 'Windows 98:Win9x',
  'Mac OS X:MacOSX', 'Mac_PowerPC:Mac', 'Macintosh:Mac', 'PPC:Mac', '68K:Mac',
  'DreamPassport:DC', 'FREEBSD:FreeBSD', 'FRINBSD:FrinBSD', 'Linux:Linux');

  for (@hash) {
    @_ = split /:/;
    if($b =~ /\Q$_[0]\E/){ $o = $_[1];last }
  }

  if ($o eq "Win9x") { $o = $b =~ /Win 9x/ ? 'WinMe' : 'Win98' }
  if ($o eq "Mac" && $b =~ /MSIE 5.22/){ $o = 'MacOSX' }
  $o ||= 'OTHER';
  return( $b, '', '', '' ) if $bn->{global}{HOST} =~ /googlebot\.com/;
  return( $b, $o, $a, $c );
}

sub get_user_ip {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  return ($gl->{IP}, $gl->{HOST}) if $gl->{IP} && $gl->{HOST};
  if ($bn->{MOD_PERL}) {
    my $c = $bn->{apache}->connection;
    $gl->{IP} = $c->remote_ip;
#    $gl->{HOST} = $c->remote_host;
  } else {
    $gl->{IP} = $ENV{REMOTE_ADDR};
#    $gl->{HOST} = $ENV{REMOTE_HOST};
  }

  $gl->{HOST} = $gl->{IP} if $gl->{HOST} eq '';
  $gl->{HOST} = gethostbyaddr( pack('C4', split /\./, $gl->{IP}), 2 ) || $gl->{IP} if $gl->{HOST} eq $gl->{IP};
  return ($gl->{IP}, $gl->{HOST});
}

sub get_date {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $TIME = defined $_[1] ? $_[1] : time + $bn->get($cf, 'TimeDiffSec');
  my @youbi=('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
  my @tuki=('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
  my($sec,$min,$hour,$mday,$month,$year,$wday) = gmtime($TIME + 2*365*24*3600);
  $bn->{sys}{cookie_limit} = sprintf( "%s, %02d-%s-%4d %02d:%02d:%02d GMT",$youbi[$wday],$mday,$tuki[$month],$year+1900,$hour,$min,$sec);
  my @T=localtime($TIME);
  return sprintf( "%4d/%02d/%02d (%s) %02d:%02d:%02d",$T[5]+1900,++$T[4],$T[3],$youbi[$T[6]],$T[2],$T[1],$T[0]);
}

sub get_cookie {
  my $bn = $_[0];
  my $cookie = {};
  my $ck = $bn->{MOD_PERL} ? $bn->{apache}->headers_in->{'Cookie'} : $ENV{HTTP_COOKIE};
  if ($ck ne '') {
    my $s = index $ck, "BBSNote=";
    if ($s != -1) {
      my $e = index $ck, ">", $s;
      $e = length $ck if $e == -1;
      for (split ',', substr $ck, $s, $e - $s) {
      my($n, $v) = split ':';
      $bn->unescape(\$v);
      $cookie->{$n} = $v;
      }
    }
  }
  $cookie;
}

sub get_palette {
  my( $bn, $pname, $ref_pname, $ref_palette ) = @_;
  my $mo = $bn->{module};
    return unless exists $mo->{Paint};
  my $sys = $bn->{sys};
  my $path = $bn->{sys}{DirSys} . '/palette.' . $bn->{sys}{Exe};
  return unless -e $path;

  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_SYS_NOTOPEN', $path);
  my @palette;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};

    while (<$fh>) {
      s/[\t\s]//g;
      @_ = split /,/,$_,3;
      my $key = shift;
      my $name = shift;

      if ($name ne '') {
        push @$ref_pname, "$key\t$name";
        if ($pname eq 'all') {
          push @palette, $_[0];
        } elsif($pname eq $key) {
          @$ref_palette = split ',', $_[0];
        }
      }
    }

    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;

  close $fh;

  return \@palette if $pname eq 'all';
}

sub write_cookie {
  my( $bn, $ref_cookie, $over_write ) = @_;
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $fl = $bn->{flag};
  my $sys = $bn->{sys};
  my $gl = $bn->{global};

  return if $fl->{write_cookied};
  $fl->{write_cookied} = 1;

  $bn->get_date unless $bn->{sys}{cookie_limit};

  my $val = $VERSION;
  my $vkey = 'BBSNote=Version';
  delete $ref_cookie->{$vkey};

  $bn->call('WriteCookieInit');

  $bn->escape(\$val);

  my $data = "$vkey:$val,";
  $data .= join ',', map{
    $val = $ref_cookie->{$_};
    $bn->escape(\$val);
    "$_:$val"
    } grep { $_ && $ref_cookie->{$_} ne '' }keys %{$ref_cookie};
  $data .= '>';

  my $cookie_path = $ENV{SCRIPT_NAME};
  $cookie_path =~ s/[^\/]*$//;

  my $cookie = $data . "; expires=$bn->{sys}{cookie_limit}; path=$cookie_path";
  if ($bn->get($cf, 'UseMetaCookie')) {
    $gl->{meta_cookie} = $cookie;
    $bn->set_call('SkinHeadFormat', sub{
      push @{ $sk->{META} },{ TAG => 'META', content => $cookie, 'http-equiv' => 'Set-Cookie' }
    });
  }

  if ($over_write) {
    if ($bn->{MOD_PERL}) { $bn->{apache}->headers_in->{'Cookie'} = $cookie }
    else { $ENV{HTTP_COOKIE} = $cookie }
  }

     if ($bn->{MOD_PERL} == 1) { $bn->{apache}->header_out('Set-Cookie' => $cookie) }
  elsif ($bn->{MOD_PERL} == 2) { $bn->{apache}->headers_out->add('Set-Cookie' => $cookie) }
  else { $bn->print("Set-Cookie: $cookie\n") }

  "Set-Cookie: $cookie\n";
}

sub get_admin_pass {
  my $bn = $_[0];
  my $in = $bn->{in};
  my $ck = $bn->{cookie};
  my($pwd, $usr);
  if (exists $in->{password}) {
    ($pwd, $usr) = @{$in}{'password','name'}
  } else {
    $bn->{flag}{AutoLogin} = 1;
    if ($ck->{adminpass}) {
      ($pwd, $usr) = @{$ck}{'adminpass','adminuser'}
    } else {
      ($pwd, $usr) = @{$ck}{'password','name'}
    }
    $usr = $in->{name} unless $usr;
    $in->{name} = $usr unless $in->{name};
    $in->{password} = $pwd;
  }
  return ($pwd, $usr);
}

sub check_password {
  my( $bn, $pass, $crypt_pass, $check_auth_off ) = @_;
  return 0 if $pass eq '';
  return 1 if $crypt_pass eq $bn->crypt_string($pass);
  return 1 if $bn->get( $bn->{cfg}, 'UseCrypt' ) == 2 && $crypt_pass eq $pass;
  return $bn->check_authority($pass) unless $check_auth_off;
  0;
}

sub check_authority {
  my( $bn, $pass ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  my(@usr, @pwd, $f);
  delete $fl->{AUTHORITY};
  delete $fl->{AUTHORITY_NAME};
  delete $fl->{AUTHORITY_PASSWORD};

  my $admin = $bn->get($cf, 'AdminPassword');
  my $subpass = $bn->get($cf, 'SubPassword');

  push @usr, $in->{name} if exists $in->{name};
  push @usr, $ck->{adminuser} if exists $ck->{adminuser};
  push @usr, $ck->{name} if exists $ck->{name};
  push @usr, $in->{login_usr} if exists $in->{login_usr};
  push @pwd, $pass;
  push @pwd, $ck->{password} if exists $ck->{password};
  push @pwd, $ck->{adminpass} if exists $ck->{adminpass};

  for ($admin, $subpass) {
    if ($_ ne '' && ref $_ eq 'ARRAY') {
      my(%c, %d);
      for my $a (@$_) {
        my @b = split /:/, $a ,2;
        $c{$b[0]} = $b[1];
        if ($b[1]) {
          $bn->escape(\$b[1]);
          $d{$b[1]} = $b[0];
        }
      }
      for my $p (@pwd) {
        next unless exists $c{$p};
        if (defined $c{$p}) {
          for (@usr) {
            my $name = $_;
            $bn->escape(\$name);
            if ($d{$name} eq $p) {
              $fl->{AUTHORITY} = $f ? 'SUB' : 'ADMIN';
              $fl->{AUTHORITY_NAME} = $_;
              $fl->{AUTHORITY_PASSWORD} = $p;
              return 1;
            }
          }
        } else {
          $fl->{AUTHORITY} = $f ? 'SUB' : 'ADMIN';
          return 1;
        }
      }
    }
    $f = 1;
  }
  return 0;
}

sub check_pool {
  my $bn = $_[0];
  my $ck = $bn->{cookie};
  return if !$ck->{pmf} || $bn->{sys}{MODE} || $bn->{in}{mode} eq 'catalog';
  my $path = "$bn->{sys}{DirSys}/pool.$bn->{sys}{Exe}";
  return if !-e $path;
  $bn->pm->check_open_page_pool;
}

sub get_reply_status {
  my( $bn, $log ) = @_;
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $path = $bn->filelog($log);
  $bn->error('ERR_DATA_NOTFOUND', ($path =~ /([^\/\\]+)$/ ? $1 : $path)) unless -e $path;

  my $fh = $bn->open($path);
  $fh or $bn->error('ERR_DATA_NOTOPEN', $path);

  my $log_size = (stat $fh)[7];
  my $log_count = -1;
  if ($bn->get($cf, 'UseThreadLock')) {
    delete $fl->{thread_lock};
    my $ex = {};
    $_ = <$fh>; $log_count++;
    $bn->get_log_main_value($_, {}, $ex);
    $fl->{thread_lock} = 1 if $ex->{thread_lock};
  }
  while ( <$fh> ) { $log_count++ }
  close $fh;
  return ($log_count, $log_size)
}

sub check_access_alert {
  my $bn = $_[0];
  my $fl = $bn->{flag};
  my $flag;
  for (qw/ALERT_ACCESS_HOST ALERT_ACCESS_IP ALERT_ACCESS_RAWIP ALERT_ACCESS_POROXY/) {
    if (exists $fl->{$_} && $fl->{$_}) {
      $bn->frame($bn->info($_,$bn->{global}{IP}, $bn->{global}{HOST},
        ($_ eq 'ALERT_ACCESS_POROXY' ? split /,/,$fl->{$_} : '')));
      last;
    }
  }
}

sub check_access {
  my( $bn, $flag ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  my $ck = $bn->{cookie};
  my $gl = $bn->{global};
  my $proxy = int $bn->get($cf, 'LimitSecurityProxy');
  $fl->{ADMIN} = 1 if $bn->check_authority(
   exists $in->{password} ? $in->{password} : undef ) && $fl->{AUTHORITY} eq 'ADMIN';
  return if !$flag && $bn->{in}{fc} eq 'setting';
  return if $proxy >= 4 &&
  $bn->get($cf, 'LimitSecurityDenyIp') eq '' &&
  $bn->get($cf, 'LimitSecurityDenyHost') eq '' &&
  $bn->get($cf, 'LimitSecurityAcceptIp') eq '' &&
  $bn->get($cf, 'LimitSecurityBookMark') eq '' &&
  $bn->get($cf, 'LimitSecurityAcceptHost') eq '' &&
  !$bn->get($cf, 'LimitSecurityAcceptUser') &&
  !$bn->get($cf, 'LimitSecurityDenyRawIp') &&
  !$bn->get($cf, 'LimitSecurityAcceptRef');

  for ( qw/ALERT_ACCESS_HOST ALERT_ACCESS_IP ALERT_ACCESS_RAWIP ALERT_ACCESS_POROXY/ )
   { delete $fl->{$_} }

  my $flags = $bn->check_host;

  $bn->call('CheckAccess');

  if (!$flags->[0]) {
    my $r;
    $r = $flags->[1] ? $fl->{ADMIN} ? $fl->{'ALERT_ACCESS_HOST'} = 1 : $bn->error('ERR_ACCESS_IP_HOST') : '';
    $r = $flags->[2] ? $fl->{ADMIN} ? $fl->{'ALERT_ACCESS_IP'} = 1 : $bn->error('ERR_ACCESS_IP_HOST') : '';
    $r = $flags->[3] ? $fl->{ADMIN} ? $fl->{'ALERT_ACCESS_RAWIP'} = 1 : $bn->error('ERR_ACCESS_RAWIP') : '';
    if ($proxy < 4) {
      my($judge, $proxy_type, $host_type) = $bn->check_proxy;
      my $r = $judge >= $proxy ? $fl->{ADMIN} ? $fl->{'ALERT_ACCESS_POROXY'} = "$judge,$proxy" :
        $bn->error( 'ERR_ACCESS_POROXY',
           $bn->{global}{HOST}, $bn->{global}{IP}, $judge, $proxy_type, $host_type):'';
    }
  }

  $bn->usr->access
    if !$fl->{ADMIN} && $bn->get($cf, 'LimitSecurityAcceptUser');

}

sub check_host {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my($ip, $host) = $bn->get_user_ip;

  my $conv = sub {
    my( $cfg, $rem ) = @_;
    $_ = $bn->get($cf, $cfg);
    return if $_ eq '';
    s/([\#\/\(\)\[\]])/\\$1/g; s/\,/\|/g;
    s/[\r\n\s]//g; s/\*/[\\w\\\/\\-\\\~]+/g;
    s/\?/[\\w\\\/\\-\\\~]/g; s/\./\\./g;
    return 1 if $rem =~ m#^($_)$#;
  };

  my(@flag,@dat);
  $flag[0] = &$conv('LimitSecurityAcceptHost', $host);
  $flag[0] = &$conv('LimitSecurityAcceptIp', $ip) unless $flag[0];
  $flag[1] = &$conv('LimitSecurityDenyHost', $host);
  $flag[2] = &$conv('LimitSecurityDenyIp', $ip);

  $flag[3] = 1 if $bn->get($cf, 'LimitSecurityDenyRawIp') && $host eq $ip;

  if ($bn->get($cf, 'LimitSecurityBookMark') && $bn->{in}{fc} !~ /paint_regist/ && !$ENV{HTTP_REFERER}) {
    $bn->error( 'ERR_ACCESS_BOOKMARK' );
  }
  if($bn->get($cf, 'LimitSecurityAcceptRef') ne '' && $bn->{in}{fc} !~ /paint_regist/ && $ENV{HTTP_REFERER} ne '') {
    $bn->error('ERR_ACCESS_REFERER')
     if $ENV{HTTP_REFERER} !~ m#https?://$ENV{HTTP_HOST}$ENV{SCRIPT_NAME}.*# &&
      ! &$conv('LimitSecurityAcceptRef', $ENV{HTTP_REFERER});
  }
  return \@flag;
}

sub check_proxy {
  my( $bn, $dat ) = @_;
  my $gl = $bn->{global};

  my( $judge, $noproxy, $keepalive, $connect_te, $host_type, $env_status, $proxy_type );

  if ($gl->{HOST} eq $gl->{IP}) {
    $host_type = 'IP Addr.';
  } else {
    if ($gl->{HOST} =~ /www.*cache|www.*proxy|webcache|delegate|gatekeeper|firewall|proxy|cache|squid|^bbs|^http|^www|^web|^dns|^ftp|^mail|^news|^cgi|^gate|^server|^pop|^smtp|^w3\.|^ns\d{0,2}\.|^fw\d{0,2}\./i) {
      $host_type = 'Via a Proxy';
    } elsif($gl->{HOST} =~ /server|gate|www|web|dns|ftp|mail|news|cgi|pop|smtp/i) {
      $host_type = '?';
    }
    my @hostz = split /\./, $gl->{HOST};
    if ($#hostz == 1) {
      $host_type .= '?';
    } elsif($host_type ne 'Via a Proxy' && ($hostz[0] !~ /\d/ || $hostz[0] =~ /\D\d$/)) {
      $host_type .= '?';
    }
  }

  for (keys %ENV) {
    if (/^HTTP_USER_AGENT$/ && $ENV{$_} =~ /via/i ) {
      $env_status   = 'Via a Proxy';
    } elsif(/^HTTP_CONNECTION$/) {
      my $con = $ENV{$_}; $con =~ s/\s//g;
      for( split /,/, $con ) {
        if ( /Keep-Alive/i ) { $noproxy++; $keepalive = 'existed' }
          elsif ( /Close/i ) { $noproxy++ }
           elsif ( /TE/i ) { $connect_te++ }
      }
      $env_status   = 'Anonymized' if !$noproxy && $env_status ne 'Via a Proxy';
    } elsif (/^HTTP_X_FORWARDED_FOR$/ && $ENV{REMOTE_ADDR} ne $ENV{HTTP_X_FORWARDED_FOR}) {
      $env_status = 'Via a Proxy';
    } elsif (/^HTTP_TE$/ && $connect_te) {
      $noproxy++;
    } elsif (/^HTTP_(VIA|.ROXY_.ONNECTION|FORWARDED|CACHE_INFO|FROM|TE|CLIENT_IP|SP_HOST|XONNECTION)$/) {
      $env_status = 'Via a Proxy';
    } elsif (/^CACHE_CONTROL$/ && $ENV{$_} !~ /no-cache/i) {
      $env_status = 'Via a Proxy';
    } elsif (!/^HTTP_(CONNECTION|HOST|PRAGMA|UA_.*|ACCEPT.*|REFERER|MIME|EXTENSION|IF_MODIFIED_SINCE|COOKIE)$/) {
      $noproxy++;
    }
  }

  # Low Suspect
  if ($noproxy >= 2 && $env_status eq '') {
    $proxy_type  = 'NoProxy!';
  } elsif ($env_status eq '' && $keepalive eq '') {
    $proxy_type   = 'Anonymized';
  }

  # NoProxy
  if ($proxy_type eq 'NoProxy!') {
    if ($host_type eq 'IP Addr.') {
      $judge = 0;
    } elsif ($host_type eq 'Via a Proxy') {
      $judge = 2;
    } elsif ($host_type eq '??') {
      $judge = 1;
    } elsif ($host_type eq '?') {
      $judge = 0;
    } else {
      $judge = 0;
    }
  # Anonymized
  } elsif($proxy_type eq 'Anonymized') {
    if ($host_type eq 'IP Addr.') {
      $judge = 1;
    } elsif ($host_type eq 'Via a Proxy') {
      $judge = 2;
    } elsif ($host_type =~ /\?/) {
      $judge = 2;
    } else {
      $judge = 1;
    }

  # Via a Proxy
  } else {
    if ($host_type eq 'IP Addr.') {
      $judge = 2;
    } elsif ($host_type eq 'Via a Proxy') {
      $judge = 4;
    } elsif ($host_type eq '??') {
      $judge = 3;
    } elsif ($host_type eq '?') {
      $judge = 2;
    } else {
      $judge = 3;
    }
  }
  return ($judge, $proxy_type, $host_type, $noproxy, $env_status);

}

sub check_reply_enable {
  my( $bn, $num, $size ) = @_;
  my $cf = $bn->{cfg};
  my $fl = $bn->{flag};
  return 0 if $bn->{sys}{MODE} =~ /DELETE|EDIT/ ||
    ($bn->{sys}{MODE} eq 'ADMIN' && $bn->{in}{cmd} eq 'delete_wide');

  return 0 if $fl->{limit_post_view} = $bn->check_limit_post_view('Reply');
  return 0 if $fl->{thread_lock};

  my $res_enable_size = $bn->get($cf, 'LimitResLogSize') > $size ? 1 : 0;
  my $res_enable_total = $bn->get($cf, 'LimitResTotal') > $num ? 1 : 0;
  my $type = $bn->get($cf, 'LimitResType');
  if ($type == 0) {return $res_enable_total}
  if ($type == 1) {return $res_enable_size}
  if ($type == 2) {return ($res_enable_total && $res_enable_size)}
  if ($type == 3) {return ($res_enable_total || $res_enable_size)}
}

sub check_limit_post {
  my( $bn, $cfg ) = @_;
  my $in = $bn->{in};
  my $cf = $bn->{cfg};
  my $ck = $bn->{cookie};
  my $flag;
  my $val;
  if ($cfg =~ /Upload/) {
    $val = $bn->get($cf, 'Use' . $cfg);
    $val = $val eq '3' ? 0 : $val;
  } else {
    $val = $bn->get($cf, 'LimitPost' . $cfg);
  }
  return 1 unless $val;
  my $auth = $val eq '1' ? 'ADMIN' : 'ADMIN|SUB';
  for ($ck->{adminpass}, $ck->{password}, $in->{password}) {
    $bn->check_authority($_);
    if ($bn->{flag}{AUTHORITY} =~ /$auth/)
     { $flag = 1; last }
  }
  return $flag;
}

sub check_limit_config {
  my( $bn, $cfg ) = @_;
  my $cf = $bn->{cfg};
  my $flag;
  if ( $bn->get( $cf, $cfg ) eq '1' ) {
    $flag = 1 if $bn->{sys}{MODE} =~ /ADMIN/;
  } elsif ( $bn->get( $cf, $cfg ) eq '2' ) {
    $bn->check_authority unless exists $bn->{flag}{AUTHORITY};
    $flag = 1 if $bn->{flag}{AUTHORITY} =~ /ADMIN/;
  } else {
    $flag = 1;
  }
  return $flag;
}
sub check_limit_post_view {
  my( $bn, $cfg ) = @_;
  my $cf = $bn->{cfg};

  if ($bn->get($cf, 'LimitPost' . $cfg)) {
    my $flag;
    if ($bn->get($cf, 'LimitViewPost' . $cfg) eq '1') {
      $flag = 1 if $bn->{sys}{MODE} =~ /REPLY|ADMIN/ && $bn->check_limit_post($cfg);
    } elsif ($bn->get($cf, 'LimitViewPost' . $cfg) eq '2') {
      $flag = $bn->check_limit_post($cfg);
    } else {
      $flag = 1;
    }
    return $flag ? 0 : 1;
  }
}

sub check_limit_up_view {
  my( $bn, $cfg ) = @_;
  my $cf = $bn->{cfg};
  my $val = $bn->get($cf, 'Use'.$cfg.'Upload');
  my $view = $bn->get($cf, 'LimitViewPostUpload');
  if ($val) { return $view ? $bn->check_limit_post($cfg.'Upload') : 1 }
  return 0;
}

sub crypt_string {
  my( $bn, $s, $f ) = @_;
  my $cf = $bn->{cfg};
  my $k = $bn->get($cf, 'UseCryptKey');
  my $c = $bn->get($cf, 'UseCrypt') ? crypt($s,$k):$s;
  $c =~ s/^$k// if $f;
  return $c;
}

sub convert {
  my( $bn, $c, $code, $from ) = @_;
  my $sys = $bn->{sys};
  if ($] > 5.008) {
    if ($from) {
      eval('require Encode');
      Encode::from_to($$c, $from, $code);
    } else {
#      # euc-jp ̏ꍇAR[h͌F ̂ŎgȂ
#      Encode::from_to($$c, 'shift-jis', 'euc-jp');
#      eval('require Encode::Guess');
#      my $enc = Encode::Guess::guess_encoding($$c, qw/shiftjis euc-jp 7bit-jis/);
#      $from = (split / or /, (ref $enc ? $enc->name : $enc) )[0];
#      Encode::encode($code, $$c);
#      $bn->end('',$$c, "$from $enc" );
#      Encode::from_to($$c, $from, $code);
    }
  } elsif($sys->{Language} eq 'jp' && exists $sys->{Convert} && $sys->{Convert} ne '') {
    eval{ require $sys->{Convert} };
    $bn->error('ERR_MOD_NOTOPEN', $sys->{Convert}, $! ) if $!;
    $code = 'sjis' if $code =~ /shift.jis|MacJapanese|cp932/i;
    $code = 'jis' if $code =~ /iso-2022-jp|jis02..-raw|7bit-jis/i;
    $code = 'utf8' if $code =~ /utf-8/i;
    if ($sys->{Convert} eq 'jcode.pl') {
      eval{ jcode::convert($c, $code) };
    } elsif ($sys->{Convert} eq 'Jcode.pm') {
      eval{ Jcode::convert($c, $code) };
    }
  }
  $bn;
}

sub get_ref {
  my( $bn, $ref, $p ) = @_;
  my $r = ref $ref;
  if ($r eq 'HASH') {
    push @$p, $ref;
  } elsif ($r eq 'REF') {
    $bn->get_ref($$ref, $p);
  } elsif ($r eq 'ARRAY'){
    for (@$ref) { $bn->get_ref($_, $p) }
  }
}

sub set_id {
  my( $bn, $val ) = @_;
  my $cf = $bn->{cfg};
  return unless $bn->get($cf, 'UseCrypt');
  my $r;
  my $str = $val->{ip} ? $val->{ip} : $val->{iphost} =~ /(\d+\.\d+\.\d+\.\d+)/ ? $1 :'';
  for (split /\./, $str,4) { $r .= sprintf "%02X", $_ }
  return $bn->crypt_string($r,1);
}

sub set_tag {
  my( $bn, @ref ) = @_;
  my( %t, @si, @st, @ed, @str );

  for (@ref) {
    my(@a, %e);
    $bn->get_ref($_, \@a);
    for my $hash (@a) {
      my $tag = $hash->{TAG} ? lc $hash->{TAG} : 'font';
      if ($t{$tag})
       { my $r = $t{$tag}; for (keys %$r) { $e{$_} = $r->{$_} } }
      $t{$tag} = \%e;
      for (keys %$hash) {
        my $l = lc $_;
        if ('tag' eq $l) { $tag = lc $hash->{$_}; $t{$tag} = \%e }
        else { $e{$l} = $hash->{$_} }
      }
    }
  }

  # /* tag elements
  for my $tag (keys %t) {
  my($noclose, $string, @s, @el, %css);
  my @style  = qw/bold italic under teletype bgcolor background/
    if $tag !~ /^(?:img|input|table|meta|link|body)$/;
  my $r = $t{$tag};
  next if $tag eq 'img' && $r->{src} eq '' || $tag eq 'a' && $r->{href} eq '';
  if ($r->{string})
   { $string = delete $r->{string};
    push @str, $string if $tag ne 'textarea' }
  if (exists $r->{noclose})
   { $noclose = $r->{noclose}; delete $r->{noclose} }

  $r->{title} = $r->{alt} if $r->{alt} && $r->{title} eq '';

  # font face +Osaka
  if ($r->{face})
   { my $f = 1;
    for (split ',', $r->{face}) { if (/Osaka/) { $f = 0; last } }
    $r->{face} .= ',Osaka' if $f }

  # css
  if (exists $r->{style})
   { for (split ';', $$r{style}) {
    my($name, $val) = split ':';
    $name =~ s/[^\w\-]//g;
    $name =~ s/(?:^\s+|\s+$)//g;
    $val =~ s/(?:^\s+|\s+$)//g;
    $css{$name} = $val if $name;
     }
    delete $r->{style} }
  # cfg css -> tag
  for (@style) {
    if (exists $r->{$_}) {
    if ($r->{$_}) {
       if (/bold/)         { push @st, '<b>'; unshift @ed,'</b>' }
      elsif (/italic/)     { push @st, '<i>'; unshift @ed,'</i>' }
      elsif (/under/)      { $css{'text-decoration'} = 'underline' }
      elsif (/delete/)     { $css{'text-decoration'} = 'line-throught' }
      elsif (/teletype/)   { push @st, '<tt>'; unshift @ed,'</tt>' }
      elsif (/bgcolor/)    { $css{'background-color'} = $r->{$_} }
      elsif (/background/) { $css{'background-image'} = "url($r->{$_})" }
    }
    delete $r->{$_};
    }
  }

  for (keys %css) { push @s, "$_:$css{$_}" }
  push @el, 'style="'.(join ';',@s).'"' if @s;
  for (keys %$r) { push @el,"$_=\"$$r{$_}\"" if $$r{$_} ne '' }
  if ( $tag =~ /^(?:img|input|hr|meta|link|body)$/ )
  { push @str, (join ' ','<'.$tag,@el).'>' }
  elsif ($tag eq 'textarea')
   { push @str,(join ' ','<'.$tag,@el).'>'.$string."</$tag>" }
  else
   { next if $tag eq 'font' && !@el;
     if ($noclose) {
     push @si,(join ' ','<'.$tag,@el).'>';
     } else {
     push @st,(join ' ','<'.$tag,@el).'>';
     unshift @ed, "</$tag>";
     } }
   }  # tag elements */

  my $s = join '', @si;
  $s .= (join '',@st).(join '',@str).(join '',@ed) if @str;
  $s;
}

sub set_app_cfg {
  my( $bn, $cfg, $cf, $app_cfg ) = @_;
  my $sys = $bn->{sys};
  my $pcf = $bn->get($cf, $cfg);
  if (ref $pcf eq '') { my %h;
    for (split /\,/,$pcf) { my($n, $v) = split /:/; $h{$n} = $v }
    $pcf = \%h }
  if ( exists $sys->{CFG}{$cfg} && ref $sys->{CFG}{$cfg} eq 'HASH' )
   { for ( keys %{$sys->{CFG}{$cfg}} ) { $pcf->{$_} = $sys->{CFG}{$cfg}{$_} } }
  if (ref $pcf eq 'HASH')
   { for (keys %$pcf) { $app_cfg->{$_} = $pcf->{$_} unless exists $app_cfg->{$_} } }
  $bn;
}

sub set_global_arg {
  my $bn = shift;
  my $flag = shift;
  push @_, $bn->{arg}{global};
  my %h;
  for my $a(@_){map{$h{$_}=$a->{$_}}keys %$a}
  return join '&',(map{"$_=$h{$_}"}grep{$h{$_}ne''}keys %h) if $flag == 0;
  return join '',(map{'<input type="hidden" name="'.$_.'" value="'.$h{$_}.'">'}keys %h) if $flag == 1;
  return %h;
}
sub set_text_arg { my $bn = shift; return $bn->set_global_arg(0, @_) }
sub set_form_arg { my $bn = shift; return $bn->set_global_arg(1, @_) }
sub set_init_arg {
  my $bn = shift;
  my $form = shift;
  my @arg = @_;
  my %h = $bn->set_global_arg(2, @arg);
  for(keys %h){
    my $form_info = [[[
     { TAG => 'INPUT', type => 'hidden', name => $_, value => $h{$_} }
     ]]];
    $bn->set_form($form,\$form_info,'insert','0,0');
  }
  $bn;
}

sub send_http_header {
  my( $bn, $type, $mtime ) = @_;
  my $fl = $bn->{flag};
  my $gl = $bn->{global};
  my $ck = $bn->{cookie};
  my $sys = $bn->{sys};

  select STDOUT;
  return $bn if $fl->{http_header};
  $fl->{http_header} = 1;
  $bn->call('HttpHeadInit');

  my @header;
  $type ||= 'text/html';
  $type .= '; charset='. $bn->{sys}{Charset}
    if $type =~ m!^text/! && $type !~ /\bcharset\b/;

  push @header, 'Expires: Thu, 01 Jan 1970 00:00:00 GMT' if $bn->{http_pragma};
  push @header, 'Last-Modified: '. $gl->{mtime} if $mtime && exists $gl->{mtime};
  push @header, ( $ENV{SERVER_PROTOCOL} eq 'HTTP/1.0' ?
   'Pragma' : 'Cache-Control' ). ': no-cache' if $bn->{http_pragma};

  $bn->write_cookie($ck) if $fl->{write_cookie};

  push @header, 'Content-Type: '. $type;

  if ( $ENV{MOD_PERL} ) {
    $bn->{apache}->send_cgi_header( join "\n", @header, "\n" );
  } else {
    $bn->print( join "\n", @header, "\n" );
  }
  $bn;
}

sub get_in {
  my $bn = shift;
  my(%IN, %in, $buf);

  if ( $bn->{MOD_PERL} ) {
    my $r = $bn->{apache};
    %IN = ( query  => $r->args.'', method => $r->method,
     content_type => $r->headers_in->{'Content-Type'},
     content_length => $r->headers_in->{'Content-Length'} );
  } else {
    %IN = ( query  => $ENV{'QUERY_STRING'}, method => $ENV{'REQUEST_METHOD'},
     content_type => $ENV{'CONTENT_TYPE'}, content_length => $ENV{'CONTENT_LENGTH'} );
  }

  if ( $IN{content_length} > 10285760 ) {
    $bn->{apache}->status(413) if $ENV{MOD_PERL};
    $bn->error('ERR_CONTENT_LENGTH');
  }

  if( $IN{content_type} =~ m#^multipart/form-data# ) {
    my $CR = "\r\n";
    $IN{method} eq 'GET' ? $buf = $IN{query} : $bn->read(\$buf, $IN{content_length});
    my $b = quotemeta ((split /$CR/, $buf, 2)[0]);

    for ( split /$CR$b(?:--)?$CR/, $buf ) {
      my( %names, $f );
      my( $head, $data ) = split /$CR$CR/, $_, 2;
      $head =~ s/$CR/; /g; $head =~ s/: /=/g; $head =~ s/"//g;
      for ( split /; /, $head )
       { my( $n, $v ) = split /=/, $_, 2;
      if ( $n eq 'name' ) { next if($f); $f = 1 }
      $names{$n} = $v if $n =~ /^(?:name|filename|Content-Type)$/ }
      for (keys %names) { $in{"$names{name}_$_"} = $names{$_} if $_ ne 'name' }
      $in{ $names{name} } = $data;
    }
  } else {
    if ( exists $bn->{module}{Paint} && $IN{query} eq 'fc=paint_captor') { $bn->pm->captor }
    else {
      if ($IN{method} eq 'GET') { $buf = $IN{query}; $bn->unescape(\$buf) }
       else { $bn->read(\$buf, $IN{content_length}) }
      $buf =~ s/\+/ /g;
      for (split '&', $buf) { my($n, $v) = split /=/,$_,2; $bn->unescape(\$v); $in{$n} = $v }
    }
  }
  return \%in;
}

sub get_src_value {
  my( $bn, $ref_data, $ref_val, $ref_ex ) = @_;
  $ref_data->{thumbnail_src} = (grep /\.(jpg|png|gif)$/i, $ref_val->{object}, $ref_ex->{thumbsrc}, $ref_ex->{thumbnail}, $ref_ex->{imagesrc})[0];
  $ref_data->{anime_src} = (grep /\.(pch|spch|jpeg|pnm)$/i, $ref_val->{file_name}, $ref_val->{object}, $ref_ex->{animesrc}, $ref_ex->{file_name},)[0];
  $ref_data->{image_src} = (grep /\.(jpg|png|gif)$/i, $ref_val->{file_name}, $ref_ex->{file_name}, $ref_ex->{filename}, $ref_ex->{imagesrc})[0];
  $ref_data->{object_src} = !$ref_data->{image_src} ? (grep $_ ne '', $ref_val->{file_name}, $ref_ex->{file_name}, $ref_ex->{imagesrc}, $ref_ex->{filename})[0] : '';
  $ref_data->{object_src_length} = (grep $_ ne '', $ref_val->{file_length}, $ref_ex->{size}, $ref_ex->{file_size})[0];
  $ref_data->{image_enable} = $ref_data->{image_src} || $ref_data->{thumbnail_src} ? 1 : 0;
  $ref_data->{data_enable} = $ref_data->{image_src} || $ref_data->{thumbnail_src} || $ref_data->{anime_src} || $ref_data->{object_src} ? 1:0;
}
sub get_log_value {
  my( $bn, $ref_data, $ref_val, $ref_ex, $format ) = @_;
  chomp $ref_data;
  my $add_format;
  if ($format eq 'MAIN') { (@$ref_val{$bn->LOG_MAIN_VALUE}, $add_format) = split "\t",$ref_data }
  else { (@$ref_val{$bn->LOG_REPLY_VALUE}, $add_format) = split "\t",$ref_data }
  for (split/\x08/, $add_format) {s/^([^=]*)=(.*)$/$ref_ex->{$1} = $2/e};
  $bn;
}
sub get_log_main_value { $_[0]->get_log_value(@_[1..3], 'MAIN') }
sub get_log_reply_value { $_[0]->get_log_value(@_[1..3], 'REPLY') }
sub get_mtime {
  my( $bn, $path ) = @_;
  my $gl = $bn->{global};
  my $sys = $bn->{sys};
  my $mod = (stat ($path ? $path : "$sys->{DirSys}/index.$sys->{Exe}"))[9];
  $gl->{mtime} = $path || $mod > $gl->{boot_mtime} ? $mod : $gl->{boot_mtime};
  $_ = gmtime($gl->{mtime});
  $gl->{mtime} = sprintf "%s, %02d %s %d %s GMT",$1,$3,$2,$5,$4
  if /(\w+)\s+(\w+)\s+(\d+)\s+(\d+:\d+:\d+)\s+(\d+)/;
  $bn;
}

sub open_index {
  my( $bn, $not_block ) = @_;
  my $gl = $bn->{global};
  my $sys = $bn->{sys};
  my $path = "$sys->{DirSys}/index.$sys->{Exe}";

  unless (-e $path) {
    my $fh = $bn->open( $path, '>' );
    $fh or $bn->error( 'ERR_SYS_NOTINDEXMAKE', $path );
    close $fh;
  }

  my $fh = $bn->open( $path, $not_block ? '<' : undef );
  $fh or $bn->error( 'ERR_SYSINDEX_NOTOPEN', $path );
  binmode $fh;

  $gl->{LogTotal} = $sys->{index}{total} = int((stat $fh)[7] / 6);
  $bn->{temp}{fh}{index} = $fh;
  return $fh;
}

sub close_index {
  my $bn = $_[0];
  my $lfh = $bn->{temp}{fh};
  if ( exists $lfh->{index} ) {
    close $lfh->{index};
    delete $lfh->{index};
  }
  $bn;
}

sub read_index {
  my( $bn, $index, $mode ) = @_;
  my $lfh = $bn->{temp}{fh};
  my $sys = $bn->{sys};
  return -1 unless ( exists $lfh->{index} );
  my $total = $sys->{index}{total};
  return -1 if abs $index >= $total;

  my $pos = $index < 0 ? 2 : 0;
  my $block = $total * 2 * $mode;
  my $fh = $lfh->{index};
  seek $fh, ( ($index * 2) + $block ), $pos
  or $bn->error('ERR_SYS_NOTREADINDEX', $index);

  my $s;
  read $fh, $s, 2
  or $bn->error('ERR_SYS_NOTREADINDEX', $index);

  return unpack 'S',$s;

}

sub cash_boot {
  my $bn = $_[0];
  my $cf = $bn->{cfg};
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};
  my $bc = $cf->{__var};
  my $bt = $cf->{__type};
  my $sc = $sk->{__var};
  my $st = $sk->{__type};
  my $tmpl = $sk->{template};

  my($fh, $tmp) = $bn->get_temp_file;
  my $path = exists $sys->{BootCgi} ? $sys->{BootCgi} :
    $sys->{DirCfgSys} . '/boot.' . $bn->{sys}{Exe};

  select $fh;
  print join "\n",('package BN::boot;','use vars qw( $bn );',
  'sub new { my $bo = bless { }, $_[0]; $bn = $_[1]; $bo }',
  'sub boot {', 'my $bo = shift;', '$bn = $bo unless $bn;',
  'my $cf = $bn->{cfg};','my $sk = $bn->{skin};',
  'my $bc = $cf->{__var};','my $bt = $cf->{__type};',
  'my $sc = $sk->{__var};','my $st = $sk->{__type};',
  'my $tmpl = $sk->{template};','');

  for my $f(keys %{$bc}) {
    print "\$bc->{'$f'}=";
    $bn->_cash_ref($bc->{$f});
    print ";\$bt->{'$f'}='$bt->{$f}';\n";
  }

  for my $f(keys %{$sc}) {
    print "\$sc->{'$f'}=";
    $bn->_cash_ref($sc->{$f});
    print ";\$st->{'$f'}='$st->{$f}';\n";
  }

  for my $f(keys %$tmpl) {
    print "\$sk->{template}{$f}=";
    $bn->_cash_ref($tmpl->{$f});
    print ";\n";
  }

  $bn->call('CashBoot');

  print "}\n1;";
  close $fh;

  select STDOUT;

  unlink $path;
  rename $tmp, $path
    or unlink $tmp && $bn->error('ERR_SYS_NOTWRITE', $path);
  $bn;
}

sub _cash_ref {
  if (ref $_[1] eq 'HASH') {
    print '{';
    my $f = 0;
    for (keys %{$_[1]}) {
      print ',' if $f;
      $f = 1;
      print "'$_'=>";
      $_[0]->_cash_ref($_[1]{$_});
    }
    print '}';
  } elsif (ref $_[1] eq 'ARRAY') {
    print '[';
    my $f = 0;
    for (my $i = 0; $i <= $#{$_[1]}; $i++) {
      print ', ' if $f;
      $f = 1;
      $_[0]->_cash_ref($_[1]->[$i]);
    }
    print ']';
  } elsif (ref $_[1] eq 'SCALAR') {
    print "{name=>'${$_[1]}'}";
  } else {
    my $s = $_[1];
    $s =~ s/\\/\\\\/g;
    $s =~ s/'/\\'/g;
    print '\'', $s, '\'';
  }
}

sub _crypt {
  my( $bn, $f, $str, $h, $k ) = @_;
  my $cf = $bn->{cfg};
  return unless $bn->get($cf, 'UseCrypt');
  my($s, $o, $i, $n, @r);
  $h = $bn->get($cf, 'AdminPassword') unless defined $h;
  $k = $bn->get($cf, 'UseCryptKey') unless defined $k;
  my $key;
  if (ref $h eq 'ARRAY') {
    ($key, $s) = split /:/, $h->[0], 2;
  } else {
    $key = $h;
  }
  my $rkey = pack "C*",reverse unpack "C*", $key;

  $key = crypt($key,$k);
  $key =~ s/^$k//;
  $rkey = crypt($rkey,$k);
  $rkey =~ s/^$k//;

  my @ck = unpack "C*", $key;
  push @ck, unpack "C*", $rkey;

  my $klen = $#ck;
  my @c = unpack "C*",join '',('%','0'..'9','A'..'Z','_','a'..'z');

  $n = $i = 0;
  for (@c) {
    $s = splice @c, $n, 1;
    $ck[$i] % 2 ? push @c, $s : unshift @c, $s;
    $n++; $i++;
    $i = 0 if $i > $klen;
  }

  unless ($f) {
    my @cm;
    for($i=0,$n=0;$n < length $str;$n++){
      $cm[$n] = $ck[$i] % 2;
      $i++;
      $i = 0 if($i>$klen);
    }
    @cm = reverse @cm;

    @r = unpack "C*",$str;
    $n = $i = 0;
    for (@r) {
      if ($cm[$i] % 2) {
        $s = pop @r;
        splice @r,$#cm - $n,0,$s;
      } else {
        $s = splice @r,0,1;
        splice @r,$#cm - $n,0,$s;
      }
      $n++;$i++;
      $i = 0 if $i > $klen;
    }
    $str = join '',pack "C*",@r;
  }

  $s = ''; $n = 0; $o = ord $k;

  for (unpack "C*",$str) {
    if (/[\x25\x30-\x39\x41-\x5A\5F\x61-\x7A]/) {
      $i = 0;
      while ($_ != $c[$i]) {
        $i++;
        $bn->error('ERR_SYS_CRYPTERROR') if$i>$#c
      }
      if($f){
        $i+=$ck[$n];
        $i+= $o;
        $o+= $_;
        while($i>$#c){$i-=@c}
      }else{
        $i-=$ck[$n];
        $i-= $o;
        while($i<0){$i+=@c}
        $o+= $c[$i];
      }
      $n++;
      $n = 0 if($n>$klen);
      $s.= chr $c[$i];
    }
  }

  if ($f) {
    @r = unpack "C*",$s;
    $n = $i = 0;
    for (@r) {
      $s = splice @r, $n, 1;
      $ck[$i] % 2 ? push @r,$s : unshift @r,$s;
      $n++;$i++;
      $i = 0 if $i > $klen;
    }
    $s = join '', pack "C*", @r;
  }

  return $s;
}
sub encrypt {my $bn  = shift;return $bn->_crypt(1,@_)}
sub decrypt {my $bn  = shift;return $bn->_crypt(0,@_)}

sub app { $_[0]->mo( 'App' )   }
sub ad  { $_[0]->mo( 'Admin' ) }
sub pm  { $_[0]->mo( 'Paint' ) }
sub po  { $_[0]->mo( 'Post' )  }
sub lo  { $_[0]->mo( 'Log' )   }
sub hp  { $_[0]->mo( 'Help' )  }
sub ic  { $_[0]->mo( 'Icon' )  }
sub pa  { $_[0]->mo( 'Panic' ) }
sub usr { $_[0]->mo( 'User' )  }
sub cms { $_[0]->mo( 'CMS' )   }
sub xml { $_[0]->mo( 'XML' )   }

sub sea {
  $_[0]->error('ERR_FUNCTION') unless $_[0]->get( $_[0]->{cfg}, 'UseSearch' );
  $_[0]->mo( 'Search' )
}

sub mo {
  my( $bn, $mo ) = @_;
  return $bn->{mo}{$mo} if exists $bn->{mo}{$mo};
  eval( 'use BN::'.$mo.';$bn->{mo}{$mo}=BN::'.$mo.'->new($bn)' );
  $bn->error( 'ERR_MOD_NOTOPEN', 'BN::'.$mo, $@ ) if $@;
  $bn->{mo}{$mo};
}

sub print {
  my $bn = shift;
  $ENV{MOD_PERL} && !$bn->{flag}{CORE_PRINT} ? $bn->{apache}->print(@_) : CORE::print(@_);
}

sub warn {
  my $bn = shift;
  $ENV{MOD_PERL} ? $bn->{apache}->warn(@_) : CORE::warn(@_);
}

sub open {
  my( $bn, $path, $mode ) = @_;
  FileHandle->new( $mode. $path );
}

sub opendir {
  my( $bn, $path ) = @_;
  if ($] > 5.006) {
    opendir( my $dh, $path ) or return undef;
    return $dh;
  } else {
    local *DIR;
    opendir( *DIR, $path ) or return undef;
    return *DIR;
  }
}

sub read {
  my( $bn, $buf, $len ) = @_;
  return $bn unless $len;
  my $max = $len;
  my $offset = 0;
  undef $$buf;
#  while ( my $byte = $ENV{MOD_PERL} ? $bn->{apache}->read($$buf, $max < 80000 ? $max : 80000, $offset)
#      : read(*STDIN, $$buf, ( $max < 80000 ? $max : 80000), $offset) ) {
  while ( my $byte = $ENV{MOD_PERL} ? $bn->{apache}->read($$buf, $max < 80000 ? $max : 80000, $offset)
      : sysread(*STDIN, $$buf, ( $max < 80000 ? $max : 80000), $offset) ) {
    $max -= $byte;
    $offset += $byte;
    last unless $max;
  }
  $bn;
}

sub text {
  my( $bn, $key, $mo ) = @_;
  my $sys = $bn->{sys};
  $mo = 'main' unless $mo;
  my $lfag = 0;

  # alarm on
  eval {
    local $SIG{ALRM} = sub{ die "alarm\n" };
    alarm $sys->{Alarm} if $sys->{Alarm};

    unless (exists $bn->{mo}{Text}{$mo}) {
    $lfag='use BN::Text::' . $sys->{Language} . '::' . $mo .';' .
      '$bn->{mo}{Text}{'.$mo.'} = BN::Text::' . $sys->{Language} . '::' . $mo .'->new';
      eval('use BN::Text::' . $sys->{Language} . '::' . $mo .';' .
      '$bn->{mo}{Text}{'.$mo.'} = BN::Text::' . $sys->{Language} . '::' . $mo .'->new');
    }

    # alarm off
    alarm 0;
  };
  $bn->Trace('ALRM') if $@ =~ /alarm\n/;

  my $s;
  if ($key ne '') {
    $s = $bn->{mo}{Text}{$mo}{$key}
      or $bn->end('SYS_ERROR','Not text '.$mo,"'$key' is not found".$lfag);
    $bn->convert(\$s, $sys->{Charset}, $bn->{mo}{Text}{$mo}{CHARSET})
      if $sys->{Charset} ne $bn->{mo}{Text}{$mo}{CHARSET};
  } else {
    $s = $bn->{mo}{Text}{$mo};
  }
  return $s;
}

sub info {
  my $bn = shift;
  my $inf = shift;
  my @arg = @_;
  my $t = $bn->text($inf, 'main');
  my $n = 1;
  for (@arg){
    $t =~ s/\[_$n\]/$arg[$n-1]/g;
    $n++;
  }
  return split /,/,$t;
}

sub error {
  my $bn = shift;
  my $gl = $bn->{global};
  $gl->{error} = shift;
  $gl->{error_arg} = \@_;
  $gl->{error_text} = $bn->text( $gl->{error}, 'error' );
  $gl->{error_mode} = 'ERROR';
  my $n;
  for ( @{ $gl->{error_arg} } ) {
    $n++;
    $gl->{error_text} =~ s/\[_$n]/$_/g;
  }

  $bn->call( 'Error' );

  $bn->access_write( join "\b",( $gl->{error}, @{ $gl->{error_arg} } ) )->end( $gl->{error_mode}, split /:/, $gl->{error_text}, 2 );
}

sub end {
  my( $bn, $err_mode, $err_title, $err_msg ) = @_;
  my $sk = $bn->{skin};
  my $sys = $bn->{sys};

  $bn->fileunlock if $bn->{flag}{LOCKED};

  select STDOUT;

  $bn->reset_javascript;
  $bn->po->default_post($err_msg) if $err_mode eq 'DEFAULTPOST';

  if ($bn->{flag}{PAINT_CAPTOR}) {
    my $err = $err_title ? "$err_title\n" : '';
    $err .= $err_msg;
    $err =~ s/<br>/\n/ig;
    $err =~ s/(<[\/\w]>)//ig;
    $bn->send_http_header( 'text/plain' )->print("error\n$err");
  } elsif ($err_mode =~ /\d\d\d/ && $bn->{apache}) {
    $bn->{apache}->status($err_mode);
    $bn->EXIT;
  } else {
    $bn->send_http_header->head($err_title);
    $err_msg =~ s/\n/<br>/ig;
    $bn->print("<center><h2>$err_title</h2>$err_msg</center>");
    $bn->foot('NotPageLink');
  }

  $bn->EXIT;
}

sub filelog {
  my $bn = shift;
  my $cf = $bn->{cfg};
  my $head = $_[1] ? $_[1] : $bn->get($cf, 'FileHeadLogs');
  my $exe = $_[2] ? $_[2] : $bn->{sys}{Exe};
  my $inc = "_$_[3]" if $_[3];
  $head =~ s/[^a-zA-Z]//g;
  return sprintf "$bn->{sys}{DirData}/$head\_%06d$inc\.$exe",$_[0];
}

sub filelock {
  my $bn = $_[0];
  return if $bn->{flag}{LOCKED};
  my $sys = $bn->{sys};
  my $try = $sys->{LockTry};

  # alarm off
  eval { alarm 0 };

  while ( !mkdir( $sys->{LockDir}, 0777 ) ) {
    if ( --$try <= 0 ) {
      my $s = (stat $sys->{LockDir})[9];
      if (time - $s > $sys->{LockSec}) {
        rmdir $sys->{LockDir}
          or $bn->error('ERR_LOCKED') if -e $sys->{LockDir};
        last;
      }
      my @txt = $bn->info('SYS_LOCKED');
      $bn->end('ERROR',$txt[1],$txt[2]) if $bn->{in}{fc} ne 'captor';
      $ENV{MOD_PERL} ? $bn->{apache}->send_cgi_header("Content-type: text/plain\n\n"):
      $bn->print( "Content-type: text/plain\n\n",
                  "error\n$txt[0]\n$txt[1]\n$txt[2]" );
      $bn->EXIT;
    }
    sleep($sys->{LockSleep});
  }
  chmod 0777, $sys->{LockDir};
  $bn->{flag}{LOCKED} = 1;
  $bn;
}

sub fileunlock {
  my $bn = $_[0];
  my $dir = $bn->{sys}{LockDir};
  rmdir( $dir ) if -d $dir;
  $bn->{flag}{LOCKED} = 0;
  $bn;
}

sub module {
  my( $bn, $mod, $c ) = @_;
  if ( ref $c eq 'HASH' ) {
    for ( split ',', ( exists $c->{Load} ? delete $c->{Load} : 'Run' ) )
     { $bn->set_call( $_, sub {eval('use '.$mod );eval{$mod->new($bn,$c)}}) }
  } else {
    for ( split ',', $c ) { $bn->set_call( $_, sub{eval('use '.$mod);
    eval{$mod->new($bn)}})};
  }
}

sub set_call {
  my( $bn, $c, $code ) = @_;
  my $hd = $bn->{handler};
  if ( ! ref $c ) {
    $hd->{$c} = () unless exists $hd->{$c};
    push @{$hd->{$c}}, $code;
  } elsif ( ref $c eq 'ARRAY' ) {
    for my $a ( @$c ) {
      $hd->{$a->[0]} = () unless exists $hd->{$a->[0]};
      push @{$hd->{$a->[0]}}, $a->[1];
    }
  } elsif ( ref $c eq 'HASH' ) {
    for my $k ( keys %$c ) {
      $hd->{$k} = () unless exists $hd->{$k};
      push @{$hd->{$k}}, $c->{$k};
    }
  }
}

sub call {
  $_[0]->{call} = $_[1];
  if ( eval '$_ = $_[0]->{handler}{'.$_[1].'}' && 'ARRAY' eq ref )
   { for my $c (@$_) { if ( 'CODE' eq ref $c ){
    my $a = $_[0]->{call}; &$c; $_[0]->{call} = $a } } }
}

sub escape_quote {
  ${$_[1]} =~ s/"/&#34;/g;
  ${$_[1]} =~ s/'/&#39;/g;
}
sub escape {
  ${$_[1]} =~ s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
}
sub unescape {
  ${$_[1]} =~ s/%([0-9A-F][0-9A-F])/pack("C", hex($1))/eg;
}

sub connect_abort {
  return $_[0]->{apache}->connection->aborted
    if $ENV{MOD_PERL} && !$_[0]->{flag}{RefleshIndex};
  return 0;
}

sub EXIT {
  my $bn = $_[0];
  # alarm off
  eval { alarm 0 };
  $bn->fileunlock if $bn->{flag}{LOCKED};
  $bn->CLOSER;
  goto EXIT;
}

sub LOG_MAIN_VALUE {
  qw/num name date title mail url message host ip agent
  file_name file_width file_height file_length password object paint_time applet/
}
sub LOG_REPLY_VALUE {
  qw/num name date message password iphost mail url browser os/
}
sub LOG_HISTORY_VALUE {qw/times mode name title message ip host file_length/}

sub CLOSER {
  my $bn = shift;
  my $sk = $bn->{skin};
  my $lfh = $bn->{temp}{fh};
  for ( qw/flag form menu template __trans __data __var __type/ )
   { for my $k (keys %{ $sk->{$_} } ){ delete $sk->{$_}{$k} } }
  for (keys %{ $lfh }) { close $lfh->{$_} };
  $bn->purge_temp_file(1);
}


sub Trace {
  my( $bn, $hook ) = @_;
  my $in = $bn->{in};
  my @err;
  $hook .= ' ';
  $bn->get_user_ip;
  push @err, 'script:'. $bn->{apache}->filename;
  push @err, "ip:$bn->{global}{IP}";
  push @err, "host:$bn->{global}{HOST}";
  if (ref $in eq 'HASH') {
    for (keys %$in) {
     if ($_ eq 'file') {
       push @err, "file:". length $in->{$_};
       next;
     }
     push @err, "$_:$in->{$_}";
    }
  }
  $bn->warn($hook . join ',', @err);
  $bn->CLOSER;
  $bn->{apache}->status(408) if $bn->{MOD_PERL};

  package DB;    # To get subroutine args.

  syswrite(STDERR, 'Caught a SIG', 12);
  syswrite(STDERR, $hook, length($hook));
  syswrite(STDERR, ' at ', 4);
  my ($pack,$file,$line) = caller;
  syswrite(STDERR, $file, length($file));
  syswrite(STDERR, ' line ', 6);
  syswrite(STDERR, $line, length($line));
  syswrite(STDERR, "\n", 1);
  # Now go for broke.
  for (my $i = 1; my($p,$f,$l,$s,$h,$w,$e,$r) = caller($i); $i++) {
    my @a = ();
  for my $arg (@DB::args) {
    $_ = $arg;
    s/([\'\\])/\\$1/g;
    s/([^\0]*)/'$1'/
      unless /^(?: -?[\d.]+ | \*[\w:]* )$/x;
    s/([\200-\377])/sprintf("M-%c",ord($1)&0177)/eg;
    s/([\0-\37\177])/sprintf("^%c",ord($1)^64)/eg;
    push(@a, $_);
  }
  $w = $w ? '@ = ' : '$ = ';
  $a = $h ? '(' . join(', ', @a) . ')' : '';
  $e =~ s/\n\s*\;\s*\Z// if $e;
  $e =~ s/[\\\']/\\$1/g if $e;
  if ($r) {
    $s = "require '$e'";
  } elsif (defined $r) {
    $s = "eval '$e'";
  } elsif ($s eq '(eval)') {
    $s = "eval {...}";
  }
  $f = "file `$f'" unless $f eq '-e';
  my $mess = "$w$s$a called from $f line $l\n";
  syswrite(STDERR, $mess, length($mess));
  }

  kill 'KILL', $$ if $hook =~ /SIG_SEGV/;
  1;
}

sub DESTROY {
  my $bn = shift;
  $bn->CLOSER;
}

1;

__END__
